# Find libssh
#
# Find the libssh includes and library
# This module defines
#  LIBSSH_INCLUDE_DIRS, where to find header, etc.
#  LIBSSH_LIBRARIES, the libraries needed to use libssh.
#  LIBSSH_FOUND, If false, do not try to use libssh.
#  LIBSSH_INCLUDE_PREFIX, include prefix for libssh



MESSAGE(STATUS "Looking for LIBSSH...")
#SET(LIBSSH_LIB)
SET(LIBSSH_ROOT_DIR)
SET(LIBSSH_INCLUDE_DIR)
SET(LIBSSH_LIBRARY)
SET(LIBSSH_INCLUDE_FILE)


IF(PKG_CONFIG_FOUND) ## looking for CPPUNIT from pkg-config (new)
  pkg_check_modules (LIBSSH libssh>=0.8.1)
  # MESSAGE(STATUS "  Looking for libssh.so... - Found  ${LIBSSH_LIBRARY}")
  set(LIBSSH_LIB "${LIBSSH_LIBRARY_DIRS}")
  set(LIBSSH_INCLUDE_DIR "${LIBSSH_INCLUDE_DIRS}")

  #MESSAGE(STATUS "  POUR INFO  [${LIBSSH_LIBRARY_DIRS}] [${LIBSSH_LIB}] [${LIBSSH_LIBRARY}] [${LIBSSH_INCLUDE_DIRS}]")

ELSE(PKG_CONFIG_FOUND)
  
  find_library(
    LIBSSH_LIB
    NAMES libssh.so
    HINTS ssh
    PATHS ENV LD_LIBRARY_PATH
    ENV PATH
    /usr/lib
    /usr/lib64
    ${LD_LIBRARY_PATH}
    DOC "LIBSSH library"
    )
  
  
  # Main task
  
  if (LIBSSH_LIB)
    
    STRING(REGEX REPLACE "(.*)/libssh.so" "\\1" LIBSSH_LIBRARY "${LIBSSH_LIB}")
    STRING(REGEX REPLACE "(.*)/lib[64]*(.*)" "\\1" LIBSSH_ROOT_DIR "${LIBSSH_LIBRARY}")
    if ("${LIBSSH_ROOT_DIR}" STREQUAL "/usr")
      SET(LIBSSH_INCLUDE_DIR /usr/include/libssh /usr/include)
    else("${LIBSSH_ROOT_DIR}" STREQUAL "/usr")
      SET(LIBSSH_INCLUDE_DIR ${LIBSSH_ROOT_DIR}/include/libssh)
    endif("${LIBSSH_ROOT_DIR}" STREQUAL "/usr")

  endif(LIBSSH_LIB)
  
  find_file(LIBSSH_INCLUDE_FILE libssh/libssh.h PATHS ${LIBSSH_INCLUDE_DIR} NO_DEFAULT_PATH)
  if (LIBSSH_INCLUDE_FILE)
    MESSAGE(STATUS "Looking for libssh/libssh.h... - Found  ${LIBSSH_INCLUDE_FILE}")
  else(LIBSSH_INCLUDE_FILE)
    MESSAGE(STATUS "Looking for libssh/libssh.h... - Not Found in ${LIBSSH_INCLUDE_DIR}")
  endif()
  
  include(FindPackageHandleStandardArgs)
  find_package_handle_standard_args(LIBSSH
    DEFAULT_MSG
    LIBSSH_LIBRARY LIBSSH_INCLUDE_DIR)
  
  mark_as_advanced (LIBSSH_INCLUDE_FILE LIBSSH_LIBRARY)
endif(PKG_CONFIG_FOUND)


#MESSAGE(STATUS "======== [${LIBSSH_INCLUDE_DIR}]  [${LIBSSH_INCLUDE_FILE}]  [${LIBSSH_FOUND}]  [${LIBSSH_LIB}]  [${LIBSSH_LIBRARY}]  [${LIBSSH_ROOT_DIR}]")


if (LIBSSH_FOUND)
    MESSAGE(STATUS "Looking for LIBSSH... - Found")
else(LIBSSH_FOUND)
    MESSAGE(STATUS "Looking for LIBSSH... - Not Found")
endif(LIBSSH_FOUND)

#-----------------------------------------------------------------------
# End of File
#-----------------------------------------------------------------------
