#-------------------------------------------------------------------------
#-----put the path variable in cmake format ------------------------------
#-------------------------------------------------------------------------
file(TO_CMAKE_PATH ${CMAKE_INSTALL_PREFIX} CMAKE_INSTALL_PREFIX)

#-------------------------------------------------------------------------
#---General Build options-------------------------------------------------
#-------------------------------------------------------------------------
# use, i.e. don't skip the full RPATH for the build tree
set(CMAKE_SKIP_BUILD_RPATH  FALSE)
# when building, don't use the install RPATH already (but later on when installing)
set(CMAKE_BUILD_WITH_INSTALL_RPATH FALSE) 
# add the automatically determined parts of the RPATH
# which point to directories outside the build tree to the install RPATH
set(CMAKE_INSTALL_RPATH_USE_LINK_PATH TRUE)


#-------------------------------------------------------------------------
#the RPATH to be used when installing-------------------------------------
#-------------------------------------------------------------------------
if(rpath)
  set(CMAKE_INSTALL_RPATH "${CMAKE_INSTALL_PREFIX}/lib")
  set(CMAKE_BUILD_WITH_INSTALL_RPATH TRUE) 
endif()


#-------------------------------------------------------------------------
#---Avoid creating dependencies to 'non-statndard' header files ----------
#-------------------------------------------------------------------------
include_regular_expression("^[^.]+$|[.]h$|[.]icc$|[.]hxx$|[.]hpp$")


#-------------------------------------------------------------------------
#---Set all directories where to install parts of Uranie up to now everything
# is installed according to the setting of CMAKE_BINARY_DIR
#-------------------------------------------------------------------------
if(CMAKE_INSTALL_PREFIX_INITIALIZED_TO_DEFAULT AND NOT gnuinstall)
  message(STATUS "Setting default installation prefix CMAKE_INSTALL_PREFIX to ${CMAKE_BINARY_DIR}")
  set(CMAKE_INSTALL_PREFIX ${CMAKE_BINARY_DIR} CACHE PATH "Default installation of URANIE" FORCE)
endif()


#-------------------------------------------------------------------------
#---Add Installation Variables--------------------------------------------
#-------------------------------------------------------------------------
include(UranieInstallDirs)


#-----------------------------------------------------------------------
#----- Coverage option
# set coverage compilation option and link the gcov library
#-------------------------------------------------------------------------
if(WITH-COVERAGE STREQUAL "ON")
  message(STATUS "Checking for coverage option...  yes")
  set(CMAKE_CXX_FLAGS
    "${CMAKE_CXX_FLAGS} --coverage"
    CACHE STRING "Flags used for coverage builds."
    FORCE)
  set(CMAKE_EXE_LINKER_FLAGS
    "${CMAKE_EXE_LINKER_FLAGS} --coverage"
    CACHE STRING "Flags used for linking binaries during coverage builds."
    FORCE)

else(WITH-COVERAGE STREQUAL "ON")
    message(STATUS "Checking for coverage option... no ")
endif(WITH-COVERAGE STREQUAL "ON")


#-------------------------------------------------------------------------


