# ROOT Options extraction module

function(root_get option_value gROOT_method key_regex)
    find_program(root_executable root REQUIRED)

    execute_process(COMMAND ${root_executable} -l -q -e
        "cout << ${gROOT_method} << endl; return 0;"
        RESULT_VARIABLE result ERROR_VARIABLE error OUTPUT_VARIABLE options)
    if(NOT result EQUAL "0")
        message(FATAL_ERROR "${CMAKE_CURRENT_FUNCTION}: failed: ${error}")
    endif()
    separate_arguments(options_list NATIVE_COMMAND ${options})

    list(FILTER options_list INCLUDE REGEX "${key_regex}")
    if(options_list)
        string(REGEX REPLACE "${key_regex}" "" without_key ${options_list})
        set(${option_value} ${without_key} PARENT_SCOPE)
    endif()
endfunction()

function(root_get_option option_value key_regex)
    set(option)
    root_get(option "gROOT->GetConfigOptions()" ${key_regex})
    set(${option_value} ${option} PARENT_SCOPE)
endfunction()

function(root_get_feature feature_value key_regex)
    set(feature)
    root_get(feature "gROOT->GetConfigFeatures()" ${key_regex})
    set(${feature_value} ${feature} PARENT_SCOPE)
endfunction()

function(root_get_configuration configuration_value optional_argument)
    if(WIN32)
        set(SCRIPT_EXTENSION ".bat")
    endif()
    find_program(root_config_executable root-config${SCRIPT_EXTENSION} REQUIRED)
    execute_process(COMMAND ${root_config_executable} ${optional_argument}
        RESULT_VARIABLE result ERROR_VARIABLE error OUTPUT_VARIABLE configuration_output)
    if(NOT result EQUAL "0")
        message(FATAL_ERROR "${CMAKE_CURRENT_FUNCTION}: failed: ${error}")
    endif()
    string(STRIP ${configuration_output} configuration_output)
    set(${configuration_value} ${configuration_output} PARENT_SCOPE)
endfunction()
