//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TBaseModel.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   ven jui 13 10:11:28 CEST 2007
 
 $Revision$
 $Date$
 
 \brief Implementation of the class URANIE::DataServer::TBaseModel
 
 */
#include "iostream.h"
using std::cout;
using std::endl;

#include "TBaseModel.h"

ClassImp (URANIE::DataServer::TBaseModel)

namespace URANIE
{
namespace DataServer
{
TBaseModel::TBaseModel() :
        _sinput(""), _shidden(""), _soutput(""), _nlanguage(kEMPTY), _sfileNameExport(
                ""), _sfunctionNameExport(""), _niteration(0), _ninitialization(
                1), _dlearn(0.0), _nseed(0)
{
    SetNameTitle("DefaultUranieModel", "Model in the uranie tool");
}
TBaseModel::TBaseModel(TString sinput, TString soutput) :
        _sinput(sinput), _shidden(""), _soutput(soutput), _nlanguage(kEMPTY), _sfileNameExport(
                ""), _sfunctionNameExport(""), _niteration(0), _ninitialization(
                1), _dlearn(0.0), _nseed(0)
{
    SetNameTitle("UranieModel",
            Form("Model in the uranie tool input[%s] output[%s]",
                    _sinput.Data(), _soutput.Data()));
}
TBaseModel::TBaseModel(TString sname, TString sinput, TString shidden,
        TString soutput) :
        _sinput(sinput), _shidden(shidden), _soutput(soutput), _nlanguage(
                kEMPTY), _sfileNameExport(sname), _sfunctionNameExport(sname), _niteration(
                0), _ninitialization(1), _dlearn(0.0), _nseed(0)
{
    SetNameTitle(sname,
            Form(
                    "Model [%s] in the uranie tool input[%s] hidden[%s] output[%s]",
                    sname.Data(), sinput.Data(), shidden.Data(),
                    soutput.Data()));
}
TBaseModel::~TBaseModel()
{
}

void TBaseModel::initExport()
{
    // Default value of File Name is the name of Model
    if (_sfileNameExport == "")
        _sfileNameExport = GetName();
    // Default value of Function Name is the name of Model
    if (_sfunctionNameExport == "")
        _sfunctionNameExport = GetName();

    switch (_nlanguage)
    {
    case kC:
        _sfileNameExport += ".c";
        break;
    case kCXX:
        _sfileNameExport += ".cxx";
        break;
    case kFORTRAN:
        _sfileNameExport += ".f";
        break;
    }
}

void TBaseModel::printLog(Option_t *option)
{
    initExport();
    cout << "TBaseModel::printLog[" << option << "]" << endl;
    cout << "Name[" << GetName() << "] Title[" << GetTitle() << "]" << endl;
    cout << "_niteration[" << _niteration << "]" << endl;
    cout << "_ninitialization[" << _ninitialization << "]" << endl;
    cout << "_nseed[" << _nseed << "]" << endl;
    cout << "_dlearn[" << _dlearn << "]" << endl;
    cout << "_sinput[" << _sinput << "] _shidden[" << _shidden
            << "] : _soutput[" << _soutput << "]" << endl;
    cout << "_sfileNameExport[" << _sfileNameExport
            << "] : _sfunctionNameExport[" << _sfunctionNameExport << "]"
            << endl;
    switch (_nlanguage)
    {
    case kEMPTY:
        cout << "**   Language[kEMPTY]" << endl;
        break;
    case kC:
        cout << "**   Language[kC]" << endl;
        break;
    case kCXX:
        cout << "**   Language[kCXX]" << endl;
        break;
    case kFORTRAN:
        cout << "**   Language[kFORTRAN]" << endl;
        break;
    }
    cout << "fin de TBaseModel::printLog[" << option << "]" << endl;
}

} // Fin du namespace DataServer
} // Fin du namespace URANIE

