//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TBaseModel.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   ven jui 13 10:11:28 CEST 2007
 
 $Revision$
 $Date$
 
 \brief Interface of the class URANIE::DataServer::TBaseModel
 
 */

/*!
 @class URANIE::DataServer::TBaseModel
 
 @brief Description of the class TBaseModel.
 
 To write by the developper.
 
 */
#ifndef TBASEMODEL_H
#define TBASEMODEL_H
// ROOT
#include "TNamed.h"

// URANIE
#include "DataServer.h"

namespace URANIE
{
namespace DataServer
{
class TBaseModel: public TNamed
{
public:
    enum ELanguage
    {
        kEMPTY, kC, kCXX, kFORTRAN
    };
private:
    TString _sinput;
    TString _shidden;
    TString _soutput;
    ELanguage _nlanguage;
    TString _sfileNameExport;
    TString _sfunctionNameExport;
    Int_t _niteration;
    Int_t _ninitialization;
    Int_t _nseed;

    Double_t _dlearn;
public:
    TBaseModel();
    TBaseModel(TString sinput, TString soutput);
    TBaseModel(TString sname, TString sinput, TString shidden, TString soutput);
    //! Default destructor
    virtual ~TBaseModel();

    //---------------------------------------------
    /** @name Other  Setting and Getting 
     **/
    //@{
    void setName(TString str)
    {
        SetName(str);
    }
    void setInput(TString str)
    {
        _sinput = str;
    }
    void setHidden(TString str)
    {
        _shidden = str;
    }
    void setOutput(TString str)
    {
        _soutput = str;
    }
    void setLanguage(ELanguage val)
    {
        _nlanguage = val;
    }
    ELanguage getLanguage()
    {
        return _nlanguage;
    }

    void initExport();
    void setFileNameExport(TString str)
    {
        _sfileNameExport = str;
    }
    void setFunctionNameExport(TString str)
    {
        _sfunctionNameExport = str;
    }
    void setNIteration(Int_t n)
    {
        _niteration = n;
    }
    void setNInitialization(Int_t n)
    {
        _ninitialization = n;
    }
    void setSeed(Int_t n)
    {
        _nseed = n;
    }
    void setLearn(Double_t dval)
    {
        _dlearn = dval;
    }

    TString getInput()
    {
        return _sinput;
    }
    TString getHidden()
    {
        return _shidden;
    }
    TString getOutput()
    {
        return _soutput;
    }
    Int_t getNIteration()
    {
        return _niteration;
    }
    Int_t getNInitialization()
    {
        return _ninitialization;
    }
    Int_t getSeed()
    {
        return _nseed;
    }
    Double_t getLearn()
    {
        return _dlearn;
    }
    //@}

    //---------------------------------------------
    /** @name The  Log
     **/
    //@{
    void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::DataServer::TBaseModel, ID_DATASERVER)
    //Classe de
};

} // Fin du namespace DataServer
} // Fin du namespace URANIE
#endif
