//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TDiscreteAttribute.h
 \author 
 \date   
 
 $Revision$
 $Date$
 
 \brief  Class of attributes following a density of probability
 
 
 */
#ifndef TDISCRETEATTRIBUTE_H
#define TDISCRETEATTRIBUTE_H

#include "TString.h"
#include <iostream>

/*! \class URANIE::DataServer::TDiscreteAttribute
 This class is the parent of the discrete random variables classes. These 
 variables can take only a finite number of states. 
 Although these states are represented by real numbers, they are not supposed 
 to have an order (i.e. value 1 is not smaller or greater than value 2). The 
 consequence is that the following properties of random variables 
 are meaningless:
 - mean, variance and other moments 
 - minimum and maximum values
 - lower and upper bounds
 - Cumulative Distribution Function    
 
 Also, it is not strictly correct to talk about Probability Density Function.
 The equivalent in the discrete world would be the Probability Mass Function. 
 However, for the sake of simplicity, we are using the getPDF name to compute 
 the PMF.  
 */

// Uranie
#include "DataServer.h"
#include "TStochAttribut.h"

namespace URANIE
{
namespace DataServer
{
class TDiscreteAttribute: public TStochasticAttribute
{

    // Attributes
protected:

public:
    // -----------------------------------------------------------------------
    //  Constructors
    // -----------------------------------------------------------------------
    //! Default Constructor                            
    /*! 
     
     */
    TDiscreteAttribute();

    TDiscreteAttribute(TString str);
    TDiscreteAttribute(TString str, ELawType value);
    TDiscreteAttribute(TAttribute *object, ELawType value);
    virtual ~TDiscreteAttribute();

    virtual Double_t getPDF(Double_t x) = 0;
    virtual Double_t getInvCDF(Double_t x) = 0;

    virtual void printLog();ClassDef(URANIE::DataServer::TDiscreteAttribute, ID_DATASERVER)
};
} // Fin du namespace DataServer
} // Fin du namespace URANIE

#endif
