//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#include <string>
#include <vector>
#include <string.h>

#include <iostream>
using std::cout;
using std::endl;

#include "TTokenizer.h"

/*
 TTokenizer::TTokenizer(const char* delimiters, const char* quotes, int comment)
 {
 setQuotes(quotes);
 setDelimiters(delimiters);    
 this->comment = comment;
 }
 */

TTokenizer::TTokenizer(const std::string& Delimiters, const std::string& Quotes,
        int Comment)
{
    setQuotes(Quotes);
    setDelimiters(Delimiters);
    this->comment = Comment;
}

void TTokenizer::setDelimiters(const std::string& Delimiters)
{
    this->delimiters = Delimiters;
    nDelims = (int) strlen(Delimiters.c_str());
}

void TTokenizer::setQuotes(const std::string& Quotes)
{
    this->quotes = Quotes;
    nQuotes = (int) strlen(Quotes.c_str());
}

void TTokenizer::setComment(char Comment)
{
    this->comment = Comment;
}

std::vector<std::string> TTokenizer::getTokens(const std::string& s)
{
    std::vector < std::string > tokens;
    int pos = 0, len = (int) s.length();
    char* str = new char[len + 1];
    memcpy(str, s.c_str(), len);
    str[len] = 0;
    int start = -1;
    bool quoteOpened = false;
    char quoteChar = -1;
    while (pos < len)
    {
        if (comment != -1 && str[pos] == comment)
            break;
        if (quoteOpened && str[pos] == quoteChar)
        {
            str[pos] = 0;
            tokens.push_back(std::string(str + start));
            start = -1;
            quoteOpened = false;
        }
        else
        {
            if (start == -1 && !isDelimiter(str[pos]))
            {
                if (quotes.size() != 0 && isQuote(str[pos]))
                {
                    quoteOpened = true;
                    quoteChar = str[pos];
                    start = pos + 1;
                }
                else
                    start = pos;
            }
            if ((start != -1 && isDelimiter(str[pos]))
                    || (quotes.size() != 0 && start != -1 && !quoteOpened
                            && str[pos] == quoteChar))
            {
                if (!quoteOpened)
                {
                    str[pos] = 0;
                    tokens.push_back(std::string(str + start));
                    start = -1;
                }
            }
        }
        pos++;
    }
    if (quoteOpened)
    {
        delete[] str;
        str = NULL;
        tokens.clear();
        return tokens;
    }
    if (start != -1)
    {
        tokens.push_back(std::string(str + start));
    }
    delete[] str;
    str = NULL;
    return tokens;
}

bool TTokenizer::isDelimiter(char chr)
{
    for (int i = 0; i < nDelims; i++)
        if (delimiters[i] == chr)
            return true;
    return false;
}

bool TTokenizer::isQuote(char chr)
{
    for (int i = 0; i < nQuotes; i++)
        if (quotes[i] == chr)
            return true;
    return false;
}
