//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TStochAttribut.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $date$

 \brief  Implementation of the class URANIE::DataServer::TTriangularPossibility

 Created: 2010-07-07 15:52:43

 */
#include <iostream>

#include "TTriangularPossibility.h"
#include "UExceptions.h"

ClassImp (URANIE::DataServer::TTriangularPossibility)
namespace URANIE
{
namespace DataServer
{
// -----------------------------------------------------------------------
//  Constructors
// -----------------------------------------------------------------------
TTriangularPossibility::TTriangularPossibility(TString str) :
        TPossibilityAttribute(str, TPossibilityAttribute::kTriangular, 0.0, 1.0,
                TPossibilityAttribute::kNotDefined), _dmode(0.5)
{

}

TTriangularPossibility::TTriangularPossibility(TString str, Double_t min,
        Double_t max, Double_t mod,
        TPossibilityAttribute::EMonotonousRelationship monotonous) :
        TPossibilityAttribute(str, TPossibilityAttribute::kTriangular, min, max,
                monotonous), _dmode(mod)
{
}

TTriangularPossibility::~TTriangularPossibility()
{
    if (_blog)
    {
        cout << "Destructeur::TTriangularPossibility[" << this << "] sname["
                << GetName() << "]" << endl;
    }

}
// -----------------------------------------------------------------------
//  setMaximumPossibility
// -----------------------------------------------------------------------
void TTriangularPossibility::setMode(Double_t dval)
{
    try
    {
        if (dval < getLowerBound())
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    Form(
                            "TTriangularPossibility::setMaximumPossibility\n the x value [%f] is lesser than the lower bound [%f]",
                            dval, getLowerBound()));
        if (dval > getUpperBound())
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    Form(
                            "TTriangularPossibility::setMaximumPossibility\n the x value [%f] is greater than the upper bound [%f]",
                            dval, getUpperBound()));

        _dmode = dval;
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
        throw(ue);
    }
}

// -----------------------------------------------------------------------
//  printLog
// -----------------------------------------------------------------------
void TTriangularPossibility::printLog()
{
    TAttribute::printLog();
    cout << "*******************************" << endl;
    cout << "** TTriangularPossibility::printLog  ******" << endl;
    cout << "**   Possibility [" << TPossibilityAttribute::kTriangular << "]"
            << endl;
    cout << "**   Inf[" << getLowerBound() << "] Mod[" << _dmode << "] Sup["
            << getUpperBound() << "]" << endl;
    TPossibilityAttribute::printLog();
    cout << "** End Of TTriangularPossibility::printLog  ******" << endl;
    cout << "*******************************" << endl;

}

} // Fin du namespace DataServer
} // Fin du namespace URANIE
