//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTAttribute.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TAttribute
 
 Cr�e le : mer nov 21 07:59:24 2007

 \par Modifications
 \li Cedric Forgues (C.S.) 2009/06/16
 Tests pour la DI 49.
 \li Cedric Forgues (C.S.) 2009/12/03
 Ajout de la m�thode testKeyCardinal, testKeyMethod1, testSetFieldOfKey1, testSetFieldOfKey2, throwExceptionTestKeyMethod, throwExceptionTestKeyMethod2, throwExceptionTestDeprecated
 // *****************************************************************************

 */

#define DBL_PRECISION_COMPARE 1.e-12
#include "TestTAttribute.h"
#include "TString.h"

#include <iostream>
using namespace std;

TAttributeTest::TAttributeTest()
{
}

TAttributeTest::~TAttributeTest()
{
}

void TAttributeTest::setUp()
{
    att1 = new URANIE::DataServer::TAttribute("att1");
    att1bis = new URANIE::DataServer::TAttribute("att1bis", "title 1bis");
    att2 = new URANIE::DataServer::TAttribute();
    att3 = new URANIE::DataServer::TAttribute("att3", 0.5, 1.5);
    attV = new URANIE::DataServer::TAttribute("attV", URANIE::DataServer::TAttribute::kVector) ;
    attS = new URANIE::DataServer::TAttribute("attS", URANIE::DataServer::TAttribute::kString) ;
    tds1 = new URANIE::DataServer::TDataServer("tds1", "my title 1");
}

void TAttributeTest::tearDown()
{
    delete att1;
    att1 = NULL;
    delete att1bis;
    att1bis = NULL;
    delete att2;
    att2 = NULL;
    delete att3;
    att3 = NULL;
    delete attV;
    attV = NULL;
    delete attS;
    attS = NULL;
    delete tds1;
    tds1 = NULL;
}

////////////////////////////////
///                          ///
///         Tests            ///
///                          ///
////////////////////////////////

//- TAttribute(const TAttribute &attr);
//- TAttribute * Clone(const char *newname) const;

void TAttributeTest::testBasisAttributes()
{
    CPPUNIT_ASSERT(att2->GetName() == TString("_uranie_attribut_att_"));
    CPPUNIT_ASSERT(att2->GetTitle() == TString("attribute title"));
    CPPUNIT_ASSERT(
            att2->getOrigin() == URANIE::DataServer::TAttribute::kAttribute);
    CPPUNIT_ASSERT(
            att2->getDataType() == URANIE::DataServer::TAttribute::kInconnu);
    CPPUNIT_ASSERT(att1->GetName() == TString("att1"));
    CPPUNIT_ASSERT(att1->GetTitle() == TString("att1"));
    CPPUNIT_ASSERT(
            att1->getOrigin() == URANIE::DataServer::TAttribute::kAttribute);
    CPPUNIT_ASSERT(
            att1->getDataType() == URANIE::DataServer::TAttribute::kDefault);

    CPPUNIT_ASSERT(att3->GetName() == TString("att3"));
    CPPUNIT_ASSERT(att3->GetTitle() == TString("att3"));
    CPPUNIT_ASSERT(
            att3->getOrigin() == URANIE::DataServer::TAttribute::kAttribute);
    CPPUNIT_ASSERT(
            att3->getDataType() == URANIE::DataServer::TAttribute::kDefault);
}

// void TAttributeTest::testClone()
// {
//   att3->setLog();
//   att3->Clone("y");
//   URANIE::DataServer::TAttribute * Y = dynamic_cast<URANIE::DataServer::TAttribute *>(gROOT->FindObject("y"));
//   CPPUNIT_ASSERT( Y->GetTitle()    == att3->GetTitle() );
//   CPPUNIT_ASSERT( Y->getOrigin()   == att3->GetTitle()  );
//   CPPUNIT_ASSERT( Y->getDataType() == att3->getDataType() );
//   CPPUNIT_ASSERT( Y->getLowerBound() == att3->getLowerBound() );
//   CPPUNIT_ASSERT( Y->getUpperBound() == att3->getUpperBound() );
// }

void TAttributeTest::testNames()
{
    TString snameExpected("toto");

    att1->SetName(snameExpected);
    att2->SetName(snameExpected);

    CPPUNIT_ASSERT(att1->GetName() == snameExpected);
    CPPUNIT_ASSERT(!strcmp(att1->GetName(), att2->GetName()));
}

void TAttributeTest::testLabels()
{
    TString snameExpected("toto"), sdefaultname("_uranie_attribut_att_");
    att1->SetName(snameExpected);

    CPPUNIT_ASSERT(att1->GetName() == snameExpected);
    CPPUNIT_ASSERT(att2->GetName() == sdefaultname);
    CPPUNIT_ASSERT(strcmp(att1->GetName(), att2->GetName()));
}

void TAttributeTest::testKeyMethod()
{
    TString sexpectedFileName = "TestFile1.in";
    TString sexpectedKey = "Att1";
    TString sexpectedFormat = "%e";
    TString sexpectedFormat2 = "%f";
    URANIE::DataServer::TAttributeFileKey::EFileType sexpectedFileType =
            URANIE::DataServer::TAttributeFileKey::kKey;
    URANIE::DataServer::TAttributeFileKey::EFileType sexpectedFileType2 =
            URANIE::DataServer::TAttributeFileKey::kNewLine;

    att1->setFileKey("TestFile1.in", "Att1");
    att1->setFileKey("TestFile2.in", "Att1", "%f",
            URANIE::DataServer::TAttributeFileKey::kNewLine);
    list<URANIE::DataServer::TAttributeFileKey *> KeyList = att1->getKeyList();
    list<URANIE::DataServer::TAttributeFileKey *>::iterator first =
            KeyList.begin();
    list<URANIE::DataServer::TAttributeFileKey *>::reverse_iterator last =
            KeyList.rbegin();
    CPPUNIT_ASSERT((*first)->_sKey == sexpectedKey);
    CPPUNIT_ASSERT(gSystem->BaseName((*first)->_sFile) == sexpectedFileName);
    CPPUNIT_ASSERT((*first)->_sFormat == sexpectedFormat);
    CPPUNIT_ASSERT((*last)->_sFormat == sexpectedFormat2);
    CPPUNIT_ASSERT((*first)->_sFileType == sexpectedFileType);
    CPPUNIT_ASSERT((*last)->_sFileType == sexpectedFileType2);

    //Number of entered keys 
    CPPUNIT_ASSERT(KeyList.size() == 2);
}

// Tests when the TDataServer is not filled with setFileKey() that KeyList.size is null.
void TAttributeTest::testKeyCardinal()
{
    list<URANIE::DataServer::TAttributeFileKey *> KeyList = att1->getKeyList();
    CPPUNIT_ASSERT(KeyList.size() == 0);

    //Tests when the TDataServer is filled with one setFileKey() that KeyList.size is 1.
    att1->setFileKey("TestFile1.in", "ATT1");
    KeyList = att1->getKeyList();
    CPPUNIT_ASSERT(KeyList.size() == 1);

    //Tests when the TDataServer is filled with 5 setFileKey() with 5 different files and 5 different file type that KeyList.size is 5.
    att1->setFileKey("TestFile2.in", "ATT2", "%e",
            URANIE::DataServer::TAttributeFileKey::kFlag);
    att1->setFileKey("TestFile3.in", "ATT3", "%e",
            URANIE::DataServer::TAttributeFileKey::kKey);
    att1->setFileKey("TestFile4.in", "ATT4", "%e",
            URANIE::DataServer::TAttributeFileKey::kNewLine);
    att1->setFileKey("TestFile5.in", "ATT5", "%e",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);
    KeyList = att1->getKeyList();
    CPPUNIT_ASSERT(KeyList.size() == 5);
}

//Tests when the TDataServer is filled with a setFileKey() with juste a file and key.
void TAttributeTest::testKeyDefaultValues()
{
    TString sexpectedFileName = "TestFile1.in";
    TString sexpectedFormat = "%e";
    URANIE::DataServer::TAttributeFileKey::EFileType sexpectedFileType =
            URANIE::DataServer::TAttributeFileKey::kKey;
    att1->setFileKey("TestFile1.in");
    list<URANIE::DataServer::TAttributeFileKey *> KeyList = att1->getKeyList();
    list<URANIE::DataServer::TAttributeFileKey *>::iterator first =
            KeyList.begin();
    CPPUNIT_ASSERT_MESSAGE(Form("Key is  %s", ((*first)->_sKey).Data()),
            (*first)->_sKey == "att1");
    CPPUNIT_ASSERT_MESSAGE(Form("File is  %s", ((*first)->_sFile).Data()),
            gSystem->BaseName((*first)->_sFile) == sexpectedFileName);
    CPPUNIT_ASSERT_MESSAGE(Form("Format is  %s", ((*first)->_sFormat).Data()),
            (*first)->_sFormat == sexpectedFormat);
    CPPUNIT_ASSERT((*first)->_sFileType == sexpectedFileType);
}

//
void TAttributeTest::testKeyMethod1()
{
    TString sexpectedFileName = "TestFile1.in";
    TString sexpectedFileName2 = "TestFile2.in";
    TString sexpectedFormat = "%e";
    URANIE::DataServer::TAttributeFileKey::EFileType sexpectedFileType =
            URANIE::DataServer::TAttributeFileKey::kKey;
    URANIE::DataServer::TAttributeFileKey::EFileType sexpectedFileType2 =
            URANIE::DataServer::TAttributeFileKey::kFlag;

    att1->setFileKey("TestFile1.in");
    att1->setFileKey("TestFile2.in", "ATT2", "%e",
            URANIE::DataServer::TAttributeFileKey::kFlag);
    list<URANIE::DataServer::TAttributeFileKey *> KeyList = att1->getKeyList();
    list<URANIE::DataServer::TAttributeFileKey *>::iterator first =
            KeyList.begin();
    list<URANIE::DataServer::TAttributeFileKey *>::reverse_iterator last =
            KeyList.rbegin();
    CPPUNIT_ASSERT((*first)->_sKey == "att1");
    CPPUNIT_ASSERT((*last)->_sKey == "ATT2");

    CPPUNIT_ASSERT(gSystem->BaseName((*first)->_sFile) == sexpectedFileName);
    CPPUNIT_ASSERT(gSystem->BaseName((*last)->_sFile) == sexpectedFileName2);

    CPPUNIT_ASSERT((*first)->_sFormat == sexpectedFormat);
    CPPUNIT_ASSERT((*last)->_sFormat == sexpectedFormat);

    CPPUNIT_ASSERT((*first)->_sFileType == sexpectedFileType);
    CPPUNIT_ASSERT((*last)->_sFileType == sexpectedFileType2);
}

// By using for a first time a setFileKey(), the TEventList _sEventList is NULL.
void TAttributeTest::testSetFieldOfKey1()
{
    att1->setFileKey("TestFile1.in");
    list<URANIE::DataServer::TAttributeFileKey *> KeyList = att1->getKeyList();
    list<URANIE::DataServer::TAttributeFileKey *>::iterator first =
            KeyList.begin();
    CPPUNIT_ASSERT((*first)->_sEventList == NULL);
}

void TAttributeTest::testSetFieldOfKey2()
{
    att1->setFileKey("TestFile1.in", "ATT1", "%e",
            URANIE::DataServer::TAttributeFileKey::kFlag);
    att1->setFieldOfKey(0, 3);
    att1->setFieldOfKey(0, 7);
    att1->setFieldOfKey(0, 12);
    list<URANIE::DataServer::TAttributeFileKey *> KeyList = att1->getKeyList();
    list<URANIE::DataServer::TAttributeFileKey *>::iterator first =
            KeyList.begin();
    list<URANIE::DataServer::TAttributeFileKey *>::reverse_iterator last =
            KeyList.rbegin();
    CPPUNIT_ASSERT((*first)->_sEventList->GetEntry(0) == 3);
    CPPUNIT_ASSERT((*first)->_sEventList->GetEntry(1) == 7);
    CPPUNIT_ASSERT((*first)->_sEventList->GetEntry(2) == 12);
}

void TAttributeTest::testGetSetOrigin()
{
    CPPUNIT_ASSERT(
            att2->getOrigin() == URANIE::DataServer::TAttribute::kAttribute);
    CPPUNIT_ASSERT(
            att1->getOrigin() == URANIE::DataServer::TAttribute::kAttribute);
    CPPUNIT_ASSERT(
            att3->getOrigin() == URANIE::DataServer::TAttribute::kAttribute);

    att3->setOrigin(URANIE::DataServer::TAttribute::kInternal);
    CPPUNIT_ASSERT(
            att3->getOrigin() == URANIE::DataServer::TAttribute::kInternal);
}

void TAttributeTest::testGetSetDataType()
{
    CPPUNIT_ASSERT(
            att1->getDataType() == URANIE::DataServer::TAttribute::kDefault);
    att1->setDataType("real");
    CPPUNIT_ASSERT(
            att1->getDataType() == URANIE::DataServer::TAttribute::kReal);
    att1->setDataType(URANIE::DataServer::TAttribute::kCategory);
    CPPUNIT_ASSERT(
            att1->getDataType() == URANIE::DataServer::TAttribute::kCategory);
}

void TAttributeTest::testGetSetTitle()
{
    CPPUNIT_ASSERT(att2->GetTitle() == TString("attribute title"));
    CPPUNIT_ASSERT(att1->GetTitle() == TString("att1"));
    CPPUNIT_ASSERT(att3->GetTitle() == TString("att3"));

    att1->setTitle("New Title");
    CPPUNIT_ASSERT(att1->GetTitle() == TString("New Title"));

    TString titlestring = "New Title";
    att1->setTitle(titlestring);
    CPPUNIT_ASSERT(att1->GetTitle() == TString("New Title"));
}

void TAttributeTest::testGetSetBounds()
{
    CPPUNIT_ASSERT(!att2->isUpperBounded());
    CPPUNIT_ASSERT(!att2->isBounded());
    CPPUNIT_ASSERT_EQUAL(att2->getLowerBound(), -1.42387e+64);
    CPPUNIT_ASSERT_EQUAL(att2->getUpperBound(), 1.42387e+64);

    CPPUNIT_ASSERT(!att1bis->isUpperBounded());
    CPPUNIT_ASSERT(!att1bis->isBounded());
    CPPUNIT_ASSERT_EQUAL(att1bis->getLowerBound(), -1.42387e+64);
    CPPUNIT_ASSERT_EQUAL(att1bis->getUpperBound(), 1.42387e+64);

    CPPUNIT_ASSERT(!att1->isUpperBounded());
    CPPUNIT_ASSERT(!att1->isBounded());
    CPPUNIT_ASSERT_EQUAL(att1->getLowerBound(), -1.42387e+64);
    CPPUNIT_ASSERT_EQUAL(att1->getUpperBound(), 1.42387e+64);

    CPPUNIT_ASSERT(att3->isUpperBounded());
    CPPUNIT_ASSERT(att3->isBounded());
    CPPUNIT_ASSERT_EQUAL(att3->getLowerBound(), 0.5);
    CPPUNIT_ASSERT_EQUAL(att3->getUpperBound(), 1.5);

    att1->setLowerBound(-2.);
    att1->setUpperBound(2.);
    CPPUNIT_ASSERT_EQUAL(att1->getLowerBound(), -2.);
    CPPUNIT_ASSERT_EQUAL(att1->getUpperBound(), 2.);
    CPPUNIT_ASSERT(att1->isBounded());

    att2->setBounds(-2., 2.);
    CPPUNIT_ASSERT_EQUAL(att2->getLowerBound(), -2.);
    CPPUNIT_ASSERT_EQUAL(att2->getUpperBound(), 2.);
    CPPUNIT_ASSERT(att2->isBounded());
    CPPUNIT_ASSERT(att2->isUpperBounded());
}

void TAttributeTest::testGetSetMeanStd()
{
    tds1->fileDataRead("flowrateUniformDesign.dat");
    tds1->computeStatistic("rw");
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("rw")->getMean(), 0.1,
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("rw")->getStd(),
            0.0312212706649727, DBL_PRECISION_COMPARE);
    tds1->getAttribute("rw")->setMean(0.5);
    tds1->getAttribute("rw")->setStd(0.25);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("rw")->getMean(), 0.5,
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("rw")->getStd(), 0.25,
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_EQUAL(tds1->getAttribute("rw")->getMean(), 0.5);
    CPPUNIT_ASSERT_EQUAL(tds1->getAttribute("rw")->getStd(), 0.25);

    URANIE::DataServer::TDataServer *tds = new URANIE::DataServer::TDataServer("tds", "my title");
    tds->fileDataRead("myDataTimeEvol.dat");
    tds->computeStatistic("t");

    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds->getAttribute("t")->getMean(0), 0, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds->getAttribute("t")->getStd(0), 0, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds->getAttribute("t")->getMean(5), 5, DBL_PRECISION_COMPARE);
    tds->getAttribute("t")->setMean(0.99,5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds->getAttribute("t")->getMean(5), 0.99, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds->getAttribute("t")->getStd(5), 0, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds->getAttribute("t")->getMean(10), 4.5, DBL_PRECISION_COMPARE);

    delete tds;
}

void TAttributeTest::testGetSetUnity()
{
    CPPUNIT_ASSERT(att1->getUnity() == "");
    att1->setUnity(TString("mms"));
    CPPUNIT_ASSERT(att1->getUnity() == TString("mms"));
}

void TAttributeTest::testGetLegend()
{
    att1->setTitle("Title");
    CPPUNIT_ASSERT(att1->getLegend() == TString("Title"));
    att1->setUnity(TString("mms"));
    CPPUNIT_ASSERT(att1->getLegend() == TString("Title [mms]"));
}

void TAttributeTest::testGetSetNote()
{
    att1->setNote(TString("Nouvelle Note"));
    CPPUNIT_ASSERT(att1->getNote() == TString("Nouvelle Note"));
    att1->setNote(TString(""));
    CPPUNIT_ASSERT(att1->getNote() == TString(""));
}

void TAttributeTest::testGetSetMinimum()
{
    tds1->fileDataRead("myDataTimeEvol.dat");
    tds1->computeStatistic("t");

    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("t")->getMinimum(0), 0, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("t")->getMinimum(5), 5, DBL_PRECISION_COMPARE);
    tds1->getAttribute("t")->setMinimum(0.99,5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("t")->getMinimum(5), 0.99, DBL_PRECISION_COMPARE);
}

void TAttributeTest::testGetSetMaximum()
{
    tds1->fileDataRead("myDataTimeEvol.dat");
    tds1->computeStatistic("t");

    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("t")->getMaximum(0), 0, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("t")->getMaximum(5), 5, DBL_PRECISION_COMPARE);
    tds1->getAttribute("t")->setMaximum(0.99,5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("t")->getMaximum(5), 0.99, DBL_PRECISION_COMPARE);
}

void TAttributeTest::testGetSetQuantile()
{
  for(unsigned int iel=0; iel<3; iel++)
    {
      att2->setQuantile(0.5, (double)iel*0.5, iel );
      att2->setQuantile(0.9, (double)iel*0.9, iel );
    }

  for(unsigned int iel=0; iel<3; iel++)
    {
      int nbQ=0;
      att2->getQuantilesSize(nbQ,iel);
      CPPUNIT_ASSERT_DOUBLES_EQUAL(nbQ,2,DBL_PRECISION_COMPARE);
    }

  double proba[2];
  for(unsigned int iel=0; iel<3; iel++)
    {
      double quantile[2];
      att2->getQuantiles(proba,quantile,iel);
      CPPUNIT_ASSERT_DOUBLES_EQUAL(proba[0],0.5,DBL_PRECISION_COMPARE);
      CPPUNIT_ASSERT_DOUBLES_EQUAL(quantile[0],(double)iel*0.5,DBL_PRECISION_COMPARE);
      CPPUNIT_ASSERT_DOUBLES_EQUAL(proba[1],0.9,DBL_PRECISION_COMPARE);
      CPPUNIT_ASSERT_DOUBLES_EQUAL(quantile[1],(double)iel*0.9,DBL_PRECISION_COMPARE);
    }

  for(unsigned int iel=0; iel<3; iel++)
    for(unsigned int iq=0; iq<2; iq++)
      {
	double quantile;
	att2->getQuantile(proba[iq],quantile,iel);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(quantile,(double)iel*proba[iq],DBL_PRECISION_COMPARE);
      }

  for(unsigned int iq=0; iq<2; iq++)
    {
      vector<double> quantile;
      att2->getQuantileVector(proba[iq],quantile);
      CPPUNIT_ASSERT_DOUBLES_EQUAL(quantile.size(),3,DBL_PRECISION_COMPARE);
      CPPUNIT_ASSERT_DOUBLES_EQUAL(quantile[0],0,DBL_PRECISION_COMPARE);
      CPPUNIT_ASSERT_DOUBLES_EQUAL(quantile[1],1*proba[iq],DBL_PRECISION_COMPARE);
      CPPUNIT_ASSERT_DOUBLES_EQUAL(quantile[2],2*proba[iq],DBL_PRECISION_COMPARE);
    }

  vector<double> quantile;
  proba[0]=0.4;
  att2->setQuantile(proba[0], 0.4, 0);
  CPPUNIT_ASSERT_THROW(att2->getQuantileVector(proba[0], quantile), URANIE::Exceptions::UWarningExceptions);
  CPPUNIT_ASSERT_DOUBLES_EQUAL(quantile.size(),0,DBL_PRECISION_COMPARE);
}

void TAttributeTest::testGetInputOutput()
{
    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kInput,
            att3->getInputOutput());
    att3->setOutput();
    CPPUNIT_ASSERT(
            att3->getInputOutput() == URANIE::DataServer::TAttribute::kOutput);
    att3->setInput();
    CPPUNIT_ASSERT(
            att3->getInputOutput() == URANIE::DataServer::TAttribute::kInput);
}

void TAttributeTest::testGetKeyList()
{
    att3->setFileKey("fichier1", "X");
    att3->setFileKey("fichier2", "xXx");
    list<URANIE::DataServer::TAttributeFileKey *> KeyList = att3->getKeyList();
    list<URANIE::DataServer::TAttributeFileKey *>::iterator it =
            KeyList.begin();
    CPPUNIT_ASSERT(gSystem->BaseName((*it)->_sFile) == TString("fichier1"));
    CPPUNIT_ASSERT((*it)->_sKey == "X");
    CPPUNIT_ASSERT((*it)->_sFormat == "%e");
    CPPUNIT_ASSERT(
            (*it)->_sFileType == URANIE::DataServer::TAttributeFileKey::kKey);
    ++it;
    CPPUNIT_ASSERT(gSystem->BaseName((*it)->_sFile) == TString("fichier2"));
    CPPUNIT_ASSERT((*it)->_sKey == "xXx");
    CPPUNIT_ASSERT((*it)->_sFormat == "%e");
    CPPUNIT_ASSERT(
            (*it)->_sFileType == URANIE::DataServer::TAttributeFileKey::kKey);
}

void TAttributeTest::testGetSetFormatSubstitue()
{
    CPPUNIT_ASSERT(att3->getFormatToSubstitute() == TString("%e"));
    att3->setFormatToSubstitute("%f");
    CPPUNIT_ASSERT(att3->getFormatToSubstitute() == TString("%f"));
}

void TAttributeTest::testIsInputOutput()
{
    att2->setInput();
    CPPUNIT_ASSERT(att2->isInput() == kTRUE);
    CPPUNIT_ASSERT(att2->isOutput() == kFALSE);
    att2->setOutput();
    CPPUNIT_ASSERT(att2->isOutput() == kTRUE);
    CPPUNIT_ASSERT(att2->isInput() == kFALSE);
    
    att2->setInput();
    att2->setOrigin(URANIE::DataServer::TAttribute::kConstant);
    CPPUNIT_ASSERT(att2->isInput() == kTRUE);
    CPPUNIT_ASSERT(att2->isOutput() == kFALSE);
    att2->setOutput();
    CPPUNIT_ASSERT(att2->isOutput() == kFALSE);
    CPPUNIT_ASSERT(att2->isInput() == kFALSE);
}

void TAttributeTest::testShareMethods()
{
    CPPUNIT_ASSERT(att2->getShare() == 1);
    att2->addShare();
    CPPUNIT_ASSERT(att2->getShare() == 2);
    att2->delShare();
    att2->delShare();
    CPPUNIT_ASSERT(att2->getShare() == 0);
    att2->initShare();
    CPPUNIT_ASSERT(att2->getShare() == 1);
}

void TAttributeTest::testGetSetDefault()
{
    TString Def="1.0";
    CPPUNIT_ASSERT(att2->hasDefault() == false);
    CPPUNIT_ASSERT(att2->getDefault()=="");
    att2->setDefault( Def );
    CPPUNIT_ASSERT(att2->hasDefault() == true);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, atof(att2->getDefault()),  DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(att2->getDefault("%1.1f")=="1.0");
    CPPUNIT_ASSERT(att2->getDefault("%1.5f")=="1.00000");
    CPPUNIT_ASSERT(att2->getDefault("%1.5f")!="1.0");
    
    vector<double> def={1.,2.,3.}, getdef;
    CPPUNIT_ASSERT(attV->hasDefault() == false);
    CPPUNIT_ASSERT(attV->getDefault()=="");
    attV->setDefault("1.0,2.0,3.0");
    CPPUNIT_ASSERT(attV->hasDefault() == true);
    CPPUNIT_ASSERT(attV->getDefaultVector(&getdef));
    for(int i=0; i<3; i++)
        CPPUNIT_ASSERT_DOUBLES_EQUAL(def.at(i), getdef.at(i), DBL_PRECISION_COMPARE);

    attV->setDefault("");
    CPPUNIT_ASSERT(attV->hasDefault() == true);
    CPPUNIT_ASSERT(attV->getDefaultVector(&getdef));
    CPPUNIT_ASSERT_EQUAL(0, int(getdef.size()));

    string defStr="tutu", defaultStr="";
    CPPUNIT_ASSERT(attS->getDefault()=="");
    CPPUNIT_ASSERT(attS->hasDefault() == false);
    attS->setDefaultString(defStr);
    CPPUNIT_ASSERT(attS->hasDefault() == true);
    CPPUNIT_ASSERT( (attS->getDefault()==defStr) );
}

void TAttributeTest::testGetSetDefaultValue()
{
    Double_t defaultValue=0;
    CPPUNIT_ASSERT(att2->hasDefault() == false);
    att2->setDefaultValue(1.0);
    CPPUNIT_ASSERT(att2->hasDefault() == true);
    CPPUNIT_ASSERT(att2->getDefaultValue(defaultValue));
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, defaultValue, DBL_PRECISION_COMPARE);
    att3->setDefaultValue(1.0);
    CPPUNIT_ASSERT(att3->getDefaultValue(defaultValue));
}

void TAttributeTest::testGetSetDefaultVector()
{
    vector<double> def={1.,2.,3.}, getdef;
    CPPUNIT_ASSERT(attV->hasDefault() == false);
    attV->setDefaultVector(def);
    CPPUNIT_ASSERT(attV->hasDefault() == true);
    CPPUNIT_ASSERT(attV->getDefaultVector(&getdef));
    for(int i=0; i<3; i++)
        CPPUNIT_ASSERT_DOUBLES_EQUAL(def.at(i), getdef.at(i), DBL_PRECISION_COMPARE);

    def.clear();
    attV->setDefaultVector(def);
    CPPUNIT_ASSERT(attV->getDefaultVector(&getdef));
    CPPUNIT_ASSERT_EQUAL(0, int(getdef.size()));
}

void TAttributeTest::testGetSetDefaultString()
{
    string defStr="tutu", defaultStr="";
    CPPUNIT_ASSERT(attS->hasDefault() == false);
    attS->setDefaultString(defStr);
    CPPUNIT_ASSERT(attS->hasDefault() == true);
    CPPUNIT_ASSERT(attS->getDefaultString(defaultStr));
    CPPUNIT_ASSERT( (defaultStr==defStr) );
}

void TAttributeTest::testGetSetStepValue()
{
    Double_t stepValue;
    att2->setStepValue(-2.0);
    CPPUNIT_ASSERT(att2->getStepValue(stepValue));
    att3->setStepValue(1.0);
    CPPUNIT_ASSERT(att3->getStepValue(stepValue));
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TAttributeTest::throwExceptionWithNegativeStd()
{
    CPPUNIT_ASSERT_THROW(att2->setStd(-2.),
            URANIE::Exceptions::UErrorExceptions);
}

void TAttributeTest::throwExceptionWithNegativeElementForStatistic()
{
  CPPUNIT_ASSERT_THROW(att2->setMinimum(2,-1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->setMaximum(2,-1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->setMean(2,-1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->setStd(2,-1), URANIE::Exceptions::UErrorExceptions);

  CPPUNIT_ASSERT_THROW(att2->getMinimum(-1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getMaximum(-1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getMean(-1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getStd(-1), URANIE::Exceptions::UErrorExceptions);
}

void TAttributeTest::throwExceptionTooBigElementForStatistic()
{
  CPPUNIT_ASSERT_THROW(att2->setMinimum(2,1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->setMaximum(2,1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->setMean(2,1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->setStd(2,1), URANIE::Exceptions::UErrorExceptions);
  
  CPPUNIT_ASSERT_THROW(att2->getMinimum(1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getMaximum(1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getMean(1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getStd(1), URANIE::Exceptions::UErrorExceptions);
}

void TAttributeTest::throwExceptionWithNegativeElementForQuantile()
{
  double proba=0.95, quantile=0; int size=0;
  vector<double> vec;
  CPPUNIT_ASSERT_THROW(att2->setQuantile(proba,quantile,-1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getQuantiles(&proba,&quantile,-1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getQuantile(proba,quantile,-1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getQuantilesSize(size,-1), URANIE::Exceptions::UErrorExceptions);
}

void TAttributeTest::throwExceptionTooBigElementForQuantile()
{
  double proba=0.95, quantile=0;  int size=0;
  vector<double> vec;
  CPPUNIT_ASSERT_THROW(att2->setQuantile(proba,quantile,1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getQuantiles(&proba,&quantile,1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getQuantile(proba,quantile,1), URANIE::Exceptions::UErrorExceptions);
  CPPUNIT_ASSERT_THROW(att2->getQuantilesSize(size,1), URANIE::Exceptions::UErrorExceptions);
}

void TAttributeTest::throwExceptionWarningQuantileNotComputed()
{
  double proba=0.95, quantile=1.25, proba2=0.75;
  att2->setQuantile(proba,quantile,0);
  vector<double> vec;
  CPPUNIT_ASSERT_THROW(att2->getQuantile(proba2,quantile,0), URANIE::Exceptions::UWarningExceptions);
  CPPUNIT_ASSERT_THROW(att2->getQuantileVector(proba2, vec), URANIE::Exceptions::UWarningExceptions);
}

void TAttributeTest::throwExceptionWithBadMean()
{
    URANIE::DataServer::TAttribute * attQuatre = new URANIE::DataServer::TAttribute(
            "attQuatre", -2., 2.);
    CPPUNIT_ASSERT_THROW(attQuatre->setMean(-4.),
            URANIE::Exceptions::UErrorExceptions);
    delete attQuatre;
}

void TAttributeTest::throwExceptionSetBounds()
{
    CPPUNIT_ASSERT_THROW(att1->setBounds(1., -1.),
            URANIE::Exceptions::UExceptions);
}

void TAttributeTest::throwExceptionSetLowerBound()
{
    att1->setUpperBound(1.);
    CPPUNIT_ASSERT_THROW(att1->setLowerBound(2.),
            URANIE::Exceptions::UExceptions);
}

void TAttributeTest::throwExceptionSetUpperBound()
{
    att1->setLowerBound(-1.);
    CPPUNIT_ASSERT_THROW(att1->setUpperBound(-2.),
            URANIE::Exceptions::UExceptions);
}

void TAttributeTest::throwExceptionSetDefault()
{
    // didn't throw exception anymore
    double defval=0;
    att1->setBounds(-1., 3.);
    att1->setDefault("4.");
    CPPUNIT_ASSERT(att1->getDefaultValue(defval));
    CPPUNIT_ASSERT_EQUAL(defval, 3.);
}

void TAttributeTest::throwExceptionSetGetHasDefaultValue()
{
    att1->setBounds(-1., 3.);
    double defVal=0;
    //CPPUNIT_ASSERT_THROW(att1->setDefaultValue(4.), URANIE::Exceptions::UExceptions);

    CPPUNIT_ASSERT_THROW(attV->setDefaultValue(4.), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(attS->setDefaultValue(4.), URANIE::Exceptions::UExceptions);

    CPPUNIT_ASSERT_THROW(attV->getDefaultValue(defVal), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(attS->getDefaultValue(defVal), URANIE::Exceptions::UExceptions);
    
    CPPUNIT_ASSERT_THROW(attV->hasDefaultValue(), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(attS->hasDefaultValue(), URANIE::Exceptions::UExceptions);
}

void TAttributeTest::throwExceptionSetGetHasDefaultVector()
{
    vector<double> def={1.,2.,3.}, getdef;

    CPPUNIT_ASSERT_THROW(att1->setDefaultVector(def), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(att1->getDefaultVector(&getdef), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(att1->hasDefaultVector(), URANIE::Exceptions::UExceptions);
    
    CPPUNIT_ASSERT_THROW(attS->setDefaultVector(def), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(attS->getDefaultVector(&getdef), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(attS->hasDefaultVector(), URANIE::Exceptions::UExceptions);
     
}

void TAttributeTest::throwExceptionSetGetHasDefaultString()
{
    string def="tutu", getdef="";

    CPPUNIT_ASSERT_THROW(att1->setDefaultString(def), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(att1->getDefaultString(getdef), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(att1->hasDefaultString(), URANIE::Exceptions::UExceptions);
    
    CPPUNIT_ASSERT_THROW(attV->setDefaultString(def), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(attV->getDefaultString(getdef), URANIE::Exceptions::UExceptions);
    CPPUNIT_ASSERT_THROW(attV->hasDefaultString(), URANIE::Exceptions::UExceptions);
}

void TAttributeTest::throwExceptionTestKeyMethod()
{
    //Users can't enter the same pair file/AttributeName more than one time
    att1->setFileKey("TestFile1.in", "ATT1");
    CPPUNIT_ASSERT_THROW(
            att1->setFileKey("TestFile1.in", "ATT1", "%e",
                    URANIE::DataServer::TAttributeFileKey::kKey),
            URANIE::Exceptions::UExceptions);
    //Users can't enter  more than one key for the same file for the same attribute (for kKey typefile).
    att2->setFileKey("TestFile1.in", "ATT1");
    CPPUNIT_ASSERT_THROW(
            att2->setFileKey("TestFile1.in", "ATT2", "%e",
                    URANIE::DataServer::TAttributeFileKey::kKey),
            URANIE::Exceptions::UExceptions);
}

