//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TBetaDistribution.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TBetaDistribution
 
 Created: 2009-12-08 09:14:10


 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/07: liberation correcte des dproba.
 \li Cedric Forgues (C.S.) 2010/11/17
 Tests with getInvCDF() added.
 \li Cedric Forgues (C.S.) 2010/06/11
 - Ajout des tests presents pour la DI49.

 
 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-6

// Uranie
#include "TestTBetaDistribution.h"

//ROOT
#include "TMath.h"

using namespace std;

TBetaDistributionTest::TBetaDistributionTest()
{
}

TBetaDistributionTest::~TBetaDistributionTest()
{
}

void TBetaDistributionTest::setUp()
{
    _dproba = new URANIE::DataServer::TBetaDistribution("beta");
}

void TBetaDistributionTest::tearDown()
{
    delete _dproba;
    _dproba = NULL;
}

////////////////////////////////
///                          ///                            
///         Tests            ///
///                          ///
////////////////////////////////

void TBetaDistributionTest::testDefaultLaw()
{
    cout << endl;

    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kJacobi);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kBeta);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353553390593274,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.79577471545948, _dproba->getPDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.63661977236758, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.79577471545948, _dproba->getPDF(0.80),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.29516723530087, _dproba->getCDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.70483276469913, _dproba->getCDF(0.8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.024471741852423, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.34549150281253, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.65450849718747, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.97552825814758, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TBetaDistribution("beta", 1.0, 2.0);

    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kJacobi);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kBeta);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.333333333333333,
            _dproba->getTheoreticalMean(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.235702260395516,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.6, _dproba->getPDF(.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPDF(.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.4, _dproba->getPDF(.80),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.36, _dproba->getCDF(.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.75, _dproba->getCDF(.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.96, _dproba->getCDF(.8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.051316701949486, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.22540333075852, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.29289321881345, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.36754446796632, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.68377223398316, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "att", 0., 1.0);
    _dproba = new URANIE::DataServer::TBetaDistribution(att, 0.5, 0.5);

    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kJacobi);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kBeta);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353553390593274,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.79577471545948, _dproba->getPDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.63661977236758, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.79577471545948, _dproba->getPDF(0.80),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.29516723530087, _dproba->getCDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.70483276469913, _dproba->getCDF(0.8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.024471741852423, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.34549150281253, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.65450849718747, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.97552825814758, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete att;
    att = NULL;
}

void TBetaDistributionTest::testOtherLaw()
{
    cout << endl;
    delete _dproba;
    _dproba = new URANIE::DataServer::TBetaDistribution("beta", 3.0, 2.0, 1.0,
            2.0);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.6, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.2, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.384, _dproba->getPDF(1.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.5, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.536, _dproba->getPDF(1.80),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(2.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0272, _dproba->getCDF(1.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3125, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8192, _dproba->getCDF(1.8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(2.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.32046058372182, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.55549999791623, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.61427243186761, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.67083349662159, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.85744068328997, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TBetaDistribution("beta", 3.0, 2.0, -1.0,
            4.0);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-1.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0768, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3072, _dproba->getPDF(3.00),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(4.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0272, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3125, _dproba->getCDF(1.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8192, _dproba->getCDF(3.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(4.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.602302918609086, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.77749998958116, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.07136215933805, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.35416748310796, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.28720341644985, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
}

void TBetaDistributionTest::testSetParameters()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353553390593274,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameters(2.0, 3.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.4, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.2, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
}

void TBetaDistributionTest::testSetParameterBeta()
{
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353553390593274,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterBeta(3.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.142857142857142,
            _dproba->getTheoreticalMean(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1. / (TMath::Sqrt(3) * 3.5),
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
}

void TBetaDistributionTest::testSetParameterAlpha()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353553390593274,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterAlpha(2.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1. / (TMath::Sqrt(3.5) * 2.5),
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterBeta(),
            DBL_PRECISION_COMPARE);
}

/////////////////////////////////////////////////////////////////
////////////////////      Exceptions         ////////////////////
/////////////////////////////////////////////////////////////////

void TBetaDistributionTest::throwExceptionConstructorWithBadBoundaries()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TBetaDistribution("beta", 3.0, 2.0, 4.0,
                    -1.0), URANIE::Exceptions::UErrorExceptions);
}

void TBetaDistributionTest::throwExceptionConstructorWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TBetaDistribution("beta", -3.0, 2.0, -4.0,
                    1.0), URANIE::Exceptions::UErrorExceptions);
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TBetaDistribution(att, -3.0, 2.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
}

void TBetaDistributionTest::throwExceptionConstructorWithNegativeBeta()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TBetaDistribution("beta", 3.0, -2.0, -4.0,
                    1.0), URANIE::Exceptions::UErrorExceptions);
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TBetaDistribution(att, 3.0, -2.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TBetaDistributionTest::throwExceptionSetParametersWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(-2.0, 3.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TBetaDistributionTest::throwExceptionSetParametersWithNegativeBeta()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(2.0, -3.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TBetaDistributionTest::throwExceptionSetParameterBetaWithNegativeBeta()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterBeta(-3.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TBetaDistributionTest::throwExceptionSetParameterAlphaWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterAlpha(-3.0),
            URANIE::Exceptions::UErrorExceptions);
}

