//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTComposedDistribution.cxx
 \author Jean-Baptiste Blanchard <jean-baptiste.blanchard@cea.fr>
 \date   Fri Aug 26 12:00:00 CET 2022
 
 \brief  Fichier de test de la classe URANIE::DataServer::TComposedDistribution
 
 \par Modifications
 Ajout des méthodes
 // *****************************************************************************

 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-7

#include "TestTInfiniteDistribution.h"
#include "TestTComposedDistribution.h"
#include "TString.h"

#include "TCauchyDistribution.h"
#include "TGammaDistribution.h"

#include <iostream>
using namespace std;

TComposedDistributionTest::TComposedDistributionTest():
    _dproba(NULL), _n1(NULL), _n2(NULL)
{
}

TComposedDistributionTest::~TComposedDistributionTest()
{
}

void TComposedDistributionTest::setUp()
{
    _dproba = new URANIE::DataServer::TComposedDistribution("compose");
    _n1 = new URANIE::DataServer::TNormalDistribution("n1", -1, 0.5);
    _n2 = new URANIE::DataServer::TNormalDistribution("n2",  1, 0.5);
}

void TComposedDistributionTest::tearDown()
{
    delete _dproba;
    _dproba = NULL;

    if( _n1 != NULL)
        delete _n1;
    _n1 = NULL;

    if( _n2 != NULL)
        delete _n2;
    _n2 = NULL;
}

void TComposedDistributionTest::testDefaultLaw()
{
    CPPUNIT_ASSERT(_dproba->getLawType()
                   == URANIE::DataServer::TStochasticAttribute::kCompose);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getNbComponent(), DBL_PRECISION_COMPARE);
    
    
}

void TComposedDistributionTest::testAddDistributionWeight()
{
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT(_dproba->addDistribution(_n1) == 0 );
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1., _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT(_dproba->addDistribution(_n2, 0.5) == 0 );
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.3333333333333333333, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = new URANIE::DataServer::TComposedDistribution("compose");

    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT(_dproba->addDistribution(_n1, 0.5) == 0 );
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1., _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT(_dproba->addDistribution(_n2) == 0 );
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3333333333333333333, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);

    
    delete _dproba;
    _dproba = new URANIE::DataServer::TComposedDistribution("compose");

    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT(_dproba->addDistribution(_n1, 0.5) == 0 );
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1., _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT(_dproba->addDistribution(_n2,0.5) == 0 );
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
}

void TComposedDistributionTest::testAddDistributionOutput()
{
    
    CPPUNIT_ASSERT(_dproba->addDistribution(_n1) == 0 );

    _n2->setLowerBound(0.4);
    CPPUNIT_ASSERT(_dproba->addDistribution(_n2) == 1 );

}

void TComposedDistributionTest::testDeleteDistributions()
{

    _dproba->addDistribution(_n1);
    _n2->delShare();
    _dproba->addDistribution(_n2);
    
}


void TComposedDistributionTest::testGetTheoreticalWhenNotPossible()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);

    _dproba->addDistribution(_n1);


    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);

    URANIE::DataServer::TCauchyDistribution *c = new URANIE::DataServer::TCauchyDistribution("cauchy", 0.3, 1.0); 
    _dproba->addDistribution(c);


    URANIE::DataServer::TGammaDistribution *g = new URANIE::DataServer::TGammaDistribution("gamma", 0.3, 1.0, 0.0); 
    _dproba->addDistribution(g);

     CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(THEO_DEF, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    delete c;
}


static void truncation_set(unordered_map<string, double>& references, bool farAway=false) {
    references["x_min"] = ((farAway) ? -2.8 : -1.4);
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = ((farAway) ? 2.8 : 2.);;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);

    // ROOT::Math::compose_cdf()
    references["pmin"] = ((farAway) ? 7.95542950861594e-05 : 0.105928095955774 );
    references["pmax"] = ((farAway) ? 0.999920445704914 : 0.988624933532616);
}

void TComposedDistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TComposedDistribution distribution("compose");
    distribution.addDistribution(_n1);
    distribution.addDistribution(_n2);

    TestTInfiniteDistribution::test_bounds(distribution, references);
    
    URANIE::DataServer::TComposedDistribution distribution2("compose2");

    truncation_set(references,true);
    distribution2.addDistribution(_n1);
    distribution2.addDistribution(_n2);
    
    TestTInfiniteDistribution::test_bounds(distribution2, references);
}


void TComposedDistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TComposedDistribution distribution("compose");
    distribution.addDistribution(_n1);
    distribution.addDistribution(_n2);

    TestTInfiniteDistribution::test_lower_bound(distribution, references);
    
    URANIE::DataServer::TComposedDistribution distribution2("compose2");

    truncation_set(references,true);
    distribution2.addDistribution(_n1);
    distribution2.addDistribution(_n2);
    
    TestTInfiniteDistribution::test_lower_bound(distribution2, references);
}

void TComposedDistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TComposedDistribution distribution("compose");
    distribution.addDistribution(_n1);
    distribution.addDistribution(_n2);

    TestTInfiniteDistribution::test_upper_bound(distribution, references);

    URANIE::DataServer::TComposedDistribution distribution2("compose2");
    
    truncation_set(references,true);
    distribution2.addDistribution(_n1);
    distribution2.addDistribution(_n2);
    
    TestTInfiniteDistribution::test_upper_bound(distribution2, references);
}


////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TComposedDistributionTest::throwExceptionFromAddingDistributions()
{

    _dproba->addDistribution(_n1);
    CPPUNIT_ASSERT_THROW(_dproba->addDistribution(_n1),
                         URANIE::Exceptions::UErrorExceptions);

    CPPUNIT_ASSERT_THROW(_dproba->addDistribution(_n2, 0),
                         URANIE::Exceptions::UErrorExceptions);

    CPPUNIT_ASSERT_THROW(_dproba->addDistribution(_n2, -1.0),
                         URANIE::Exceptions::UErrorExceptions);

}


