//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTComposedDistribution.h
 \author Jean-Baptiste Blanchard <jean-baptiste.blanchard@cea.fr>
 \date   Fri Aug 26 12:00:00 CET 2022

 
 \brief  Fichier de test de la classe URANIE::DataServer::TComposedDistribution

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TCOMPOSEDDISTRIBUTION_H
#define CPPUNITEST_TCOMPOSEDDISTRIBUTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TComposedDistribution.h"
#include "TAttribute.h"
#include "UExceptions.h"
#include "TNormalDistribution.h"

/*! \class TComposedDistributionTest
 * \brief Unit tests for TComposedDistribution.
 */
class TComposedDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TComposedDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (testAddDistributionWeight);
    CPPUNIT_TEST (testAddDistributionOutput);
    CPPUNIT_TEST (testDeleteDistributions);
    CPPUNIT_TEST (testTruncationBounds);
    CPPUNIT_TEST (testTruncationLowerBound);
    CPPUNIT_TEST (testTruncationUpperBound);
    CPPUNIT_TEST (testGetTheoreticalWhenNotPossible);
    CPPUNIT_TEST (throwExceptionFromAddingDistributions);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TComposedDistributionTest object.
     */
    TComposedDistributionTest();

    /// Destructor.
    virtual ~TComposedDistributionTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests compose distribution with set parameters (set mean and sigma).
    void testDefaultLaw();
    //! Tests output form addDistribution
    void testAddDistributionWeight();
    //! Tests output form addDistribution
    void testAddDistributionOutput();
    //! Tests the deletion of Distribution
    void testDeleteDistributions();
    //! Tests the theoretical properties when not available
    void testGetTheoreticalWhenNotPossible();
    
    //! Tests truncation on compose distribution.
    void testTruncationBounds();
    void testTruncationLowerBound();
    void testTruncationUpperBound();

    //! Throws exception when giving a negative sigma.
    void throwExceptionFromAddingDistributions();

    //@}

private:
    URANIE::DataServer::TComposedDistribution *_dproba;
    URANIE::DataServer::TNormalDistribution *_n1, *_n2;
};
#endif  // CPPUNITEST_TCOMPOSEDDISTRIBUTION_H
