//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTCustomDistribution.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$

 \brief  Fichier de test de la classe URANIE::DataServer::TCustomDistribution

 Cree le : mer nov 21 07:59:24 2007

 */

#include "TestTCustomDistribution.h"
#include "TString.h"

#include <iostream>
#include <cstdlib>

using namespace std;

TCustomDistributionTest::TCustomDistributionTest()
{
    dproba = NULL;
}

TCustomDistributionTest::~TCustomDistributionTest()
{
}

void TCustomDistributionTest::setUp()
{
}

void TCustomDistributionTest::tearDown()
{
}

void TCustomDistributionTest::testNbValues()
{
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };

    URANIE::DataServer::TCustomDistribution dice("D6", 6, D6, freqOK);
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test de getNbValues", 6, dice.getNbValues());
}

void TCustomDistributionTest::testValues()
{
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };

    URANIE::DataServer::TCustomDistribution dice("D6", 6, D6, freqOK);

    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getValue(): premiere valeur", D6[0],
            dice.getValue(0));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getValue(): derniere valeur",
            D6[dice.getNbValues() - 1], dice.getValue(dice.getNbValues() - 1));

    Double_t *retrievedValues = dice.getAllValues();
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getAllValues(): deuxieme valeur", D6[1],
            retrievedValues[1]);
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getAllValues(): avant-derniere valeur",
            D6[dice.getNbValues() - 2],
            retrievedValues[dice.getNbValues() - 2]);
}

void TCustomDistributionTest::testFrequencies()
{
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };

    URANIE::DataServer::TCustomDistribution dice("D6", 6, D6, freqOK);

    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getFrequency(): premiere valeur",
            freqOK[0], dice.getFrequency(0));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getFrequency(): derniere valeur",
            freqOK[dice.getNbValues() - 1],
            dice.getFrequency(dice.getNbValues() - 1));

    Double_t *retrievedFrequencies = dice.getAllFrequencies();
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getAllFrequencies: deuxieme valeur",
            freqOK[1], retrievedFrequencies[1]);
    CPPUNIT_ASSERT_EQUAL_MESSAGE(
            "Test getAllFrequencies: avant derniere valeur",
            freqOK[dice.getNbValues() - 2],
            retrievedFrequencies[dice.getNbValues() - 2]);
}

void TCustomDistributionTest::testProba()
{
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };

    URANIE::DataServer::TCustomDistribution dice("D6", 6, D6, freqOK);

    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getProba(): premiere valeur",
            freqOK[0] / 6, dice.getProba(0));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getProba(): derniere valeur",
            freqOK[dice.getNbValues() - 1] / 6,
            dice.getProba(dice.getNbValues() - 1));
}

void TCustomDistributionTest::testSetDistribution()
{
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };

    URANIE::DataServer::TCustomDistribution dice("D6", 6, D6, freqOK);

    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test ancienne distribution", 6,
            dice.getNbValues());
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test ancienne distribution", D6[0],
            dice.getValue(0));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test ancienne distribution",
            freqOK[dice.getNbValues() - 1],
            dice.getFrequency(dice.getNbValues() - 1));

    Double_t D10[10] =
    { 1.0, 2.0, 3.0, 6.0, 5.0, 4.0, 7.0, 8.0, 9.0, 0.0 };
    Double_t freqProba[10] =
    { 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1 };

    dice.setDistribution(10, D10, freqProba);

    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test nouvelle distribution", 10,
            dice.getNbValues());
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test nouvelle distribution", D10[0],
            dice.getValue(0));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test nouvelle distribution",
            freqProba[dice.getNbValues() - 1],
            dice.getFrequency(dice.getNbValues() - 1));
}

void TCustomDistributionTest::testNewValues()
{
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
    Double_t newValue = 7.0;
    Double_t newD6[6] =
    { 0.0, 2.0, 4.0, 6.0, 8.0, 10.0 };

    URANIE::DataServer::TCustomDistribution dice("D6", 6, D6, freqOK);

    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test ancienne valeur", D6[0],
            dice.getValue(0));

    dice.setValue(0, newValue);
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test setValue()", newValue, dice.getValue(0));

    dice.setAllValues(newD6);
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test setAllValues()", newD6[0],
            dice.getValue(0));
}

void TCustomDistributionTest::testNewFrequencies()
{
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
    Double_t freqTriche[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 4.0 };
    Double_t newFrequency = 2.0;

    URANIE::DataServer::TCustomDistribution dice("D6", 6, D6, freqOK);

    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test ancienne Frequence",
            freqOK[dice.getNbValues() - 1],
            dice.getFrequency(dice.getNbValues() - 1));

    dice.setFrequency(dice.getNbValues() - 1, newFrequency);
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test setFrequency()", newFrequency,
            dice.getFrequency(dice.getNbValues() - 1));

    dice.setAllFrequencies(freqTriche);
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test setAllFrequencies()",
            freqTriche[dice.getNbValues() - 1],
            dice.getFrequency(dice.getNbValues() - 1));
}

void TCustomDistributionTest::testGetPDF()
{
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };

    URANIE::DataServer::TCustomDistribution dice("D6", 6, D6, freqOK);

    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getPDF(): valeur non existente", 0.0,
            dice.getPDF(7.0));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getPDF(): valeur existente",
            freqOK[5] / 6, dice.getPDF(6.0));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getPDF(): valeur dans la tolerance",
            freqOK[4] / 6, dice.getPDF(5.001, 1e-2));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getPDF(): valeur dans la tolerance",
            freqOK[3] / 6, dice.getPDF(3.999, 1e-2));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("Test getPDF(): valeur hors tolerance", 0.0,
            dice.getPDF(3.1, 1e-2));
}

void TCustomDistributionTest::testGetInvCDF()
{
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqTriche[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 4.0 };

    Int_t nbOne = 0;
    Int_t nbSix = 0;
    Int_t nbSamples = 10000;

    URANIE::DataServer::TCustomDistribution dice("D6", 6, D6, freqTriche);

    for (Int_t i = 0; i < nbSamples; i++)
    {
        Double_t val = dice.getInvCDF(Double_t(rand()) / RAND_MAX);
        if (val == 1.0)
            nbOne++;
        else if (val == 6.0)
            nbSix++;
    }
    CPPUNIT_ASSERT(
            TMath::Abs(dice.getProba(0) - Double_t(nbOne) / nbSamples) < 1e-2);
    CPPUNIT_ASSERT(
            TMath::Abs(dice.getProba(5) - Double_t(nbSix) / nbSamples) < 1e-2);
}

/////////////////////////////////////////////////////////////////
////////////////////      Exceptions         ////////////////////
/////////////////////////////////////////////////////////////////

void TCustomDistributionTest::throwExceptionGetValue()
{
    cout << endl;
    dproba = new URANIE::DataServer::TCustomDistribution();
    CPPUNIT_ASSERT_THROW(dproba->getValue(0),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
    cout << endl;

    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
    dproba = new URANIE::DataServer::TCustomDistribution("D6", 6, D6, freqOK);
    CPPUNIT_ASSERT_THROW(dproba->getValue(6),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
}

void TCustomDistributionTest::throwExceptionGetFrequency()
{
    cout << endl;
    dproba = new URANIE::DataServer::TCustomDistribution();
    CPPUNIT_ASSERT_THROW(dproba->getFrequency(0),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
    cout << endl;

    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
    dproba = new URANIE::DataServer::TCustomDistribution("D6", 6, D6, freqOK);
    CPPUNIT_ASSERT_THROW(dproba->getFrequency(6),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
}

void TCustomDistributionTest::throwExceptionGetProba()
{
    cout << endl;
    dproba = new URANIE::DataServer::TCustomDistribution();
    CPPUNIT_ASSERT_THROW(dproba->getProba(0),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
    cout << endl;

    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
    dproba = new URANIE::DataServer::TCustomDistribution("D6", 6, D6, freqOK);
    CPPUNIT_ASSERT_THROW(dproba->getProba(6),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
}

void TCustomDistributionTest::throwExceptionSetDistributionWithNegativeFrequency()
{
    cout << endl;
    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, -1.0, 1.0, 1.0 };

    CPPUNIT_ASSERT_THROW(
            dproba = new URANIE::DataServer::TCustomDistribution("D6", 6, D6,
                    freqOK), URANIE::Exceptions::UErrorExceptions);
    if (dproba)
    {
        delete dproba;
        dproba = NULL;
    }

    freqOK[3] = 1.0;
    dproba = new URANIE::DataServer::TCustomDistribution("D6", 6, D6, freqOK);
    CPPUNIT_ASSERT_THROW(dproba->setFrequency(1, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
}

void TCustomDistributionTest::throwExceptionSetValue()
{
    cout << endl;
    dproba = new URANIE::DataServer::TCustomDistribution();
    CPPUNIT_ASSERT_THROW(dproba->setValue(0, 7.0),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
    cout << endl;

    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
    dproba = new URANIE::DataServer::TCustomDistribution("D6", 6, D6, freqOK);
    CPPUNIT_ASSERT_THROW(dproba->setValue(6, 7.0),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
}

void TCustomDistributionTest::throwExceptionSetFrequency()
{
    cout << endl;
    dproba = new URANIE::DataServer::TCustomDistribution();
    CPPUNIT_ASSERT_THROW(dproba->setFrequency(0, 7.0),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
    cout << endl;

    Double_t D6[6] =
    { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
    Double_t freqOK[6] =
    { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 };
    dproba = new URANIE::DataServer::TCustomDistribution("D6", 6, D6, freqOK);
    CPPUNIT_ASSERT_THROW(dproba->setFrequency(6, 4.0),
            URANIE::Exceptions::UErrorExceptions);
    delete dproba;
    dproba = NULL;
}

void TCustomDistributionTest::throwExceptionComputeTheoreticalInformation()
{
    cout << endl;
    dproba = new URANIE::DataServer::TCustomDistribution();
    CPPUNIT_ASSERT_THROW(dproba->computeTheoreticalInformation(),
            URANIE::Exceptions::UWarningExceptions);
    delete dproba;
    dproba = NULL;
}

void TCustomDistributionTest::throwExceptionGetCDF()
{
    cout << endl;
    dproba = new URANIE::DataServer::TCustomDistribution();
    CPPUNIT_ASSERT_THROW(dproba->getCDF(0),
            URANIE::Exceptions::UWarningExceptions);
    delete dproba;
    dproba = NULL;
}

