//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TGammaDistribution.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TGammaDistribution

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.  
 Created: 2009-12-08 09:14:10
 
 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTInfiniteDistribution.h"
#include "TestTGammaDistribution.h"

using namespace std;

TGammaDistributionTest::TGammaDistributionTest()
{
}

TGammaDistributionTest::~TGammaDistributionTest()
{
}

void TGammaDistributionTest::setUp()
{
    _dproba = new URANIE::DataServer::TGammaDistribution("gamma");
}

void TGammaDistributionTest::tearDown()
{
    delete _dproba;
    _dproba = NULL;
}

////////////////////////////////
///                          ///                            
///         Tests            ///
///                          ///
////////////////////////////////

void TGammaDistributionTest::testDefaultLaw()
{

    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8187307530779818, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1353352832366127, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.002478752176666359, _dproba->getPDF(6.0),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.693147180559945, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGammaDistribution("gamma", 2.0, 3.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(6.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(sqrt(18.0), _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.02078904411181373, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1140926931183538, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.09022352215774181, _dproba->getPDF(6.0),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.03504097004997, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
        "Att");
    _dproba = new URANIE::DataServer::TGammaDistribution(att, 2.0, 3.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(6.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(sqrt(18.0), _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.02078904411181373, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1140926931183538, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.09022352215774181, _dproba->getPDF(6.0),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.03504097004997, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);

    delete att;
    att = NULL;
}

void TGammaDistributionTest::testOtherLaw()
{

    delete _dproba;
    _dproba = new URANIE::DataServer::TGammaDistribution("gamma", 2.0, 3.0,-2.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(sqrt(18.0), _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1174079624886176, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1171542836069897, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.06176306775360138, _dproba->getPDF(6.0),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.03504097004997, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);

    
    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
        "Att");
    _dproba = new URANIE::DataServer::TGammaDistribution(att, 2.0, 3.0, -2.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(sqrt(18.0), _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1174079624886176, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1171542836069897, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.06176306775360138, _dproba->getPDF(6.0),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.03504097004997, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);



    
    delete att;
    att = NULL;
}

static void truncation_set(unordered_map<string, double>& references) {
    references["shape"] = 3.;
    references["scale"] = 1.;

    references["x_min_default"] = 0.;
    references["x_min_default_prev"] = nextafter(references.at("x_min_default"), -HUGE_VAL);
    references["x_min"] = 1.5;
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = 6.;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);

    // ROOT::Math::gamma_cdf()
    references["pmin_default"] = 0.;
    references["pmin"] = 1.9115316946194183e-01;
    references["pmax"] = 9.3803119558334103e-01;
}

void TGammaDistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGammaDistribution distribution("gamma", references.at("shape"),
        references.at("scale"));
    TestTInfiniteDistribution::test_bounds(distribution, references);
}

void TGammaDistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGammaDistribution distribution("gamma", references.at("shape"),
        references.at("scale"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);
}

void TGammaDistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGammaDistribution distribution("gamma", references.at("shape"),
        references.at("scale"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TGammaDistributionTest::throwExceptionWithNegativeAlpha()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGammaDistribution("gamma", -1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGammaDistribution(att, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TGammaDistributionTest::throwExceptionWithNegativeScale()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGammaDistribution("gamma", 1.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGammaDistribution(att, 1.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TGammaDistributionTest::throwExceptionSetParametersWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(-1.0, 2.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGammaDistributionTest::throwExceptionSetParametersWithNegativeScale()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(1.0, -2.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGammaDistributionTest::throwExceptionSetParameterAlphaWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterAlpha(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGammaDistributionTest::throwExceptionSetParameterScaleWithNegativeScale()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterScale(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}


