//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTGeneralizedNormalDistribution.cxx
 \author Rudy Chocat <rudy.chocat@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TGeneralizedNormalDistribution

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.  
 Created: 2009-12-08 09:14:10
 
 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTInfiniteDistribution.h"
#include "TestTGeneralizedNormalDistribution.h"

using namespace std;

TGeneralizedNormalDistributionTest::TGeneralizedNormalDistributionTest()
{
}

TGeneralizedNormalDistributionTest::~TGeneralizedNormalDistributionTest()
{
}

void TGeneralizedNormalDistributionTest::setUp()
{
    _dproba = new URANIE::DataServer::TGeneralizedNormalDistribution("gennormal");
}

void TGeneralizedNormalDistributionTest::tearDown()
{
    delete _dproba;
    _dproba = NULL;
}

////////////////////////////////
///                          ///                            
///         Tests            ///
///                          ///
////////////////////////////////

void TGeneralizedNormalDistributionTest::testDefaultLaw()
{

    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormal);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7071067811865476, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterBeta(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMu(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5420673935524315, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.010333492677046027, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(6.349117335933279e-08, _dproba->getPDF(4.0),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.4769362762044697, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);
    
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGeneralizedNormalDistribution("gennormal", 0.0, 1.0, 2.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormal);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7071067811865476, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterBeta(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMu(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5420673935524315, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.010333492677046027, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(6.349117335933279e-08, _dproba->getPDF(4.0),
                                 DBL_PRECISION_COMPARE);
    
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.4769362762044697, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;
    
    
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
        "Att");
    _dproba = new URANIE::DataServer::TGeneralizedNormalDistribution(att, 0.0, 1.0, 2.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormal);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7071067811865476, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterBeta(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMu(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5420673935524315, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.010333492677046027, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(6.349117335933279e-08, _dproba->getPDF(4.0),
                                 DBL_PRECISION_COMPARE);
    
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.4769362762044697, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);

    delete att;
    att = NULL;
}

void TGeneralizedNormalDistributionTest::testOtherLaw()
{

    delete _dproba;
    _dproba = new URANIE::DataServer::TGeneralizedNormalDistribution("gennormal", 2.0, 3.0, 3.0);
//     CPPUNIT_ASSERT(
//         _dproba->getLawType()
//         == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormal);
//     CPPUNIT_ASSERT(
//         _dproba->getStochasticBasis()
//         == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.832904679781946, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterBeta(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterMu(),
                                 DBL_PRECISION_COMPARE);
//     
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.15038331253863194, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.18664108695369766, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.017440883578879816, _dproba->getPDF(6.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.3713411877025123, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);
    
    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
        "Att");
    _dproba = new URANIE::DataServer::TGeneralizedNormalDistribution(att, 2.0, 3.0, 3.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormal);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.832904679781946, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterBeta(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterMu(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.15038331253863194, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.18664108695369766, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.017440883578879816, _dproba->getPDF(6.0),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.3713411877025123, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);



    
    delete att;
    att = NULL;
}

static void truncation_set(unordered_map<string, double>& references, bool farAway=false) {
    references["location"] = 1.;
    references["shape"] = 3.;
    references["scale"] = 1.;

    references["x_min"] = ((farAway) ? -140 : -1.4);
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = ((farAway) ? 200 : 2.);;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);
;

    // ROOT::Math::generalizedNormal_cdf()
    references["pmin"] = ((farAway) ? 0 : 0.22466448205861084 );
    references["pmax"] = ((farAway) ? 1 : 0.6417343447131054  );
}

void TGeneralizedNormalDistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGeneralizedNormalDistribution distribution("gennormal", references.at("location"), references.at("shape"),
        references.at("scale"));
    TestTInfiniteDistribution::test_bounds(distribution, references);
}

void TGeneralizedNormalDistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGeneralizedNormalDistribution distribution("gennormal",references.at("location"), references.at("shape"),
        references.at("scale"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);
}

void TGeneralizedNormalDistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGeneralizedNormalDistribution distribution("gennormal",references.at("location"), references.at("shape"),
        references.at("scale"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TGeneralizedNormalDistributionTest::throwExceptionWithNegativeAlpha()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGeneralizedNormalDistribution("gennormal", 1.0, -1.0,  1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGeneralizedNormalDistribution(att, 1.0, -1.0,  1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TGeneralizedNormalDistributionTest::throwExceptionWithNegativeBeta()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGeneralizedNormalDistribution("gennormal", 1.0, 1.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGeneralizedNormalDistribution(att, 1.0, 1.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TGeneralizedNormalDistributionTest::throwExceptionSetParametersWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(-1.0, -2.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGeneralizedNormalDistributionTest::throwExceptionSetParametersWithNegativeBeta()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(1.0, -2.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGeneralizedNormalDistributionTest::throwExceptionSetParameterAlphaWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterAlpha(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGeneralizedNormalDistributionTest::throwExceptionSetParameterBetaWithNegativeBeta()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterBeta(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}


