//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTGumbelMaxDistribution.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TGumbelMaxDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba. 
 \li Cedric Forgues (C.S.) 2010/06/16
 Ajout des m�thodes pour la DI 49
 // *****************************************************************************

 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

#include "TestTInfiniteDistribution.h"
#include "TestTGumbelMaxDistribution.h"
#include "TString.h"


#include <iostream>
using namespace std;

TGumbelMaxDistributionTest::TGumbelMaxDistributionTest()
{
}

TGumbelMaxDistributionTest::~TGumbelMaxDistributionTest()
{
}

void TGumbelMaxDistributionTest::setUp()
{
    _att = new URANIE::DataServer::TAttribute("gumbel");
    _dproba = new URANIE::DataServer::TGumbelMaxDistribution("gumbel");
}

void TGumbelMaxDistributionTest::tearDown()
{
    delete _att;
    _att = NULL;
    delete _dproba;
    _dproba = NULL;
}

void TGumbelMaxDistributionTest::testDefaultLaw()
{

    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kGumbel);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.577215664901533,
            _dproba->getTheoreticalMean(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.28254983016186,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.179374078734017, _dproba->getPDF(-1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.367879441171442, _dproba->getPDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.254646380043583, _dproba->getPDF(1.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0659880358453125, _dproba->getCDF(-1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.367879441171442, _dproba->getCDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.692200627555346, _dproba->getCDF(1.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.834032445247956, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0874215717907552, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.366512920581664, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.671726992092122, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.25036732731245, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelMaxDistribution("gumbel", 1., 2.);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kGumbel);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.15443132980307, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.56509966032373,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.158520960538971, _dproba->getPDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.183939720585721, _dproba->getPDF(1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.165352149445209, _dproba->getPDF(2.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.192295645547965, _dproba->getCDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.367879441171442, _dproba->getCDF(1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.545239211892605, _dproba->getCDF(2.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.668064890495911, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.17484314358151, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.73302584116333, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.34345398418424, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.50073465462489, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelMaxDistribution(_att, 1., 2.);

    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kGumbel);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.15443132980307, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.56509966032373,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.158520960538971, _dproba->getPDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.183939720585721, _dproba->getPDF(1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.165352149445209, _dproba->getPDF(2.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.192295645547965, _dproba->getCDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.367879441171442, _dproba->getCDF(1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.545239211892605, _dproba->getCDF(2.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.668064890495911, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.17484314358151, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.73302584116333, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.34345398418424, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.50073465462489, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
}

void TGumbelMaxDistributionTest::testGetSetScale()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterScale(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterScale(2.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterScale(),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelMaxDistribution("gumbel", 0.5, 1.);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterScale(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterScale(2.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterScale(),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelMaxDistribution(_att, 0.5, 1.);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterScale(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterScale(2.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterScale(),
            DBL_PRECISION_COMPARE);
}

void TGumbelMaxDistributionTest::testGetSetLocation()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterMode(0.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelMaxDistribution("gumbel", 2.0, 1.);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterMode(0.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelMaxDistribution(_att, 0.0, 1.);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterMode(0.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
}

static void truncation_set(unordered_map<string, double>& references) {
    references["mode"] = 2.5;
    references["scale"] = 3.;

    references["x_min"] = -1.;
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = 12.;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);

    // scipy.stats.gumbel_r.cdf()
    references["pmin"] = 4.0305371012981442e-02;
    references["pmax"] = 9.5873186329779814e-01;
}

void TGumbelMaxDistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGumbelMaxDistribution distribution("gumbel_max", references.at("mode"),
        references.at("scale"));
    TestTInfiniteDistribution::test_bounds(distribution, references);
}

void TGumbelMaxDistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGumbelMaxDistribution distribution("gumbel_max", references.at("mode"),
        references.at("scale"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);
}

void TGumbelMaxDistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGumbelMaxDistribution distribution("gumbel_max", references.at("mode"),
        references.at("scale"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);
}

void TGumbelMaxDistributionTest::throwExceptionWithNegativeBeta()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGumbelMaxDistribution("gumbel", 0.5, -1.),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGumbelMaxDistribution(_att, 0.5, -1.),
            URANIE::Exceptions::UErrorExceptions);
}

