//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TInvGammaDistribution.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TInvGammaDistribution

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.  
 Created: 2009-12-08 09:14:10
 
 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTInfiniteDistribution.h"
#include "TestTInvGammaDistribution.h"

using namespace std;

TInvGammaDistributionTest::TInvGammaDistributionTest()
{
}

TInvGammaDistributionTest::~TInvGammaDistributionTest()
{
}

void TInvGammaDistributionTest::setUp()
{
    _dproba = new URANIE::DataServer::TInvGammaDistribution("invgamma");
}

void TInvGammaDistributionTest::tearDown()
{
    delete _dproba;
    _dproba = NULL;
}

////////////////////////////////
///                          ///                            
///         Tests            ///
///                          ///
////////////////////////////////

void TInvGammaDistributionTest::testDefaultLaw()
{

    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kInvGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_THROW(_dproba->getTheoreticalMean(), URANIE::Exceptions::UWarningExceptions);
    CPPUNIT_ASSERT_THROW(_dproba->getTheoreticalStdDev(), URANIE::Exceptions::UWarningExceptions);     

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1684486749771367, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3678794411714423, _dproba->getPDF(1.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.2497661528351802, _dproba->getPDF(1.4),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.44269504088896, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TInvGammaDistribution("invgamma", 2.0, 3.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kInvGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_THROW(_dproba->getTheoreticalStdDev(), URANIE::Exceptions::UWarningExceptions);     
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0003441401105645543, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.4480836153107756, _dproba->getPDF(1.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3847931832537381, _dproba->getPDF(1.4),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.78747304213309, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
        "Att");
    _dproba = new URANIE::DataServer::TInvGammaDistribution(att, 2.0, 3.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kInvGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_THROW(_dproba->getTheoreticalStdDev(), URANIE::Exceptions::UWarningExceptions);     
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0003441401105645543, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.4480836153107756, _dproba->getPDF(1.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3847931832537381, _dproba->getPDF(1.4),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.78747304213309, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);

    delete att;
    att = NULL;
}

void TInvGammaDistributionTest::testOtherLaw()
{

    delete _dproba;
    _dproba = new URANIE::DataServer::TInvGammaDistribution("invgamma", 2.0, 3.0,-2.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kInvGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
     CPPUNIT_ASSERT_THROW(_dproba->getTheoreticalStdDev(), URANIE::Exceptions::UWarningExceptions);     
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.2161497407229438, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1226264803904808, _dproba->getPDF(1.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.09475556921932454, _dproba->getPDF(1.4),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.212526957866907, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);

    
    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
        "Att");
    _dproba = new URANIE::DataServer::TInvGammaDistribution(att, 2.0, 3.0, -2.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kInvGamma);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_THROW(_dproba->getTheoreticalStdDev(), URANIE::Exceptions::UWarningExceptions);     
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, _dproba->getParameterScale(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getParameterLocation(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.2161497407229438, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1226264803904808, _dproba->getPDF(1.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.09475556921932454, _dproba->getPDF(1.4),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.212526957866907, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);



    
    delete att;
    att = NULL;
}

static void truncation_set(unordered_map<string, double>& references) {
    references["shape"] = 2.5;
    references["scale"] = 0.5;
    references["location"] = 0.;

    references["x_min_default"] = 0.;
    references["x_min_default_prev"] = nextafter(references.at("x_min_default"), -HUGE_VAL);
    references["x_min"] = 0.1;
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = 1.6;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);

    // scipy.stats.invgamma.cdf()
    references["pmin_default"] = 0.;
    references["pmin"] = 7.5235246146512169e-02;
    references["pmax"] = 9.8683001318401242e-01;
}

void TInvGammaDistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TInvGammaDistribution distribution("inv_gamma", references.at("shape"),
        references.at("scale"), references.at("location"));
    TestTInfiniteDistribution::test_bounds(distribution, references);
}

void TInvGammaDistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TInvGammaDistribution distribution("inv_gamma", references.at("shape"),
        references.at("scale"), references.at("location"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);
}

void TInvGammaDistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TInvGammaDistribution distribution("inv_gamma", references.at("shape"),
        references.at("scale"), references.at("location"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);
}


////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TInvGammaDistributionTest::throwExceptionWithNegativeAlpha()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TInvGammaDistribution("invgamma", -1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TInvGammaDistribution(att, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TInvGammaDistributionTest::throwExceptionWithNegativeScale()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TInvGammaDistribution("invgamma", 1.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TInvGammaDistribution(att, 1.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TInvGammaDistributionTest::throwExceptionSetParametersWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(-1.0, 2.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TInvGammaDistributionTest::throwExceptionSetParametersWithNegativeScale()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(1.0, -2.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TInvGammaDistributionTest::throwExceptionSetParameterAlphaWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterAlpha(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TInvGammaDistributionTest::throwExceptionSetParameterScaleWithNegativeScale()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterScale(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}


