//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTInvGammaDistribution.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TInvGammaDistribution
 
 Created: 2009-12-08 09:14:10
 
 */
#ifndef CPPUNITEST_TINVGAMMADISTRIBUTION_H
#define CPPUNITEST_TINVGAMMADISTRIBUTION_H

//CPPUNIT
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

//URANIE
#include "TInvGammaDistribution.h"

class TInvGammaDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TInvGammaDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (testOtherLaw);
    CPPUNIT_TEST (testTruncationBounds);
    CPPUNIT_TEST (testTruncationLowerBound);
    CPPUNIT_TEST (testTruncationUpperBound);
    CPPUNIT_TEST (throwExceptionWithNegativeAlpha);
    CPPUNIT_TEST (throwExceptionWithNegativeScale);
    CPPUNIT_TEST (throwExceptionSetParametersWithNegativeAlpha);
    CPPUNIT_TEST (throwExceptionSetParametersWithNegativeScale);
    CPPUNIT_TEST (throwExceptionSetParameterAlphaWithNegativeAlpha);
    CPPUNIT_TEST (throwExceptionSetParameterScaleWithNegativeScale);
    CPPUNIT_TEST_SUITE_END();

public:
    TInvGammaDistributionTest();

    /// Destructor.
    virtual ~TInvGammaDistributionTest();

    void setUp();
    void tearDown();
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests the default constructor with three invgamma laws : InvGamma(0.5, 0.5) on [0,1], on [1,2] and [-1, 4]
    void testDefaultLaw();
    //! Tests the constructor with a specif law parameter 3.0, 2.0 with three invgamma laws : InvGamma(0.5, 0.5) on [0,1], on [1,2] and [-1, 4]
    void testOtherLaw();

    void testTruncationBounds();
    void testTruncationLowerBound();
    void testTruncationUpperBound();

    //! Throws exceptions when a negative lambda is given.
    void throwExceptionWithNegativeAlpha();

    //! Throws exceptions when a negative Scale is given.
    void throwExceptionWithNegativeScale();

    //! Throws exceptions when a negative lambda is given in SetParameters.
    void throwExceptionSetParametersWithNegativeAlpha();

    //! Throws exceptions when a negative Scale is given.
    void throwExceptionSetParametersWithNegativeScale();

    //! Throws exceptions when a negative lambda is given in setParameterAlpha().
    void throwExceptionSetParameterAlphaWithNegativeAlpha();

    //! Throws exceptions when a negative Scale is given in setParameterScale().
    void throwExceptionSetParameterScaleWithNegativeScale();

    //@}

private:
    URANIE::DataServer::TInvGammaDistribution *_dproba;
};

#endif  // CPPUNITEST_TINVGAMMADISTRIBUTION_H
