//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTLogNormalDistribution.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TLogNormalDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2010/04/26
 Ajout des m�thodes
 // *****************************************************************************

 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-7

#include "TestTInfiniteDistribution.h"
#include "TestTLogNormalDistribution.h"
#include "TString.h"

#include <iostream>
using namespace std;

TLogNormalDistributionTest::TLogNormalDistributionTest()
{
}

TLogNormalDistributionTest::~TLogNormalDistributionTest()
{
}

void TLogNormalDistributionTest::setUp()
{
    _att = new URANIE::DataServer::TAttribute("lognormal", 0.0, 0.5);
}

void TLogNormalDistributionTest::tearDown()
{
    delete _att;
    _att = NULL;
}

void TLogNormalDistributionTest::testDefaultLaw()
{
    _dproba = new URANIE::DataServer::TLogNormalDistribution("lognormal");
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kLogNormal);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kHermite);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.01, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00440780001309409,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterErrorFactor(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.09430986768442e-11, _dproba->getPDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.02436463098898e-20, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.23902258833985e-25, _dproba->getPDF(0.80),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getCDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getCDF(0.8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00533243546238467, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00822401505401333, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0091505172861615, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0101813975356817, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.015702387247814, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TLogNormalDistribution("loguniform", 0.5,
            1.5);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kLogNormal);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kHermite);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.125137441258357,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.5, _dproba->getParameterErrorFactor(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0126737272832885, _dproba->getPDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.21258282554901, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.25770381609585, _dproba->getPDF(0.80),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0011834686419e-05, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0001626866473758374, _dproba->getCDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5490421370760121, _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.978825741070261, _dproba->getCDF(0.8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999997679426679, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353664936159926, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.455677460426809, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.485039862317027, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.516294283715858, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.665216265403634, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TLogNormalDistribution(_att, 0.5, 1.5);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kLogNormal);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kHermite);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.125137441258357,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.5, _dproba->getParameterErrorFactor(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0126737272832885, _dproba->getPDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.21258282554901, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(0.80),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0001626866473758374, _dproba->getCDF(0.20),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5490421370760121, _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(0.8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353664936159926, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.455677460426809, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.485039862317027, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.516294283715858, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.665216265403634, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;
}

void TLogNormalDistributionTest::testOtherLaw()
{
    _dproba = new URANIE::DataServer::TLogNormalDistribution("lognormal1", 0.5,
            1.5, -2.0);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kLogNormal);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kHermite);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.625687206291783,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.5, _dproba->getParameterErrorFactor(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0025347454566577, _dproba->getPDF(-1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.59604483362527, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.642516565109802, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.487699618617006, _dproba->getPDF(0.80),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.152772108614189, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.000162686647375837, _dproba->getCDF(-1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.217087554388989, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5490421370760121, _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.720061425465552, _dproba->getCDF(0.8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.931668428213507, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.231675319200371, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.27838730213404, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.425199311585136, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.581471418579288, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.32608132701817, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att2 = new URANIE::DataServer::TAttribute(
            "lognormal2");
    _dproba = new URANIE::DataServer::TLogNormalDistribution(att2, 0.5, 1.5,
            -2.0);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kLogNormal);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kHermite);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.625687206291783,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.5, _dproba->getParameterErrorFactor(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0025347454566577, _dproba->getPDF(-1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.59604483362527, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.642516565109802, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.487699618617006, _dproba->getPDF(0.80),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.152772108614189, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.000162686647375837, _dproba->getCDF(-1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.217087554388989, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5490421370760121, _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.720061425465552, _dproba->getCDF(0.8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.931668428213507, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.231675319200371, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.27838730213404, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.425199311585136, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.581471418579288, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.32608132701817, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete att2;
    att2 = NULL;
    delete _dproba;
    _dproba = NULL;
}

static void truncation_set(unordered_map<string, double>& references) {
    references["mean"] = 1.5;
    references["error_factor"] = 1.8;

    references["x_min_default"] = -0.5;
    references["x_min_default_prev"] = nextafter(references.at("x_min_default"), -HUGE_VAL);
    references["x_min"] = 0.6;
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = 3.1;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);

    // ROOT::Math::lognormal_cdf()
    references["pmin_default"] = 0.;
    references["pmin"] = 6.7526584323654215e-02;
    references["pmax"] = 9.6589814632780924e-01;
}

void TLogNormalDistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TLogNormalDistribution distribution("lognormal", references.at("mean"),
        references.at("error_factor"), references.at("x_min_default"));
    TestTInfiniteDistribution::test_bounds(distribution, references);
}

void TLogNormalDistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TLogNormalDistribution distribution("lognormal", references.at("mean"),
        references.at("error_factor"), references.at("x_min_default"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);
}

void TLogNormalDistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TLogNormalDistribution distribution("lognormal", references.at("mean"),
        references.at("error_factor"), references.at("x_min_default"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TLogNormalDistributionTest::throwExceptionWithNegativeMean()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(URANIE::DataServer::TLogNormalDistribution("lognormal", -1.0 , 2); ,URANIE::Exceptions::UErrorExceptions );
    CPPUNIT_ASSERT_THROW(
            URANIE::DataServer::TLogNormalDistribution(_att, -1.0, 2),
            URANIE::Exceptions::UErrorExceptions);
}

void TLogNormalDistributionTest::throwExceptionWithNegativeLowerBound()
{
    cout << endl;
    URANIE::DataServer::TAttribute * attCrt =
            new URANIE::DataServer::TAttribute("lognormal", -5.0, 0.5);
    CPPUNIT_ASSERT_THROW(
            URANIE::DataServer::TLogNormalDistribution(attCrt, 1.0, 2),
            URANIE::Exceptions::UErrorExceptions);
    delete attCrt;
}
