//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTLogTriangularDistribution.h
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TLogTriangularDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TLOGTRIANGULARDISTRIBUTION_H
#define CPPUNITEST_TLOGTRIANGULARDISTRIBUTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TLogTriangularDistribution.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TLogTriangularDistributionTest
 * \brief Unit tests for TLogTriangularDistribution.
 */
class TLogTriangularDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TLogTriangularDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (throwExceptionWithNegativeMode);CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TLogTriangularDistributionTest object.
     */
    TLogTriangularDistributionTest();

    /// Destructor.
    virtual ~TLogTriangularDistributionTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests the default parameters for log triangular distribution.
    void testDefaultLaw();
    //! Tests new parameters for boundaries and a set probable value of the attribute of log triangular distribution.
    void testOtherLaw();

    //@}

    //---------------------------------------------
    /** @name Unit Tests
     Exceptions
     **/
    //@{
    //! Throws exception when mod is negative;
    void throwExceptionWithNegativeMode();
    //@}

private:
    URANIE::DataServer::TLogTriangularDistribution *_dproba;
    URANIE::DataServer::TAttribute * _att;
};
#endif  // CPPUNITEST_TLOGTRIANGULARDISTRIBUTION_H
