//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTLogUniformDistribution.h
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TLogUniformDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TLOGUNIFORMDISTRIBUTION_H
#define CPPUNITEST_TLOGUNIFORMDISTRIBUTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TLogUniformDistribution.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TLogUniformDistributionTest
 * \brief Unit tests for TLogUniformDistribution.
 */
class TLogUniformDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TLogUniformDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (testLogUniformTruncation);
    CPPUNIT_TEST (throwExceptionWithNegativeLowerBound);
    CPPUNIT_TEST (throwExceptionWithBadBoundaries);CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TLogUniformDistributionTest object.
     */
    TLogUniformDistributionTest();

    /// Destructor.
    virtual ~TLogUniformDistributionTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests loguniform distribution with set parameters (set mean and sigma).
    void testDefaultLaw();

    //! Tests restriction on bounds
    void testLogUniformTruncation();

    //! Throws Exception when negative lowerbound is given
    void throwExceptionWithNegativeLowerBound();

    //! Throws Exception when negative lowerbound is given
    void throwExceptionWithBadBoundaries();

    //@}

private:
    URANIE::DataServer::TLogUniformDistribution *_dproba;
    URANIE::DataServer::TAttribute * _att;
};
#endif  // CPPUNITEST_TLOGUNIFORMDISTRIBUTION_H
