//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTTrapeziumDistribution.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TTrapeziumDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.
 \li Cedric Forgues (C.S.) 2010/06/16
 Ajout des m�thodes pour la DI 49
 // *****************************************************************************

 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

#include "TestTTrapeziumDistribution.h"
#include "TString.h"

#include <iostream>
using namespace std;

TTrapeziumDistributionTest::TTrapeziumDistributionTest()
{
}

TTrapeziumDistributionTest::~TTrapeziumDistributionTest()
{
}

void TTrapeziumDistributionTest::setUp()
{
    _att = new URANIE::DataServer::TAttribute("att", 0.0, 1.0);
    _dproba = new URANIE::DataServer::TTrapeziumDistribution("trapezium");
}

void TTrapeziumDistributionTest::tearDown()
{
    delete _att;
    _att = NULL;
    delete _dproba;
    _dproba = NULL;
}

void TTrapeziumDistributionTest::testDefaultLaw()
{

    CPPUNIT_ASSERT(_dproba->getLawType() == URANIE::DataServer::TStochasticAttribute::kTrapezium);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.25, _dproba->getTheoreticalMode(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.25, _dproba->getParameterXlower(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.75, _dproba->getParameterXupper(),
				 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.5),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5333333333333, _dproba->getPDF(0.1),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.3333333333333, _dproba->getPDF(0.5),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.3333333333333, _dproba->getPDF(0.75),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(2.0),
				 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0266666666667, _dproba->getCDF(0.1),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(0.50),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8333333333333, _dproba->getCDF(0.75),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(2.0),
				 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.19364916731, _dproba->getInvCDF(0.10),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.425, _dproba->getInvCDF(0.40),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getInvCDF(0.50),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.575, _dproba->getInvCDF(0.60),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.806350832689629, _dproba->getInvCDF(0.90),
				 DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TTrapeziumDistribution("trapezium", 0.0, 1.0);

    CPPUNIT_ASSERT(_dproba->getLawType() == URANIE::DataServer::TStochasticAttribute::kTrapezium);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.25, _dproba->getTheoreticalMode(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.25, _dproba->getParameterXlower(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.75, _dproba->getParameterXupper(),
				 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.5),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5333333333333, _dproba->getPDF(0.1),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.3333333333333, _dproba->getPDF(0.5),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.3333333333333, _dproba->getPDF(0.75),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(2.0),
				 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0266666666667, _dproba->getCDF(0.1),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(0.50),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8333333333333, _dproba->getCDF(0.75),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(2.0),
				 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.19364916731, _dproba->getInvCDF(0.10),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.425, _dproba->getInvCDF(0.40),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getInvCDF(0.50),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.575, _dproba->getInvCDF(0.60),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.806350832689629, _dproba->getInvCDF(0.90),
				 DBL_PRECISION_COMPARE);
    
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TTrapeziumDistribution(_att, 0.2,0.8);

    CPPUNIT_ASSERT(
	_dproba->getLawType()
	== URANIE::DataServer::TStochasticAttribute::kTrapezium);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3, _dproba->getTheoreticalMode(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.2, _dproba->getParameterXlower(),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8, _dproba->getParameterXupper(),
				 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.5),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.625, _dproba->getPDF(0.1),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.25, _dproba->getPDF(0.5),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.25, _dproba->getPDF(0.75),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(2.0),
				 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.03125, _dproba->getCDF(0.1),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(0.50),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8125, _dproba->getCDF(0.75),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(2.0),
				 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1788854382, _dproba->getInvCDF(0.10),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.42, _dproba->getInvCDF(0.40),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getInvCDF(0.50),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.58, _dproba->getInvCDF(0.60),
				 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8211145618, _dproba->getInvCDF(0.90),
				 DBL_PRECISION_COMPARE);
}


void TTrapeziumDistributionTest::throwExceptionWithBadBoundaries()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW( new URANIE::DataServer::TTrapeziumDistribution("trapezium", 1.0, 0.0), URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW( new URANIE::DataServer::TTrapeziumDistribution("trapezium", 0.0, 1.0, -0.2, 0.8), URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW( new URANIE::DataServer::TTrapeziumDistribution("trapezium", 0.0, 1.0, 0.2, 1.2), URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW( new URANIE::DataServer::TTrapeziumDistribution("trapezium", 0.0, 1.0,-0.2, 1.8), URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW( new URANIE::DataServer::TTrapeziumDistribution("trapezium", 0.0, 1.0, 0.3, 0.2), URANIE::Exceptions::UErrorExceptions);
    
}

