//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTWeibullDistribution.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TWeibullDistribution
 
 Created: 2009-12-08 09:14:10
 
 */
#ifndef CPPUNITEST_TWEIBULLDISTRIBUTION_H
#define CPPUNITEST_TWEIBULLDISTRIBUTION_H

//CPPUNIT
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

//URANIE
#include "TWeibullDistribution.h"

class TWeibullDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TWeibullDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (testOtherLaw);
    CPPUNIT_TEST (testTruncationBounds);
    CPPUNIT_TEST (testTruncationLowerBound);
    CPPUNIT_TEST (testTruncationUpperBound);
    CPPUNIT_TEST (throwExceptionWithNegativeLambda);
    CPPUNIT_TEST (throwExceptionWithNegativeK);
    CPPUNIT_TEST (throwExceptionSetParametersWithNegativeLambda);
    CPPUNIT_TEST (throwExceptionSetParametersWithNegativeK);
    CPPUNIT_TEST (throwExceptionSetParameterLambdaWithNegativeLambda);
    CPPUNIT_TEST (throwExceptionSetParameterKWithNegativeK);
    CPPUNIT_TEST_SUITE_END();

public:
    TWeibullDistributionTest();

    /// Destructor.
    virtual ~TWeibullDistributionTest();

    void setUp();
    void tearDown();
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests the default constructor with three weibull laws : Weibull(0.5, 0.5) on [0,1], on [1,2] and [-1, 4]
    void testDefaultLaw();
    //! Tests the constructor with a specif law parameter 3.0, 2.0 with three weibull laws : Weibull(0.5, 0.5) on [0,1], on [1,2] and [-1, 4]
    void testOtherLaw();

    //! Tests truncation on the law.
    void testTruncationBounds();
    void testTruncationLowerBound();
    void testTruncationUpperBound();

    //! Throws exceptions when a negative lambda is given.
    void throwExceptionWithNegativeLambda();

    //! Throws exceptions when a negative K is given.
    void throwExceptionWithNegativeK();

    //! Throws exceptions when a negative lambda is given in SetParameters.
    void throwExceptionSetParametersWithNegativeLambda();

    //! Throws exceptions when a negative K is given.
    void throwExceptionSetParametersWithNegativeK();

    //! Throws exceptions when a negative lambda is given in setParameterLambda().
    void throwExceptionSetParameterLambdaWithNegativeLambda();

    //! Throws exceptions when a negative K is given in setParameterK().
    void throwExceptionSetParameterKWithNegativeK();

    //@}

private:
    URANIE::DataServer::TWeibullDistribution *_dproba;
};

#endif  // CPPUNITEST_TWEIBULLDISTRIBUTION_H
