 //////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TLoadLevelerCluster.h
 \author Thierry Crestaux <thierry.crestaux@c-s.fr>
 \date   2014-04-30 11:09:42
 
 \brief Interface of the class URANIE::Launcher::TLoadLevelerCluster
 
 */
/**
 
 * @class URANIE::Launcher::TLoadLevelerCluster
 *
 * @brief Description of the class TLoadLevelerCluster.
 *
 * To be written by the developper.
 *
 * Example of environment variable \b LOADL_PROCESSOR_LIST
 <code>
 LOADL_PROCESSOR_LIST=poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare047-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr poincare056-adm.maisondelasimulation.fr
 </code>
 */
#ifndef TLOADLEVELERCLUSTER_H
#define TLOADLEVELERCLUSTER_H

#include <iostream>
#include <sstream>

using std::cout;
using std::endl;
using std::istringstream;
using std::string;

//ROOT
#include "Rtypes.h"

// Uranie
#include "TCluster.h"

namespace URANIE
{
namespace Launcher
{
class TLoadLevelerCluster: public TCluster
{
private:
    static const char *nodeList();
public:
    static bool eligible();

    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TLoadLevelerCluster();
    //! Default destructor
    virtual ~TLoadLevelerCluster();
    //@}

    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Construct and return the list of nodes
    virtual void init();
    virtual void getCommandPrefix(char *, char *);
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Launcher::TLoadLevelerCluster, ID_LAUNCHER)
    //Classe de
};

} // Fin du namespace Launcher
} // Fin du namespace URANIE
#endif
