//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TOutputFileXML.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   
 
 \brief Interface of the class URANIE::Launcher::TOutputFileXML
 
 */
/**
 
 @class URANIE::Launcher::TOutputFileXML

 @brief Description of the class TOutputFileXML.

 This classe enables to raise information from an XML file

 */
#ifndef TOUTPUTFILEXML_H
#define TOUTPUTFILEXML_H

// ROOT
#include "Rtypes.h"
#include "TString.h"
#include "TList.h"
// URANIE
#include "Launcher.h"
#include "TOutputFile.h"

#include <vector>

namespace URANIE
{
namespace Launcher
{
class TOutputFileXML: public TOutputFile
{
public:

    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    /*!
     \param str Name of the XML output file
     */
    TOutputFileXML(TString str);
    //! Default destructor
    virtual ~TOutputFileXML();
    //@}

    //---------------------------------------------
    /** @name Add attribut to the File
     **/
    //@{
    //!   Add the attribute in the file
    /*!
     \param att (URANIE::DataServer::TAttribute*) the pointer to the attribute
     \param xmlPath (TString) the xml path for the attribute in the file.
     \param FileType (URANIE::DataServer::TAttributeFileKey::EFileType) type of the output file (must be kXMLAttribute or kXMLField)
     */
    void addAttribute(URANIE::DataServer::TAttribute* att, TString xmlPath,
            URANIE::DataServer::TAttributeFileKey::EFileType FileType);
    //! Add the attribute by this name in the file); 
    //! Add the attribute by this name in the file
    /*!
     \param sname (TString)  the name of the attribute to created.
     \param xmlPath (TString) the xml path for the attribute in the file.
     \param FileType (URANIE::DataServer::TAttributeFileKey::EFileType) type of the output file (must be kXMLAttribute or kXMLField)
     */
    void addAttribute(TString sname, TString xmlPath,
            URANIE::DataServer::TAttributeFileKey::EFileType FileType);
    //@}

    //---------------------------------------------
    /** @name Computes Output
     **/
    //@{
    //! Looking for output usindouble only and transmitting matrix (not used by launcher anymore)
    Bool_t searchOutput(Int_t &n, Double_t *val, TList *lstOut = NULL);
    //! Looking for output using a map to be able to cope with vectors/strings
    Bool_t searchOutput(vector<URANIE::DataServer::UEntry*> *out);
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Launcher::TOutputFileXML, ID_LAUNCHER)
    //Classe de
};

} // Fin du namespace Launcher
} // Fin du namespace URANIE
#endif
