//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
// TPlatineCluster lun aoû 4 14:19:19 CEST 2008
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TPlatineCluster.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mon august 4 14:19:19 CEST 2008
 
 \brief Implementation of the class URANIE::Launcher::TPlatineCluster
 
 */

#include "TPlatineCluster.h"

#include "TSystem.h"

ClassImp (URANIE::Launcher::TPlatineCluster)

namespace URANIE
{
namespace Launcher
{
// class method
bool TPlatineCluster::eligible()
{
    const char *tmp = gSystem->Getenv("SLURM_NODELIST");
    return (tmp != NULL && tmp[0]);
}

const char *TPlatineCluster::nodeList()
{
    return gSystem->Getenv("SLURM_NODELIST");
}

// Constructor
TPlatineCluster::TPlatineCluster() :
        TCluster("platine", TPlatineCluster::nodeList(), 8)
{
    if (_blog)
        cout << " ** Constructor TPlatineCluster::TPlatineCluster ["
                << GetName() << "] Title[" << GetTitle() << "]" << endl;
    init();
}
TPlatineCluster::~TPlatineCluster()
{
    if (_blog)
        cout << " ** Destroy TPlatineCluster [" << GetName() << "] Title["
                << GetTitle() << "]" << endl;
}

///////////////////////////////////////////////////////////
//  Construct and return the list of node
///////////////////////////////////////////////////////////
void TPlatineCluster::init()
{
    TString snodes = GetTitle();
    if (_blog)
        cout << " ** TCluster::getListOfNodes Node[" << snodes << "]" << endl;
    if (snodes.CountChar('[') == 0)
    {
        addNodeName(snodes);
    }
    else
    {
        Int_t nbegin = snodes.Index('[');
        Int_t nend = snodes.Index(']');
        TString ssnodes = snodes(nbegin + 1, nend - 1 - nbegin);
        if (_blog)
            cout << " ** snode[" << snodes << "] ssnode[" << ssnodes << "]"
                    << endl;
        ssnodes.ReplaceAll(",", " ");
        istringstream inStream((char *) ssnodes.Data());
        string inName;
        size_t found;
        while (inStream >> inName)
        {
            if (_blog)
                cout << " ** inName[" << inName << "] -------- ";
            found = inName.find("-");
            if (found != string::npos)
            {
                if (_blog)
                    cout << "found at: " << int(found);
                inName.replace(int(found), 1, " ");
                if (_blog)
                    cout << " **** inName[" << inName << "]" << endl;
                istringstream inStream2(inName);
                string sfirst, slast;
                inStream2 >> sfirst;
                Int_t nfirst = atoi(sfirst.c_str());
                inStream2 >> slast;
                Int_t nlast = atoi(slast.c_str());
                if (_blog)
                    cout << " ** ****** First\"" << sfirst << "\" nfirst["
                            << nfirst << "] -- last\"" << slast << "\" nlast["
                            << nlast << "]" << endl;
                for (Int_t icrt = nfirst; icrt <= nlast; icrt++)
                    addNodeName(Form("%s%d", GetName(), icrt));

            }
            else
            {
                if (_blog)
                    cout << "not found" << endl;
                addNodeName(Form("%s%s", GetName(), inName.c_str()));
            }
        }

    }

    if (_blog)
        cout << " TCluster::getListOfNodes _slistOfNodes["
                << TCluster::getListOfNodes() << "]" << endl;
    TCluster::getListOfNodes();
}

void TPlatineCluster::getCommandPrefix(char *mpiexe, char* procs)
{
    cout << "MPIRUN FOR PLATINE" << endl;
    strcpy(mpiexe, "srun --nodelist=");
    strcat(mpiexe, procs);
    strcat(mpiexe, " -n 1 -N 1");
}

///////////////////////////////////////////////////////////
//  Log
///////////////////////////////////////////////////////////
void TPlatineCluster::printLog(Option_t *option)
{
    cout << "TPlatineCluster::printLog[" << option << "]" << endl;
    TCluster::printLog(option);
}

} // Fin du namespace Launcher
} // Fin du namespace URANIE

