//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TSGECluster.cxx
 \author Cedric Forgues <uranietm@cea.fr>
 \date   mar apr 28 14:19:19 CEST 2008
 
 \brief Implementation of the class URANIE::Launcher::TSGECluster
 
 */

#include "TSGECluster.h"
#include "TSystem.h"

ClassImp (URANIE::Launcher::TSGECluster)

namespace URANIE
{
namespace Launcher
{
// class method
TString TSGECluster::sRawListOfHosts = "";

bool TSGECluster::eligible()
{
    const char *tmp = gSystem->Getenv("PE_HOSTFILE");
    return (tmp != NULL && tmp[0]);
}

const char * TSGECluster::rawNodeList()
{
    std::ifstream fichier;

    fichier.open(gSystem->Getenv("PE_HOSTFILE"));
    if (fichier.is_open())
    {
        std::string ligne;
        while (std::getline(fichier, ligne))
            sRawListOfHosts = sRawListOfHosts + ligne + ":";
    }
    fichier.close();
    return sRawListOfHosts;
}

// Constructeur
TSGECluster::TSGECluster() :
        TCluster("SGE", TSGECluster::rawNodeList(), 1)
{
    if (_blog)
        cout << " ** Constructor TSGECluster::TSGECluster [" << GetName()
                << "] Title[" << GetTitle() << "]" << endl;
    init();
}
TSGECluster::~TSGECluster()
{
    if (_blog)
        cout << " ** Destroy TSGECluster [" << GetName() << "] Title["
                << GetTitle() << "]" << endl;
}

///////////////////////////////////////////////////////////
//  Construct and return the list of node
///////////////////////////////////////////////////////////
void TSGECluster::init()
{
    TString snodes = GetTitle();

    if (_blog)
        cout << " ** TCluster::getListOfNodes Node[" << snodes << "]" << endl;

    Int_t nbegin = 0;
    Int_t nbproc_heart;
    TString nbproc_heart_alpha = "";
    TString node = "";
    TString ssnodes = "";

    while (not snodes.IsNull())
    {
        nbegin = snodes.Index(' ');
        node = snodes(0, nbegin);
        snodes.ReplaceAll(node + " ", "");
        nbegin = snodes.Index(' ');
        nbproc_heart_alpha = snodes(0, nbegin);
        nbproc_heart = atoi(nbproc_heart_alpha.Data());
        nbegin = snodes.Index(":");
        snodes = snodes(nbegin + 1, snodes.Sizeof() - 1);
        addNodeName(node, nbproc_heart);
        setNNodes(nbproc_heart);

        std::fstream nodefile;
        nodefile.open(
                gSystem->Getenv("TMPDIR") + TString("/") + node
                        + TString(".txt"), std::ios::out | std::ios::trunc);
        if (nodefile.is_open())
            nodefile << node << endl;
        nodefile.close();
    }

    TCluster::getListOfNodes();

}

void TSGECluster::getCommandPrefix(char *mpiexe, char *procs)
{
    if (_blog)
        cout << " ** MPIRUN for SGE on host[" << gSystem->Getenv("HOSTNAME")
                << "]" << endl;
    strcpy(mpiexe, "mpirun ");
    strcat(mpiexe, " --machinefile ");
    TString machine = gSystem->Getenv("TMPDIR") + TString("/") + procs
            + TString(".txt");
    strcat(mpiexe, machine);
    strcat(mpiexe, " -np 1 ");
}

///////////////////////////////////////////////////////////
//  Log
///////////////////////////////////////////////////////////
void TSGECluster::printLog(Option_t *option)
{
    cout << "TSGECluster::printLog[" << option << "]" << endl;
    TCluster::printLog(option);
}

} // Fin du namespace Launcher
} // Fin du namespace URANIE
