//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TPlatineCluster.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   Mon August 4 14:19:19 CEST 2008
 
 \brief Interface of the class URANIE::Launcher::TPlatineCluster
 
 */
/**
 
 * @class URANIE::Launcher::TPlatineCluster
 *
 * @brief Description of the class TPlatineCluster.
 *
 * To be written by the developper.
 *
 */
#ifndef TSLURMCLUSTER_H
#define TSLURMCLUSTER_H

#include <iostream>
#include <sstream>

using std::cout;
using std::endl;
using std::istringstream;
using std::string;
#define BUF_SIZE 128
//ROOT
#include "Rtypes.h"

// Uranie
#include "TCluster.h"

namespace URANIE
{
namespace Launcher
{

class TSlurmCluster: public TCluster
{
private:
    static const char *nodeList();

public:

    static bool eligible();

    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TSlurmCluster();
#ifdef WITH_LIBSSH
    TSlurmCluster(URANIE::DataServer::TDataServer *tds, TCode *code);
    TSlurmCluster(TCode *code);
#endif
    
    //! Default destructor
    virtual ~TSlurmCluster();
    //@}

    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Construct and return the list of nodes
    virtual void init();
    virtual void getCommandPrefix(char *, char *);
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    virtual void printLog(Option_t *option = "") ;
    //@}
    ////////////
    /// SSH
    ////////////
#ifdef WITH_LIBSSH
    //! Sets the Cluster name
    /*!
     * \param cluster_name (TString) name of the cluster
     */
    void setCluster(TString cluster_name); 

    void selectHeader(TString SkeletonHeaderName="header_skeleton.in",TString localHeaderFolderPath=gSystem->pwd() );
    void generateHeader(Option_t *option = "");
    void generateHeaderSplit(Option_t *option = "");
    TString getJobStatusCommand(TString JobIDs);
    TString checkJobStatus(TString JobId, TString Command);

#endif
    
     ClassDef(URANIE::Launcher::TSlurmCluster , ID_LAUNCHER)

};

} // Fin du namespace Launcher
} // Fin du namespace URANIE
#endif
