//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTCode.cxx
 \author Thierry Crestaux
 \date   30/07/2014
 
 \brief  Fichier de test de la classe URANIE::Launcher::TCode
 
// ***************************************************************
 
 */

//CPPUNIT
//! The double precision to compare from CFO/CS
#define DBL_PRECISION_COMPARE 1.e-2
//! A true double precision to compare (FGA)
#define DBL_DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTCode.h"

#include <iostream>
using namespace std;

TCodeTest::TCodeTest()
{

}

TCodeTest::~TCodeTest()
{
  
}

void TCodeTest::setUp()
{

// FlowRate code ////////////////////////////////////
    _tdsFlowrate = new URANIE::DataServer::TDataServer("tdsFlowrate",
            "TDs for the Flowrate Use Case");
//   _tdsFlowrate->fileDataRead("flowrate_sampler_launcher_500.dat");

    _tdsFlowrate->addAttribute(new TUniformDistribution("rw", 0.05, 0.15));
    _tdsFlowrate->addAttribute(new TUniformDistribution("r", 100.0, 50000.0));
    _tdsFlowrate->addAttribute(new TUniformDistribution("tu", 63070.0, 115600.0));
    _tdsFlowrate->addAttribute(new TUniformDistribution("tl", 63.1, 116.0));
    _tdsFlowrate->addAttribute(new TUniformDistribution("hu", 990.0, 1110.0));
    _tdsFlowrate->addAttribute(new TUniformDistribution("hl", 700.0, 820.0));
    _tdsFlowrate->addAttribute(new TUniformDistribution("l", 1120.0, 1680.0));
    _tdsFlowrate->addAttribute(new TUniformDistribution("kw", 9855.0, 12045.0));
    _tdsFlowrate->getAttribute("rw")->delShare();
    _tdsFlowrate->getAttribute("r")->delShare();
    _tdsFlowrate->getAttribute("tu")->delShare();
    _tdsFlowrate->getAttribute("tl")->delShare();
    _tdsFlowrate->getAttribute("hu")->delShare();
    _tdsFlowrate->getAttribute("hl")->delShare();
    _tdsFlowrate->getAttribute("l")->delShare();
    _tdsFlowrate->getAttribute("kw")->delShare();
    
    TString sJDDReferenceKey = gSystem->pwd();
    sJDDReferenceKey += TString("/flowrate_input_with_row.in");

    _tdsFlowrate->getAttribute("rw")->setFileKey(sJDDReferenceKey, "Rw", "%f",
            TAttributeFileKey::kNewRow);
    _tdsFlowrate->getAttribute("r")->setFileKey(sJDDReferenceKey, "R", "%f",
            TAttributeFileKey::kNewRow);
    _tdsFlowrate->getAttribute("tu")->setFileKey(sJDDReferenceKey, "Tu", "%f",
            TAttributeFileKey::kNewRow);
    _tdsFlowrate->getAttribute("tl")->setFileKey(sJDDReferenceKey, "Tl", "%f",
            TAttributeFileKey::kNewRow);
    _tdsFlowrate->getAttribute("hu")->setFileKey(sJDDReferenceKey, "Hu", "%f",
            TAttributeFileKey::kNewRow);
    _tdsFlowrate->getAttribute("hl")->setFileKey(sJDDReferenceKey, "Hl", "%f",
            TAttributeFileKey::kNewRow);
    _tdsFlowrate->getAttribute("l")->setFileKey(sJDDReferenceKey, "L", "%f",
            TAttributeFileKey::kNewRow);
    _tdsFlowrate->getAttribute("kw")->setFileKey(sJDDReferenceKey, "Kw", "%f",
            TAttributeFileKey::kNewRow);

    _foutDataServer = new TOutputFileDataServer("_output_flowrate_withRow_.dat");
    _foutDataServer->addAttribute(new TAttribute("yhat"));
    _foutDataServer->getAttribute(0)->delShare();
    
    _codeFlowrate =
            new TCode(_tdsFlowrate,
                      Form("%s/%s", gSystem->pwd(),
                            "../../../../launCHER/teST/coDE/flowREATE/flowrate -r flowrate_input_with_row.in >>/dev/null"));
    _codeFlowrate->addOutputFile(_foutDataServer);  
}

void TCodeTest::tearDown()
{
    if (_codeFlowrate)
    {
        delete _codeFlowrate;
	_codeFlowrate=NULL;
    }

    if (_foutDataServer != NULL)
    {
        delete _foutDataServer;
	_foutDataServer=NULL;
    }

    
    if (_tdsFlowrate)
    {
        delete _tdsFlowrate;
	_tdsFlowrate=NULL;
    }
    
}

void TCodeTest::testGetTDataServer()
{
    CPPUNIT_ASSERT(_codeFlowrate->getTDataServer() == _tdsFlowrate);
    
    
}
