//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTCodeFMU.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mon oct 9th 14:49:25 CEST 2006

 \brief  Implement TUs for TCodeFMU class
 
// ***************************************************************
 
 */

//CPPUNIT
//! Low precision
#define LOW_PRECISION_COMPARE 1.e-2
//! Double precision
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTCodeFMU.h"
#include "UDisableTest.h"
#include "uranie_commands.h"

#include <iostream>
using namespace std;

TCodeFMUTest::TCodeFMUTest():
    _tds1FMIv1(nullptr), _tds1FMIv2(nullptr)
{
    _bLog = kFALSE;
    _spwd = gSystem->pwd();
//     cout << " *** _bLog[" << _bLog << "] _spwd[" << _spwd << "]" << endl;
}

TCodeFMUTest::~TCodeFMUTest()
{
    // Clean
//    cout << " *** Method::" << __func__ << "." << endl;
//    gDirectory->ls();
//    //gDirectory->Delete("tdsUseCase1_FMI*;9999");
//    gDirectory->ls();
//    cout << " *** End of Method::" << __func__ << "." << endl;
}

void TCodeFMUTest::setUp()
{
    gSystem->cd(_spwd);
    cout << endl;
    //    cout << endl << " *** [Method::" << __func__ << "] _spwd[" << _spwd << "]" << endl;
    //    cout << " *** PWD[" << gSystem->pwd() << "]" << endl;

//    TString sFileFMU = Form("%s/useCase1_fmi2_cs.fmu", gSystem->pwd());
    TString sFileFMU = "";

    // useCase1 - FMIv1
    {
        // TDS uscase 1 with FMI v1
        _tds1FMIv1 = new URANIE::DataServer::TDataServer("tdsUseCase1_FMIv1", "TDs for the use Case 1 - FMI v1");
        _tds1FMIv1->unsetLog();
        _tds1FMIv1->fileDataRead("useCase1FMIv1.dat");

        sFileFMU = "useCase1_fmi1_cs.fmu";
        cout << " **** sFileFMU[" << sFileFMU << "]" << endl;

        _tds1FMIv1->getAttribute("mu")->setFileFMU(sFileFMU, "BOUNCE_COF");

        _foutFMU1FMIv1 = new URANIE::Launcher::TOutputFileFMU(sFileFMU);
        _foutFMU1FMIv1->unsetLog();
        _foutFMU1FMIv1->addAttribute(new TAttribute("yhat0", TAttribute::kVector), "HIGHT");
        _foutFMU1FMIv1->addAttribute(new TAttribute("yhat1", TAttribute::kVector), "HIGHT_SPEED");
        //    _foutFMU1FMIv1->addAttribute(new TAttribute("yhat0", TAttribute::kVector), "x0");
        //    _foutFMU1FMIv1->addAttribute(new TAttribute("yhat1", TAttribute::kVector), "x1");

        _foutFMU1FMIv1->getAttribute(0)->delShare();
        _foutFMU1FMIv1->getAttribute(1)->delShare();

        _code1FMIv1 = new TCodeFMU(_tds1FMIv1, sFileFMU, "loglevel=all");
        _code1FMIv1->unsetLog();
    }

    // useCase1 - FMIv2
    {
        // TDS usecase 1 with FMI v2
        _tds1FMIv2 = new URANIE::DataServer::TDataServer("tdsUseCase1_FMIv2", "TDs for the use Case 1 - FMI v2");
        _tds1FMIv2->unsetLog();
        _tds1FMIv2->fileDataRead("useCase1FMIv2.dat");

        sFileFMU = "useCase1_fmi2_cs.fmu";
        cout << " **** sFileFMU[" << sFileFMU << "]" << endl;

        //    _tds1FMIv2->getAttribute("mu")->setFileFMU(sFileFMU, "mu");
        _tds1FMIv2->getAttribute("mu")->setFileFMU(sFileFMU, "BOUNCE_COF");

        _foutFMU1FMIv2 = new URANIE::Launcher::TOutputFileFMU(sFileFMU);
        _foutFMU1FMIv2->unsetLog();
        _foutFMU1FMIv2->addAttribute(new TAttribute("yhat0", TAttribute::kVector), "HIGHT");
        _foutFMU1FMIv2->addAttribute(new TAttribute("yhat1", TAttribute::kVector), "HIGHT_SPEED");
        //    _foutFMU1FMIv2->addAttribute(new TAttribute("yhat0", TAttribute::kVector), "x0");
        //    _foutFMU1FMIv2->addAttribute(new TAttribute("yhat1", TAttribute::kVector), "x1");

        _foutFMU1FMIv2->getAttribute(0)->delShare();
        _foutFMU1FMIv2->getAttribute(1)->delShare();

        _code1FMIv2 = new TCodeFMU(_tds1FMIv2, sFileFMU, "loglevel=info");
        _code1FMIv2->unsetLog();
    }
}

void TCodeFMUTest::tearDown()
{
    gSystem->cd(_spwd);
    if (_code1FMIv1)
        delete _code1FMIv1;
    _code1FMIv1=nullptr;

    if (_code1FMIv2)
        delete _code1FMIv2;
    _code1FMIv2=nullptr;

    if (_foutFMU1FMIv1)
        delete _foutFMU1FMIv1;
    _foutFMU1FMIv1=nullptr;

    if (_foutFMU1FMIv2)
        delete _foutFMU1FMIv2;
    _foutFMU1FMIv2=nullptr;


    if (_tds1FMIv2!=nullptr)
        delete _tds1FMIv2;
    _tds1FMIv2=nullptr;

    if (_tds1FMIv1!=nullptr)
        delete _tds1FMIv1;
    _tds1FMIv1=nullptr;

    // Clean
    UranieTests::removeFolder(gSystem->pwd(),"URANIE");
}

////////////////////////////////
///                          ///
///    Tests Units           ///
///                          ///
////////////////////////////////
void TCodeFMUTest::tuTCodeFMUConstructorFMIv1()
{
#ifdef WIN32
    URANIE_IGNORED_TEST()
#endif
    Int_t nipf_ref = 0;
    Int_t nopf_ref = 0;
    Int_t npat_ref = _tds1FMIv1->getNPatterns();
    Int_t natt_ref = _tds1FMIv1->getNAttributes();

    Int_t nipf = _code1FMIv1->getNInputFiles();
    Int_t nopf = _code1FMIv1->getNOutputFiles();


    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of Input files (%d) is not equal to reference(%d)",
                                      __func__, nipf_ref, nipf), nipf, nipf_ref);
    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of Output files (%d) is not equal to reference(%d)",
                                      __func__, nopf_ref, nopf), nopf, nopf_ref);

    _code1FMIv1->initFileType();

    nipf_ref = 1;
    nipf = _code1FMIv1->getNInputFiles();

    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of Input files (%d) is not equal to reference(%d)",
                                      __func__, nipf, nipf_ref), nipf, nipf_ref);
    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of Output files (%d) is not equal to reference(%d)",
                                      __func__, nopf, nopf_ref), nopf, nopf_ref);

    _code1FMIv1->addOutputFile(_foutFMU1FMIv1);

    _code1FMIv1->setStepSimulation(0.1);
    _code1FMIv1->run();

    Int_t npat = _tds1FMIv1->getNPatterns();
    Int_t natt = _tds1FMIv1->getNAttributes();

    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of patterns (%d) is not equal to reference(%d)",
                                      __func__, npat, npat_ref), npat, npat_ref);

    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of attributes (%d) is not equal to reference(%d)",
                                      __func__, natt, natt_ref), natt, natt_ref);
    _code1FMIv1->terminate();
}

void TCodeFMUTest::tuTCodeFMUConstructorFMIv2()
{
#ifdef WIN32
    URANIE_IGNORED_TEST()
#endif
    Int_t nipf_ref = 0;
    Int_t nopf_ref = 0;
    Int_t npat_ref = _tds1FMIv2->getNPatterns();
    Int_t natt_ref = _tds1FMIv2->getNAttributes();

    Int_t nipf = _code1FMIv2->getNInputFiles();
    Int_t nopf = _code1FMIv2->getNOutputFiles();


    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of Input files (%d) is not equal to reference(%d)",
                                      __func__, nipf_ref, nipf), nipf, nipf_ref);
    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of Output files (%d) is not equal to reference(%d)",
                                      __func__, nopf_ref, nopf), nopf, nopf_ref);

    _code1FMIv2->initFileType();

    nipf_ref = 1;
    nipf = _code1FMIv2->getNInputFiles();

    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of Input files (%d) is not equal to reference(%d)",
                                      __func__, nipf_ref, nipf), nipf, nipf_ref);
    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of Output files (%d) is not equal to reference(%d)",
                                      __func__, nopf_ref, nopf), nopf, nopf_ref);

    _code1FMIv2->addOutputFile(_foutFMU1FMIv2);

    _code1FMIv2->setStepSimulation(0.1);
    _code1FMIv2->run();

    Int_t npat = _tds1FMIv2->getNPatterns();
    Int_t natt = _tds1FMIv2->getNAttributes();

    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of patterns (%d) is not equal to reference(%d)",
                                      __func__, npat_ref, npat), npat, npat_ref);

    CPPUNIT_ASSERT_EQUAL_MESSAGE(Form("Method::%s The number of attributes (%d) is not equal to reference(%d)",
                                      __func__, natt_ref, natt), natt, natt_ref);

    _code1FMIv2->terminate();
}

////////////////////////////////
///                          ///
///    Throw Exceptions      ///
///                          ///
////////////////////////////////
void TCodeFMUTest::teFileNotFound()
{
    // Clean
    if (_code1FMIv2->getLog())
        cout << endl;
    if (_code1FMIv2)
        delete _code1FMIv2;
    _code1FMIv2=nullptr;


    if (_tds1FMIv2!=nullptr)
        delete _tds1FMIv2;
    _tds1FMIv2=nullptr;

    // TDS usecase 1 with FMI v2
    _tds1FMIv2 = new URANIE::DataServer::TDataServer("tdsUseCase1_FMIv2", "TDs for the use Case 1 - FMI v2");
    _tds1FMIv2->unsetLog();
    _tds1FMIv2->fileDataRead("useCase1FMIv2.dat");

    TString sFileFMU = "useCase1_fmi2_cddds.fmu"   ;
    // cout << " *** sFileFMU[" << sFileFMU << "]" << endl;

    _tds1FMIv2->getAttribute("mu")->setFileFMU(sFileFMU, "BOUNCE_COF");

    _code1FMIv2 = new TCodeFMU(_tds1FMIv2, sFileFMU, "loglevel=nothing");
    _code1FMIv2->unsetLog();
    _code1FMIv2->initFileType();

    CPPUNIT_ASSERT_THROW(_code1FMIv2->init(), URANIE::Exceptions::UExceptions);
}

void TCodeFMUTest::teFMUFileMEv1()
{
    // Clean
    if (_code1FMIv1->getLog())
        cout << endl;
    if (_code1FMIv1)
        delete _code1FMIv1;
    _code1FMIv1=nullptr;

    if (_tds1FMIv1!=nullptr)
        delete _tds1FMIv1;
    _tds1FMIv1=nullptr;

    // TDS usecase 1 with FMI v1
    _tds1FMIv1 = new URANIE::DataServer::TDataServer("tdsUseCase1_FMIv1", "TDs for the use Case 1 - FMI v1");
    _tds1FMIv1->unsetLog();
    _tds1FMIv1->fileDataRead("useCase1FMIv1.dat");

    TString sFileFMU = "useCase1_fmi1_me.fmu"   ;
    cout << " **** sFileFMU[" << sFileFMU << "]" << endl;

    _tds1FMIv1->getAttribute("mu")->setFileFMU(sFileFMU, "BOUNCE_COF");

    _code1FMIv1 = new TCodeFMU(_tds1FMIv1, sFileFMU, "loglevel=debug");
    _code1FMIv1->unsetLog();
    _code1FMIv1->initFileType();

    CPPUNIT_ASSERT_THROW(_code1FMIv1->init(), URANIE::Exceptions::UExceptions);
}

void TCodeFMUTest::teFMUFileMEv2()
{
    // Clean
    if (_code1FMIv2->getLog())
        cout << endl;
    if (_code1FMIv2)
        delete _code1FMIv2;
    _code1FMIv2=nullptr;

    if (_tds1FMIv2!=nullptr)
        delete _tds1FMIv2;
    _tds1FMIv2=nullptr;

    // TDS usecase 1 with FMI v2
    _tds1FMIv2 = new URANIE::DataServer::TDataServer("tdsUseCase1_FMIv2", "TDs for the use Case 1 - FMI v2");
    _tds1FMIv2->unsetLog();
    _tds1FMIv2->fileDataRead("useCase1FMIv2.dat");

    TString sFileFMU = "useCase1_fmi2_me.fmu"   ;
    cout << " **** sFileFMU[" << sFileFMU << "]" << endl;

    _tds1FMIv2->getAttribute("mu")->setFileFMU(sFileFMU, "BOUNCE_COF");

    _code1FMIv2 = new TCodeFMU(_tds1FMIv2, sFileFMU, "loglevel=verbose");
    _code1FMIv2->unsetLog();
    _code1FMIv2->initFileType();

    CPPUNIT_ASSERT_THROW(_code1FMIv2->init(), URANIE::Exceptions::UExceptions);
}

// End of File

