//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestTCodeFMU.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mon oct 9th 14:49:25 CEST 2006
 
 \brief TUs for TCodeFMU class
 
 */
#ifndef CPPUNITEST_TCODEFMU_H
#define CPPUNITEST_TCODEFMU_H

#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "UExceptions.h"
#include "TCodeFMU.h"
#include "TDataServer.h"


using namespace URANIE::DataServer;
using namespace URANIE::Exceptions;
using namespace URANIE::Launcher;


/**

 * @class TCodeFMUTest
 *
 * @brief Description of the class TCodeFMUTest.
 *
 *  The class TCodeFMUTest is the unit test class of the class TCodeFMU.
 *  
 *
 */

class TCodeFMUTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TCodeFMUTest);
    CPPUNIT_TEST (tuTCodeFMUConstructorFMIv1);
    CPPUNIT_TEST (tuTCodeFMUConstructorFMIv2);
    CPPUNIT_TEST (teFileNotFound);
    CPPUNIT_TEST (teFMUFileMEv1);
    CPPUNIT_TEST (teFMUFileMEv2);
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TCodeFMUTest();

    //! Default destructor
    virtual ~TCodeFMUTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
	The list of unit test
    **/
    //@{
    void tuTCodeFMUConstructorFMIv1();
    void tuTCodeFMUConstructorFMIv2();
    //@}

    //---------------------------------------------
    /** @name Throw Exceptions
        The list of Throw Exceptions
    **/
    //@{

    /*!
     * \brief FMU File not found
     */
    void teFileNotFound();

    /*!
     * \brief FMU File v1 in mode Model Echange
     */
    void teFMUFileMEv1();
    /*!
     * \brief FMU File v2 in mode Model Echange
     */
    void teFMUFileMEv2();
    //@}

private:
   
    Bool_t _bLog;
    TString _spwd;
    
//    URANIE::DataServer::TDataServer *_tds1;
    URANIE::DataServer::TDataServer *_tds1FMIv1;
    URANIE::DataServer::TDataServer *_tds1FMIv2;
    URANIE::Launcher::TOutputFileFMU *_foutFMU1FMIv1;
    URANIE::Launcher::TOutputFileFMU *_foutFMU1FMIv2;

    URANIE::Launcher::TCodeFMU *_code1FMIv1;
    URANIE::Launcher::TCodeFMU *_code1FMIv2;

};

#endif  // CPPUNITEST_TCODEFMU_H
