{
  // Load the function flowrateCalib2DVizir
  gROOT->LoadMacro("UserFunctions.C");
  
  // Input reference file   
  TString ExpData="Ex2DoE_n100_sd1.75.dat";
  
  // define the reference
  TDataServer *tdsRef = new TDataServer("tdsRef","doe_exp_Re_Pr");
  tdsRef->fileDataRead(ExpData.Data());
  
  // define the parameters
  TDataServer *tdsPar = new TDataServer("tdsPar","pouet");
  tdsPar->addAttribute( new TAttribute("hu", 1020.0, 1080.0) );
  tdsPar->addAttribute( new TAttribute("hl", 720.0, 780.0) );
  
  // Create the output attribute
  TAttribute *out = new TAttribute("out");
  
  // Create interface to assessors
  TCIntEval *Model = new TCIntEval("flowrateCalib2D");
  Model->addInput(tdsPar->getAttribute("hu"));
  Model->addInput(tdsPar->getAttribute("hl"));
  Model->addInput(tdsRef->getAttribute("rw"));
  Model->addInput(tdsRef->getAttribute("l"));
  Model->addOutput(out);
  
  // Set the runner
  TSequentialRun *runner = new TSequentialRun(Model);    
  
  // Set the calibration object
  TMinimisation *cal = new TMinimisation(tdsPar,runner,1);
  cal->setDistanceAndReference("relativeLS",tdsRef,"rw:l","Qexp");
  // Set optimisaiton properties
  URANIE::Reoptimizer::TVizirGenetic solv;
  solv.setSize(24,15000,100);
  cal->setOptimProperties(&solv); 
  // ((URANIE::Reoptimizer::TVizir2*)cal->getOptimMaster()->setTolerance(1e-6);
  cal->estimateParameters();

  // Draw the Residual
  TCanvas *canRes = new TCanvas("CanRes","CanRes",1200,800);
  TPad *padRes = new TPad("padRes","padRes",0, 0.03, 1, 1); padRes->Draw(); padRes->cd();
  cal->drawResidues("tutu","*","","nonewcanvas");
  
  // Draw the box plot of parameters
  TCanvas *canPar = new TCanvas("CanPar","CanPar",1200,800);
  tdsPar->getTuple()->SetMarkerStyle(20); tdsPar->getTuple()->SetMarkerSize(0.8);
  tdsPar->Draw("hu:hl");

  // Look at the correlation and statistic
  tdsPar->computeStatistic("hu:hl");
  TMatrixD corr=tdsPar->computeCorrelationMatrix("hu:hl");
  corr.Print();

  cout<<"hl is "<<tdsPar->getAttribute("hl")->getMean()<<" +- "<<tdsPar->getAttribute("hl")->getStd()<<endl;
  cout<<"hu is "<<tdsPar->getAttribute("hu")->getMean()<<" +- "<<tdsPar->getAttribute("hu")->getStd()<<endl;  
}