{
  // Create dataserver and add attributes
  TDataServer *tds = new TDataServer("pouet","foo");
  tds->addAttribute(new TUniformDistribution("pressure",0,10));
  tds->addAttribute(new TLogUniformDistribution("density",0.01,100));

  // Define the input file and the keys for 
  TString sFileName=TString("withblank_input.dat");
  tds->getAttribute("pressure")->setFileKey(sFileName, "water model.brewery pit pressure ");
  tds->getAttribute("density")->setFileKey(sFileName, "residual brew density model.factor for glass ");

  // Create the input file interface before it's done by the TCode to precise information
  TInputFileKey *in = new TInputFileKey(gSystem->PrependPathName(gSystem->pwd(),sFileName));
  // Set the separator to '=' so that the line could contains blank spaces (specific)
  in->setFieldSeparatorCharacter("=");
  in->setSeparatorCharacter(" ");
  in->addAttribute( tds->getAttribute("pressure") );
  in->addAttribute( tds->getAttribute("density") );

  // Generate a doe
  TBasicSampling *sampl = new TBasicSampling(tds,"srs",10);
  sampl->generateSample();

  // Define the output
  TString sOutFileName="withblank_output.dat";
  TOutputFileKey *fout = new TOutputFileKey(sOutFileName);
  // Set the separator to '=' so that the line could contains blank spaces (specific)
  fout->setFieldSeparatorCharacter("=");
  fout->setSeparatorCharacter("\n");

  // Define the attributes and their keys
  TAttribute *outAtt = new TAttribute("outpressure");
  outAtt->setFileKey(sOutFileName,"Test doubling.water model.brewery pit pressure ");
  TAttribute *outAtt2 = new TAttribute("outdensity");
  outAtt2->setFileKey(sOutFileName,"Chocolat doubling.residual brew density model.factor for glass ");
  fout->addAttribute(outAtt);
  fout->addAttribute(outAtt2);
    
  // Create a TCode object with the TDS (attribute and input files) and the command to execute
  TCode *mycode = new TCode(tds, "withblank");
  // Add the output file
  mycode->addInputFile( in );    
  mycode->addOutputFile( fout );
    
  // Launcher of the code on the Design of experiments (DoE) in the TDS
  TLauncher *tlch = new TLauncher(tds, mycode);
  tlch->setSave(-1);
  tlch->setClean();
  tlch->run();

  tds->Scan("pressure:outpressure:density:outdensity");
  
}
  