
void launchCodeFlowrateKeywithFlagSampling(Int_t nS = 100)
{
  
  // Define the DataServer
  TDataServer *tds = new TDataServer("tdsFlowrate", "Design of experiments for Flowrate");

  // Add the study attributes ( min, max and nominal values)
  tds->addAttribute( new TUniformDistribution("rw", 0.05, 0.15));
  tds->addAttribute( new TUniformDistribution("r", 100.0, 50000.0));
  tds->addAttribute( new TUniformDistribution("tu", 63070.0, 115600.0));
  tds->addAttribute( new TUniformDistribution("tl", 63.1, 116.0));
  tds->addAttribute( new TUniformDistribution("hu", 990.0, 1110.0));
  tds->addAttribute( new TUniformDistribution("hl", 700.0, 820.0));
  tds->addAttribute( new TUniformDistribution("l", 1120.0, 1680.0));
  tds->addAttribute( new TUniformDistribution("kw", 9855.0, 12045.0));

  // The reference input file with key
  TString sInKey = TString("flowrate_input_with_keys.in");
  
  // Set the reference input file and the key for each input attributes
  tds->getAttribute("rw")->setFileKey(sInKey, "Rw");
  tds->getAttribute("r")->setFileKey(sInKey, "R");
  tds->getAttribute("tu")->setFileKey(sInKey, "Tu");
  tds->getAttribute("tl")->setFileKey(sInKey, "Tl");
  tds->getAttribute("hu")->setFileKey(sInKey, "Hu");
  tds->getAttribute("hl")->setFileKey(sInKey, "Hl");
  tds->getAttribute("l")->setFileKey(sInKey, "L");
  tds->getAttribute("kw")->setFileKey(sInKey, "Kw");
  
  // Generate the Design of Experiments
  TSampling *sampling = new TSampling(tds, "lhs", nS);  
  sampling->generateSample();   

  // The output file of the code
  TOutputFileRow *fout = new TOutputFileRow("_output_flowrate_withRow_.dat");
  // The attribute in the output file
  fout->addAttribute(new TAttribute("yhat"));
  
  // Instanciation de mon code
  TCode *mycode = new TCode(tds, "flowrate -s -k");
  // on ajoute le fichier de sortie du code
  mycode->setReferenceDirectory( gSystem->pwd() );
  mycode->addInputFile("flowrate_input_with_flags.in");
  
  mycode->addOutputFile( fout );
  
  // Lancement du code
  TLauncher *lanceur = new TLauncher(tds, mycode);
  //lanceur->setWorkingDirectory(gSystem->Getenv("PWD") + TString("/tmpLanceurUranie/flowrate"));
  lanceur->setVarDraw("yhat:rw","","");

  lanceur->run();

  // Visualisation
  TCanvas  *Canvas = new TCanvas("c1", "Graph for the Macro launchCodeFlowrateOATMinMax",5,64,1270,667);
  gStyle->SetPalette(1);
  tds->draw("yhat:rw");
  
  

  tds->startViewer();
}
