"""
Example of flowrate code launching with dataserver output
"""
from rootlogon import ROOT, DataServer, Launcher, Sampler

nS = 100
# Define the DataServer
tds = DataServer.TDataServer("tdsFlowrate", "DoE for Flowrate")

# Add the study attributes ( min, max and nominal values)
tds.addAttribute(DataServer.TUniformDistribution("rw", 0.05, 0.15))
tds.addAttribute(DataServer.TUniformDistribution("r", 100.0, 50000.0))
tds.addAttribute(DataServer.TUniformDistribution("tu", 63070.0, 115600.0))
tds.addAttribute(DataServer.TUniformDistribution("tl", 63.1, 116.0))
tds.addAttribute(DataServer.TUniformDistribution("hu", 990.0, 1110.0))
tds.addAttribute(DataServer.TUniformDistribution("hl", 700.0, 820.0))
tds.addAttribute(DataServer.TUniformDistribution("l", 1120.0, 1680.0))
tds.addAttribute(DataServer.TUniformDistribution("kw", 9855.0, 12045.0))

# The input file
sFileName = "flowrate_input_with_values_rows.in"

# Set input file with "key = value" format CREATED ON FLY one row per attribute
InType = DataServer.TAttributeFileKey.kNewRow
tds.getAttribute("rw").setFileKey(sFileName, "Rw", "", InType)
tds.getAttribute("r").setFileKey(sFileName, "R", "", InType)
tds.getAttribute("tu").setFileKey(sFileName, "Tu", "", InType)
tds.getAttribute("tl").setFileKey(sFileName, "Tl", "", InType)
tds.getAttribute("hu").setFileKey(sFileName, "Hu", "", InType)
tds.getAttribute("hl").setFileKey(sFileName, "Hl", "", InType)
tds.getAttribute("l").setFileKey(sFileName, "L", "", InType)
tds.getAttribute("kw").setFileKey(sFileName, "Kw", "", InType)

# Generate a Design of experiments (DoE) from the TDataServer
sampling = Sampler.TSampling(tds, "lhs", nS)
sampling.generateSample()

# The output file is based on dataserver format (#COLUMN_NAMES: yhat | d)
fout = Launcher.TOutputFileDataServer("_output_flowrate_withRow_.dat")
# The attributes in the output file
fout.addAttribute("d")  # fout.addAttribute(DataServer.TAttribute("d"))
fout.addAttribute("yhat")  # fout.addAttribute(DataServer.TAttribute("yhat"))

# Create a TCode object with the TDS and the command to execute
mycode = Launcher.TCode(tds, "flowrate -s -r")
# Add the output file
mycode.addOutputFile(fout)

# Launcher of the code on the Design of experiments (DoE) in the TDS
tlch = Launcher.TLauncher(tds, mycode)
tlch.setSave()
tlch.setClean()
# tlch.setWorkingDirectory(ROOT.gSystem.pwd() + "/tmpLanceurUranie/flowrate")
tlch.setVarDraw("yhat:rw", "", "")

# Evaluate the DoE
tlch.run()

# tds.exportData("_flowrate_sampler_launcher_.dat")

# Visualisation
Canvas = ROOT.TCanvas("c1",
                      "launchCodeFlowrateRowRecreateSamplingOutputDataServer",
                      565, 62, 560, 619)
pad = ROOT.TPad("pad", "pad", 0, 0.03, 1, 1)
pad.Draw()
ROOT.gStyle.SetPalette(1)
pad.Divide(1, 3)
pad.cd(1)
tds.draw("yhat:rw")

pad.cd(2)
# Histogramm of the output attribute
tds.draw("yhat")

pad.cd(3)
# The parallel plot
tds.draw("rw:r:tu:tl:hu:hl:l:kw:yhat", "", "para")
para = ROOT.gPad.GetListOfPrimitives().FindObject("ParaCoord")

# The output attribute
axis = para.GetVarList().FindObject("yhat")
axis.AddRange(ROOT.TParallelCoordRange(axis, 15.0, 20.0))
para.AddSelection("blue")
para.GetCurrentSelection().SetLineColor(ROOT.kBlue)

axis.AddRange(ROOT.TParallelCoordRange(axis, 155.0, 160.0))
para.AddSelection("blue")
