void launchCodeFlowrateXMLSampling(Int_t nS = 100){
  // Define the DataServer
  TDataServer *tds = new TDataServer("tdsFlowrate", "Design of experiments for Flowrate");

  // Add the study attributes ( min, max and nominal values)
  tds->addAttribute( new TUniformDistribution("rw", 0.05, 0.15));
  tds->addAttribute( new TUniformDistribution("r", 100.0, 50000.0));
  tds->addAttribute( new TUniformDistribution("tu", 63070.0, 115600.0));
  tds->addAttribute( new TUniformDistribution("tl", 63.1, 116.0));
  tds->addAttribute( new TUniformDistribution("hu", 990.0, 1110.0));
  tds->addAttribute( new TUniformDistribution("hl", 700.0, 820.0));
  tds->addAttribute( new TUniformDistribution("l", 1120.0, 1680.0));
  tds->addAttribute( new TUniformDistribution("kw", 9855.0, 12045.0));

  // The reference input file
  TString sIn = TString("flowrate_input_with_xml.in");

  tds->getAttribute("rw")->setFileKey(sIn, "wall_friction/@rw", "%e", TAttributeFileKey::kXMLAttribute);
  tds->getAttribute("r")->setFileKey(sIn, "wall_friction/@r", "%e", TAttributeFileKey::kXMLAttribute);
  tds->getAttribute("tu")->setFileKey(sIn, "parameter[@name='Tu']/value/double", "%e", TAttributeFileKey::kXMLField);
  tds->getAttribute("tl")->setFileKey(sIn, "parameter[@name='Tl']/value/double", "%e", TAttributeFileKey::kXMLField);
  tds->getAttribute("hu")->setFileKey(sIn, "parameter[tonode='mesher' and toport='dt_hu']/value/double", "%e", TAttributeFileKey::kXMLField);
  tds->getAttribute("hl")->setFileKey(sIn, "parameter[tonode='mesher' and toport='dt_hl']/value/double", "%e", TAttributeFileKey::kXMLField);
  tds->getAttribute("l")->setFileKey(sIn, "parameter[@name='L']/@precision", "%e", TAttributeFileKey::kXMLAttribute);
  tds->getAttribute("kw")->setFileKey(sIn, "kW/@value", "%e", TAttributeFileKey::kXMLAttribute);
    
  
  // Generate the Design of Experiments
  TSampling *sampling = new TSampling(tds, "lhs", nS);  
  sampling->generateSample();	
    

  // The output file of the code
  TOutputFileXML *fout = new TOutputFileXML("_output_flowrate_withXML_.dat");
  // The attribute in the output file
  fout->addAttribute(new TAttribute("yhat"), "/steady_state[@name='flowrate']/parameter/value/double", TAttributeFileKey::kXMLField);
  
  // Instanciation de mon code
  TCode *mycode = new TCode(tds, "flowrate -s -x");
  // on ajoute le fichier de sortie du code
  mycode->addOutputFile( fout );
  
  // Lancement du code
  TLauncher *lanceur = new TLauncher(tds, mycode);
  lanceur->setSave();
  lanceur->setClean();
  lanceur->setVarDraw("yhat:rw","","");

  lanceur->run();
  
  tds->exportData("_flowrate_sampler_launcher_.dat");
  
      
  // Visualisation
  TCanvas  *Canvas = new TCanvas("c1", "Graph for the Macro launchCodeFlowrateOATMinMax",5,64,1270,667);
  TPad *pad = new TPad("pad","pad",0, 0.03, 1, 1); pad->Draw();
  gStyle->SetPalette(1);
  pad->Divide(2, 2);
  pad->cd(1); tds->draw("yhat:rw");

  pad->cd(3); tds->draw("yhat");
  pad->cd(2); tds->draw("yhat:rw","","colz");
  
  pad->cd(4);
  // The parallel plot
  tds->draw("rw:r:tu:tl:hu:hl:l:kw:yhat","","para");
  TParallelCoord* para = (TParallelCoord*)gPad->GetListOfPrimitives()->FindObject("ParaCoord");
  
  // The output attribute
  TParallelCoordVar* axis = (TParallelCoordVar*)para->GetVarList()->FindObject("yhat");
  axis->AddRange(new TParallelCoordRange(axis,15.0,80.0));
  para->AddSelection("blue");
  para->GetCurrentSelection()->SetLineColor(kBlue);
      
}
