"""
Example of readmultitype code launching with column output format
"""
from rootlogon import ROOT, DataServer, Launcher

# Create dataserver with a database of 10 runs of the "multitype -mt" code
tds = DataServer.TDataServer("foo", "pouet")
tds.fileDataRead("readmultitype_sampling.dat")

# Explain the name of the input file and the format chosen
InFile = "_output_multitype_mt_Column_.dat"
InType = DataServer.TAttributeFileKey.kNewColumn
tds.getAttribute("w1").setFileKey(InFile, "", "%e", InType)
tds.getAttribute("v1").setFileKey(InFile, "", "%e", InType)
tds.getAttribute("v2").setFileKey(InFile, "", "%e", InType)
tds.getAttribute("f1").setFileKey(InFile, "", "%e", InType)
tds.getAttribute("w2").setFileKey(InFile, "", "%e", InType)

# Create the Class that will handle re-writting the input file
FullFileName = ROOT.gSystem.PrependPathName(ROOT.gSystem.pwd(), InFile)
inFile = Launcher.TInputFileRecreate(FullFileName)

# Add attribute in the correct (needed) order
inFile.addAttribute(tds.getAttribute("w1"))
inFile.addAttribute(tds.getAttribute("v1"))
inFile.addAttribute(tds.getAttribute("v2"))
inFile.addAttribute(tds.getAttribute("f1"))
inFile.addAttribute(tds.getAttribute("w2"))

# Create the output file interface and state that there will be 2 outputs
out = Launcher.TOutputFileColumn("_output_multitype_readmt_Column_.dat")
out.addAttribute(DataServer.TAttribute("thev1"))
out.addAttribute(DataServer.TAttribute("thev2"))

# Create the corresponding interface to code
tc1 = Launcher.TCode(tds, "multitype -ReadmtCol")
tc1.addInputFile(inFile)
tc1.addOutputFile(out)

# Create the launcher and run the code
tl1 = Launcher.TLauncher(tds, tc1)
tl1.run()

tds.Scan("thev1:thev2")
