"""
Example of function optimisation creating new inputs and outputs on the fly
"""
from rootlogon import ROOT, DataServer, Optimizer

# Load the function
ROOT.gROOT.LoadMacro("UserFunctions.C")

# Les variables de l'etude
x = DataServer.TAttribute("x", -1.5, 1.5)
x.setDefaultValue(-1.2)
x.setStepValue(0.01)

y = DataServer.TAttribute("y", -1.5, 1.5)
y.setDefaultValue(1.)
y.setStepValue(0.01)

tdsRos = DataServer.TDataServer("tdsRosenbrock", "Optimize Rosenbrock")
tdsRos.addAttribute(x)
tdsRos.addAttribute(y)

tdsRos.addAttribute("xshift", "x-0.1")
tdsRos.addAttribute("yshift", "y+0.2")

# Graph
Canvas = ROOT.TCanvas("c1", "Graph optimizeFunctionRosenbrockNewInputOutput",
                      5, 64, 1270, 667)

# Create an TOptimizer object from TDataServer and an anlystical function
topt = Optimizer.TOptimizer(tdsRos, "rosenbrock", "xshift:yshift", "out")
topt.addOutputVariable("out+1:out*out:out*3")
topt.selectCost("out+1")
# topt.setMethod(Optimizer.TOptimizer.kSimplex)
# topt.setTolerance(1e-5)
# topt.setPrintLevel(5)
# topt.setMaxIterations(3)
# topt.setMaxFunctionCalls(10)
topt.optimize("same")
tdsRos.exportData("_tds_rosenbrock_.dat")
