
{
  // Create the TDataServer
  TDataServer *tds = new TDataServer("foo","test");

  // Define the attribute that should be considered as constant
  TAttribute r("r");
  
  // Add the study attributes ( min, max and nominal values)
  tds->addAttribute( new TUniformDistribution("rw", 0.05, 0.15));
  tds->addAttribute( new TUniformDistribution("tu", 63070.0, 115600.0));
  tds->addAttribute( new TUniformDistribution("tl", 63.1, 116.0));
  tds->addAttribute( new TUniformDistribution("hu", 990.0, 1110.0));
  tds->addAttribute( new TUniformDistribution("hl", 700.0, 820.0));
  tds->addAttribute( new TUniformDistribution("l", 1120.0, 1680.0));
  tds->addAttribute( new TUniformDistribution("kw", 9855.0, 12045.0));

  // The reference input file
  TString sIn = TString("flowrate_input_with_keys.in");

  int nS=15;
  // Generate the Design of Experiments
  TSampling *sampling = new TSampling(tds, "lhs", nS);
  sampling->generateSample();   

  // Create the input files
  TKeyScript inputFile( sIn.Data() );
  inputFile.addInput(tds->getAttribute("rw"),"Rw");
  inputFile.addInput(&r,"R"); // Add the constant attribute as an input
  inputFile.addInput(tds->getAttribute("tu"),"Tu");
  inputFile.addInput(tds->getAttribute("tl"),"Tl");
  inputFile.addInput(tds->getAttribute("hu"),"Hu");
  inputFile.addInput(tds->getAttribute("hl"),"Hl");
  inputFile.addInput(tds->getAttribute("l"),"L");
  inputFile.addInput(tds->getAttribute("kw"),"Kw"); 

  // Create the output attribute
  TAttribute *yhat = new TAttribute("yhat");
  TAttribute *d = new TAttribute("d");

  // Create the output files
  TKeyResult outputFile("_output_flowrate_withKey_.dat");
  outputFile.addOutput(yhat, "yhat");
  outputFile.addOutput(d, "d");
  
  // Create the user's evaluation function
  TCodeEval eval("flowrate -s -k");
  eval.addInputFile(&inputFile);
  eval.addOutputFile(&outputFile);
  
  // Create the sequential runner
  TSequentialRun run(&eval);
  run.startSlave(); //Start the master (necessary even for a sequential)
  if (run.onMaster())
  {
      TLauncher2 lanceur(tds, &run);
      // State to the master : r is constant with value 108
      // By default the value is not kept in the tds.
      // The third argument says : yes, keep it for bookkeeping
      lanceur.addConstantValue(&r,108,true);

      // resolution
      lanceur.solverLoop();
      run.stopSlave(); // Stop the slaves (necessary even for a sequential)
  }

  tds->scan("*");
  
   
}
