
/* possible solver 
 * direct
#define SOLV TNloptCobyla
 * lagragien direct
#define SOLV TNloptBobyqa //doesn't work on this problem
#define SOLV TNloptPraxis
#define SOLV TNloptNelderMead
#define SOLV TNloptSubplexe
 * */
#define SOLV TNloptCobyla

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;
using namespace URANIE::Reliability;

/********** User Function *****************/
static void fselect(double *in, double *res)
{
  double x,fe,M;

  x=in[0]; fe=in[1]; M=in[2];
  /* critere */
  res[0] = M-x*fe;
}

/********** Optimization  *****************/
void reliabilityFormSorm()
{
   
  /* inputs */
  TNormalDistribution x("x", 40, 5),
                      fe("x2", 50, 2.5),
                      M("M", 1000, 200);
  /* outputs */
  TAttribute cont("seuil");
  /* starting point */
  vector<double> start{-1., -1., 1.};
  
  /* code */
  TSimpleTransform fobj;
  fobj.setParameters(3, &x, &fe, &M);
  
  TCIntEval fcont("fselect");
  fcont.setInputs(3, &x, &fe, &M);
  fcont.setOutputs(1, &cont);
  
  TFormEval code(&fobj, &fcont);
  TGreaterFit it(0.0);
  code.addConstraint(&cont, &it);
  
  /* runner */
  TSequentialRun run(&code);
  run.startSlave();
  if (run.onMaster()) {
    /* tds */
    TDataServer tds("toto", "tds for vizir test");
    code.addAllInputs(&tds);
    
    /*** FORM ***/
    /* optimizer */
    SOLV solv;
    TNlopt nlo(&tds, &run, &solv);
    code.addObjective(&nlo);
    
    /* resolution */
    nlo.setStartingPoint(start.size(),&start[0]);
    nlo.solverLoop();
 
    /* results */
    //tds.getTuple()->Scan("*");
    
    /*** SORM ***/
    TSorm sorm(&tds, &run);
    sorm.solverLoop();
    
    /* results */
    tds.getTuple()->Scan("*");
    
    /* cleanup */
    run.stopSlave();
  } 
}