"""
Example of reliability form / sorm usage
"""
import numpy as np
from rootlogon import DataServer, Reliability, Relauncher, Reoptimizer


def fselect(x_var, fe_var, m_var):
    """Compute the criterion."""
    # critere
    return [m_var-x_var*fe_var]


# inputs
x = DataServer.TNormalDistribution("x", 40, 5)
fe = DataServer.TNormalDistribution("x2", 50, 2.5)
M = DataServer.TNormalDistribution("M", 1000, 200)

# outputs
cont = DataServer.TAttribute("seuil")
# starting point
start = np.array([-1., -1., 1.], dtype=np.float64)

# code
fobj = Reliability.TSimpleTransform()
fobj.addParameter(x)
fobj.addParameter(fe)
fobj.addParameter(M)

fcont = Relauncher.TPythonEval(fselect)
fcont.addInput(x)
fcont.addInput(fe)
fcont.addInput(M)
fcont.addOutput(cont)

code = Reliability.TFormEval(fobj, fcont)
it = Reoptimizer.TGreaterFit(0.0)
code.addConstraint(cont, it)

# runner
run = Relauncher.TSequentialRun(code)
run.startSlave()
if run.onMaster():
    # tds
    tds = DataServer.TDataServer("toto", "tds for vizir test")
    code.addAllInputs(tds)

    # FORM
    # optimizer
    solv = Reoptimizer.TNloptCobyla()
    nlo = Reoptimizer.TNlopt(tds, run, solv)
    code.addObjective(nlo)

    # resolution
    nlo.setStartingPoint(len(start), start)
    nlo.solverLoop()
    # results
    # tds.getTuple().Scan("*")

    # SORM
    sorm = Reliability.TSorm(tds, run)
    sorm.solverLoop()

    # results
    tds.getTuple().Scan("*")

    # cleanup
    run.stopSlave()
