#define TOLERANCE 0.001
#define NBmaxEVAL 1200
#define SIZE 500

bool LaunchVizir(int RunNumber, TCanvas *fig1)
{
    
    // variables
    TAttribute x("x", 0.0, 1.0),
      y("y", 0.0, 1.0),
      thick("thick"),
      sect("sect"),
      def("def");

    TCIntEval code("barAllCost");
    code.addInput(&x);
    code.addInput(&y);
    code.addOutput(&thick);
    code.addOutput(&sect);
    code.addOutput(&def);

    // Create a runner
    TSequentialRun runner(&code);
    runner.startSlave();

    // Output to state whether convergence is reached
    bool hasConverged=false; 
    if(runner.onMaster())
    {
        // Create the TDS
        TDataServer tds("vizirDemo", "Param de l'opt vizir pour la barre");
        tds.addAttribute(&x);
        tds.addAttribute(&y);
    
        TVizirGenetic solv;
        // Name of the file that will contain
        string filename="genetic.dump";
        std::vector<char> cstr(filename.c_str(), filename.c_str() + filename.size() + 1);
        /* Test whether genetic.dump exists. If not, it creates it and returns false, so
        that the "else" part is done to start the initialisation of the vizir algorithm. */
        if ( solv.setResume(NBmaxEVAL, &cstr[0]))
            cout << "Restarting Vizir" << endl;
        else solv.setSize(SIZE, NBmaxEVAL);        

        // Create the multi-objective constrained optimizer
        TVizir2 opt(&tds, &runner, &solv); 
        opt.setTolerance(TOLERANCE);
        // add the objective
        opt.addObjective(&sect);
        opt.addObjective(&def);
        TGreaterFit positiv(0.4);
        opt.addConstraint(&thick,&positiv);
        
        /* resolution */
        opt.solverLoop();
        hasConverged=opt.isConverged();
        // Stop the slave processes
        runner.stopSlave();
        
        fig1->cd(RunNumber+1);
        tds.getTuple()->SetMarkerColor(2);
        tds.draw("def:sect");
        stringstream tit; tit << "Run number "<<RunNumber+1;
        if(hasConverged) tit << ": Converged !";
        ((TH1F*)gPad->GetPrimitive("__tdshisto__0"))->SetTitle( tit.str().c_str() );
    }

    return hasConverged;
}
    
int reoptimizeHollowBarVizirSplitRuns()
{
    using namespace URANIE::DataServer;
    using namespace URANIE::Relauncher;
    using namespace URANIE::Reoptimizer;
  
    gROOT->LoadMacro("UserFunctions.C");
    // Delete previous file if it exists
    gSystem->Unlink("genetic.dump"); 

    bool finished=false;
    int i=0;
    TCanvas *fig1 = new TCanvas("fig1","fig1",1200,800);
    fig1->Divide(2,2);
    while ( ! finished )
    {
        finished=LaunchVizir(i, fig1);
        i++;
    }
           
   return 1;
}
