{
    // Definition of the reference dataserver
    TDataServer *tdsError = new TDataServer("tdsError", "Objectives values");
    tdsError->fileDataRead("rosenbrock_ref.dat");

    TString sinputfile ="input_rosenbrock_with_keys.dat";
    tdsError->getAttribute("x1")->setFileKey(sinputfile, "x");
    tdsError->getAttribute("x2")->setFileKey(sinputfile, "y");

    // Definition of the output file of the code
    TOutputFileRow *fout = new TOutputFileRow("_output_rosenbrock_with_values_rows_.dat");
    fout->addAttribute(new TAttribute("yhat"), 3);

    // OS abstraction
    string to_null =
        string(gSystem->GetBuildArch()) == "win64" ? " > NUL" : " >> /dev/null";

    // Definition of the code
    TCode *mycode = new TCode(tdsError, "rosenbrock -k" + to_null);
    mycode->addOutputFile(fout);

    // Launcher on the TDS Ref
    TLauncher * tl = new TLauncher(tdsError, mycode);
    tl->setDrawProgressBar(kFALSE);
    tl->run(); 

    // Export the reference values for the (a,b) parameters
    tdsError->exportData("_output_rosenbrock_references_.dat","x1:x2:yhat:y");

    // Compute the L1, L2 and LMax norms for the (a,b) parameters
    tdsError->addAttribute("squareDiff", "(yhat-y)^2.");
    tdsError->computeStatistic("squareDiff");

    Double_t normL2 = tdsError->getAttribute("squareDiff")->getMean();

    ofstream outFile("_rosenbrock_error_.dat",ios::out);
    outFile << "normL2 = " << normL2 << ";" << endl;
    outFile << endl;
    outFile.close();
}
