
void sensitivityMorrisFunctionFlowrateRunner(Int_t nk = 5)
{
      
  gROOT->LoadMacro("UserFunctions.C");

  // Define the attributes
  TUniformDistribution rw("rw", 0.05, 0.15);
  TUniformDistribution r("r", 100.0, 50000.0);
  TUniformDistribution tu("tu", 63070.0, 115600.0);
  TUniformDistribution tl("tl", 63.1, 116.0);
  TUniformDistribution hu("hu", 990.0, 1110.0);
  TUniformDistribution hl("hl", 700.0, 820.0);
  TUniformDistribution l("l", 1120.0, 1680.0);
  TUniformDistribution kw("kw", 9855.0, 12045.0);

  // Create the evaluator
  TCIntEval code("flowrateModel");
  // Create output attribute
  TAttribute yout("flowrateModel");
  // Provide input/output attributes to the assessor
  code.setInputs(8, &rw, &r, &tu, &tl, &hu, &hl, &l, &kw);
  code.setOutputs(1, &yout);

  TSequentialRun run(&code); // To be replaced to distribute the computation
  run.startSlave();
  if( run.onMaster())
  {
    // Create the dataserver
    TDataServer *tds = new TDataServer("sobol", "foo bar pouet chocolat");
    tds->addAttribute(&rw);
    tds->addAttribute(&r);
    tds->addAttribute(&tu);
    tds->addAttribute(&tl);
    tds->addAttribute(&hu);
    tds->addAttribute(&hl);
    tds->addAttribute(&l);
    tds->addAttribute(&kw);    
    Int_t nreplique = 3;
    Int_t nlevel = 10;
    // Create the Morris object     
    TMorris * scmo = new TMorris(tds, &run, nreplique, nlevel); 
    scmo->setDrawProgressBar(kFALSE);
    scmo->generateSample();
    
    tds->exportData("_morris_sampling_.dat");
    scmo->computeIndexes();
    
    tds->exportData("_morris_launching_.dat");

    TTree *ntresu = scmo->getMorrisResults();
    ntresu->Scan("*");
     
    // Graph
    TCanvas  *cc = new TCanvas("c1", "Graph for the Macro sensitivityMorrisFunctionFlowrateRunner",5,64,1270,667);
    TPad *pad = new TPad("pad","pad",0, 0.03, 1, 1); pad->Draw();
    pad->Divide(2);
    pad->cd(1);
    scmo->drawSample("", -1,"nonewcanv");
    pad->cd(2);
    scmo->drawIndexes("mustar,nonewcanv");
   
  }
}