"""
Example of Morris computation
"""
from rootlogon import ROOT, DataServer, Sensitivity

ROOT.gROOT.LoadMacro("UserFunctions.C")

# Define the DataServer
tds = DataServer.TDataServer("tdsflowrate", "DataBase flowrate")
tds.addAttribute(DataServer.TUniformDistribution("rw", 0.05, 0.15))
tds.addAttribute(DataServer.TUniformDistribution("r", 100.0, 50000.0))
tds.addAttribute(DataServer.TUniformDistribution("tu", 63070.0, 115600.0))
tds.addAttribute(DataServer.TUniformDistribution("tl", 63.1, 116.0))
tds.addAttribute(DataServer.TUniformDistribution("hu", 990.0, 1110.0))
tds.addAttribute(DataServer.TUniformDistribution("hl", 700.0, 820.0))
tds.addAttribute(DataServer.TUniformDistribution("l", 1120.0, 1680.0))
tds.addAttribute(DataServer.TUniformDistribution("kw", 9855.0, 12045.0))

# Set the Morris method parameters
nreplica = 3
nlevel = 10

# Create a TMorris object
scmo = Sensitivity.TMorris(tds, "flowrateModel", nreplica, nlevel)

# generate a Morris sample
scmo.generateSample()

# Save the sample
tds.exportData("_morris_sampling_.dat")

# compute the Morris screening measures
scmo.computeIndexes()

# Display graphically the Morris screanig measures
cc = ROOT.TCanvas("can", "histogramme")
apad = ROOT.TPad("apad", "apad", 0, 0.03, 1, 1)
apad.Draw()
apad.cd()
scmo.drawIndexes("mustar", "", "nonewcanv")
