//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#ifndef CPPUNITEST_TLINEARBAYESIAN_H
#define CPPUNITEST_TLINEARBAYESIAN_H

// CppUnit
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/Test.h>

//URANIE
#include "TDataServer.h"
#include "TLinearBayesian.h"
#include "TSequentialRun.h"
#include "TThreadedRun.h"
#include "TNormalDistribution.h"
#include "TCJitEval.h"
#include "TCodeEval.h"
#include "TCode.h"
#include "TOutputFileRow.h"
#include "TAttribute.h"
#include "TKeyScript.h"
#include "TKeyResult.h"

using namespace URANIE::DataServer;
using namespace URANIE::Calibration;
using namespace URANIE::Relauncher;
using namespace URANIE::Launcher;

class TLinearBayesianTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TLinearBayesianTest);

    CPPUNIT_TEST( runFlow1DCJitNoPlots );
    CPPUNIT_TEST( runFlow1DNoPlotsCodeEval );
    CPPUNIT_TEST( runFlow1DNoPlotsCodeEvalPar );
    CPPUNIT_TEST( runFlow1DNoPlots );
    CPPUNIT_TEST( runFlow1DPointerFunctionNoPlots );
    CPPUNIT_TEST( runFlow1DNoPlotsCode );
    
    CPPUNIT_TEST ( throwExceptionConstruction );
    CPPUNIT_TEST ( throwExceptionRegressorsAndCovMat );
    CPPUNIT_TEST ( throwExceptionReference );

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TLinearBayesianTest();

    //! Default destructor
    virtual ~TLinearBayesianTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void setUpException();
    void setUpFlow1D(string type="TCJitEval");
    void tearDown();

    // Transformation function
    static void transf(double *x, double *res);

    void cleanWorkingDirectories();
    
    //@}
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void runFlow1DCJitNoPlots();
    void runFlow1DNoPlotsCodeEval();
    void runFlow1DNoPlotsCodeEvalPar();
    void runFlow1DNoPlots();
    void runFlow1DPointerFunctionNoPlots();
    void runFlow1DNoPlotsCode();
    
    void throwExceptionConstruction();
    void throwExceptionRegressorsAndCovMat();
    void throwExceptionReference();
    //@}

private:
    TString _workingTestPath;

    TDataServer *_tds;
    TDataServer *_tdsRef;
    TLinearBayesian *_tlb;
    TSequentialRun *_seqRun;
    TThreadedRun *_thrRun;
    TCJitEval *_evalJit;
    TCodeEval *_evalCode;
    TCode *_code;
    TOutputFileRow *_outRow;
    TKeyScript *_inp;
    TKeyResult *_outp;
    TNormalDistribution *_norm;
    TAttribute *_out;
};

#endif

