//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#ifndef CPPUNITEST_TMINIMISATION_H
#define CPPUNITEST_TMINIMISATION_H

// CppUnit
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/Test.h>

//URANIE
#include "TDataServer.h"
#include "TMinimisation.h"
#include "TSequentialRun.h"
#include "TNormalDistribution.h"
#include "TCJitEval.h"
#include "TCodeEval.h"
#include "TAttribute.h"
#include "TKeyScript.h"
#include "TKeyResult.h"

using namespace URANIE::DataServer;
using namespace URANIE::Calibration;
using namespace URANIE::Relauncher;

class TMinimisationTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TMinimisationTest);
#ifdef WITH_NLOPT
    CPPUNIT_TEST( runFlow1DNoPlots );
    CPPUNIT_TEST( runFlow1DNoPlotsCode );
#endif
#ifdef WITH_VIZIR
    CPPUNIT_TEST( runFlow1DVizirNoPlots);
    CPPUNIT_TEST( runFlow2DVizirNoPlots);
#endif    
    CPPUNIT_TEST ( throwExceptionWrongConstruction );
    CPPUNIT_TEST ( throwExceptionConstruction );
#ifdef WITH_NLOPT
    CPPUNIT_TEST ( throwExceptionOptimproperties );
#endif
    CPPUNIT_TEST ( throwExceptionReference );

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TMinimisationTest();

    //! Default destructor
    virtual ~TMinimisationTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void setUpException();
    void setUpFlow1D(bool function);
    void setUpFlow2D();
    void tearDown();

    void removeFile(const char* folder, const char* filename);
    void removeFolder(const char* workfolder, const char* folder);
    void cleanWorkingDirectories();

    //@}
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
#ifdef WITH_NLOPT    
    void runFlow1DNoPlots();
    void runFlow1DNoPlotsCode();
#endif
#ifdef WITH_VIZIR    
    void runFlow1DVizirNoPlots();
    void runFlow2DVizirNoPlots();
#endif

    void throwExceptionWrongConstruction();
    void throwExceptionConstruction();
#ifdef WITH_NLOPT
    void throwExceptionOptimproperties();
#endif    
    void throwExceptionReference();
    //@}

private:
    TString _workingTestPath;
    
    TDataServer *_tds;
    TDataServer *_tdsRef;
    TMinimisation *_tmi;
    TSequentialRun *_seqRun;
    TCJitEval *_evalJit;
    TCodeEval *_evalCode;
    TKeyScript *_inp;
    TKeyResult *_outp;
    TNormalDistribution *_norm;
    TAttribute *_out;
};

#endif

