//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestUserFunctions.cxx
 \author Jean-Baptiste Blanchard
 \date   2020/08/27

 \brief Implement the user function 

 */

// ROOT
#include "TROOT.h"
#include "TMath.h"

void EmptyFunction(double *x, double *y)
{
    y[0] = 2*x[0];
}

void flowrateModel(double *x, double *y)
{
  Double_t drw = x[1], dr  = 25050;
  Double_t dtu = 89335, dtl = 89.55;
  Double_t dhu = 1050, dhl = x[0];
  Double_t dl  = x[2], dkw = 10950;

  Double_t dnum = 2.0 * TMath::Pi() * dtu * ( dhu -dhl);
  Double_t dlnronrw = TMath::Log( dr / drw);
  Double_t dden = dlnronrw * ( 1.0 +  ( 2.0 * dl * dtu ) / ( dlnronrw * drw * drw * dkw) + dtu / dtl );

  y[0] = dnum / dden;
}

void flowrateModelnoH(double *x, double *y)
{
  Double_t drw = x[0], dr  = 25050;
  Double_t dtu = 89335, dtl = 89.55;
  Double_t dl  = x[1], dkw = 10950;

  Double_t dnum = 2.0 * TMath::Pi() * dtu;
  Double_t dlnronrw = TMath::Log( dr / drw);
  Double_t dden = dlnronrw * ( 1.0 +  ( 2.0 * dl * dtu ) / ( dlnronrw * drw * drw * dkw) + dtu / dtl );

  y[0] = dnum / dden;
}
