// --- C++ ---
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2019 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TEgoHjdynSolver.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2019-12-05
 
 \brief  hjma dynamic search for EI next point
*/


#include "TEgoHjDynSolver.h"

ClassImp(URANIE::MetaModelOptim::TEgoHjDynSolver);

// constructeur
URANIE::MetaModelOptim::TEgoHjDynSolver::TEgoHjDynSolver() :
    TEgoHjSolver(), _signif(1.0e-9)
{
}

URANIE::MetaModelOptim::TEgoHjDynSolver::~TEgoHjDynSolver()
{
}

// protected

void URANIE::MetaModelOptim::TEgoHjDynSolver::fitnessChanged()
{
    _algo->fitnessChanged();
}

void URANIE::MetaModelOptim::TEgoHjDynSolver::extend() {}

// public
void URANIE::MetaModelOptim::TEgoHjDynSolver::build(URANIE::Relauncher::TRun *run,
    vector<URANIE::DataServer::TAttribute*>& in, URANIE::DataServer::TAttribute *ego)
{
    int i;
    
    URANIE::MetaModelOptim::TEgoShareSolver::build(run, in, ego);
    // tds
    _tds = new URANIE::DataServer::TDataServer("eihjTDS", "tds for EI search via HJMA");
    _tds->keepFinalTuple(kFALSE);
    for (i=0; _in[i]; i++) _tds->addAttribute(_in[i]);
    // algo
    _algo = new THJMA(_tds, _run);
    _algo->setSize(_Nfirst, _Nstep);
    _algo->addObjective(ego, &_maxim);
    _algo->setUsable(&_signif);
}

