// --- C++ ---
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2019 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TEgoStdSolver.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2019-11-04
 
 \brief  recherche du point minimisant l'EI
*/

#include "TEgoStdSolver.h"

ClassImp(URANIE::MetaModelOptim::TEgoStdSolver);

// constructeur
URANIE::MetaModelOptim::TEgoStdSolver::TEgoStdSolver() :
    TEgoVzrSolver()
{
    _NG = 10000;
}

URANIE::MetaModelOptim::TEgoStdSolver::~TEgoStdSolver()
{
}

// protected

/* récupération de l'optimum EI après utilisation vizir
 ***/
void URANIE::MetaModelOptim::TEgoStdSolver::fitnessChanged()
{
    int j;
    
    if (_tds) delete _tds;
    if (_viz) delete _viz;
    
    // tds
    _tds = new URANIE::DataServer::TDataServer("eiTDS", "tds for EI");
    _tds->keepFinalTuple(kFALSE);
    for (j=0; _in[j]; j++) _tds->addAttribute(_in[j]);
    // viz
    _viz = new URANIE::Reoptimizer::TVizir2(_tds, _run, _algo);
    _viz->setTolerance(1.e-20);
    _viz->addObjective(_ego, &_maxim);
    _viz->addConstraint(_ego, &_signif);
}

void URANIE::MetaModelOptim::TEgoStdSolver::extend()
{
    _viz->extend(_NG);
}

// public
void URANIE::MetaModelOptim::TEgoStdSolver::build(URANIE::Relauncher::TRun *run, 
    vector<URANIE::DataServer::TAttribute*>& in, URANIE::DataServer::TAttribute *ego)
{
    URANIE::MetaModelOptim::TEgoShareSolver::build(run, in, ego);
    if (!_algo) {
        _algo = new URANIE::Reoptimizer::TVizirGenetic();
        _algo->setSize(320, 100000, 160);
    }
    fitnessChanged();
}

int URANIE::MetaModelOptim::TEgoStdSolver::getNext(long ev, int rien, 
    vector<vector<URANIE::DataServer::UEntry*>> &sampl, vector<char> &mark)
{
    UNUSED(rien);
    
    int i, id, rank, info;
    double bfront[4];
    vector<double> bzone(_NinDat);

    // recherche du meilleur
    id = 0;
    _viz->item(id, bzone.data() , bfront, &rank, NULL, &info, NULL);
    if (bfront[1] > -_valid_ei) return 0;

    //getBest(tmp);
    std::cout << "EGO expect " << ev << " " << -bfront[1] << std::endl;
    //copie
    mark[ev] = FILL;
    *sampl[ev][0]->val.dou = ev ;
    for (i=0; i<_NinDat; i++) *sampl[ev][i+1]->val.dou = bzone[i];
    
    return 1;
}

