/*/////////////////////////////////////////////////////////////
 // Copyright (C) 2013-2023 CEA/DES
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Lesser General Public License as published
 // by the Free Software Foundation, either version 3 of the License, or any
 // later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 // $Id$
 // $Author$
 // $Date$
 // $Revision$
 // $State$
 /////////////////////////////////////////////////////////////*/
/*!
 \file   beschb.c
 \author Jean-Marc Martinez <jean-marc.martinez@cea.fr>
 \date   2012/07/19 15:38:00

 \brief
 */
#define NUSE1 5
#define NUSE2 5

/* -- ngi: modifications du 25/10/2012 --
 ** - beschb retourne maintenant un code d'erreur entier (1 en cas d'erreur, 0 sinon)
 ** - le test de la fonction chebev verifiant l'appartenance de xx a l'interval ]-1.0, 1.0[ a ete 
 **   transfere dans la fonction beschb
 */
int beschb(double x, double *gam1, double *gam2, double *gampl, double *gammi)
{
    double chebev(double a, double b, double c[], int m, double x);
    double xx;
    static double c1[] =
    { -1.142022680371172e0, 6.516511267076e-3, 3.08709017308e-4,
            -3.470626964e-6, 6.943764e-9, 3.6780e-11, -1.36e-13 };
    static double c2[] =
    { 1.843740587300906e0, -0.076852840844786e0, 1.271927136655e-3,
            -4.971736704e-6, -3.3126120e-8, 2.42310e-10, -1.70e-13, -1.0e-15 };

    xx = 8.0 * x * x - 1.0;
    if ((xx - (-1.0)) * (xx - 1.0) > 0.0)
        return (1);
    *gam1 = chebev(-1.0, 1.0, c1, NUSE1, xx);
    *gam2 = chebev(-1.0, 1.0, c2, NUSE2, xx);
    *gampl = *gam2 - x * (*gam1);
    *gammi = *gam2 + x * (*gam1);

    return (0);
}
#undef NUSE1
#undef NUSE2

