/*//////////////////////////////////////////////////////////////
 // Copyright (C) 2013-2023 CEA/DES
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Lesser General Public License as published
 // by the Free Software Foundation, either version 3 of the License, or any
 // later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 // $Id$
 // $Author$
 // $Date$
 // $Revision$
 // $State$
 //////////////////////////////////////////////////////////////*/
/*!
 \file   gpDposl.c
 \author Jean-Marc Martinez <jean-marc.martinez@cea.fr>
 \date   2012/07/19 15:38:00

 \brief
 */
# include "blas1_d.h"
# include "linpack_d.h"
#include <stdio.h>
#ifdef WIN32
#include "libpgauss_Export.h"
#else
#define libpgauss_EXPORT
#endif

/****************************************************************************/

libpgauss_EXPORT void gpDposl(double a[], int lda, int n, double b[])

/****************************************************************************/
/*  =======================================================
 JM Martinez 29/04/2012
 Nouvelle fonction definie a partir de la fonction dposl
 Renvoie la solution x telle que Lx = y
 ou L est une matrice triangulaire inferieure
 L est obtenue par la decomposition Cholesky dpofa
 =======================================================
 */
{
    int k;
    double t;
    /*
     **  Solve R' * Y = B.
     */
    for (k = 1; k <= n; k++)
    {
        t = uranie_ddot(k - 1, a + 0 + (k - 1) * lda, 1, b, 1);
        b[k - 1] = (b[k - 1] - t) / a[k - 1 + (k - 1) * lda];
    }
    return;
}

