/*//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////*/

#include <stdlib.h>
#include <math.h>
#include "linpack_d.h"
#ifdef WIN32
#include "libpgauss_Export.h"
#else
#define libpgauss_EXPORT
#endif

/*
              IMPLEMENTED ON 2019/09/25
*/
/* Calcul de des predictions - sans tendance deterministe       */
/* Methode gpPred0                                      	*/
/* Input Kp : matrice new x n de covariances new x n		*/
/* Input Rp : matrice de variance new x new			*/
/* Input variance : variance du processus			*/
/* Input gamma : vecteur de taille n calcule par gpMOd0         */
/* Input iL : matrice n x n calcule par gpMOd0        		*/
/* Input new : nombre de ligne de Kp                   		*/
/* Input n : nombre de colonnes de Kp et taille de gamma	*/
/* Output yp : vecteur des new predictions			*/
/* Output vp : matrice de variance new x new			*/
libpgauss_EXPORT void gpPred0Cov(double *Kp, double *Rp, double variance, double *gamma, double *iL, int new, int n, double *yp, double *vp) {
    int i, j, inew, jnew, k;
    double vt1, vt2, *pgamma;

    /* Unchanged from pred0 - JBB */
    /* calcul des predictions */
    for(i=0;i<new;i++) {
        pgamma = gamma;
        for(vt1=0.,j=0;j<n;j++) vt1 += Kp[i+j*new] * *pgamma++;
        yp[i] = vt1;
    }
  
    /* compute the matrix Kp * iL*/
    double *KpiL = calloc(sizeof(double),new*n);
    for(i=0;i<new;i++) {
        for(j=0;j<n;j++) {
            KpiL[i*n+j]=0;
            for(k=0;k<=j;k++) KpiL[i*n+j] += Kp[i+k*new] * iL[k+j*n];
        }
    }
  
    /* Modified to take into account Cov between new points pred0 - JBB */
    /* calcul des variances */
    for(inew=0;inew<new;inew++) {
        for(jnew=0;jnew<new;jnew++) {
            vp[inew*new+jnew] = Rp[inew*new+jnew] ;
            for(vt2=0.,k=0;k<n;k++) vt2 += KpiL[inew*n+k] * KpiL[jnew*n+k];
            vp[inew*new+jnew] -= vt2;
            if (inew==jnew && vp[inew*new+jnew] < 0.) vp[inew*new+jnew] = 0.;
        }
    }
  
    free(KpiL); KpiL=NULL;
  
}
