/*//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////*/

#include <stdlib.h>
#include <math.h>
#include "linpack_d.h"
#ifdef WIN32
#include "libpgauss_Export.h"
#else
#define libpgauss_EXPORT
#endif

/*
	VERIFIE 25/06/2012
*/
/* Calcul de des predictions - avec tendance deterministe       */
/* Methode gpPredhBayes identique a gpPredh                     */
/* Input Hp : matrice new x p des regresseurs			*/
/* Input Kp : matrice new x n de covariances	   		*/
/* Input variance : variance du processus			*/
/* Input hbeta : coefficients de la tendance			*/
/* Input iL : matrice n x n calcule par gpModhBayes    	    	*/
/* Input gamma : vecteur de taille n calcule par gpModhBayes    */
/* Input iR : matrice p x p calculee par gpModhBayes    	*/
/* Input M  : matrice n x p calculee par gpModhBayes         	*/
/* Input new : nombre de predictions                  		*/
/* Input n : nombre de points utilises pour la selection	*/
/* Input p : nombre de regresseurs				*/
/* Output yp : vecteur des new predictions			*/
/* Output vp : vecteur des new variances			*/
libpgauss_EXPORT void gpPredhBayes(double *Hp, double *Kp, double variance, double *hbeta, double *gamma, double *iL, double *iR, double *M, int new, int n, int p, double *yp, double *vp) {
  void gpPredh(double *Hp, double *Kp, double variance, double *hbeta, double *gamma, double *iL, double *iR, double *M, int new, int n, int p, double *yp, double *vp);
  gpPredh(Hp, Kp, variance, hbeta, gamma, iL, iR, M, new, n, p, yp, vp);
}
