//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TBootstrap.h
 \author Raphael Da Costa
 \date   Wed Apr 20 11:04:58 2005
 
 \brief  Header of the class TBootstrap
 
 */
#ifndef _TBootstrap
#define _TBootstrap

#include "Rtypes.h"

#include "TMultiLayerPerceptron.h"
#include "TNtuple.h"
#include "TObject.h"
#include "TNamed.h"
#include "TPatternsEventList.h"

#include <ctime>

class TBootstrap: public TNamed
{

private:
    Double_t _dLowerQuantile;   /// The lower quantile [0.10]
    Double_t _dUpperQuantile;   /// The upper quantile [0.90]

public:

    TMultiLayerPerceptron* mlp; //!< Perceptron associated
    Int_t nb_bases; //!< Number of bootstraped basis to generate
    Bool_t bLog; //!< Log Printing
    URANIE::DataServer::TPatternsEventList *_belApp; //// The bootstrap list of input pattern
    TEventList *_telApp;     //// The original list of input pattern

public:
    TBootstrap(TMultiLayerPerceptron* mlp_cons, Int_t nbbases_cons,
            Bool_t bLog_cons = kFALSE);
    virtual ~TBootstrap();
    void Train(Int_t nb_epochs, Option_t * option, TString pmml_name);
    Bool_t BuildBootTraining(Int_t iter);
    void setQuantiles(Double_t dlow, Double_t dupp);
    Double_t RMSmu(Long64_t n, const Double_t *a, Double_t mu);ClassDef(TBootstrap, ID_MODELER)
};

#endif
