//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <vector>
#include "polynome.hpp"
#include "SPC.hpp"
#include "util.hpp"

using namespace std;

namespace Anisp
{

//-------------------------------------------------------------------------
 
SPC:: SPC ()  // constructeur par defaut
   {
      nx = 0;
      no = 0;
      npol = 0;
      lois = NULL;
      
      double* le;
      double* he;
      double* lo;
      double* la;
     
      
    le = new double[1];
    legendre.push_back(le);
    legendre[0][0] = 1;
    he = new double[1];
    hermite.push_back(he);
    hermite[0][0] = 1;
    lo = new double[1];
    log_uniforme.push_back(lo);
    log_uniforme[0][0] = 1;
    la = new double[1];
    laguerre.push_back(la);
    laguerre[0][0] = 1;
         
   } 
 
//-------------------------------------------------------------------------

SPC::SPC(int ndim, char kind) // constructeur qui met toutes les
                                      // variable � la m�me loi
    {
       nx = ndim;
       no = 0; 
       npol = 1; // calcul de npol � l'aide de nx et no
       
 //---allocation de la m�moire---------
      
       lois = new char[nx];
       beta.reserve(10000);
       legendre.reserve(30);
       hermite.reserve(30); 
       log_uniforme.reserve(30);
       laguerre.reserve(30);
       

        double* le;
	double* he;
	double* lo;
	double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }

    /*   for(int i=0;i<=no;i++){
            legendre[i] = new double[i+1];
            hermite[i] = new double[i+1];
	    log_uniforme[i] = new double[i+1];
	    laguerre[i] = new double[i+1];
          }*/

	npt.reserve(1000); 
	int*v = new int[nx];
        npt.push_back(v);
	 
	np.reserve(1000);
	
	num.reserve(1000);  
//---initialisation des donn�es--------	
 
      //---initialisation de lois --------
 
       for(int i=0;i<nx;i++)
	   lois[i]= kind;

      //---initialisation de beta -------- 
      
	for(int i=0;i<npol;i++)
	     beta.push_back(0); //beta[i] = 0;
 	
      //---initialisation de legendre --------

        polynome * phi;
	phi = build_legendre(no);
	double * norme;// = new double[no+1];
	norme = norme_legendre(no);

	for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  legendre[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de hermite --------

         phi = build_hermite(no);
	 norme = norme_hermite(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  hermite[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
	delete[] phi;
    delete[] norme;
    norme = NULL;

      //---initialisation de log_uniforme --------

          phi = build_log_uniforme(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	   log_uniforme[i][j]=phi[i].coef[j];
	   }
    delete[] phi;
      //---initialisation de laguerre --------

          phi = build_laguerre(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	    laguerre[i][j]=phi[i].coef[j];
	   }


     delete[] phi;
	
      //---initialisation de npt, np et num --------
         
	for(int i=0;i<nx;i++)
	   npt[0][i] = 0;
	np.push_back(0);      
	num.push_back(0); 
   
    }

//--------------------------------------------------------
    
 SPC::SPC(int ndim, char* law) // constructeur qui met toutes les
                                      // variable � la m�me loi
    {
       nx = ndim;
       no = 0; 
       npol = 1; // calcul de npol � l'aide de nx et no
       
 //---allocation de la m�moire---------
      
       lois = new char[nx];
       beta.reserve(10000);
       legendre.reserve(30);
       hermite.reserve(30); 
       log_uniforme.reserve(30);
       laguerre.reserve(30);

        double* le;
	double* he;
	double* lo;
	double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }

     /*  for(int i=0;i<=no;i++){
            legendre[i] = new double[i+1];
            hermite[i] = new double[i+1];
	    log_uniforme[i] = new double[i+1];
	    laguerre[i] = new double[i+1];
          }*/

	npt.reserve(1000); 
	int*v = new int[nx];
        npt.push_back(v);
	 
	np.reserve(1000);
	num.reserve(1000);
	  
//---initialisation des donn�es--------	
 
      //---initialisation de lois --------
 
       for(int i=0;i<nx;i++)
	   lois[i]=law[i];

      //---initialisation de beta -------- 

	for(int i=0;i<npol;i++)
	    beta.push_back(0); //[i] = 0;
 	
      //---initialisation de legendre --------

        polynome * phi;
	phi = build_legendre(no);
	double * norme;// = new double[no+1];
	norme = norme_legendre(no);

	for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  legendre[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de hermite --------

         phi = build_hermite(no);
	 norme = norme_hermite(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  hermite[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de log_uniforme --------

          phi = build_log_uniforme(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	   log_uniforme[i][j]=phi[i].coef[j];
	   }
    delete[] phi;
      //---initialisation de laguerre --------

          phi = build_laguerre(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	    laguerre[i][j]=phi[i].coef[j];
	   }


        delete[] phi;
	

      //---initialisation de npt et np --------
         
	for(int i=0;i<nx;i++)
	   npt[0][i] = 0;
	np.push_back(0);      
	num.push_back(0);   
        
    }

   
    
    
//-------------------------------------------------------------------------

SPC::SPC(int ndim, int nord, char kind) // constructeur qui met toutes les variable � la m�me loi
    {
       nx = ndim;
       no = nord; 
       npol = 1; // calcul de npol � l'aide de nx et no
       
 //---allocation de la m�moire---------
      
       lois = new char[nx];
       beta.reserve(10000);
       legendre.reserve(30);
       hermite.reserve(30); 
       log_uniforme.reserve(30);
       laguerre.reserve(30);

        double* le;
	double* he;
	double* lo;
	double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }

      /* for(int i=0;i<=no;i++){
            legendre[i] = new double[i+1];
            hermite[i] = new double[i+1];
	    log_uniforme[i] = new double[i+1];
	    laguerre[i] = new double[i+1];
          }*/

	npt.reserve(1000); 
	int*v = new int[nx];
        npt.push_back(v);
	 
	np.reserve(1000);
	num.reserve(1000);
	  
//---initialisation des donn�es--------	
 
      //---initialisation de lois --------
 
      for(int i=0;i<nx;i++)
	   lois[i]= kind;
   

      //---initialisation de beta -------- 
      
	for(int i=0;i<npol;i++)
        beta.push_back(0);
	//beta[i] = 0;
 	
      //---initialisation de legendre --------

        polynome * phi;
	phi = build_legendre(no);
	double * norme;// = new double[no+1];
	norme = norme_legendre(no);

	for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  legendre[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de hermite --------

         phi = build_hermite(no);
	 norme = norme_hermite(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  hermite[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de log_uniforme --------

          phi = build_log_uniforme(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	   log_uniforme[i][j]=phi[i].coef[j];
	   }
    delete[] phi;
      //---initialisation de laguerre --------

          phi = build_laguerre(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	    laguerre[i][j]=phi[i].coef[j];
	   }


        delete[] phi;

      //---initialisation de npt et np --------
         
	for(int i=0;i<nx;i++)
	   npt[0][i] = 0;
	np.push_back(0);      
	num.push_back(0);   
        
    }


//-------------------------------------------------------------------------
SPC::SPC(int ndim, int nord, char* law) // constructeur qui met toutes les variable � la m�me loi
    {
       nx = ndim;
       no = nord; 
       npol = 1; // calcul de npol � l'aide de nx et no
       
 //---allocation de la m�moire---------
      
       lois = new char[nx];
       beta.reserve(10000);
       legendre.reserve(30);
       hermite.reserve(30); 
       log_uniforme.reserve(30);
       laguerre.reserve(30);

        double* le;
	double* he;
	double* lo;
	double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }

     /*  for(int i=0;i<=no;i++){
            legendre[i] = new double[i+1];
            hermite[i] = new double[i+1];
	    log_uniforme[i] = new double[i+1];
	    laguerre[i] = new double[i+1];
          }*/

	npt.reserve(1000); 
	int*v = new int[nx];
        npt.push_back(v);
	 
	np.reserve(1000);
	num.reserve(1000);
	  
//---initialisation des donn�es--------	
 
      //---initialisation de lois --------
 
      for(int i=0;i<nx;i++)
	   lois[i]= law[i];
   

      //---initialisation de beta -------- 
      
	for(int i=0;i<npol;i++)
	    beta.push_back(0); //[i] = 0;
 	
      //---initialisation de legendre --------

        polynome * phi;
	phi = build_legendre(no);
	double * norme;// = new double[no+1];
	norme = norme_legendre(no);

	for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  legendre[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de hermite --------

         phi = build_hermite(no);
	 norme = norme_hermite(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  hermite[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de log_uniforme --------

          phi = build_log_uniforme(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	   log_uniforme[i][j]=phi[i].coef[j];
	   }
    delete[] phi;
      //---initialisation de laguerre --------

          phi = build_laguerre(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	    laguerre[i][j]=phi[i].coef[j];
	   }


        delete[] phi;


      //---initialisation de npt et np --------
         
	for(int i=0;i<nx;i++)
	   npt[0][i] = 0;
	np.push_back(0);      
	num.push_back(0);  
       
      
    }


//-------------------------------------------------------------------------
 SPC::SPC(int ndim, int nord, int npoly, char kind, vector<double> bet, vector<int*> npt1)// constructeur qui met toutes les variables � la m�me loi et initialise beta avec les valeurs de bet
    {
       nx = ndim;
       no = nord; 
       npol = npoly;
       
 //---allocation de la m�moire---------
     
       lois = new char[nx];
       beta.reserve(10000);
       legendre.reserve(30);
       hermite.reserve(30); 
       log_uniforme.reserve(30);
       laguerre.reserve(30);

        double* le;
	double* he;
	double* lo;
	double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }

     /*  for(int i=0;i<=no;i++){
            legendre[i] = new double[i+1];
            hermite[i] = new double[i+1];
	    log_uniforme[i] = new double[i+1];
	    laguerre[i] = new double[i+1];
          }*/

	npt.reserve(10000); 
	 
	np.reserve(10000);
	num.reserve(10000); 


//---initialisation des donn�es--------	

      //---initialisation de lois --------
 
       for(int i=0;i<nx;i++)
	       lois[i]= kind;
   
      //---initialisation de beta -------- 
      
	beta.assign(bet.begin(),bet.end());
 	
     //---initialisation de legendre --------

        polynome * phi;
	phi = build_legendre(no);
	double * norme;// = new double[no+1];
	norme = norme_legendre(no);

	for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  legendre[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de hermite --------

         phi = build_hermite(no);
	 norme = norme_hermite(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  hermite[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
     //---initialisation de log_uniforme --------

          phi = build_log_uniforme(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	   log_uniforme[i][j]=phi[i].coef[j];
	   }
    delete[] phi;
      //---initialisation de laguerre --------

         phi = build_laguerre(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  laguerre[i][j]=phi[i].coef[j];
	  }

        delete[] phi;

	
      //---initialisation de npt et np --------
      int* vec = new int[nx];
      int it;
       for(int i=0;i<npol;i++)
          {
	   it = 0;
	   for(int j=0;j<nx;j++)
	     {
	      vec[j] = npt1[i][j]; 
	      it = it + vec[j];
	     } 
	   npt.push_back(vec);
	   np.push_back(it);
	   vec = new int[nx];
           num.push_back(calcul_num(npt[i],nx));
	  }   
      
    }

//-------------------------------------------------------------------------

SPC::SPC(int ndim, int nord, int npoly, char* law, vector<double> bet, vector<int*> npt1)// constructeur qui met toutes les variables � des lois diff�rentes et initialise beta avec les valeurs de bet
    {
       nx = ndim;
       no = nord; 
       npol = npoly;
       
 //---allocation de la m�moire---------
     
       lois = new char[nx];
       beta.reserve(10000);
       legendre.reserve(30);
       hermite.reserve(30); 
       log_uniforme.reserve(30);
       laguerre.reserve(30);

        double* le;
	double* he;
	double* lo;
	double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }


    /*   for(int i=0;i<=no;i++){
            legendre[i] = new double[i+1];
            hermite[i] = new double[i+1];
	    log_uniforme[i] = new double[i+1];
	    laguerre[i] = new double[i+1];
          }*/

	npt.reserve(10000); 
	 
	np.reserve(10000);
	num.reserve(10000);  


//---initialisation des donn�es--------	

      //---initialisation de lois --------
 
       for(int i=0;i<nx;i++)
	       lois[i]= law[i];
   
      //---initialisation de beta -------- 
      
	beta.assign(bet.begin(),bet.end());
 	
     //---initialisation de legendre --------

        polynome * phi;
	phi = build_legendre(no);
	double * norme;// = new double[no+1];
	norme = norme_legendre(no);

	for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  legendre[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de hermite --------

         phi = build_hermite(no);
	 norme = norme_hermite(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  hermite[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
     //---initialisation de log_uniforme --------

          phi = build_log_uniforme(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	   log_uniforme[i][j]=phi[i].coef[j];
	   }
    delete[] phi;
      //---initialisation de laguerre --------

         phi = build_laguerre(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  laguerre[i][j]=phi[i].coef[j];
	  }

        delete[] phi;

	
      //---initialisation de npt et np --------
      int* vec = new int[nx];
      int it;
       for(int i=0;i<npol;i++)
          {
	   it = 0;
	   for(int j=0;j<nx;j++)
	     {
	      vec[j] = npt1[i][j]; 
	      it = it + vec[j];
	     } 
	   npt.push_back(vec);
	   np.push_back(it);
	   vec = new int[nx];
           num.push_back(calcul_num(npt[i],nx));
	  }   
      
    }


//-------------------------------------------------------------------------
 
 SPC::SPC(int ndim, int nord, int npoly, char kind, vector<double> bet, vector<int*> npt1, vector<int> num1)// constructeur qui met toutes les variables � la m�me loi et initialise beta avec les valeurs de bet
    {
       nx = ndim;
       no = nord; 
       npol = npoly;
       
 //---allocation de la m�moire---------
     
       lois = new char[nx];
       beta.reserve(10000);
       legendre.reserve(30);
       hermite.reserve(30); 
       log_uniforme.reserve(30);
       laguerre.reserve(30);

        double* le;
	double* he;
	double* lo;
	double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }

     /*  for(int i=0;i<=no;i++){
            legendre[i] = new double[i+1];
            hermite[i] = new double[i+1];
	    log_uniforme[i] = new double[i+1];
	    laguerre[i] = new double[i+1];
          }*/

	npt.reserve(10000); 
	 
	np.reserve(10000);
	num.reserve(10000); 


//---initialisation des donn�es--------	

      //---initialisation de lois --------
 
       for(int i=0;i<nx;i++)
	       lois[i]= kind;
   
      //---initialisation de beta -------- 
      
	beta.assign(bet.begin(),bet.end());
 	
     //---initialisation de legendre --------

        polynome * phi;
	phi = build_legendre(no);
	double * norme;// = new double[no+1];
	norme = norme_legendre(no);

	for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  legendre[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de hermite --------

         phi = build_hermite(no);
	 norme = norme_hermite(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  hermite[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
     //---initialisation de log_uniforme --------

          phi = build_log_uniforme(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	   log_uniforme[i][j]=phi[i].coef[j];
	   }
    delete[] phi;
      //---initialisation de laguerre --------

         phi = build_laguerre(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  laguerre[i][j]=phi[i].coef[j];
	  }

        delete[] phi;

	
      //---initialisation de npt et np --------
      int* vec = new int[nx];
      int it;
       for(int i=0;i<npol;i++)
          {
	   it = 0;
	   for(int j=0;j<nx;j++)
	     {
	      vec[j] = npt1[i][j]; 
	      it = it + vec[j];
	     } 
	   npt.push_back(vec);
	   np.push_back(it);
	   vec = new int[nx];
           num.push_back(num1[i]);
	  }   
      
    }

//-------------------------------------------------------------------------





 //-------------------------------------------------------------------------
 
 
 SPC::SPC(int ndim, int nord, int npoly, char* law, vector<double> bet, vector<int*> npt1, vector<int> num1 )// constructeur qui met toutes les variables � des lois diff�rentes et initialise beta avec les valeurs de bet
    {
       nx = ndim;
       no = nord; 
       npol = npoly;
       
 //---allocation de la m�moire---------
     
       lois = new char[nx];
       beta.reserve(10000);
       legendre.reserve(30);
       hermite.reserve(30); 
       log_uniforme.reserve(30);
       laguerre.reserve(30);

        double* le;
	double* he;
	double* lo;
	double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }


      /* for(int i=0;i<=no;i++){
            legendre[i] = new double[i+1];
            hermite[i] = new double[i+1];
	    log_uniforme[i] = new double[i+1];
	    laguerre[i] = new double[i+1];
          }*/

	npt.reserve(10000); 
	 
	np.reserve(10000);
	num.reserve(10000);  


//---initialisation des donn�es--------	

      //---initialisation de lois --------
 
       for(int i=0;i<nx;i++)
	   lois[i]= law[i];
   
      //---initialisation de beta -------- 
      
	beta.assign(bet.begin(),bet.end());
 	
     //---initialisation de legendre --------

        polynome * phi;
	phi = build_legendre(no);
	double * norme;// = new double[no+1];
	norme = norme_legendre(no);

	for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  legendre[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
      //---initialisation de hermite --------

         phi = build_hermite(no);
	 norme = norme_hermite(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  hermite[i][j]=phi[i].coef[j]/sqrt(norme[i]);
	  }
    delete[] phi;
    delete[] norme;
    norme = NULL;
     //---initialisation de log_uniforme --------

          phi = build_log_uniforme(no);

	  for(int i=0;i<=no;i++)
	   {
	   for(int j=0;j<=i;j++)
	   log_uniforme[i][j]=phi[i].coef[j];
	   }
    delete[] phi;
      //---initialisation de laguerre --------

         phi = build_laguerre(no);

	 for(int i=0;i<=no;i++)
	  {
	  for(int j=0;j<=i;j++)
	  laguerre[i][j]=phi[i].coef[j];
	  }

        delete[] phi;

	
      //---initialisation de npt et np --------
      int* vec = new int[nx];
      int it;
       for(int i=0;i<npol;i++)
          {
	   it = 0;
	   for(int j=0;j<nx;j++)
	     {
	      vec[j] = npt1[i][j]; 
	      it = it + vec[j];
	     } 
	   npt.push_back(vec);
	   np.push_back(it);
	   vec = new int[nx];
           num.push_back(num1[i]);
	  }   
      
    }
 
//-------------------------------------------------------------------------

SPC::SPC(const SPC & psi) // constructeur par copie
{
 //------ allocation de la m�moire et mise aux valeurs --------         
	 nx = psi.nx;
	 no = psi.no;
	 npol = psi.npol;
	 
	 lois = new char[nx];
	
	 
         beta.reserve(10000);
         legendre.reserve(30);
         hermite.reserve(30); 
         log_uniforme.reserve(30);
         laguerre.reserve(30);
         
	 double* le;
	 double* he;
	 double* lo;
	 double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }
	  
        npt.reserve(10000); 
	 
	np.reserve(10000);
	num.reserve(10000);

	int* vec = new int[nx];
        int it;
        for(int i=0;i<npol;i++)
          {
	   beta.push_back(psi.beta[i]);
	   it = 0;
	   for(int j=0;j<nx;j++)
	     {
	      vec[j] = psi.npt[i][j]; 
	      it = it + vec[j];
	     } 
	   npt.push_back(vec);
	   np.push_back(it);
	   vec = new int[nx];
           num.push_back(psi.num[i]);
	  }   
	
	 
	 for(int i=0;i<nx;i++)
	    lois[i] = psi.lois [i];
	 
	 
	 for(int i=0;i<=no;i++)
	    {
	    for(int j=0;j<=i;j++)
	       {
	       legendre[i][j] = psi.legendre[i][j];
	       hermite[i][j] = psi.hermite[i][j];
	       log_uniforme[i][j] = psi.log_uniforme[i][j];
	       laguerre[i][j] = psi.laguerre[i][j];
	       }
	    }

      

}


//-------------------------------------------------------------------------
 
 
 
//-------------------------------------------------------------------------
SPC:: ~SPC ()  //destructeur
{
delete[] lois;

if(nx>0){
    for(unsigned i=0;i<legendre.size();i++){
        delete[] legendre[i];
        legendre[i] = NULL;
        }
    
    for(unsigned i=0;i<hermite.size();i++){
        delete[] hermite[i];
        hermite[i] = NULL;
        }

    for(unsigned i=0;i<log_uniforme.size();i++){
        delete[] log_uniforme[i];
        log_uniforme[i] = NULL;
        }

    for(unsigned i=0;i<laguerre.size();i++){
        delete[] laguerre[i];
        laguerre[i] = NULL;
        }

    for(unsigned i=0;i<npt.size();i++)
    {
        delete[] npt[i];
        npt[i] = NULL;
    }
}

}

//-------------------------------------------------------------------------
 
SPC & 
SPC::operator = (const SPC & psi)  // operateur =
{
    if(this == & psi) 
        return *this;
      
      //------ effacement de la m�moire existante --------
         
         delete lois;
	 
	 if(legendre.size()!=0){
            for(int i=0;i<int(legendre.size());i++){
               delete legendre[i];
              }
           }
	 
	  if(hermite.size()!=0){   
             for(int i=0;i<int(hermite.size());i++){
                 delete hermite[i];
               }
           }
	   
	 if(log_uniforme.size()!=0){
              for(int i=0;i<int(log_uniforme.size());i++){
                 delete log_uniforme[i];
                }
              }
	      
         if(laguerre.size()!=0){
           for(int i=0;i<int(laguerre.size());i++){
                delete laguerre[i];
               }
            }
	 
	 if(npt.size()!=0){ 
            for(int i=0;i<int(npt.size());i++)
               delete npt[i];
	    }
      
      //------ allocation de la m�moire et mise aux valeurs -------- 
	 nx = psi.nx;
	 no = psi.no;
	 npol = psi.npol;
	 
	 lois = new char[nx];
	
	 
         beta.reserve(10000);
         legendre.reserve(30);
         hermite.reserve(30); 
         log_uniforme.reserve(30);
         laguerre.reserve(30);
         
	 double* le;
	 double* he;
	 double* lo;
	 double* la;
	 
         for(int i=0;i<=no;i++){
	    le = new double[i+1];
	    legendre.push_back(le);
            he = new double[i+1];
	    hermite.push_back(he); 
            lo = new double[i+1];
	    log_uniforme.push_back(lo); 
	    la = new double[i+1];
	    laguerre.push_back(la);
          }
	  
        npt.reserve(10000); 
	 
	np.reserve(10000);
	num.reserve(10000);

	int* vec = new int[nx];
        int it;
        for(int i=0;i<npol;i++)
          {
	   beta.push_back(psi.beta[i]);
	   it = 0;
	   for(int j=0;j<nx;j++)
	     {
	      vec[j] = psi.npt[i][j]; 
	      it = it + vec[j];
	     } 
	   npt.push_back(vec);
	   np.push_back(it);
	   vec = new int[nx];
           num.push_back(psi.num[i]);
	  }   
	
	 
	 for(int i=0;i<nx;i++)
	    lois[i] = psi.lois [i];
	 
	 
	 for(int i=0;i<=no;i++)
	    {
	    for(int j=0;j<=i;j++)
	       {
	       legendre[i][j] = psi.legendre[i][j];
	       hermite[i][j] = psi.hermite[i][j];
	       log_uniforme[i][j] = psi.log_uniforme[i][j];
	       laguerre[i][j] = psi.laguerre[i][j];
	       }
	    }
     
   return *this;
}


 //-------------------------------------------------------------------------
/*
PC &
PC::  operator += (const PC & p)
{
PC psi;
int ordre;
ordre = max(p.no,no);
psi = PC(nx,ordre,lois);

for(int i=0;i<=psi.npol;i++)
   {
    if(p.no>no)
      {
       if(i<=npol)
         psi[i] = p.beta[i] + beta[i];
       else
         psi[i] = p.beta[i];	 
      }
    else
      {
       if(i<=p.npol)
         psi[i] = p.beta[i] + beta[i];
       else
         psi[i] = beta[i];
      }    
   }

delete beta;
delete lois;
delete np;
npol = psi.npol;
no = psi.no;
nx = psi.nx;

         
 delete legendre;        
 delete hermite; 
 delete log_uniforme;
 delete laguerre;
 delete npt;


beta = new double[npol+1];
lois = new char[nx];
np = new int[npol+1];

 legendre = new double*[no+1];
 hermite = new double*[no+1];
 log_uniforme = new double*[no+1];
 laguerre = new double*[no+1];

 for(int i=0;i<=no;i++){
    legendre[i] = new double[i+1];
    hermite[i] = new double[i+1];
    log_uniforme[i] = new double[i+1];
    laguerre[i] = new double[i+1];
    }
    
 npt = new int*[npol+1];

 for(int i=0;i<npol+1;i++)
       npt[i] = new int[nx];

memcpy(beta,psi.beta,(npol+1)*sizeof(double)); 
memcpy(lois,psi.lois,(nx)*sizeof(char));
memcpy(np,psi.np,(npol+1)*sizeof(int));

for(int i=0;i<npol+1;i++) 
    {
     memcpy(npt[i],psi.npt[i],(nx)*sizeof(int));
    }
 
 for(int i=0;i<=no;i++)
    {
     memcpy(legendre[i],psi.legendre[i],(i+1)*sizeof(double));
     memcpy(hermite[i],psi.hermite[i],(i+1)*sizeof(double));
     memcpy(log_uniforme[i],psi.log_uniforme[i], (i+1)*sizeof(double));
     memcpy(laguerre[i],psi.laguerre[i],(i+1)*sizeof(double));
    }
  
   
return (*this);
} 
*/



//-------------------------------------------------------------------------

SPC
SPC::  operator + (const SPC & p)
{
SPC psi;
psi = p;
for(int i=0;i<npol;i++)
   {
    psi.ajoute(beta[i],num[i]);
   }


return psi;
} 


//-------------------------------------------------------------------------

SPC
SPC::  operator - (const SPC & p)
{
SPC psi;
psi = p;
for(int i=0;i<npol;i++)
   {
    psi.ajoute(-beta[i],num[i]);
   }


return psi;
} 


//-------------------------------------------------------------------------
/*
PC
PC::  operator * (const PC & p)
{
PC psi;
PC psi1(p);
int ordre,N;
ordre = p.no+no;
N = calcul_npol(ordre,p.nx)+1;
double** xx = new double*[N];
double* y = new double[N];

time_t *adr= new time_t;
srand(int(time(adr)));
delete adr;

for(int i=0;i<N;i++)
    xx[i] =  new double[p.nx];
  
for(int i=0;i<N;i++)
   {
    for(int j=0;j<p.nx;j++)
       xx[i][j] = double(rand())/RAND_MAX;  
   }

for(int i=0;i<N;i++)
   y[i] = this->eval_PC(xx[i])*psi1(xx[i]);

psi = fct_regression(ordre, p.nx,p.lois,N, xx, y);              
   

return psi;
}*/

//-------------------------------------------------------------------------
/*
SPC operator * (double a,const SPC & p)
{
SPC psi;
psi = p;
//SPC psi(p.nx,p.no,p.lois);
//psi = SPC(p.nx;p.no;p.lois);
for(int i=0;i<psi.npol;i++)
   {
    psi[i] = a*psi.beta[i] ;
   }

return psi;
}*/


//-------------------------------------------------------------------------

double 
SPC::operator () (double * x) const // operateur qui donne la valeur du PC en x
{
// double * leg;
// double * her;
double * val;
double ** tab_phi;
double y;

val = new double[npol+1];
// leg = new double[no+1];
// her = new double[no+1];
tab_phi = new double*[nx];
// for(int i=0;i<nx;i++)
// tab_phi[i] = new double[no+1];

for(int j=0;j<nx;j++)
{
if(lois[j]=='l')
         tab_phi[j] = eval_legendre(x[j]);
else{ if(lois[j]=='h')
         tab_phi[j] = eval_hermite(x[j]);
     else{ if(lois[j]=='u')
         tab_phi[j] = eval_log_uniforme(x[j]);
	  else if(lois[j]=='e')
             tab_phi[j] = eval_laguerre(x[j]);
	 }
     }
}  

  

y = 0;
for(int i=0;i<npol;i++)
  {
  val[i] = beta[i];  
  for(int j=0;j<nx;j++) 
     {            
         val[i] = val[i] * tab_phi[j][npt[i][j]];	     
     } 
   y = y + val[i];  
  }
  
// Clean
//     delete[] leg;
//     delete[] her;
    delete[] val;
    for (int idel = 0; idel < nx; idel++)
    {
        delete[] tab_phi[idel];
        tab_phi[idel] = NULL;
    }
    delete[] tab_phi;
    tab_phi = NULL;
    
//   delete val;
//   delete her;
//   delete leg;
//   for(int i=0;i<nx;i++)
//     delete  tab_phi[i];
//   delete tab_phi;
    
  return y;
}
//-------------------------------------------------------------------------
 
double &
SPC::operator [] (int i)  //operateur qui retourne la valeur du i�me coef beta
{
return beta[i];
} 
 
//-------------------------------------------------------------------------




//-------------------------------------------------------------------------

void
SPC::ajoute(double bet,int* npt1)
{
int* v = new int[nx];
int num1,m,ind,no1;
int n = 0;
num1 = calcul_num(npt1,nx);

m = 0;
for(int i=0;i<npol;i++)
 {
  if(num[i]==num1)
    {
     m = 1;
     ind = i;
     delete[] v;
     v = NULL;
     break;
    }
 }

if(m==1)
{
 beta[ind] = beta[ind] + bet;
}
else
{
for(int i=0;i<nx;i++)
 {
  n = n + npt1[i];
  v[i] = npt1[i];
 }


beta.push_back(bet);
np.push_back(n);
npt.push_back(v);
num.push_back(num1);
npol = npol+1;



no1 = no;
if(n>no) 
 { 
  no=n; 
  double* le;
  double* he;
  double* lo;
  double* la;	
  polynome * phi;
  double * norme;// = new double[no+1];

  for(int i=no1+1;i<=no;i++)
   {
    le = new double[i+1];
    legendre.push_back(le);
    he = new double[i+1];
    hermite.push_back(he);
    lo = new double[i+1];
    log_uniforme.push_back(lo); 
    la = new double[i+1];
    laguerre.push_back(la);
   }

  //for(int i=no1+1;i<=no;i++)
  // {
  //  le = new double[i+1];
  //  legendre.push_back(le);
    phi = build_legendre(no);
    norme = norme_legendre(no);
    for(int k=no1+1;k<=no;k++)
    {
     for(int j=0;j<=k;j++)
      legendre[k][j]=phi[k].coef[j]/sqrt(norme[k]);
    }
    delete[] phi;
    delete[] norme;
    norme = NULL;

   // he = new double[i+1];
   // hermite.push_back(he); 
    phi = build_hermite(no);
    norme = norme_hermite(no);
    for(int k=no1+1;k<=no;k++)
    {
     for(int j=0;j<=k;j++)
      hermite[k][j]=phi[k].coef[j]/sqrt(norme[k]);
    }
    delete[] phi;
    delete[] norme;
    norme = NULL;

   // lo = new double[i+1];
   // log_uniforme.push_back(lo); 
   
   
    phi = build_log_uniforme(no);
    norme = norme_log_uniforme(no);
    for(int k=no1+1;k<=no;k++)
    {
     for(int j=0;j<=k;j++)
      log_uniforme[k][j]=phi[k].coef[j]/sqrt(norme[k]);
    }
    delete[] phi;
    delete[] norme;
    norme = NULL;

   // la = new double[i+1];
   // laguerre.push_back(la);
    phi = build_laguerre(no);
    norme = norme_laguerre(no);
    for(int k=no1+1;k<=no;k++)
    {
     for(int j=0;j<=k;j++)
      laguerre[k][j]=phi[k].coef[j]/sqrt(norme[k]);
    }
    delete[] phi;
    delete[] norme;
    norme = NULL;
    
 // }
 }

}

}

//-------------------------------------------------------------------------

void
SPC::ajoute(double bet,int n1)
{
int* v = new int[nx];
int m,ind,no1;
int n = 0;
calcul_npt_num(v,n1,nx);

m = 0;
for(int i=0;i<npol;i++)
 {
  if(num[i]==n1)
    {
     m = 1;
     ind = i;
     break;
    }
 }

if(m==1)
{
 beta[ind] = beta[ind] + bet;
}
else
{
for(int i=0;i<nx;i++)
  n = n + v[i];

//if(n>no) no=n;
beta.push_back(bet);
np.push_back(n);
npt.push_back(v);
num.push_back(n1);
npol = npol+1;


no1 = no;
if(n>no) 
 { 
  no=n; 
  double* le;
  double* he;
  double* lo;
  double* la;	
  polynome * phi;
  double * norme;// = new double[no+1];

  for(int i=no1+1;i<=no;i++)
   {
    le = new double[i+1];
    legendre.push_back(le);
    he = new double[i+1];
    hermite.push_back(he);
    lo = new double[i+1];
    log_uniforme.push_back(lo); 
    la = new double[i+1];
    laguerre.push_back(la);
   }
  
  //for(int i=no1+1;i<=no;i++)
  // {
  //  le = new double[i+1];
  //  legendre.push_back(le);
    phi = build_legendre(no);
    norme = norme_legendre(no);
    for(int k=no1+1;k<=no;k++)
    {
     for(int j=0;j<=k;j++)
      legendre[k][j]=phi[k].coef[j]/sqrt(norme[k]);
    }
    
   // he = new double[i+1];
   // hermite.push_back(he); 
    phi = build_hermite(no);
    norme = norme_hermite(no);
    for(int k=no1+1;k<=no;k++)
    {
     for(int j=0;j<=k;j++)
      hermite[k][j]=phi[k].coef[j]/sqrt(norme[k]);
    }
    
   // lo = new double[i+1];
   // log_uniforme.push_back(lo); 
    phi = build_log_uniforme(no);
    norme = norme_log_uniforme(no);
    for(int k=no1+1;k<=no;k++)
    {
     for(int j=0;j<=k;j++)
      log_uniforme[k][j]=phi[k].coef[j]/sqrt(norme[k]);
    }
    
   // la = new double[i+1];
   // laguerre.push_back(la);
    phi = build_laguerre(no);
    norme = norme_laguerre(no);
    for(int k=no1+1;k<=no;k++)
    {
     for(int j=0;j<=k;j++)
      laguerre[k][j]=phi[k].coef[j]/sqrt(norme[k]);
    }
    
    //Clean
    delete[] norme;
    norme = NULL;
    
 // }
 }

}

}


//-------------------------------------------------------------------------

void
SPC::filtre(double eps)
{
 int n = 0;
 int m = npol;
 
 for(int k=0;k<int(beta.size());k++) 
  {
   if(fabs(beta[k]) < eps)
    {
     beta.erase(beta.begin()+k);
     num.erase(num.begin()+k);
     delete[] npt[k];
     npt.erase(npt.begin()+k);
     np.erase(np.begin()+k);
     npol = npol-1;
     k = k-1;
    }
   else n = max(n,np[k]);
  }
  
 if(n<no)
   {
    if(nx>0){
    for(int i=n+1;i<=no;i++){
      delete[] legendre[i];
      legendre[i] = NULL;
    }
  
    for(int i=n+1;i<=no;i++){
      delete[] hermite[i];
      hermite[i] = NULL;
    }

    for(int i=n+1;i<=no;i++){
      delete[] log_uniforme[i];
      log_uniforme[i] = NULL;
    }

    for(int i=n+1;i<=no;i++){
      delete[] laguerre[i];
      laguerre[i] = NULL;
    }
   }
   no = n;
  }

}


//-------------------------------------------------------------------------

void 
SPC::ordonne()
{
 int *T1 = new int[npol];
 int *T2 = new int[npol];
 
 int sortLength = 1, de1, de2, de3, i;
 
for( i=0;i<npol;i++)
   {
   T1[i] = i;
   }

 while(sortLength < npol)
    {
      de1 = 0;
      while(de1 < npol)
        {
         de2 = de1 + sortLength;
         de3 = de2 + sortLength;
         if(de2>npol) 
             de2 = npol;
         if(de3>npol) 
             de3 = npol;
         
         fusion(num, T1, T2, de1, de2-1, de2, de3-1, de3-de1, de1);
         de1 = de3;
         
        }
 
      for(i = 0 ; i < npol ; i++) T1[i] = T2[i];
         sortLength *= 2;
    }
 
 

//  int * u = new int[nx];
 double * D = new double[npol];
 int ** N = new int*[npol]; 
 // int a;
 // double b;
 for(i=0;i<npol;i++)
  { 
    T2[i] = num[T1[i]];
  }
 for(i=0;i<npol;i++)
  { 
    num[i] = T2[i];
  }

  for(i=0;i<npol;i++)
  { 
    T2[i] = np[T1[i]];
    D[i] = beta[T1[i]];
    N[i] = npt[T1[i]];
  }
 for(i=0;i<npol;i++)
  { 
    np[i] = T2[i];
    beta[i] = D[i];
    npt[i] = N[i];
  }
  
// Clean
    delete[] T1;
    T1 = NULL;
    delete[] T2;
    T2 = NULL;
    delete[] D;
    D = NULL;
    delete[] N;
  

}

//-------------------------------------------------------------------------


//-------------------------------------------------------------------------

void 
SPC::affiche_beta()
{
for(int k=0;k<npol;k++)
cout<<"beta"<<num[k]<<"="<<beta[k]<<endl;
}

//-------------------------------------------------------------------------
 
void 
SPC::affiche_npol()
{
cout<<"nombre de polynomes: npol="<<npol<<endl;
}
 
//-------------------------------------------------------------------------

void 
SPC::affiche_nx()
{
cout<<"nombre de dimensions: nx="<<nx<<endl;
}


//-------------------------------------------------------------------------
 
void 
SPC::affiche_no()
{
cout<<"ordre du developpement: no="<<no<<endl;
}
 
//-------------------------------------------------------------------------


void 
SPC::affiche_lois()
{
for(int k=0;k<nx;k++){
 if(lois[k]=='l')
 cout<<"la variable x"<<k+1<<" suit une loi uniforme"<<endl;
 if(lois[k]=='h')
 cout<<"la variable x"<<k+1<<" suit une loi gaussienne"<<endl;
 }
}
 

//-------------------------------------------------------------------------
 
void 
SPC::affiche_SPC()
{
this->affiche_beta();

this->affiche_num();

this->affiche_npol(); 

this->affiche_nx();

this->affiche_no();

this->affiche_lois();

this->affiche_npt();
}
 
//-------------------------------------------------------------------------


void 
SPC::affiche_np()
{
for(int k=0;k<npol;k++)
cout<<"ordre global du "<<k+1<<"-eme PC: "<<np[k]<<endl;
}


//-------------------------------------------------------------------------


void 
SPC::affiche_num()
{
for(int k=0;k<npol;k++)
cout<<" numeros du "<<k+1<<"-eme PC: "<<num[k]<<endl;
}



//-------------------------------------------------------------------------
 
void 
SPC::affiche_npt()
{
cout<<"npt ="<<endl;
for(int i=0;i<npol;i++){
  cout<<"      ";
  for(int j=0;j<nx;j++)
  cout<<npt[i][j]<<"  ";
  cout<<endl;
  }
}
 
//-------------------------------------------------------------------------

void 
SPC::affiche_legendre()
{
cout<<"legendre ="<<endl;
for(int i=0;i<=no;i++){
  cout<<"      ";
  for(int j=0;j<=i;j++)
  cout<<legendre[i][j]<<"  ";
  cout<<endl;
  }
}


//-------------------------------------------------------------------------

void 
SPC::affiche_hermite()
{
cout<<"hermite ="<<endl;
for(int i=0;i<=no;i++){
  cout<<"      ";
  for(int j=0;j<=i;j++)
  cout<<hermite[i][j]<<"  ";
  cout<<endl;
  }
}


//-------------------------------------------------------------------------



void 
SPC::affiche_laguerre()
{
cout<<"laguerre ="<<endl;
for(int i=0;i<=no;i++){
  cout<<"      ";
  for(int j=0;j<=i;j++)
  cout<<laguerre[i][j]<<"  ";
  cout<<endl;
  }
}


//-------------------------------------------------------------------------
 
 void 
 SPC::modif_beta(double* tab)
 {
 for(int i=0;i<npol;i++)
 beta[i]=tab[i];
 } 
 
//-------------------------------------------------------------------------

 void 
 SPC::modif_beta(int k,double val)
 {
 beta[k]=val;
 } 
 
//-------------------------------------------------------------------------

void 
SPC::modif_lois(char* law)
{
for(int i=0;i<nx;i++)
   lois[i]=law[i];
}

//-------------------------------------------------------------------------

void
SPC::modif_lois(int i, char law)
{
   lois[i]=law;
}

//-------------------------------------------------------------------------
double*
SPC::valeur_beta()
{
double* a = new double[npol];
for(int i=0;i<npol;i++)
   a[i] = beta[i];
return a;
}

//-------------------------------------------------------------------------

int**
SPC::valeur_npt()
{
int** npt1 = new int*[npol];
for(int i=0;i<npol;i++) 
  {
   npt1[i] = new int[nx];
   for(int j=0;j<nx;j++)
      npt1[i][j] = npt[i][j];
  }
return npt1;
}

//--------------------------------------------------------------------------

int 
SPC::valeur_npol() // retourne la valeur de npol
{
return npol;
}

//-------------------------------------------------------------------------

double
SPC:: eval_SPC(double * x) // retourne la valeur du PC en x
{

double * val;
double ** tab_phi; 
double y;

val = new double[npol+1];
tab_phi = new double*[nx];
for(int i=0;i<nx;i++)
tab_phi[i] = new double[no+1];



for(int j=0;j<nx;j++)
{
if(lois[j]=='l')
      eval_legendre(tab_phi[j],x[j]);
         //tab_phi[j] = eval_legendre(x[j]);
else{ if(lois[j]=='h')
         tab_phi[j] = eval_hermite(x[j]);
      else{ if(lois[j]=='u')
             tab_phi[j] = eval_log_uniforme(x[j]);
            else if(lois[j]=='e')
               tab_phi[j] = eval_laguerre(x[j]);
       }
   }

}

       //--------tensorisation des valeurs des polyn�mes id--------  

y = 0;
for(int i=0;i<npol;i++)
  {
  val[i] = beta[i];
  for(int j=0;j<nx;j++) 
     {     
         val[i] = val[i] * tab_phi[j][npt[i][j]];
	
     }
   
   y = y + val[i];  
  }
  delete val;
  for(int i=0;i<nx;i++)
    delete  tab_phi[i];
  delete tab_phi;
  return y;
}
 
//-------------------------------------------------------------------------


double *
SPC:: eval_SPC( int a, double ** x) //retourne les valeurs du PC pour les points
                                  // de l'�chantillon x de taille a
{

double * y;
y = new double[a];
for(int i=0;i<a;i++)
  {
  y[i] = eval_SPC(x[i]);
  }

  return y;
}


//------------------------------------------------
double
SPC:: eval_psi(int a, double * x) // retourne la valeur de l'�l�ment a de la base en x
{
double y;
y = 1;
  /*for(int j=0;j<nx;j++) 
     {   

       if(lois[j]=='l')
         y = y *(eval_legendre(x[j]))[npt[a][j]];
      else{ if(lois[j]=='h')
         y = y *(eval_hermite(x[j]))[npt[a][j]];
      else{ if(lois[j]=='u')
              y = y *(eval_log_uniforme(x[j]))[npt[a][j]];
            else if(lois[j]=='e')
               y = y *(eval_laguerre(x[j]))[npt[a][j]];
       }
   }*/
   for(int j=0;j<nx;j++) 
     {   
//cout<<npt[a][j]<<endl;
       if(lois[j]=='l')
         y = y *eval_legendre(x[j],npt[a][j]);
      else{ if(lois[j]=='h')
         y = y *eval_hermite(x[j],npt[a][j]);
      else{ if(lois[j]=='u')
              y = y *eval_log_uniforme(x[j],npt[a][j]);
            else if(lois[j]=='e')
               y = y *eval_laguerre(x[j],npt[a][j]);
       }
   }

  }  


  return y;
}
 




//-------------------------------------------------------------------------


double *
SPC:: eval_legendre(double x) const  // retourne les valeurs des polyn�mes de
                                    //legendre en x
{
double * leg;
leg = new double[no+1];
for(int i=0;i<=no;i++)
  {
    if(i==0)
      leg[i] = legendre[0][0];
    else{
    leg[i] = legendre[i][i-1] + legendre[i][i]*x;
    for(int j=i-2;j>=0;j--)
       {
         leg[i] = legendre[i][j] + x*leg[i];
       }
     }

  }
return  leg;
}


//-------------------------------------------------------------------------


double 
SPC:: eval_legendre(double x, int m) const  // retourne les valeurs des polyn�mes de
                                    //legendre en x
{
double leg;


    if(m==0)
      leg = legendre[0][0];
    else{
    leg = legendre[m][m-1] + legendre[m][m]*x;
    for(int j=m-2;j>=0;j--)
       {
         leg = legendre[m][j] + x*leg;
       }
     }

return  leg;
}


//-------------------------------------------------------------------------

//-------------------------------------------------------------------------


void
SPC:: eval_legendre(double * leg, double x) const  // retourne la valeur du polyn�me de
                                    //legendre de degr� en x
{
for(int i=0;i<=no;i++)
  {
    if(i==0)
      leg[i] = legendre[0][0];
    else{
    leg[i] = legendre[i][i-1] + legendre[i][i]*x;
    for(int j=i-2;j>=0;j--)
       {
         leg[i] = legendre[i][j] + x*leg[i];
       }
     }

  }

}

//-------------------------------------------------------------------------




double *
SPC:: eval_hermite(double x) const  // retourne les valeurs des polyn�mes de
                                    //hermite en x
{
double * her;
her = new double[no+1];
for(int i=0;i<=no;i++)
  {
    
     if(i==0)
      her[i] = hermite[0][0];
    else{  
    her[i] = hermite[i][i-1] + hermite[i][i]*x;
    for(int j=i-2;j>=0;j--)
       {
         her[i] = hermite[i][j] + x*her[i];
       }
     }
     
  }
return  her;
} 
 

//-------------------------------------------------------------------------


double 
SPC:: eval_hermite(double x, int m) const  // retourne la valeur du polyn�mes de
                                    //hermite de degr� m en x
{
double her;


    if(m==0)
      her = hermite[0][0];
    else{
    her = hermite[m][m-1] + hermite[m][m]*x;
    for(int j=m-2;j>=0;j--)
       {
         her = hermite[m][j] + x*her;
       }
     }

return  her;
}


//-------------------------------------------------------------------------

double *
SPC:: eval_log_uniforme(double x) const  // retourne les valeurs des polyn�mes
                                    //log_uniforme  en x
{
double * logu;
logu = new double[no+1];
for(int i=0;i<=no;i++)
  {
    if(i==0)
      logu[i] = log_uniforme[0][0];
    else{
    logu[i] = log_uniforme[i][i-1] + log_uniforme[i][i]*x;
    for(int j=i-2;j>=0;j--)
       {
         logu[i] = log_uniforme[i][j] + x*logu[i];
       }
     }

  }
return  logu;
}

//-------------------------------------------------------------------------


double 
SPC:: eval_log_uniforme(double x, int m) const  // retourne la valeur du polyn�me de
                                    //log_uniforme de degr� m en x
{
double logu;


    if(m==0)
      logu = log_uniforme[0][0];
    else{
    logu = log_uniforme[m][m-1] + log_uniforme[m][m]*x;
    for(int j=m-2;j>=0;j--)
       {
        logu = log_uniforme[m][j] + x*logu;
       }
     }

return  logu;
}



//-------------------------------------------------------------------------


double *
SPC:: eval_laguerre(double x) const  // retourne les valeurs des polyn�mes de
                                    //legendre en x
{
double * lag;
lag = new double[no+1];
for(int i=0;i<=no;i++)
  {
    if(i==0)
      lag[i] = laguerre[0][0];
    else{
    lag[i] = laguerre[i][i-1] + laguerre[i][i]*x;
    for(int j=i-2;j>=0;j--)
       {
         lag[i] = laguerre[i][j] + x*lag[i];
       }
     }

  }
return  lag;
}



//-------------------------------------------------------------------------


double 
SPC:: eval_laguerre(double x, int m) const  // retourne la valeur du polyn�me de
                                    //laguerre de degr� en x
{
double lag;


    if(m==0)
      lag = laguerre[0][0];
    else{
    lag = laguerre[m][m-1] + laguerre[m][m]*x;
    for(int j=m-2;j>=0;j--)
       {
        lag = laguerre[m][j] + x*lag;
       }
     }

return  lag;
}



//------------------------------------------------------------------------- 
 
double *
SPC:: sensibilite()  // calcul les indices de sensibilit� pour le PC
                    // � modifier 
{
double D;
double * S;
double * S1;
//double * norme;
int a ;
int ind;
// int ifs;
a = int(pow(2.,double(nx))-1);
S = new double[a];
S1 = new double[a];
//norme = new double[npol+1];
//norme = norme_base();

int** I = new int*[a];;
for(int i=0;i<a;i++)
  {
   I[i] = new int[nx];
   for(int j=0;j<nx;j++)
     I[i][j] = 0;
  } 
indices(I,nx); 





for(int i=0;i<a;i++)
S1[i] = 0;

D = 0;

for(int i=0;i<npol;i++)
  {
   if(num[i]!=0)
   {
    D = D + beta[i]*beta[i];
    ind = 0;
    for(int j=0;j<nx;j++)
       {     
          if(npt[i][j] != 0)
            ind = ind + int(pow(2.,double(j)));                    
       }
    ind = ind-1;
    S1[ind] = S1[ind] + beta[i]*beta[i];
   }
  }

for(int i=0;i<a;i++)
 S1[i] = S1[i]/D;  


for(int i=0;i<a;i++)
  {
   ind = 0;
   for(int j=0;j<nx;j++)
       {           
          if(I[i][j] != 0)
            ind = ind + int(pow(2.,double(j)));                    
       }
   ind = ind -1;
   S[i] = S1[ind]; 
   
  } 


return S;

delete S;
} 
 


//-------------------------------------------------------------------------


double *
SPC:: ST()
{
 double * ST;
 double * S;
 int a ;
a = int(pow(2.,double(nx))-1);
S = new double[a];
ST = new double[nx];

S = sensibilite();

int** I = new int*[a];;
for(int i=0;i<a;i++)
  {
   I[i] = new int[nx];
   for(int j=0;j<nx;j++)
     I[i][j] = 0;
  } 
indices(I,nx); 

for(int i=0;i<nx;i++)
   ST[i] = 0;

for(int i=0;i<a;i++)
  {
   for(int j=0;j<nx;j++)
       {
	if(I[i][j]==0)
	  ST[j] =ST[j] + S[i];
       }
  }
for(int i=0;i<nx;i++)
   ST[i] = 1 - ST[i]; 

 return ST;
}




//-------------------------------------------------------------------------



/*double **
PC::build_Z(int N, double** B) // retourne la matrice Z des valeurs des
//polyn�mes de la base de PC en les points de l'�chantillon B de taille N
{
double ** Z;

Z = new double*[N];
for(int i=0;i<N;i++)
  Z[i] = new double[npol+1];

double ** tab_phi;

tab_phi = new double*[nx];
for(int i=0;i<nx;i++)
tab_phi[i] = new double[no+1];

for( int k=0;k<N;k++)
  {
   for(int j=0;j<nx;j++)
     {
      if(lois[j]=='l')
          tab_phi[j] = eval_legendre(B[k][j]);
      else{ if(lois[j]=='h')
          tab_phi[j] = eval_hermite(B[k][j]);

	  else{ if(lois[j]=='u')
             tab_phi[j] = eval_log_uniforme(B[k][j]);
	        else if(lois[j]=='e')
                   tab_phi[j] = eval_laguerre(B[k][j]);
	     }
	  }

      }


   Z[k][0] = 1;

   for(int i=1;i<=npol;i++)
     {
       Z[k][i] = 1 ;
       for(int j=0;j<nx;j++)
         {
          Z[k][i] = Z[k][i] * tab_phi[j][npt[i][j]];

         }
      }

  }

  for(int i=0;i<nx;i++)
    delete  tab_phi[i];
  delete tab_phi;



return Z;
}*/

//-------------------------------------------------------------------------


/*
void
PC::build_Z(double **Z, int N, double** B) // retourne la matrice Z des valeurs des
//polyn�mes de la base de PC en les points de l'�chantillon B de taille N
{
double ** tab_phi;

tab_phi = new double*[nx];
for(int i=0;i<nx;i++)
tab_phi[i] = new double[no+1];

for( int k=0;k<N;k++)
  {
   for(int j=0;j<nx;j++)
     {
      if(lois[j]=='l')
          tab_phi[j] = eval_legendre(B[k][j]);
      else{ if(lois[j]=='h')
          tab_phi[j] = eval_hermite(B[k][j]);

	  else{ if(lois[j]=='u')
             tab_phi[j] = eval_log_uniforme(B[k][j]);
	        else if(lois[j]=='e')
                   tab_phi[j] = eval_laguerre(B[k][j]);
	     }
	  }

      }


   Z[k][0] = 1;

   for(int i=1;i<=npol;i++)
     {
       Z[k][i] = 1 ;
       for(int j=0;j<nx;j++)
         {
          Z[k][i] = Z[k][i] * tab_phi[j][npt[i][j]];

         }
      }

  }

  for(int i=0;i<nx;i++)
    delete  tab_phi[i];
  delete tab_phi;

}*/



//-------------------------------------------------------------------------




//-------------------------------------------------------------------------




//-------------------------------------------------------------------------
 
 



//-------------------------------------------------------------------------
  


//-------------------------------------------------------------------------
  



//-------------------------------------------------------------------------




//-------------------------------------------------------------------------
  


//-------------------------------------------------------------------------


 
//------------------------------------------------------------------------- 
 
/*double *
PC:: sensibilite()  // calcul les indices de sensibilit� pour le PC
                    // � modifier 
{
double D;
double * S;
//double * norme;
int a ;
int ind;
int ifs;
a = int(pow(2.,double(nx))-1);
S = new double[a];
//norme = new double[npol+1];
//norme = norme_base();


int **H;
H = new int*[a+1];
for(int i=0;i<=a;i++)
H[i] = new int[nx];
//H = calculH(nx);
ifs = 0;
for(int id=0;id<nx;id++)
H[ifs][id] = 0;




for(int i=0;i<a;i++)
S[i] = 0;

D = 0;

for(int i=1;i<=npol;i++)
  {
    D = D + beta[i]*beta[i];
    ind = 0;
    for(int j=0;j<nx;j++)
       {           
          if(npt[i][j] != 0)
            ind = ind + int(pow(2.,double(j)));                    
       }
    ind = ind-1;          
    S[ind] = S[ind] + beta[i]*beta[i];
  }
  
  
for(int i=0;i<a;i++)
 S[i] = S[i]/D;  


for(int i=0;i<=a;i++)
delete H[i];
delete H;

return S;

delete S;
} */
 
//-------------------------------------------------------------------------

/*void 
SPC:: zeros()
   {
    for(int i=0;i<npol+1;i++)
       beta[i] = 0.0;
   }*/

//-------------------------------------------------------------------------



//-----------------------------------------------------------------------------




//-------------------------------------------------------------------------





//-------------------------------------------------------------------------



//-------------------------------------------------------------------------



//-------------------------------------------------------------------------
 
 
//-------------------------------------------------------------------------

double * norme_legendre(int no) // retourne la norme de la famille de
//polyn�mes de legendre d'ordre no
{
double * y;
y = new double[no+1];
for(int i=0;i<=no;i++)
y[i] = 1./(2*i+1);


return y;
}

//-------------------------------------------------------------------------

double * norme_hermite(int no)  // retourne la norme de la famille de
//polyn�mes de hermite d'ordre no
{
double * y;
y = new double[no+1];
for(int i=0;i<=no;i++)
y[i] = fact(double(i));

return y;
}
//-------------------------------------------------------------------------

double * norme_log_uniforme(int no)  // retourne la norme de la famille de
//polyn�mes de log_uniforme d'ordre no
{
double * y;
y = new double[no+1];
for(int i=0;i<=no;i++)
y[i] = 1;

return y;
}
//-------------------------------------------------------------------------

double * norme_laguerre(int no)  // retourne la norme de la famille de
//polyn�mes de hermite d'ordre no
{
double * y;
y = new double[no+1];
for(int i=0;i<=no;i++)
y[i] = 1;

return y;
}
//-------------------------------------------------------------------------

int calcul_num(int *v,int nx)
{
 int no,num,s;

 num = 0;
 no = 0;
 for(int i=0;i<nx;i++)
   no = no + v[i];

 if(no == 0) 
   num = 0;
 else if(nx == 1) num = no;
 else 
  {
   num = calculer_npol(no-1,nx); 
  
   s=0;
   for(int i=0;i<nx-1;i++)
    {
     s = s + v[i];
     if(no-s==0) break; 
     num = num + calculer_npol(no-s-1,nx-i-1)+1;
   
    }
     num = num +1;
  }

 return num;
}


//-------------------------------------------------------------------------

void calcul_npt_num(int* npt1,int num,int nx)
{
 int* v = new int[nx];
 int m,n,num1,num2;
 n = 0;
 num1 = 0;
 m = 0;

 for(int i=0;i<nx;i++)
   { 
     npt1[i] = 0;
     v[i] = 0;
   }

 if(num!=0)
  {
   while(num1<num)
    {
     n = n + 1;
     num1 = calculer_npol(n,nx); 
    }
   num1 = calculer_npol(n-1,nx);  
   
   for(int i=0;i<nx-1;i++)
     {
      m = n;
      num2 = calculer_npol(0,nx-i-1)+1;
      while((num1+num2)<num)
       {
        m = m-1;
        num2 = calculer_npol(n-m,nx-i-1)+1;
       }
      npt1[i] = m;
      num1=num1+ calculer_npol(n-m-1,nx-i-1)+1;
      n = n - m;
      
      if(n==0) break;
     }
    npt1[nx-1] = n;
  }

}

//-------------------------------------------------------------------------


void calcul_ind(int * v, int it, int nx)
{

for(int i=0;i<nx;i++)
  v[i]=0; 

int n = 0;
int num = 0;
int itt = it+1;
int j;

while(num<itt)
  {
     n = n + 1;
     num = num + comb(nx,n); 
  }
num = num - comb(nx,n);  
j=0;itt = itt - num;
while(n>0)
 {
  j=j+1;
  //itt = itt - num;
  num = comb(nx-j,n-1);
   if(itt<=num)
     {
      v[j-1]=1;
      n = n-1;
     }
   else itt = itt - num;
 }

 
}

//-------------------------------------------------------------------------

int calcul_ni(int * v, int nx)
{
int no,ni;
// int s;

 ni = 0;
 no = 0;
 for(int i=0;i<nx;i++)
   no = no + v[i];

 if(no == 0) 
   ni = 0;
 else if(nx == 1) ni = no;
 else 
  {
   for(int i=1;i<no;i++)
      ni = ni + comb(nx,i); 
   
   for(int i=0;i<nx;i++)
    {
     if(v[i]==0)
       ni = ni + comb(nx-i-1,no-1);
     else
      no = no-1;
     if(no==0)break;
    }    

  }

return ni;
}



//-------------------------------------------------------------------------

/*SPC fct_regression(int no, int nx,char* law, int N, double** xx, double* y)
{
int npol;  // le nbre de PC de la base
double * bet; // tableau contenant les beta calcul�s par r�gression
double ** Z; // la matrice Z des valeurs des polyn�mes de la base de PC en les
// points de l'�chantillon de taille N
double wmax;
double wmin;

npol = calcul_npol(no,nx);  // calcul du nbre de PC de la base

bet = new double[npol+1];
Z = new double*[N];
for(int i=0;i<N;i++)
   {
   Z[i] = new double[npol+1];
   }



SPC psi(nx,no,law); // construction du PC

Z = psi.build_Z(N,xx); // construction de la matrice Z


      //------calcul des betas par regression -------------------

double ** A;
double ** v;
double * w;
int n; // taille de la matrice A
double sum;

n = npol+1;

A = new double*[N];
v = new double*[n];
w = new double[n];


for(int i=0;i<n;i++){
   v[i] = new double[n];
   }

for(int i=0;i<N;i++){
   A[i] = new double[n];
   }

for(int i=0;i<N;i++){
    for(int j=0;j<n;j++){
         A[i][j] = Z[i][j];
        }
   }



svdcmp( A, N, n, w, v );
wmax = 0.0;
for(int j=0;j<n;j++)
   {
    if(w[j]>wmax)
      wmax = w[j];
   }
wmin = wmax * 1.0e-16;
for(int j=0;j<n;j++)
   {
    if(w[j]<wmin)
      w[j] = 0.0;
   }
svbksb( A, w, v, N, n, y, bet );

psi.modif_beta(bet); // modification des beta dans notre PC


      //----- fin du calcul des betas ------------------------


for(int i=0;i<n;i++){
   delete v[i];
   }

for(int i=0;i<N;i++){
    delete A[i];
   }


delete A;
delete v;


for(int i=0;i<N;i++)
   {
  delete Z[i] ;
   }

delete w;
delete Z;
delete bet;


return psi;
}

*/


//---------------------------------------------------------------------------

//---------------------------------------------------------------------------

void indices(int** J, int d)
{
 int ind = 0;
 
 for(int p=1;p<=d;p++)
    {
     indices2(J,d,p,0,ind);
     ind = ind + comb(d,p);
    }
}  


//-------------------------------------------------------------------------


void indices2(int** J, int d, int p, int i, int k)
{
 if(d>0 && p>0)
 {
 if(p>1 && d!=p)
   {
    for(int j=0;j<comb(d-1,p-1);j++)
       {
        J[k+j][i] = 1;
       }
    for(int j=comb(d-1,p-1);j<comb(d-1,p-1)+comb(d-1,p)-1;j++)
       {
        J[k+j][i] = 0;
       } 
    indices2(J,d-1,p-1,i+1,k);
    indices2(J,d-1,p,i+1,k+comb(d-1,p-1));
   }
 else
   {
    if(p==1)
      {
       for(int j=0;j<d;j++)
          J[k+j][i+j] = 1;
      }
    else
      {
       for(int j=0;j<d;j++)
          J[k][i+j] = 1;
      } 
   } 
 }    
}




//---------------------------------------------------------------------------



void fusion(vector<int> v, int* t, int* tmp, int de1, int vers1, int de2, int vers2, int count, int posInTmp) 
{
       int i;
       for(i = 0 ; i < count ; i++)
       {
               if (de2 > vers2)   // Si fin de la liste 2, on prend dans liste 1
                       tmp[posInTmp++] = t[de1++];
               else{ if (de1 > vers1)   // Idem si fin de la liste 1
                       tmp[posInTmp++] = t[de2++];
               else{ if (v[t[de1]] <= v[t[de2]])   // Enfin, sinon, on compare
                    {   tmp[posInTmp++] = t[de1++];} 
               else 
                    {    tmp[posInTmp++] = t[de2++];}
                } 
              }
       }
}




//-------------------------------------------------------------------------





//-------------------------------------------------------------------------

}

