//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#include <iostream>
#include <stdlib.h>
#include <math.h>
#include "polynome.hpp"

using namespace std;

namespace Anisp
{

//-------------------------------------------------------------------------
polynome:: polynome ()
      { 
          d = 0;
          coef = NULL;
      }

//-------------------------------------------------------------------------

polynome::~polynome()
      {
          delete[] coef;       
      }

//-------------------------------------------------------------------------


polynome::polynome (int deg)
     { 
          d = deg;
          coef = new double[deg+1];
	  for(int i=0;i<=deg;i++)
	     { coef[i]=1;}
      }

//-------------------------------------------------------------------------

polynome::polynome (int deg, double * coefs)
     { 
          d = deg;
          coef = new double[deg+1];
	  for(int i=0;i<=deg;i++)
	     { coef[i]=coefs[i];}
      }

//-------------------------------------------------------------------------

//constructeur par copie
polynome::polynome(const polynome & p0) //:d(p0.d),coef(new double[d+1])   
      { 
	  d = p0.d;
	  coef = new double[d+1]; 
	  *this = p0; 
	  }

//-------------------------------------------------------------------------

polynome &
polynome:: operator = (const polynome & p0) //op�rateur =
      {
            if(this != & p0)
              {
	        delete[] coef;
            coef = NULL;
		d = p0.d;
		coef=new double[d+1];
		memcpy(coef,p0.coef,(d+1)*sizeof(double));
                
              }
	      return(*this);
      }

//-------------------------------------------------------------------------

polynome &
polynome:: operator += (const polynome & p) //op�rateur +=
      {
        polynome r;
        int deg;
        deg = max( p.d , d );
        r=polynome(deg);

        for(int i=0;i<=deg;i++)
         {
          if ( p.d <= d )
             {
               if( i<= p.d)
                  {r.coef[i] = coef[i] + p.coef[i];}
               else
                  {r.coef[i] = coef[i] ;}
             }
          else
             { 
              if( i<= d)
                {r.coef[i] = coef[i] + p.coef[i];}
              else
                { r.coef[i] = p.coef[i] ;}
              }  
          }
	  
	delete[] coef;
    coef = NULL;
        d = r.d;
        coef=new double[d+1];
	memcpy(coef,r.coef,(d+1)*sizeof(double));
	return(*this);
      }

//-------------------------------------------------------------------------

polynome &
polynome:: operator -= (const polynome & p) //op�rateur -=
      {
        polynome r;
        int deg;
        deg = max( p.d , d );
        r=polynome(deg);

        for(int i=0;i<=deg;i++)
         {
          if ( p.d <= d )
             {
               if( i<= p.d)
                  {r.coef[i] = coef[i] - p.coef[i];}
               else
                  {r.coef[i] = coef[i] ;}
             }
          else
             { 
              if( i<= d)
                {r.coef[i] = coef[i] - p.coef[i];}
              else
                { r.coef[i] = - p.coef[i] ;}
              }  
          }
	  
	delete[] coef;
    coef = NULL;
        d = r.d;
        coef=new double[d+1];
	memcpy(coef,r.coef,(d+1)*sizeof(double));
	return(*this);
      }

//-------------------------------------------------------------------------

polynome 
polynome:: operator + (const polynome & p) 
{
polynome r;
int deg;
deg = max( p.d , d );
r=polynome(deg);

for(int i=0;i<=deg;i++)
 {
 if ( p.d <= d )
    {
       if( i<= p.d)
         {r.coef[i] = coef[i] + p.coef[i];}
       else
         {r.coef[i] = coef[i] ;}
    }
 else
    { 
       if( i<= d)
         {r.coef[i] = coef[i] + p.coef[i];}
       else
         {r.coef[i] = p.coef[i] ;}
    }  
 }
 
return r;
}


//-------------------------------------------------------------------------

polynome 
polynome:: operator - (const polynome & p) 
{
polynome r;
int deg;
deg = max( p.d , d );
r=polynome(deg);

for(int i=0;i<=deg;i++)
 {
 if ( p.d <= d )
    {
       if( i<= p.d)
         {r.coef[i] = coef[i] - p.coef[i];}
       else
         {r.coef[i] = coef[i] ;}
    }
 else
    { 
       if( i<= d)
         {r.coef[i] = coef[i] - p.coef[i];}
       else
         {r.coef[i] = -p.coef[i] ;}
    }  
 }
 
return r;
}


//-------------------------------------------------------------------------

polynome 
polynome:: operator * (const polynome & p) 
{
polynome r;
int deg;
deg = p.d + d ;
r=polynome(deg);
r.zeros();
for(int i=0;i<=d;i++)
   {
   for(int j=0;j<=p.d;j++)
     {
     r.coef[i+j] = r.coef[i+j] + coef[i] * p.coef[j];
     }
   }

return r;
}

//-------------------------------------------------------------------------

polynome operator * (double a,const polynome & p) 
{
polynome r;
r=polynome(p.d);
r.zeros();
for(int i=0;i<=p.d;i++)
   {
     r.coef[i] = a * p.coef[i] ;
   }
return r;
}

//-------------------------------------------------------------------------

polynome
polynome :: operator / (double a)
{
polynome r;
r=polynome(d);
r.zeros();
for(int i=0;i<=d;i++)
   {
     r.coef[i] = coef[i]/a ;
   }
return r;
}
//-------------------------------------------------------------------------

bool 
polynome:: operator == (const polynome & p )
{
bool a=1;
   if (d != p.d)
     { a = 0;}
   else
     {
      for(int i=0;i<=d;i++)
        {
	  if (coef[i] != p.coef[i])
	   {
	    a = 0;
	    break;
	   }
	}
     }  
return a;
}

//-------------------------------------------------------------------------


bool 
polynome:: operator != (const polynome & p )
{
bool a=0;
   if (d != p.d)
     { a = 1;}
   else
     {
      for(int i=0;i<=d;i++)
        {
	  if (coef[i] != p.coef[i])
	   {
	    a = 1;
	    break;
	   }
	}
     }  
return a;
}

//-------------------------------------------------------------------------


double &
polynome:: operator [] (int i) const //op�rateur []
      {
        return coef[i] ;   
      }

//-------------------------------------------------------------------------

double 
polynome:: operator () (double x ) const 
{
double y;
/*y = coef[0];
 for(int i=1;i< d +1;i++)
 {
 y = y + coef[i]*pow(x,i);
 }*/
 if(d==0)
 y = coef[0];
 else{
   y = coef[d-1] + coef[d]*x;
   for(int i=d-2;i>=0;i--)
    {
     y = coef[i] + x*y;
    }
 }
return y;
}

//-------------------------------------------------------------------------

void puissance( int n , polynome & p)
{
polynome q(p);
for(int i=0;i<n-1;i++)
p = p * q;
}

//-------------------------------------------------------------------------

void 
polynome::zeros()
{
 for(int i=0;i<=d;i++)
   { coef[i]=0;}
}
//-------------------------------------------------------------------------

void
polynome:: modifcoef( double c , int i )  // modifie un coefficient  
      {
        if(i <= d)
	   {
	     coef[i]=c;
	   } 
	else { cout<<"l'indice est trop grand"<<endl;}  
      }

//-------------------------------------------------------------------------

double 
polynome::eval_poly( double x) const
{
 double y;
 // y = coef[0];
 if(d==0)
 y = coef[0];
 else{
     y = coef[d-1] + coef[d]*x;
     for(int i=d-2;i>=0;i--)
       {
       //y = y + coef[i]*pow(x,i);
         y = coef[i] + x*y;
       }
   }    
 return y;
 } 

//-------------------------------------------------------------------------

int
polynome:: lire_degre() const
{
return d;
}

//-------------------------------------------------------------------------

void 
polynome::affiche()
{
cout<<coef[0];
for(int i=1;i<=d;i++)
   {
   cout<<" + "<<coef[i]<<"x^"<<i;
   }
   cout<<" "<<endl;
}

//-------------------------------------------------------------------------
 
polynome *  build_legendre(int n) 
{
polynome q(1);
q.coef[0] = 0;
polynome * p;
p = new polynome[n+1];
p[0] = polynome(0);
if(n!=0){
p[1] = polynome(1);
p[1].coef[0]=0;
}

for(int i=1;i<n;i++)
  {
   p[i+1]=( (2*i+1)*q*p[i] - i*p[i-1] )/(i+1);
  } 
return p;
} 
//-------------------------------------------------------------------------

polynome *  build_hermite(int n)
{
polynome q(1);
q.coef[0] = 0;
polynome * p;
p = new polynome[n+1];
p[0] = polynome(0);
if(n!=0){
p[1] = polynome(1);
p[1].coef[0]=0;
}
for(int i=1;i<n;i++)
  {
   p[i+1]=q*p[i] - i*p[i-1];
  }
return p;
}
//-------------------------------------------------------------------------

polynome *  build_log_uniforme(int no)
{
polynome q(no);
polynome r;
polynome * p;
p = new polynome[no+1];
double t;

p[0] = polynome(0);
q.zeros();
double alpha;

for(int i=1;i<=no;i++)
   {
    q[i] = 1;
    p[i] = q;
	
    for(int j=0;j<i;j++)
       {
        r = q*p[j];
	    alpha = r[0];
	    for(int k=1;k<=i+j;k++)
	      {
	       alpha = alpha + r[k]*
	       (pow(10.0,double(k))-pow(10.0,-double(k)))/(k*2*log(10.0));
	      }
	    p[i] = p[i] - alpha*p[j];
       }
	
    r = p[i]*p[i];
    t = r[0];
    for(int k=1;k<=2*i;k++)
        {
	     t = t + r[k]*
	     (pow(10.0,double(k))-pow(10.0,-double(k)))/(k*2*log(10.0));
	    }
    t = 1.0/sqrt(t);
    p[i] = t*p[i];
    q[i] = 0;
   }

return p;
}
 
 
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------

polynome *  build_laguerre(int n)
{
polynome q(1);
q.coef[0] = 0;
polynome * p;
p = new polynome[n+1];
p[0] = polynome(0);
if(n!=0){
p[1] = polynome(1);
p[1].coef[1]=-1;
}
for(int i=1;i<n;i++)
  {
   p[i+1]=( (2*i+1)*p[i] - q*p[i] - i*p[i-1] )/(i+1);
  }
return p;
}
//-------------------------------------------------------------------------


//-------------------------------------------------------------------------
 
 
 
//-------------------------------------------------------------------------




//-------------------------------------------------------------------------
 
 
 
//-------------------------------------------------------------------------

}

