//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#include "util.hpp"
#include "polynome.hpp"
#define EPS 3.0e-14
#define PIM4 0.7511255444649425
#define MAXIT 10
#define M_SQRT2PI 2.506683

namespace Anisp
{

using namespace std; 


static double borne     = 1.e-13;
static double precision = 1.e-16;

static double p_min = 0.02425;
static double p_max = 0.97575;

static double a[]={0.,
  -3.969683028665376e+01,
  +2.209460984245205e+02,
  -2.759285104469687e+02,
  +1.383577518672690e+02,
  -3.066479806614716e+01,
  +2.506628277459239e+00};

static double b[]={0.,
  -5.447609879822406e+01,
  +1.615858368580409e+02,
  -1.556989798598866e+02,
  +6.680131188771972e+01,
  -1.328068155288572e+01,};

static double c[]={0.,
  -7.784894002430293e-03,
  -3.223964580411365e-01,
  -2.400758277161838e+00,
  -2.549732539343734e+00,
  +4.374664141464968e+00,
  +2.938163982698783e+00};

static double d[]={0.,
  +7.784695709041462e-03,
  +3.224671290700398e-01,
  +2.445134137142996e+00,
  +3.754408661907416e+00};





//-----------------------------------------------------------------------------

/*void svdcmp(double **a, int m, int n, double *w, double **v)
// retourne la d�composition SVD
{
	double pythag(double a, double b);
	int flag,i,its,j,jj,k,l,nm;
	double anorm,c,f,g,h,s,scale,x,y,z,*rv1;

	rv1=new double[n];
	g=scale=anorm=0.0;	
	for (i=0;i<n;i++) {
		l=i+1;
		rv1[i]=scale*g;
		g=s=scale=0.0;
		if (i <= m) {
			for (k=i;k<m;k++) scale += fabs(a[k][i]);
			if (scale) {
				for (k=i;k<m;k++) {
					a[k][i] /= scale;
					s += a[k][i]*a[k][i];
				}
				f=a[i][i];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][i]=f-g;
				for (j=l;j<n;j++) {
					for (s=0.0,k=i;k<m;k++) s += a[k][i]*a[k][j];
					f=s/h;
					for (k=i;k<m;k++) a[k][j] += f*a[k][i];
				}
				for (k=i;k<m;k++) a[k][i] *= scale;
			}
		}
		w[i]=scale *g;
		g=s=scale=0.0;
		if (i <= m && i != n) {
			for (k=l;k<n;k++) scale += fabs(a[i][k]);
			if (scale) {
				for (k=l;k<n;k++) {
					a[i][k] /= scale;
					s += a[i][k]*a[i][k];
				}
				f=a[i][l];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][l]=f-g;
				for (k=l;k<n;k++) rv1[k]=a[i][k]/h;
				for (j=l;j<m;j++) {
					for (s=0.0,k=l;k<n;k++) s += a[j][k]*a[i][k];
					for (k=l;k<n;k++) a[j][k] +=s*rv1[k]; 
					
				}
				for (k=l;k<n;k++) a[i][k] *= scale;
			}
		}
		anorm=DMAX(anorm,(fabs(w[i])+fabs(rv1[i])));
	}
	for (i=n-1;i>=0;i--) {
		if (i < n) {
			if (g) {
				for (j=l;j<n;j++)
					v[j][i]=(a[i][j]/a[i][l])/g;
				for (j=l;j<n;j++) {
					for (s=0.0,k=l;k<n;k++) s += a[i][k]*v[k][j];
					for (k=l;k<n;k++) v[k][j] += s*v[k][i];
				}
			}
			for (j=l;j<n;j++) v[i][j]=v[j][i]=0.0;
		}
		v[i][i]=1.0;
		g=rv1[i];
		l=i;
	}	
	for (i=IMIN(m,n)-1;i>=0;i--) {
		l=i+1;
		g=w[i];
		for (j=l;j<n;j++) a[i][j]=0.0;
		if (g) {
			g=1.0/g;
			for (j=l;j<n;j++) {
				for (s=0.0,k=l;k<m;k++) s += a[k][i]*a[k][j];
				f=(s/a[i][i])*g;
				for (k=i;k<m;k++){ a[k][j] += f*a[k][i];
				                  }
			}
			for (j=i;j<m;j++){ a[j][i] *= g;}
		} else for (j=i;j<m;j++) a[j][i]=0.0;
		++a[i][i];
	}
	for (k=n-1;k>=0;k--) {
		for (its=1;its<=30;its++) {
			flag=1;
			for (l=k;l>=0;l--) {
				nm=l-1;
				if ((double)(fabs(rv1[l])+anorm) == anorm) {
					flag=0;
					break;
				}
				if ((double)(fabs(w[nm])+anorm) == anorm) break;
			}
			if (flag) {
				c=0.0;
				s=1.0;
				for (i=l;i<=k;i++) {
					f=s*rv1[i];
					rv1[i]=c*rv1[i];
					if ((double)(fabs(f)+anorm) == anorm) break;
					g=w[i];
					h=pythag(f,g);
					w[i]=h;
					h=1.0/h;
					c=g*h;
					s = -f*h;
					for (j=0;j<m;j++) {
						y=a[j][nm];
						z=a[j][i];
						a[j][nm]=y*c+z*s;
						a[j][i]=z*c-y*s;
					}
				}
			}
			z=w[k];
			if (l == k) {
				if (z < 0.0) {
					w[k] = -z;
					for (j=0;j<n;j++) v[j][k] = -v[j][k];
				}
				break;
			}
			char mess_err[100] = "no convergence in 50 svdcmp iterations";
			if (its == 50) nrerror(mess_err);
			x=w[l];
			nm=k-1;
			y=w[nm];
			g=rv1[nm];
			h=rv1[k];
			f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
			g=pythag(f,1.0);
			f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
			c=s=1.0;
			for (j=l;j<=nm;j++) {
				i=j+1;
				g=rv1[i];
				y=w[i];
				h=s*g;
				g=c*g;
				z=pythag(f,h);
				rv1[j]=z;
				c=f/z;
				s=h/z;
				f=x*c+g*s;
				g = g*c-x*s;
				h=y*s;
				y *= c;
				for (jj=0;jj<n;jj++) {
					x=v[jj][j];
					z=v[jj][i];
					v[jj][j]=x*c+z*s;
					v[jj][i]=z*c-x*s;
				}
				z=pythag(f,h);
				w[j]=z;
				if (z) {
					z=1.0/z;
					c=f*z;
					s=h*z;
				}
				f=c*g+s*y;
				x=c*y-s*g;
				for (jj=0;jj<m;jj++) {
					y=a[jj][j];
					z=a[jj][i];
					a[jj][j]=y*c+z*s;
					a[jj][i]=z*c-y*s;
				}
			}
			rv1[l]=0.0;
			rv1[k]=f;
			w[k]=x;
		}
	}
	delete rv1;
}
*/
/* (C) Copr. 1986-92 Numerical Recipes Software Z4k!'2150K%521.<. */
//-----------------------------------------------------------------------------

// void svbksb(double **u, double *w, double **v, int m, int n, double *b, double *x)
// //la resolution d'un syst�me lin�aire � l'aide de la d�composition SVD
// {
// 	int jj,j,i;
// 	double s,*tmp;
// 
// 	tmp=new double[n];
// 	for (j=0;j<n;j++) {
// 		s=0.0;
// 		if (w[j]) {
// 			for (i=0;i<m;i++) s += u[i][j]*b[i];
// 			s /= w[j];
// 		}
// 		tmp[j]=s;
// 	}
// 	for (j=0;j<n;j++) {
// 		s=0.0;
// 		for (jj=0;jj<n;jj++) s += v[j][jj]*tmp[jj];
// 		x[j]=s;
// 	}
// 	delete tmp;
// }

/* (C) Copr. 1986-92 Numerical Recipes Software Z4k!'2150K%521.<. */
//-----------------------------------------------------------------------------


// void qrdcmp(double **a, int n, double *c, double *d, int *sing)
// {
// 	int i,j,k;
// 	double scale=0.0,sigma,sum,tau;
// 
// 	*sing=0;
// 	for (k=1;k<n;k++) {
// 		for (i=k;i<=n;i++) scale=DMAX(scale,fabs(a[i][k]));
// 		if (scale == 0.0) {
// 			*sing=1;
// 			c[k]=d[k]=0.0;
// 		} else {
// 			for (i=k;i<=n;i++) a[i][k] /= scale;
// 			for (sum=0.0,i=k;i<=n;i++) sum += DSQR(a[i][k]);
// 			sigma=SIGN(sqrt(sum),a[k][k]);
// 			a[k][k] += sigma;
// 			c[k]=sigma*a[k][k];
// 			d[k] = -scale*sigma;
// 			for (j=k+1;j<=n;j++) {
// 				for (sum=0.0,i=k;i<=n;i++) sum += a[i][k]*a[i][j];
// 				tau=sum/c[k];
// 				for (i=k;i<=n;i++) a[i][j] -= tau*a[i][k];
// 			}
// 		}
// 	}
// 	d[n]=a[n][n];
// 	if (d[n] == 0.0) *sing=1;
// }
// 

/* (C) Copr. 1986-92 Numerical Recipes Software Z4k!'2150K%521.<. */
//-----------------------------------------------------------------------------

// void qrupdt(double **r, double **qt, int n, double *u, double *v)
// {
// 	void rotate(double **r, double **qt, int n, int i, double a, double b);
// 	int i,j,k;
// 
// 	for (k=n;k>=1;k--) {
// 		if (u[k]) break;
// 	}
// 	if (k < 1) k=1;
// 	for (i=k-1;i>=1;i--) {
// 		rotate(r,qt,n,i,u[i],-u[i+1]);
// 		if (u[i] == 0.0) u[i]=fabs(u[i+1]);
// 		else if (fabs(u[i]) > fabs(u[i+1]))
// 			u[i]=fabs(u[i])*sqrt(1.0+DSQR(u[i+1]/u[i]));
// 		else u[i]=fabs(u[i+1])*sqrt(1.0+DSQR(u[i]/u[i+1]));
// 	}
// 	for (j=1;j<=n;j++) r[1][j] += u[1]*v[j];
// 	for (i=1;i<k;i++)
// 		rotate(r,qt,n,i,r[i][i],-r[i+1][i]);
// }

/* (C) Copr. 1986-92 Numerical Recipes Software Z4k!'2150K%521.<. */
//-----------------------------------------------------------------------------


// void qrsolv(double **a, int n, double *c, double *d, double *b)
// {
// 	void rsolv(double **a, int n, double *d, double *b);
// 	int i,j;
// 	double sum,tau;
// 
// 	for (j=1;j<n;j++) {
// 		for (sum=0.0,i=j;i<=n;i++) sum += a[i][j]*b[i];
// 		tau=sum/c[j];
// 		for (i=j;i<=n;i++) b[i] -= tau*a[i][j];
// 	}
// 	rsolv(a,n,d,b);
// }


/* (C) Copr. 1986-92 Numerical Recipes Software Z4k!'2150K%521.<. */
//-----------------------------------------------------------------------------


// void rsolv(double **a, int n, double *d, double *b)
// {
// 	int i,j;
// 	double sum;
// 
// 	b[n] /= d[n];
// 	for (i=n-1;i>=1;i--) {
// 		for (sum=0.0,j=i+1;j<=n;j++) sum += a[i][j]*b[j];
// 		b[i]=(b[i]-sum)/d[i];
// 	}
// }


/* (C) Copr. 1986-92 Numerical Recipes Software Z4k!'2150K%521.<. */
//-----------------------------------------------------------------------------


// void rotate(double **r, double **qt, int n, int i, double a, double b)
// {
// 	int j;
// 	double c,fact,s,w,y;
// 
// 	if (a == 0.0) {
// 		c=0.0;
// 		s=(b >= 0.0 ? 1.0 : -1.0);
// 	} else if (fabs(a) > fabs(b)) {
// 		fact=b/a;
// 		c=SIGN(1.0/sqrt(1.0+(fact*fact)),a);
// 		s=fact*c;
// 	} else {
// 		fact=a/b;
// 		s=SIGN(1.0/sqrt(1.0+(fact*fact)),b);
// 		c=fact*s;
// 	}
// 	for (j=i;j<=n;j++) {
// 		y=r[i][j];
// 		w=r[i+1][j];
// 		r[i][j]=c*y-s*w;
// 		r[i+1][j]=s*y+c*w;
// 	}
// 	for (j=1;j<=n;j++) {
// 		y=qt[i][j];
// 		w=qt[i+1][j];
// 		qt[i][j]=c*y-s*w;
// 		qt[i+1][j]=s*y+c*w;
// 	}
// }

/* (C) Copr. 1986-92 Numerical Recipes Software Z4k!'2150K%521.<. */
//-----------------------------------------------------------------------------

// void facto_QR(double **A, int N, int M, double *beta, double *diagr)
// {
// double s,alpha,piv,gamma;
// double *v;
// v = new double[M];
// 
// for(int k=0;k<N;k++)
//   {
//    s = 0;
//    for(int i=k;i<M;i++)
//       {
//        v[i] = A[i][k];
//        s = s + v[i]*v[i];
//       }
//    alpha = sqrt(s);
//    piv = A[k][k];
//    if(piv>0) alpha = -alpha;
//    diagr[k] = alpha;
//    beta[k] = s - piv*alpha;
//    piv = piv - alpha;
//    v[k] = piv;
//    A[k][k] = piv;double stdnormal_inv(double p)
// {
//  const double a[6] = {
//   -3.969683028665376e+01,  2.209460984245205e+02,
//   -2.759285104469687e+02,  1.383577518672690e+02,
//   -3.066479806614716e+01,  2.506628277459239e+00
//  };
//  const double b[5] = {
//   -5.447609879822406e+01,  1.615858368580409e+02,
//   -1.556989798598866e+02,  6.680131188771972e+01,
//   -1.328068155288572e+01
//  };
//  const double c[6] = {
//   -7.784894002430293e-03, -3.223964580411365e-01,
//   -2.400758277161838e+00, -2.549732539343734e+00,
//    4.374664141464968e+00,  2.938163982698783e+00
//  };
//  const double d[4] = {
//    7.784695709041462e-03,  3.224671290700398e-01,
//    2.445134137142996e+00,  3.754408661907416e+00
//  };
// 
//  register double q, t, u;
// 
//  if (isnan(p) || p > 1.0 || p < 0.0)
//   exception();//return NaN;
//  if (p == 0.0)
//   exception();//return nan;
//  if (p == 1.0)
//   exception();//return  _nan;
//  q = DMIN(p,1-p);
//  if (q > 0.02425) {
//   /* Rational approximation for central region. */
//   u = q-0.5;
//   t = u*u;
//   u = u*(((((a[0]*t+a[1])*t+a[2])*t+a[3])*t+a[4])*t+a[5])
//     /(((((b[0]*t+b[1])*t+b[2])*t+b[3])*t+b[4])*t+1);
//  } else {
//   /* Rational approximation for tail region. */
//   t = sqrt(-2*log(q));
//   u = (((((c[0]*t+c[1])*t+c[2])*t+c[3])*t+c[4])*t+c[5])
//    /((((d[0]*t+d[1])*t+d[2])*t+d[3])*t+1);
//  }
//  /* The relative error of the approximation has absolute value less
//     than 1.15e-9.  One iteration of Halley's rational method (third
//     order) gives full machine precision... */
//  t = erf(u)-q;    /* error */
//  t = t*M_SQRT2PI*exp(u*u/2);   /* f(u)/df(u) */
//  u = u-t/(1+u*t/2);     /* Halley's method */
// 
//  return (p > 0.5 ? -u : u);
// };

//    for(int j=k+1;j<N;j++)
//       {
//        s = 0;
//        for(int i=k;i<M;i++)
//            s = s + v[i]*A[i][j];
//        gamma = s/beta[k];
//        for(int i=k;i<M;i++)
//            A[i][j] = A[i][j] - gamma*v[i];
// 
//       }
// 
//   }
// 
//   delete v;
// 
// }


//-----------------------------------------------------------------------------

// void regression_QR(double **Z, double *bet, double *y, int N, int M)
// {
// double **A;
// double *b, *d, *diagr, *yy;
// b = new double[N];
// d = new double[N];
// diagr = new double[N];
// yy = new double[N];
// A = new double*[M];
// for(int i=0;i<M;i++)
//    A[i] = new double[N];
// 
// for(int i=0;i<M;i++)
//    {
//     for(int j=0;j<N;j++)
//        A[i][j] = Z[i][j];
//    }
// 
// facto_QR(A,N,M,d,diagr); // factorisation QR
// 
// 
// for(int i=0;i<N;i++)    // calcul de Z^t y
//    {
//    b[i] = 0;
//    for(int j=0;j<M;j++)
//        b[i] = b[i] + Z[j][i]*y[j];
//    }
// 
//     //----descente-------//
//   yy[0] = b[0]/diagr[0];
//   for(int i=1;i<N;i++)
//       {
//        yy[i] = b[i];
//        for(int j=0;j<i;j++)
//            yy[i] = yy[i] - yy[j]*A[j][i];
//        yy[i] = yy[i]/diagr[i];
//       }
// 
//    //----remont�e-------//
//   bet[N-1] = yy[N-1]/diagr[N-1];
//   for(int i=N-2;i>=0;i--)
//       {
//        bet[i] = yy[i];
//        for(int j=i+1;j<N;j++)
//            bet[i] = bet[i] - bet[j]*A[i][j];
//        bet[i] = bet[i]/diagr[i];
//       }
// 
// for(int i=0;i<M;i++)
//    delete A[i];
// 
// delete b;
// delete d;
// delete diagr;
// delete yy;
// 
// }

//-----------------------------------------------------------------------------

// void update_QR(double **A, int N, int M, double *beta, double *diagr)
// {
// double s,alpha,piv,gamma;
// double *v;
// v = new double[M];
// 
// for(int k=0;k<N-1;k++)
//   {
// 
//    for(int j=N-1;j<N;j++)
//       {
//        s = 0;
//        for(int i=k;i<M;i++)
//            s = s + A[i][k]*A[i][j];
//        gamma = s/beta[k];
//        for(int i=k;i<M;i++)
//            A[i][j] = A[i][j] - gamma*A[i][k];	   	 
//           
//       }   
//   
//   } 
//         
//    int k = N-1;
//    s = 0;
//    for(int i=k;i<M;i++)
//       {
//        v[i] = A[i][k];
//        s = s + v[i]*v[i];
//       }
//    alpha = sqrt(s);
//    piv = A[k][k];
//    if(piv>0) alpha = -alpha;
//    diagr[k] = alpha;
//    beta[k] = s - piv*alpha;
//    piv = piv - alpha;
//    v[k] = piv;
//    A[k][k] = piv;
//    
// 
// delete v;
//   
// }
// 



//-----------------------------------------------------------------------------


// double test_col_QR(double **A, int N, int M, double *beta, double *diagr, 
// double *b)
// {
// double s, t, alpha, piv, gamma, gamma1, m, n;
// double *v;
// v = new double[M];
// 
// for(int k=0;k<N-1;k++)
//   {        
//        s = 0;
//        t = 0;
//        for(int i=k;i<M;i++)
//           {
//            s = s + A[i][k]*A[i][N-1];
// 	   t = t + A[i][k]*b[i];
// 	  } 
//        gamma = s/beta[k];
//        gamma1 = t/beta[k];
//        for(int i=k;i<M;i++)
//           { 
//            A[i][N-1] = A[i][N-1] - gamma*A[i][k];
// 	   b[i] = b[i] - gamma1*A[i][k];
// 	  } 	   	            
//   }  
// 
// 
// m = A[M-1][N-1]*b[M-1];
// n = A[M-1][N-1]*A[M-1][N-1];
// for(int i=N-1;i<M-1;i++)
//   { 
//    m = m + A[i][N-1]*b[i]; 
//    n = n + A[i][N-1]*A[i][N-1];
//   }    
// m = m*m/n;
// 
// delete v;
// 
// return m;  
// }




//-----------------------------------------------------------------------------


// double test_col_reg(double **A, int N, int M, double *beta, double *diagr,
// double *b)
// {
// double s, t, alpha, piv, gamma, gamma1, m, n;
// double *b1;
// double **A1 = new double*[M];
// b1 = new double[M];
// for(int i=0;i<M;i++)
//     A1[i] = new double[N];
// 
// for(int i=0;i<M;i++)
//    {
//     for(int j=0;j<N;j++)
//        A1[i][j] = A[i][j];
//     b1[i] = b[i];
//    }
// 
// for(int k=0;k<N-1;k++)
//   {
//        s = 0;
//        t = 0;
//        for(int i=k;i<M;i++)
//           {
//            s = s + A1[i][k]*A1[i][N-1];
// 	   t = t + A1[i][k]*b1[i];
// 	  }
//        gamma = s/beta[k];
//        gamma1 = t/beta[k];
//        for(int i=k;i<M;i++)
//           {
//            A1[i][N-1] = A1[i][N-1] - gamma*A1[i][k];
// 	   b1[i] = b1[i] - gamma1*A1[i][k];
// 	  }
//   }
// 
// 
// m = A1[M-1][N-1]*b1[M-1];
// n = A1[M-1][N-1]*A1[M-1][N-1];
// for(int i=N-1;i<M-1;i++)
//   {
//    m = m + A1[i][N-1]*b1[i];
//    n = n + A1[i][N-1]*A1[i][N-1];
//   }
// m = m*m/n;
// 
// for(int i=0;i<M;i++)
//    delete A1[i] ;
// 
// delete A1;
// delete b1;
// 
// return m;
// }




//-----------------------------------------------------------------------------


// double DSQR(double a)
// {
// 	return a*a;
// }


//-----------------------------------------------------------------------------

// double pythag(double a, double b)
// {
// 	return (sqrt(a*a +b*b));
// }


//-----------------------------------------------------------------------------

void nrerror(char error_text[100])
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}


//-----------------------------------------------------------------------------

// double DMAX(double a, double b)  //la fonction max pour les double
// {
// if(a>b) return a;
// else return b;
// }


//-----------------------------------------------------------------------------

// double DMIN(double a, double b)  //la fonction min pour les double
// {
// if(a<b) return a;
// else return b;
// }


//-----------------------------------------------------------------------------

int IMAX(int a, int b)  // la fonction max pour les int
{
if(a>b) return a;
else return b;
}


//-----------------------------------------------------------------------------


int IMIN(int a, int b)  // la fonction min pour les int
{
if(a<b) return a;
else return b;
}


//-----------------------------------------------------------------------------

double SIGN(double a, double b)
{
if(b<0) return -a;
else return a;
}


//-----------------------------------------------------------------------------

// double ** inverse_svd(double **a, int n)  // inverse la matrice � l'aide de la SVD
// {
// double ** inv;
// double ** v;
// double * w;
// double * x;
// double * b;
// inv = new double*[n];
// v = new double*[n];
// w = new double[n];
// x = new double[n];
// b = new double[n]; 
// 
// for(int i=0;i<n;i++)
//    inv[i] = new double[n];
//    
// for(int i=0;i<n;i++)
//    v[i] = new double[n];
// 
// svdcmp( a, n, n, w, v );
//    
// for(int j=0;j<n;j++)   
//     {
//     for(int i=0;i<n;i++)
//        b[i] = delta(i,j);
//        
//     svbksb( a, w, v, n, n, b, x );
//     
//     for(int i=0;i<n;i++)
//        inv[i][j] = x[i];
//       
//     }   
// 
// 
//    
// for(int i=0;i<n;i++)
//    delete v[i];
//    
// 
// delete v; 
// delete w;
// delete b;
// delete x;  
// 
// return inv;
//    
// }


//-----------------------------------------------------------------------------

int delta(int i, int j) // la fonction de kronecker
{
if(i==j) return 1;
else return 0;
}

//-----------------------------------------------------------------------------

int comb(int n , int p) // la fonction de combinaison
{
int c = 1;
if((n-p)>p)
 {
  for(int i=n-p+1;i<=n;i++)
     c = c*i;
  c = c/fact(p);   
 }
else
 {
 for(int i=p+1;i<=n;i++)
     c = c*i;
  c = c/fact(n-p); 
 } 
//c=fact(n)/(fact(p)*fact(n-p));

return c;
}


//-----------------------------------------------------------------------------

double comb(double n , double p) // la fonction de combinaison
{
double c = 1;
if((n-p)>p)
 {
  for(int i=int(n-p)+1;i<=n;i++)
     c = c*i;
  c = c/fact(p);   
 }
else
 {
 for(int i=int(p)+1;i<=n;i++)
     c = c*i;
  c = c/fact(n-p); 
 } 
//c=fact(n)/(fact(p)*fact(n-p));

return c;
}

//-----------------------------------------------------------------------------


 int fact(int n) // la fonction factoriel
 {
  int a;
 if(n==0)
   {a=1;}
 else{
      if( n<13 )
      {
       a=1;
       for(int i=2;i<=n;i++)
       a=a*i;
      }
      else
        {  
         cout<<" fonction factoriel echoue "<<endl;
         //exit(-1);
         a=1;
        }
    }  
 return a;
 }

//-----------------------------------------------------------------------------


 double fact(double n) // la fonction factoriel
 {
  double a;
 if(n==0)
   {a=1;}
 else{
       a=1;
       for(int i=2;i<=n;i++)
       a=a*i;  
    }  
 return a;
 }



//------------------------------------------------------------------------

int fact_partiel(int m, int n) //produit:m*(m+1)*....*(n-1)*n 
{
int a;
if(n==0)
   {a=1;}
 else{
    if(n==m) {a=1;}
    else{
      a=m+1;
      for(int i=m+2;i<=n;i++)
       a=a*i;
      } 
     }  
return a;
}


//-----------------------------------------------------------------------------


// double get_user_time(void)
// {
// struct rusage used;
// getrusage(RUSAGE_SELF,&used);
// return(used.ru_utime.tv_sec+(used.ru_utime.tv_usec)/1e6);
// }


//-----------------------------------------------------------------------------


double gauss() // retourne un nombre al�atoire suivant la gaussienne centr�e r�duite
{
const double pi2=8*atan(1.);
const double epsi=1.e-50;
time_t *adr= new time_t;
srand(int(time(adr)));
delete adr;
return sqrt(epsi-2.*log(epsi+rand()/(double)RAND_MAX))*cos(rand()*pi2/RAND_MAX);
}



//-----------------------------------------------------------------------------

void use_gnuplot()
{
//system("gnuplot ");
int nret = system("gnuplot \"taille_2d\" ");
//system("quit");

}


//-----------------------------------------------------------------------------

double** lhs(int dim, int n, char loi)
{
double** A = new double*[n];
vector<double> v(n,0);
time_t *adr= new time_t;
srand(int(time(adr)));
delete adr;

for(int i=0;i<n;i++)
   A[i] = new double[dim];

if(loi == 'l')
  {
   for(int j=0;j<dim;j++)
      {
       for(int i=0;i<n;i++)
          v[i] = -1 + 2*double(i)/n + (2./n)*double(rand())/RAND_MAX;

       random_shuffle(v.begin(),v.end());

       for(int i=0;i<n;i++)
          A[i][j] = v[i];
      }
  }
else if(loi == 'e')
  {
   for(int j=0;j<dim;j++)
      {
       for(int i=0;i<n-1;i++)
          v[i] = -log(1-double(i)/n) +
	         log(double(n-i)/(n-i-1))*double(rand())/RAND_MAX;
          
	v[n-1] = -log(1-double(n-1)/n) +
                 2*log(2.0)*double(rand())/RAND_MAX;


       random_shuffle(v.begin(),v.end());

       for(int i=0;i<n;i++)
          A[i][j] = v[i];
      }
  }

return A;
}

//-----------------------------------------------------------------------------

void lhs(double** A, int dim, int n, char loi)
{
//double** A = new double*[n];
vector<double> v(n,0);
time_t *adr= new time_t;
srand(int(time(adr)));
delete adr;

/*or(int i=0;i<n;i++)
   A[i] = new double[dim];*/

if(loi == 'l')
  {
   for(int j=0;j<dim;j++)
      {
       for(int i=0;i<n;i++)
          v[i] = -1 + 2*double(i)/n + (2./n)*double(rand())/RAND_MAX;

       random_shuffle(v.begin(),v.end());

       for(int i=0;i<n;i++)
          A[i][j] = v[i];
      }
  }
else if(loi == 'e')
  {
   for(int j=0;j<dim;j++)
      {
       for(int i=0;i<n-1;i++)
          v[i] = -log(1-double(i)/n) +
	         log(double(n-i)/(n-i-1))*double(rand())/RAND_MAX;
          
	v[n-1] = -log(1-double(n-1)/n) +
                 2*log(2.0)*double(rand())/RAND_MAX;


       random_shuffle(v.begin(),v.end());

       for(int i=0;i<n;i++)
          A[i][j] = v[i];
      }
  }

//return A;
}



//-----------------------------------------------------------------------------

double** lhs(int dim, int n, char* loi)
{
double** A = new double*[n];
vector<double> v(n,0);
time_t *adr= new time_t;
srand(int(time(adr)));
delete adr;
double a;

for(int i=0;i<n;i++)
   A[i] = new double[dim];

   for(int j=0;j<dim;j++)
    { 
      if(loi[j]=='l')
       {
       for(int i=0;i<n;i++)
          v[i] = -1 + 2*double(i)/n + (2./n)*double(rand())/RAND_MAX;
	}  
      else if(loi[j]=='e')
       {
       for(int i=0;i<n-1;i++)
          v[i] = -log(1-double(i)/n) +
	         log(double(n-i)/(n-i-1))*double(rand())/RAND_MAX;
          
	v[n-1] = -log(1-double(n-1)/n) +
                 2*log(2.0)*double(rand())/RAND_MAX;
       }
      else if(loi[j]=='u')
        {
	 for(int i=0;i<n-1;i++)
	    {
	    a = double(rand())/RAND_MAX;
	    v[i] = (1-a)*pow(10.0,(double(2*i)-n)/n) + a*pow(10.0,(double(2*(i+1))-n)/n);
	    }
	}
      
      
       random_shuffle(v.begin(),v.end());

       for(int i=0;i<n;i++)
          A[i][j] = v[i];
     }
  
return A;
}



//-----------------------------------------------------------------------------






double** lhs2(int dim, int n, char loi)
{
double** A = new double*[n];
int p,m,a;
double pj;
p = int(pow(2.,double(dim)));
m = n-p;
vector<double> v(m,0);
time_t *adr= new time_t;
srand(int(time(adr)));
delete adr;

for(int i=0;i<n;i++)
   A[i] = new double[dim];

if(loi == 'l')
  {
   for(int j=0;j<dim;j++)
      {
       for(int i=0;i<m;i++)
          v[i] = -1 + 2*double(i)/m + (2./m)*double(rand())/RAND_MAX;
	
       random_shuffle(v.begin(),v.end());

       for(int i=0;i<m;i++)
          A[i][j] = v[i];

       pj = pow(2.,double(j));
       a = 1;
       for(int i=0;i<p;i++)
          {
	   if(fmod(double(i),pj) == 0)
	     a = -a;
	   A[m+i][j] = a;
	  }
      }
  }   


return A;
}


//-----------------------------------------------------------------------------

double prod_scal(int n, double* u, double* v)
{
double ps = 0;
for(int i=0;i<n;i++)
   ps = ps + u[i]*v[i];

return ps;
}

//-----------------------------------------------------------------------------
double** build_sampling(int dim, int N,int n, char loi)
{
int p = 1;
for(int i=0;i<dim;i++)
   p = p*n;

double** xx = new double*[p];
for(int i=0;i<p;i++)
   xx[i] = new double[dim];

 xx = build_pool(dim,n,loi);
 //xx = lhs(dim,10000,loi);

vector<int>v1(p);
for(int i=0;i<p;i++)
   v1[i] = i;

time_t *adr= new time_t;
srand(int(time(adr)));
delete adr;

double** xxx = new double*[N];
for(int i=0;i<N;i++)
   xxx[i] = new double[dim];

int it = rand()%p ;
xxx[0] = xx[v1[it]];
v1.erase(v1.begin()+it);
double d;
double d1;
double d2;
double q;
if(loi == 'l')
{
for(int i=1;i<N;i++)
   {
   d = 0;
    for(int j=0;j<int(v1.size());j++)
       {
       d1 = 10000;
         for(int k=0;k<i;k++)
	   {
	    d2 = 0;
            for(int l=0;l<dim;l++)
	       {
               q = xx[v1[j]][l] - xxx[k][l];
	       d2 = d2 + q*q;
	       }
	    d2 = sqrt(d2);
	    if(d2<d1)
	      {
	       d1 = d2;
	      }
           }
	   if(d1>d)
	     {
	      d = d1;
              it = j;
	     }
       }

    xxx[i] = xx[v1[it]];
    v1.erase(v1.begin()+it);
   }
 }
 else if(loi == 'e')
   {
    for(int i=1;i<N;i++)
      {
       d = 0;
       for(int j=0;j<int(v1.size());j++)
         {
          d1 = 10000;
            for(int k=0;k<i;k++)
	      {
	       d2 = 0;
               for(int l=0;l<dim;l++)
	         {
                  q = fabs(exp(-xx[v1[j]][l]) - exp(-xxx[k][l]));
	          d2 = d2 + q*q;
	         }
	       d2 = sqrt(d2);
	       if(d2<d1)
	         {
	          d1 = d2;
	         }
               }
	     if(d1>d)
	       {
	        d = d1;
                it = j;
	       }
           }

      xxx[i] = xx[v1[it]];
      v1.erase(v1.begin()+it);
     }
   }

return xxx;
}



//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
double** build_sampling(double (*fct)(double*),int dim, int N,int n, char loi)
{
int p = 1;
for(int i=0;i<dim;i++)
   p = p*n;

double** xx = new double*[p];
for(int i=0;i<p;i++)
   xx[i] = new double[dim];

 xx = build_pool(dim,n,loi);
 //xx = lhs(dim,10000,loi);

vector<int>v1(p);
for(int i=0;i<p;i++)
   v1[i] = i;

time_t *adr= new time_t;
srand(int(time(adr)));
delete adr;

double** xxx = new double*[N];
for(int i=0;i<N;i++)
   xxx[i] = new double[dim];
   
double* y = new double[N];

int it = rand()%p ;
xxx[0] = xx[v1[it]];
v1.erase(v1.begin()+it);
y[0] = fct(xxx[0]);
double ymax = fabs(y[0]);
double d;
double d1;
double d2;
double q;
for(int i=1;i<N;i++)
   {
   d = 0;
    for(int j=0;j<int(v1.size());j++)
       {
       d1 = 10000;
         for(int k=0;k<i;k++)
	   {
	    d2 = 0;
            for(int l=0;l<dim;l++)
	       {
               q = xx[v1[j]][l] - xxx[k][l];
	       d2 = d2 + q*q;
	       }
	    q = y[k]*(double(dim))/(sqrt(double(i))*ymax);
            //q = y[k]/(ymax);
	    d2 = d2 + q*q;
	    d2 = sqrt(d2);
	    if(d2<d1)
	      {
	       d1 = d2;
	      }
           }
	   if(d1>d)
	     {
	      d = d1;
              it = j;
	     }
       }

    xxx[i] = xx[v1[it]];
    v1.erase(v1.begin()+it);
    y[i] = fct(xxx[i]);
     if(fabs(y[i])>ymax)
       {
        ymax = fabs(y[i]);
       }

   }

return xxx;
}



//-----------------------------------------------------------------------------



double** re_sampling(int dim, int Nr, int N,double** yy,
int n, char loi)
{
int p = 1;
for(int i=0;i<dim;i++)
   p = p*n;

double** xx = new double*[p];
for(int i=0;i<p;i++)
   xx[i] = new double[dim];

 xx = build_pool(dim,n,loi);
 //xx = lhs(dim,10000,loi);

vector<int>v1(p+1);
for(int i=0;i<p;i++)
   v1[i] = i;


double** xxx = new double*[Nr];
for(int i=0;i<Nr;i++)
   xxx[i] = new double[dim];

int it=-1;
double d;
double d1;
double d2;
double q;
for(int i=0;i<Nr;i++)
   {
   d = 0;
    for(int j=0;j<int(v1.size());j++)
       {
       d1 = 10000;
         for(int k=0;k<i;k++)
	   {
	    d2 = 0;
            for(int l=0;l<dim;l++)
	       {
               q = xx[v1[j]][l] - xxx[k][l];
	       d2 = d2 + q*q;
	       }
	    d2 = sqrt(d2);
	    if(d2<d1)
	      {
	       d1 = d2;
	      }
           }
          for(int k=0;k<N;k++)
	    {
	     d2 = 0;
             for(int l=0;l<dim;l++)
	        {
                q = xx[v1[j]][l] - yy[k][l];
	        d2 = d2 + q*q;
	        }
	     d2 = sqrt(d2);
	     if(d2<d1)
	       {
	        d1 = d2;
	       }
            }
	   if(d1>d)
	     {
	      d = d1;
              it = j;
	     }
       }

    xxx[i] = xx[v1[it]];
    v1.erase(v1.begin()+it);
   }

return xxx;
}


//-----------------------------------------------------------------------------



double** re_sampling(int dim, int Nr, int N,double** xx, double** yy, int n, char loi, int p)
{
    (void)(n); (void)(loi);
    /*int p = 1;
for(int i=0;i<dim;i++)
   p = p*n;*/

vector<int>v1(p+1);
for(int i=0;i<p;i++)
   v1[i] = i;

double** xxx = new double*[Nr];
for(int i=0;i<Nr;i++)
   xxx[i] = new double[dim];

int it=-1;
double d;
double d1;
double d2;
double q;


for(int i=0;i<Nr;i++)
   {
   d = 0;
    for(int j=0;j<int(v1.size());j++)
       {
       d1 = 10000;
         for(int k=0;k<i;k++)
	   {
	    d2 = 0;
            for(int l=0;l<dim;l++)
	       {
               q = xx[v1[j]][l] - xxx[k][l];
	       d2 = d2 + q*q;
	       }
	    d2 = sqrt(d2);
	    if(d2<d1)
	      {
	       d1 = d2;
	      }
           }
          for(int k=0;k<N;k++)
	    {
	     d2 = 0;     
             for(int l=0;l<dim;l++)
	        {
                q = xx[v1[j]][l] - yy[k][l];
	        d2 = d2 + q*q;
	        }
	     d2 = sqrt(d2);
	     if(d2<d1)
	       {
	        d1 = d2;
	       }
            }
	   if(d1>d)
	     {
	      d = d1;
              it = j;
	     }
       }

    xxx[i] = xx[v1[it]];
    v1.erase(v1.begin()+it);
   }

return xxx;
}




//-----------------------------------------------------------------------------


double dist_minimale(int dim, int N,double** xx, double* x,
 char loi)
{
double d;
double d1 = 1000;
double q;
for(int i=0;i<N;i++)
  {
  d = 0;
  if(loi == 'l'){              // distance euclidienne
  for(int j=0;j<dim;j++)
     {
      q = xx[i][j] - x[j];
      d = d + q*q;
     }
  d = sqrt(d);
  }
  
  if(d<d1)
      d1 = d;   
  }

return d1;
}




//-----------------------------------------------------------------------------






//-----------------------------------------------------------------------------

bool fct_comp(double* a, double* b)
{
bool c;
if(a[1]<b[1])
  c = true ;
else
  c = false ;

return c;
}

//---------------------------------------------------------------------------------------

double** build_pool(int dim, int n, char loi)
{
int p = 1;
for(int i=0;i<dim;i++)
   p = p*n;

double** xx = new double*[p];
for(int i=0;i<p;i++)
   xx[i] = new double[dim];

double* x = new double[n];
if(loi == 'l')
{
double q = 2./(n-1);
for(int i=0;i<n;i++)
  {
   x[i] = -1 + q*i;
  }
}
else if(loi == 'e')
  {
   for(int i=0;i<n;i++)
     {
      x[i] = -log(1-double(i)/n);
     }
  }


//int it = n;
for(int i=0;i<dim;i++)
   {
   for(int j=0;j<p;j++)
      {
        xx[j][i] = x[(j/int(pow(double(n),double(dim-i-1))))%n];
      }
      //it = it * n;
   }

return xx;
}

//-----------------------------------------------------------------------------

// void gauher(double* x, double* w, int n)//calcule les n points de quadrature de gauss-hermite (x) et les poids (w) 
// {
// 	void nrerror(char error_text[100]);
// 	int i,its,j,m;
// 	double p1,p2,p3,pp,z,z1;
// 
// 	m=(n+1)/2;
// 	for (i=1;i<=m;i++) {
// 		if (i == 1) {
// 			z=sqrt((double)(2*n+1))-1.85575*pow((double)(2*n+1),-0.16667);
// 		} else if (i == 2) {
// 			z -= 1.14*pow((double)n,0.426)/z;
// 		} else if (i == 3) {
// 			z=1.86*z-0.86*x[0];
// 		} else if (i == 4) {
// 			z=1.91*z-0.91*x[1];
// 		} else {
// 			z=2.0*z-x[i-3];
// 		}
// 		for (its=1;its<=MAXIT;its++) {
// 			p1=PIM4;
// 			p2=0.0;
// 			for (j=1;j<=n;j++) {
// 				p3=p2;
// 				p2=p1;
// 				p1=z*sqrt(2.0/j)*p2-sqrt(((double)(j-1))/j)*p3;
// 			}
// 			pp=sqrt((double)2*n)*p2;
// 			z1=z;
// 			z=z1-p1/pp;
// 			if (fabs(z-z1) <= EPS) break;
// 		}
// 		char err_mess[100] = "too many iterations in gauher";
// 		if (its > MAXIT) nrerror(err_mess);
// 		x[i-1]=z;
// 		x[n-i] = -z;
// 		w[i-1]=2.0/(pp*pp);
// 		w[n-i]=w[i-1];
// 	}
// }


//-----------------------------------------------------------------------------

// void gaujac(double* x, double* w, int n, double alf, double bet) //calcule les n points de quadrature de gauss-jacobi (x) et les poids (w) avec les parametre alf et bet
// {
// 	float gammln(double xx);
// 	void nrerror(char error_text[100]);
// 	int i,its,j;
// 	double alfbet,an,bn,r1,r2,r3;
// 	double a,b,c,p1,p2,p3,pp,temp,z,z1;
// 
// 	for (i=1;i<=n;i++) {
// 		if (i == 1) {
// 			an=alf/n;
// 			bn=bet/n;
// 			r1=(1.0+alf)*(2.78/(4.0+n*n)+0.768*an/n);
// 			r2=1.0+1.48*an+0.96*bn+0.452*an*an+0.83*an*bn;
// 			z=1.0-r1/r2;
// 		} else if (i == 2) {
// 			r1=(4.1+alf)/((1.0+alf)*(1.0+0.156*alf));
// 			r2=1.0+0.06*(n-8.0)*(1.0+0.12*alf)/n;
// 			r3=1.0+0.012*bet*(1.0+0.25*fabs(alf))/n;
// 			z -= (1.0-z)*r1*r2*r3;
// 		} else if (i == 3) {
// 			r1=(1.67+0.28*alf)/(1.0+0.37*alf);
// 			r2=1.0+0.22*(n-8.0)/n;
// 			r3=1.0+8.0*bet/((6.28+bet)*n*n);
// 			z -= (x[0]-z)*r1*r2*r3;
// 		} else if (i == n-1) {
// 			r1=(1.0+0.235*bet)/(0.766+0.119*bet);
// 			r2=1.0/(1.0+0.639*(n-4.0)/(1.0+0.71*(n-4.0)));
// 			r3=1.0/(1.0+20.0*alf/((7.5+alf)*n*n));
// 			z += (z-x[n-4])*r1*r2*r3;
// 		} else if (i == n) {
// 			r1=(1.0+0.37*bet)/(1.67+0.28*bet);
// 			r2=1.0/(1.0+0.22*(n-8.0)/n);
// 			r3=1.0/(1.0+8.0*alf/((6.28+alf)*n*n));
// 			z += (z-x[n-3])*r1*r2*r3;
// 		} else {
// 			z=3.0*x[i-2]-3.0*x[i-3]+x[i-4];
// 		}
// 		alfbet=alf+bet;
// 		for (its=1;its<=MAXIT;its++) {
// 			temp=2.0+alfbet;
// 			p1=(alf-bet+temp*z)/2.0;
// 			p2=1.0;
// 			for (j=2;j<=n;j++) {
// 				p3=p2;
// 				p2=p1;
// 				temp=2*j+alfbet;
// 				a=2*j*(j+alfbet)*(temp-2.0);
// 				b=(temp-1.0)*(alf*alf-bet*bet+temp*(temp-2.0)*z);
// 				c=2.0*(j-1+alf)*(j-1+bet)*temp;
// 				p1=(b*p2-c*p3)/a;
// 			}
// 			pp=(n*(alf-bet-temp*z)*p1+2.0*(n+alf)*(n+bet)*p2)/(temp*(1.0-z*z));
// 			z1=z;
// 			z=z1-p1/pp;
// 			if (fabs(z-z1) <= EPS) break;
// 		}
// 		char err_mess[100] = "too many iterations in gaujac";
// 		if (its > MAXIT) nrerror(err_mess);
// 		x[i-1]=z;
// 		w[i-1]=exp(gammln(alf+n)+gammln(bet+n)-gammln(n+1.0)-
// 			gammln(n+alfbet+1.0))*temp*pow(2.0,alfbet)/(pp*p2);
// 	}
// }

//-----------------------------------------------------------------------------

// float gammln(double xx)
// {
// 	double x,y,tmp,ser;
// 	static double cof[6]={76.18009172947146,-86.50532032941677,
// 		24.01409824083091,-1.231739572450155,
// 		0.1208650973866179e-2,-0.5395239384953e-5};
// 	int j;
// 
// 	y=x=xx;
// 	tmp=x+5.5;
// 	tmp -= (x+0.5)*log(tmp);
// 	ser=1.000000000190015;
// 	for (j=0;j<=5;j++) ser += cof[j]/++y;
// 	return -tmp+log(2.5066282746310005*ser/x);
// }

//-----------------------------------------------------------------------------

// void gaulag(double* x, double* w, int n, double alf)//calcule les n points de quadrature de gauss-laguerre (x) et les poids (w) pour le param�tre
// {
// 	float gammln(double xx);
// 	void nrerror(char error_text[100]);
// 	int i,its,j;
// 	float ai;
// 	double p1,p2,p3,pp,z,z1;
// 
// 	for (i=1;i<=n;i++) {
// 		if (i == 1) {
// 			z=(1.0+alf)*(3.0+0.92*alf)/(1.0+2.4*n+1.8*alf);
// 		} else if (i == 2) {
// 			z += (15.0+6.25*alf)/(1.0+0.9*alf+2.5*n);
// 		} else {
// 			ai=i-2;
// 			z += ((1.0+2.55*ai)/(1.9*ai)+1.26*ai*alf/
// 				(1.0+3.5*ai))*(z-x[i-3])/(1.0+0.3*alf);
// 		}
// 		for (its=1;its<=MAXIT;its++) {
// 			p1=1.0;
// 			p2=0.0;
// 			for (j=1;j<=n;j++) {
// 				p3=p2;
// 				p2=p1;
// 				p1=((2*j-1+alf-z)*p2-(j-1+alf)*p3)/j;
// 			}
// 			pp=(n*p1-(n+alf)*p2)/z;
// 			z1=z;
// 			z=z1-p1/pp;
// 			if (fabs(z-z1) <= EPS) break;
// 		}
// 		char err_mess[100] = "too many iterations in gaulag";
// 		if (its > MAXIT) nrerror(err_mess);
// 		x[i-1]=z;
// 		w[i-1] = -exp(gammln(alf+n)-gammln((float)n))/(pp*n*p2);
// 	}
// }
// 


//-----------------------------------------------------------------------------

// void gauleg(double x1, double x2, double* x, double* w, int n) //calcule les n points de quadrature de gauss-legendre (x) et les poids (w) pour un int�gration entre x1 et x2
// {
// 	int m,j,i;
// 	double z1,z,xm,xl,pp,p3,p2,p1;
// 
// 	m=(n+1)/2;
// 	xm=0.5*(x2+x1);
// 	xl=0.5*(x2-x1);
// 	for (i=1;i<=m;i++) {
// 		z=cos(3.141592654*(i-0.25)/(n+0.5));
// 		do {
// 			p1=1.0;
// 			p2=0.0;
// 			for (j=1;j<=n;j++) {
// 				p3=p2;
// 				p2=p1;
// 				p1=((2.0*j-1.0)*z*p2-(j-1.0)*p3)/j;
// 			}
// 			pp=n*(z*p1-p2)/(z*z-1.0);
// 			z1=z;
// 			z=z1-p1/pp;
// 		} while (fabs(z-z1) > EPS);
// 		x[i-1]=xm-xl*z;
// 		x[n-i]=xm+xl*z;
// 		w[i-1]=2.0*xl/((1.0-z*z)*pp*pp);
// 		w[n-i]=w[i-1];
// 	}
// }

//-----------------------------------------------------------------------------

/*
 * The inverse standard normal distribution.
 *
 *   Author:      Peter J. Acklam <pjacklam@online.no>
 *   URL:         http://home.online.no/~pjacklam
 *
 * This function is based on the MATLAB code from the address above,
 * translated to C, and adapted for our purposes.
 */
// double stdnormal_inv(double p)
// {
//  const double a[6] = {
//   -3.969683028665376e+01,  2.209460984245205e+02,
//   -2.759285104469687e+02,  1.383577518672690e+02,
//   -3.066479806614716e+01,  2.506628277459239e+00
//  };
//  const double b[5] = {
//   -5.447609879822406e+01,  1.615858368580409e+02,
//   -1.556989798598866e+02,  6.680131188771972e+01,
//   -1.328068155288572e+01
//  };
//  const double c[6] = {
//   -7.784894002430293e-03, -3.223964580411365e-01,
//   -2.400758277161838e+00, -2.549732539343734e+00,
//    4.374664141464968e+00,  2.938163982698783e+00
//  };
//  const double d[4] = {
//    7.784695709041462e-03,  3.224671290700398e-01,
//    2.445134137142996e+00,  3.754408661907416e+00
//  };
// 
//  register double q, t, u;
// 
//  if (isnan(p) || p > 1.0 || p < 0.0)
//   exception();//return NaN;
//  if (p == 0.0)
//   exception();//return nan;
//  if (p == 1.0)
//   exception();//return  _nan;
//  q = DMIN(p,1-p);
//  if (q > 0.02425) {
//   /* Rational approximation for central region. */
//   u = q-0.5;
//   t = u*u;
//   u = u*(((((a[0]*t+a[1])*t+a[2])*t+a[3])*t+a[4])*t+a[5])
//     /(((((b[0]*t+b[1])*t+b[2])*t+b[3])*t+b[4])*t+1);
//  } else {
//   /* Rational approximation for tail region. */
//   t = sqrt(-2*log(q));
//   u = (((((c[0]*t+c[1])*t+c[2])*t+c[3])*t+c[4])*t+c[5])
//    /((((d[0]*t+d[1])*t+d[2])*t+d[3])*t+1);
//  }
//  /* The relative error of the approximation has absolute value less
//     than 1.15e-9.  One iteration of Halley's rational method (third
//     order) gives full machine precision... */
//  t = erf(u)-q;    /* error */
//  t = t*M_SQRT2PI*exp(u*u/2);   /* f(u)/df(u) */
//  u = u-t/(1+u*t/2);     /* Halley's method */
// 
//  return (p > 0.5 ? -u : u);
// };

//-----------------------------------------------------

double exp_inv(double x)
{
  return -log((1-x)/2.0);
}


//-----------------------------------------------------------------------------

void clenshaw_curtis(double* x, double* w, int l) //calcule les n points de quadrature de clenshaw-curtis (x) et les poids (w)
{
double n,m;
double pi;
if(l==1)
  {
    x[0]=0; w[0]=2;
  }
else{
    n = pow(2.0,l-1.0) +1.0;
    m = n-1;
    //pi = 4*atan(1);
    pi = 3.141592653589793238462643383;
    
    for(int i=1;i<=n;i++)
      x[i-1] = -cos( pi*(i-1.0)/m );
    
    w[0] = 1.0/(n*(n-2));
    w[int(m)] = w[0];
    
    for(int i=1;i<m;i++)
       {
        w[i] = 1;
	for(int j=1;j<m/2;j++)
	   {
	    w[i] = w[i] + (2.0/(1.0-4*j*j))* cos(2*pi*j*(i)/m );
	   }
	w[i] = w[i] + (1.0/(1.0-m*m))* cos(2*pi*(m/2.0)*(i)/m); 
	w[i] = w[i]*2.0/m;  
       }        

   }
} 


//-----------------------------------------------------------------------------

void fejer2(double* x, double* w, int l) //calcule les n points de quadrature de clenshaw-curtis (x) et les poids (w)
{
double n,m,a,b;
double pi;
if(l==1)
  {
    x[0]=0; w[0]=2;
  }
else{
    n = pow(2.0,l+0.0) -2.0;
    m = n+2.0;
    //pi = 4*atan(1);
    pi = 3.141592653589793238462643383;
    a = pi/m;
    
    for(int i=0;i<=n;i++)
      x[i] = cos( a*(i+1.0));
    
    for(int i=0;i<=n;i++)
       {
        b = a*(i+1.0); 
        w[i] = sin(b);
	for(int j=2;j<=m/2;j++)
	   {
	    w[i] = w[i] +  sin((2*j-1.0)*b)/(2*j-1.0);
	   }
	w[i] = w[i]*sin(b)*4.0/m;  
       }        

   }
} 


//-----------------------------------------------------------------------------

void fejer1(double* x, double* w, int l) //calcule les n points de quadrature de clenshaw-curtis (x) et les poids (w)
{
double n;
double pi;
if(l==2)
  {
    x[0]=0; w[0]=2;
  }
else{
    n = pow(2.0,l-1.0);
    //pi = 4*atan(1);
    pi = 3.141592653589793238462643383;
    
    for(int i=0;i<n;i++)
      x[i] = cos( pi*(i+0.5)/n );
    
    for(int i=0;i<n;i++)
       {
        w[i] = 1;
	for(int j=1;j<=n/2;j++)
	   {
	    w[i] = w[i] - 2*cos(j*(2*i+1.5)*pi/n)/(4*j*j-1.0);
	   }
	w[i] = w[i]*2.0/n;  
       }        

   }
} 

//-----------------------------------------------------------------------------





//-----------------------------------------------------------------------------

void smolyak(int d, int l,char fich[13] )
{

int nld; // nombre de points de la formule
double ** Delta; // tableau des formules de diff�rences 1d
double ** q; // formule de quadrature 1d;
double ** Q; // formule de quadrature de smolyak

int n1d = int(pow(2.0,l+0.0)) -1;

Delta = new double*[n1d];
for(int i=0;i<n1d;i++)
   Delta[i] = new double[l+1];
   
q = new double*[2];
q[0] = new double[n1d]; 
q[1] = new double[n1d];

//-----calcul du nombre de points de la formule-------------------

nld = 0;

if(d-1<l-1){
  for(int j=0;j<l;j++)
    {
     nld = nld + fact_partiel(j,j+d-1)*int(pow(2.0,j+0.0))/fact(d-1);  
    } 
  }
else{
  for(int j=0;j<l;j++)
    {
     nld = nld + fact_partiel(d-1,j+d-1)*int(pow(2.0,j+0.0))/fact(j);  
    } 
  }  

cout<<" nombre de points de la formule : "<<nld<<endl;  
  
//-----fin du calcul du nombre de points -------------------------

Q = new double*[nld];
for(int i=0;i<nld;i++)
  Q[i] = new double[d+1];
  
if(d==1)
  {
   fejer2(q[0],q[1],l) ;
   for(int i=0;i<nld;i++)
      {
       Q[i][0] = q[0][i];
       Q[i][1] = 0.5*q[1][i];
      }
  }  
else{    
	 
//-------calcul de npt ----------------------------------------	 

	 int N = calculer_npol(l-1,d);
	 int ** npt; npt = new int*[N+1];
         int n;
	 int isum;
	 int IP;
	 int * ic;
	 ic = new int[d];
	 int * ict;
	 ict = new int[d];  
	 
	 for(int i=0;i<=N;i++)
	    npt[i] = new int[d];
	
	 n=0;
	 for(int i=0;i<=N;i++){
	    for(int j=0;j<d;j++)
	      npt[i][j] = 0;
	 }
	 
	 
	 for(int i=0;i<d;i++)
	 npt[0][i] = 0;
         
	 if(l-1!=0)
	   {
	    for(int i=0;i<d;i++)
	      {
	      n = n + 1;
	      npt[n][i] = 1;
	      ic[i] = 1;
	      }
	      
	    if(l-1!=1)
	      {
	      for(int k=1;k<l-1;k++)
	         {
		 
		  IP = n;
		  for(int j=0;j<d;j++){
		     isum = 0;		     
		     for(int l=d-1;l>=j;l--)
		     isum = isum + ic[l];		     
		     ict[j] = isum;		     
		     } 
		   for(int j=0;j<d;j++)
		   ic[j] = ict[j];		   
		   for(int jj=0;jj<d;jj++){
		       
		        for(int j=IP-ic[jj];j<IP;j++){
			     n = n+1;
			     for(int i=0;i<d;i++){
			     npt[n][i] = npt[j+1][i];
			     
			     }
			   
			     npt[n][jj] = npt[n][jj] + 1;
		 
			   }
			
		      }
		  
	         } 
		 
             }

        }
for(int i=0;i<=N;i++)
  {
   for(int j=0;j<d;j++)
     {
      npt[i][j] = npt[i][j] +1;
     } 
   }
//-------fin du calcul de npt ----------------------   

//--------calcul de Delta ----------------------------

for(int i=0;i<n1d;i++)
  {
    for(int j=0;j<l+1;j++)
       Delta[i][j] = 0;
  }     

for(int k=1;k<l;k++)
  {
   fejer2(q[0],q[1],k);
   
   for(int r=0;r<int(pow(2.0,k+0.0))-1;r++)
     {
      Delta[int(pow(2.0,l-k+0.0))*r+int(pow(2.0,l-k+0.0))-1][k] = Delta[int(pow(2.0,l-k+0.0))*r+int(pow(2.0,l-k+0.0))-1][k] + 0.5*q[1][r] ;
      
      Delta[int(pow(2.0,l-k+0.0))*r+int(pow(2.0,l-k+0.0))-1][k+1]= Delta[int(pow(2.0,l-k+0.0))*r+int(pow(2.0,l-k+0.0))-1][k+1] - 0.5*q[1][r] ;
     }
     
  }
  
  fejer2(q[0],q[1],l);  
  for(int r=0;r<int(pow(2.0,l+0.0))-1;r++)
    {
     Delta[r][l] = Delta[r][l] + 0.5*q[1][r];
     Delta[r][0] = q[0][r];
    }
  
//--------fin du calcul de Delta --------------------------

//------- calcul de Q ----------------------------------------

int ** pt = new int*[nld];
for(int i=0;i<nld;i++)
  {
   pt[i] = new int[d];
   for(int j=0;j<d;j++)
    {
     pt[i][j] = -1;
    } 
  }


for(int i=0;i<nld;i++)
   Q[i][d] = 0;
   
int ind = 0;
int nid;
int p;
int a;
int b;
int aa;
int bb;
int it;
double* K = new double[d+1];
int* K1 = new int[d];

for(int i=0;i<N+1;i++)
  {    
   nid = 1;
   for(int j=0;j<d;j++)
     {
      nid = nid*(int(pow(2.0,npt[i][j]+0.0))-1); 
     } 
       
   for(int j=0;j<nid;j++)
     {
      K[d]=1; 
      p = nid;
      for(int k=0;k<d;k++)
        {
	 a = npt[i][k];
	 b = int(pow(2.0,a+0.0))-1;
	 p = p/(b);
	 aa = int(pow(2.0,l-a+0.0))*( (j/p)%b +1)-1;
	 K1[k] = aa;
	 K[k] = Delta[aa][0];
	 K[d] = K[d] * Delta[aa][a];
	}
    
      bb=0;
      it=-1;
      while(it<ind && bb==0)
        {
	bb = 1; 
	it = it+1;
         for(int k=0;k<d;k++)
           {
	    if(pt[it][k]!=K1[k])
	     {
	      bb = 0; 
	      break;
	     } 
	   }
	  
	 }
    	
	 if(it<ind)
	   {    
	    Q[it][d] = Q[it][d] + K[d]; 
	   }
	 else{
	    for(int k=0;k<d;k++)
	      {
	       pt[ind][k] = K1[k];
	       Q[ind][k] = K[k];
	      } 
	    Q[ind][d] = K[d];   
	    if(ind<nld)	   
	       ind = ind + 1;  
	   }  
     }   
  }   

//-------fin du calcul de Q ----------------------------------- 
 
}
 
//char nomfich[12] = "quad.dat";
//ofstream quad (nomfich,ios::out);
ofstream quad (fich,ios::out);

//quad<<"# nombre de dimensions : "<<d<<endl;
//quad<<"# niveau de la formule : "<<l<<endl;
//quad<<"# nombre de points de la formule : "<<nld<<endl;
quad<<nld<<endl;
quad<<endl;
quad<<endl;
for(int i=0;i<nld;i++)
   {
    for(int j=0;j<d+1;j++)
      quad<<setprecision(20)<<Q[i][j]<<"   ";
    quad<<endl; 
   }

   
   
}

//-----------------------------------------------------------------------------

void smolyak_CC(int d, int l,char fich[13] )
{

int nld; // nombre de points de la formule
double ** Delta; // tableau des formules de diff�rences 1d
double ** q; // formule de quadrature 1d;
double ** Q; // formule de quadrature de smolyak

int n1d;
if(l!=1)
  n1d = int(pow(2.0,l-1.0)) +1;
else n1d = 1;

Delta = new double*[n1d];
for(int i=0;i<n1d;i++)
   Delta[i] = new double[l+1];
   
q = new double*[2];
q[0] = new double[n1d]; 
q[1] = new double[n1d];


	 
//-------calcul de npt ----------------------------------------	 

	 int N = calculer_npol(l-1,d);
	 int ** npt; npt = new int*[N+1];
         int n;
	 int isum;
	 int IP;
	 int * ic;
	 ic = new int[d];
	 int * ict;
	 ict = new int[d];  
	 
	 for(int i=0;i<=N;i++)
	    npt[i] = new int[d];
	
	 n=0;
	 for(int i=0;i<=N;i++){
	    for(int j=0;j<d;j++)
	      npt[i][j] = 0;
	 }
	 
	 
	 for(int i=0;i<d;i++)
	 npt[0][i] = 0;
         
	 if(l-1!=0)
	   {
	    for(int i=0;i<d;i++)
	      {
	      n = n + 1;
	      npt[n][i] = 1;
	      ic[i] = 1;
	      }
	      
	    if(l-1!=1)
	      {
	      for(int k=1;k<l-1;k++)
	         {
		 
		  IP = n;
		  for(int j=0;j<d;j++){
		     isum = 0;		     
		     for(int l=d-1;l>=j;l--)
		     isum = isum + ic[l];		     
		     ict[j] = isum;		     
		     } 
		   for(int j=0;j<d;j++)
		   ic[j] = ict[j];		   
		   for(int jj=0;jj<d;jj++){
		       
		        for(int j=IP-ic[jj];j<IP;j++){
			     n = n+1;
			     for(int i=0;i<d;i++){
			     npt[n][i] = npt[j+1][i];
			     
			     }
			   
			     npt[n][jj] = npt[n][jj] + 1;
		 
			   }
			
		      }
		  
	         } 
		 
             }

        }
for(int i=0;i<=N;i++)
  {
   for(int j=0;j<d;j++)
     {
      npt[i][j] = npt[i][j] +1;
     } 
   }
//-------fin du calcul de npt ----------------------   

//-----calcul du nombre de points de la formule-------------------

nld = 0;
int nj;
if(d>1){
for(int i=0;i<N+1;i++)
  {
   nj=1;
   for(int j=0;j<d;j++)
     {
      if(npt[i][j]==2)
        nj = nj*2;
      else if(npt[i][j]>2)                                
             { nj = nj*int(pow(2.0,npt[i][j]-2.0));} 
     }
   nld = nld + nj;  
  }
}  
else nld = int(pow(2.0,l-1.0))+1;
cout<<" nombre de points de la formule : "<<nld<<endl;  
  
//-----fin du calcul du nombre de points -------------------------

Q = new double*[nld];
for(int i=0;i<nld;i++)
  Q[i] = new double[d+1];

if(d==1)
  {
   fejer2(q[0],q[1],l) ;
   for(int i=0;i<nld;i++)
      {
       Q[i][0] = q[0][i];
       Q[i][1] = 0.5*q[1][i];
      }
  }  
else{    
  
//--------calcul de Delta ----------------------------

for(int i=0;i<n1d;i++)
  {
    for(int j=0;j<l+1;j++)
       Delta[i][j] = 0;
  }     

for(int k=1;k<l;k++)
  {
   clenshaw_curtis(q[0],q[1],k);
   
   for(int r=0;r<int(pow(2.0,k-1.0))+1;r++)
     {
      Delta[int(pow(2.0,l-k+0.0))*r][k] = Delta[int(pow(2.0,l-k+0.0))*r][k] + 0.5*q[1][r] ;
      
      Delta[int(pow(2.0,l-k+0.0))*r][k+1]= Delta[int(pow(2.0,l-k+0.0))*r][k+1] - 0.5*q[1][r] ;
     }
     
  }
  
  clenshaw_curtis(q[0],q[1],l);  
  for(int r=0;r<int(pow(2.0,l-1.0))+1;r++)
    {
     Delta[r][l] = Delta[r][l] + 0.5*q[1][r];
     Delta[r][0] = q[0][r];
    }
  
//--------fin du calcul de Delta --------------------------

//------- calcul de Q ----------------------------------------

int ** pt = new int*[nld];
for(int i=0;i<nld;i++)
  {
   pt[i] = new int[d];
   for(int j=0;j<d;j++)
    {
     pt[i][j] = -1;
    } 
  }


for(int i=0;i<nld;i++)
   Q[i][d] = 0;
   
int ind = 0;
int nid;
int p;
int a;
int b;
int aa;
int bb;
int it;
double* K = new double[d+1];
int* K1 = new int[d];

for(int i=0;i<N+1;i++)
  {    
   nid = 1;
   for(int j=0;j<d;j++)
     {
      if(npt[i][j]>1)
         nid = nid*(int(pow(2.0,npt[i][j]-1.0))+1); 
     }  
   for(int j=0;j<nid;j++)
     {
      K[d]=1; 
      p = nid;
      for(int k=0;k<d;k++)
        {
	 a = npt[i][k];
	 if(a>1)
	    b = int(pow(2.0,a-1.0))+1;
	 else b=1;   
	 p = p/b;
	 aa = int(pow(2.0,l-a+0.0))*((j/p)%b);
	 K1[k] = aa;
	 K[k] = Delta[aa][0];
	 K[d] = K[d] * Delta[aa][a];
	}
     
      bb=0;
      it=-1;
      while(it<ind && bb==0)
        {
	bb = 1; 
	it = it+1;
         for(int k=0;k<d;k++)
           {
	    if(pt[it][k]!=K1[k])
	     {
	      bb = 0; 
	      break;
	     } 
	   }
	  
	 }
    	
	 if(it<ind)
	   {    
	    Q[it][d] = Q[it][d] + K[d]; 
	   }
	 else{
	    for(int k=0;k<d;k++)
	      {
	       pt[ind][k] = K1[k];
	       Q[ind][k] = K[k];
	      } 
	    Q[ind][d] = K[d];   
	    if(ind<nld)	   
	       ind = ind + 1;  
	   }  
     }   
  }   

//-------fin du calcul de Q ----------------------------------- 
 
}
 
//char nomfich[12] = "quad.dat";
//ofstream quad (nomfich,ios::out);
 ofstream quad (fich,ios::out);

//quad<<"# nombre de dimensions : "<<d<<endl;
//quad<<"# niveau de la formule : "<<l<<endl;
//quad<<"# nombre de points de la formule : "<<nld<<endl;
quad<<nld<<endl;
quad<<endl;
quad<<endl;
for(int i=0;i<nld;i++)
   {
    for(int j=0;j<d+1;j++)
      quad<<setprecision(20)<<Q[i][j]<<"   ";
    quad<<endl; 
   }  
   
}

//-----------------------------------------------------------------------------

// void polint(double* x1, double* y1, int n, double x, double* y)
// {
//  int ns=1;
//  double den, dif, dift, ho, hp, w;
//  double* c = new double[n+1];
//  double* d = new double[n+1];
// double* xa = new double[n+1];
//  double* ya = new double[n+1];
// for(int i=1;i<=n;i++)
//   {
//    xa[i] = x1[i-1];
//    ya[i] = y1[i-1];
//   }
// 
//  dif = fabs(x-xa[1]);
//  
//  for(int i=1;i<=n;i++)
//    {
//     dift=fabs(x-xa[i]);
//     if( dift < dif)
//       {
//        ns = i;
//        dif = dift;
//       }
//     c[i] = ya[i];
//     d[i] = ya[i];
//    }
//  
//  *y = ya[ns--];
//   
//  for(int m=1;m<n;m++)
//    {
//     for(int i=1;i<=n-m;i++)
//       {
//        ho = xa[i]-x;
//        hp = xa[i+m]-x;
//        w = c[i+1] - d[i];
//        den = ho-hp;
//        if( den == 0.0) 
//          {
//           cout<<"Error in routine polint"<<endl;
//           exit(1);
//          }
//        den = w/den;
//        d[i] = hp*den;
//        c[i] = ho*den; 
//       } 
//     *y += (2*ns < (n-m) ? c[ns+1] : d[ns--]);
//    } 
//   delete d;
//   delete c;
// }

//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------

double* chgt_var(double * x, int d, char * loi)
{
double* y = new double[d];

if(d==1)
 y[0] = chgt_var(x[0],d,loi[0]);
else{

for(int i=0;i<d;i++)
  {
   if(loi[i] == 'u')
     y[i] = x[i];
   else{
     if(loi[i] == 'n')
       y[i] = InvFcn((x[i]+1.0)/2.0);
     else{
        if(loi[i] == 'e')
          y[i] = -log((1-x[i])/2.0);
       } 
    }
    
  } 

}

return y;
}

//-----------------------------------------------

double chgt_var(double x, int d, char loi)
{
double y=0.;
(void)(d);

   if(loi == 'u')
     y = x;
   else{
     if(loi == 'n')
       y = InvFcn((x+1.0)/2.0);
     else{
        if(loi == 'e')
          y = -log((1-x)/2.0);
       } 
    }
   
  
return y;
}





//-----------------------------------------------------------------------------

// void vander(double* x, double* w, double* q, int n)
// {
// 	int i,j,k,k1;
// 	double b,s,t,xx;
// 	double* c;
// 
// 	c = new double[n];
// 	if (n == 1) w[0]=q[0];
// 	else {
// 		for (i=1;i<=n;i++) c[i-1]=0.0;
// 		c[n-1] = -x[0];
// 		for (i=2;i<=n;i++) {
// 			xx = -x[i-1];
// 			for (j=(n+1-i);j<=(n-1);j++) c[j-1] += xx*c[j];
// 			c[n-1] += xx;
// 		}
// 		for (i=1;i<=n;i++) {
// 			xx=x[i-1];
// 			t=b=1.0;
// 			s=q[n-1];
// 			k=n;
// 			for (j=2;j<=n;j++) {
// 				k1=k-1;
// 				b=c[k-1]+xx*b;
// 				s += q[k1-1]*b;
// 				t=xx*t+b;
// 				k=k1;
// 			}
// 			w[i-1]=s/t;
// 		}
// 	}
// 	delete c;
// }


//-----------------------------------------------------------------------------

void pause(double t)
{

time_t start;
time_t current;

time(&start);
do{
time(&current);
}while(difftime(current,start) < t);

}



//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------

double* prod(double a, double b)
{
double rh,rl,x,xh,xl,y,yh,yl;
double* r = new double[2];
int s = 27;
int q = 1;

for(int i=0;i<s;i++)
   q = q*2;
q = q + 1;

x = a*q;
xh = (a-x)+x;
xl = a-xh;

y = b*q;
yh = (b-y)+y;
yl = b-yh;

rh = a*b;
rl = (((xh*yh-rh)+xh*yl)+xl*yh)+xl*yl;

r[0] = rh;
r[1] = rl;

return r;
}

//-----------------------------------------------------------------------------

double inv(double b)
{
double x,xx;
x = 1/(2*b);
for(int i=0;i<12;i++)
  {
   xx = x*(2-b*x);
   x = xx;
  }

return x;
}


//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------

void quadrature_normale(double* x, double* w, int l)
{// l<=6
if(l>6)
 {
  cout<<" l trop grand "<<endl;
  exit(1);
 }
else
 {
 int m = int(pow(2.0,6.0-l));
 int nl = int(pow(2.0,l+0.0))-1;
 double a,b;
 a = sqrt(2.0);
 b =  1.0/sqrt(4*atan(1.0));
 double** Q = new double*[63];
 for(int i=0;i<63;i++)
  { 
   Q[i] = new double[7];
   for(int j=0;j<7;j++)
     Q[i][j] = 0;
  }

   Q[0][0] = 0.11332306972524087E+02;
   Q[0][6] = 0.76469084197984854E-43;
   Q[1][0] = 0.98669848158693654E+01;  
   Q[1][6] = -0.98780064563723156E-38;
   Q[1][5] = -0.16215806876014959E-25;
   Q[2][0] = 0.89192128292673090E+01;  
   Q[2][6] = 0.62575068789043616E-34;
   Q[3][0] = 0.81946859372550751E+01;  
   Q[3][6] = 0.44347090468179334E-29;
   Q[3][5] = -0.26235488314176124E-22;
   Q[3][4] = 0.11046855508033972E-08;
   Q[4][0] = 0.75967611847999832E+01; 
   Q[4][6] = 0.47670840466677874E-25;
   Q[5][0] = 0.70809577848465306E+01;  
   Q[5][6] = 0.81134454290031239E-22;
   Q[5][5] = 0.11699976050427028E-19;
   Q[6][0] = 0.66229208492676035E+01;  
   Q[6][6] = 0.38781336628362138E-19;
   Q[7][0] = 0.62077875657087676E+01;  
   Q[7][6] = 0.72913045294202246E-17;
   Q[7][5] = 0.13619852169858379E-16;
   Q[7][4] = -0.11839463761824145E-06;
   Q[7][3] = 0.26307521119969726E-03;
   Q[8][0] = 0.58257823262764052E+01; 
   Q[8][6] = 0.66969107027577369E-15;
   Q[9][0] = 0.54701110501540784E+01;   
   Q[9][6] = 0.34824396741849340E-13;
   Q[9][5] = 0.69680868513084190E-13;
   Q[10][0] = 0.51358470189242444E+01;  
   Q[10][6] = 0.11385718036108109E-11;
   Q[11][0] = 0.48192946553014941E+01;  
   Q[11][6] = 0.25280787679865530E-10;
   Q[11][5] = 0.50561012461100265E-10;
   Q[11][4] = 0.28972232360498436E-05;
   Q[12][0] = 0.45176039332373916E+01;  
   Q[12][6] = 0.40410215118152162E-09;
   Q[13][0] = 0.42285252142956615E+01;  
   Q[13][6] = 0.48645225003442528E-08;
   Q[13][5] = 0.97290510877570424E-08;
   Q[14][0] = 0.39502470903230296E+01;  
   Q[14][6] = 0.45694494841131884E-07;
   Q[15][0] = 0.36812855105782880E+01;  
   Q[15][6] = 0.34459424533140665E-06;
   Q[15][5] = 0.68918844742336551E-06;
   Q[15][4] = -0.29591636416842310E-04;
   Q[15][3] = -0.73848821532446641E-02;
   Q[15][2] = 0.32697607222317751E-01;
   Q[16][0] = 0.34204057947880262E+01;  
   Q[16][6] = 0.21349274133661362E-05;
   Q[17][0] = 0.31665664130326094E+01;  
   Q[17][6] = 0.11073747996883468E-04;
   Q[17][5] = 0.22147496243807775E-04;
   Q[18][0] = 0.29188775713880926E+01;  
   Q[18][6] = 0.48845979884032087E-04;
   Q[19][0] = 0.26765701097522370E+01;  
   Q[19][6] = 0.18562581305924723E-03;
   Q[19][5] = 0.37125162547351105E-03;
   Q[19][4] = 0.92266892334255158E-03;
   Q[20][0] = 0.24389717319444255E+01; 
   Q[20][6] = 0.61440870419948918E-03;
   Q[21][0] = 0.22054885440752847E+01;  
   Q[21][6] = 0.17875789013035665E-02;
   Q[21][5] = 0.35751578076493149E-02;
   Q[22][0] = 0.19755904965155668E+01;  
   Q[22][6] = 0.46069463066549365E-02;
   Q[23][0] = 0.17487997352598685E+01;  
   Q[23][6] = 0.10585772562599707E-01;
   Q[23][5] = 0.21171545131053612E-01;
   Q[23][4] = 0.41723625991498071E-01;
   Q[23][3] = 0.17429756834617838E+00; 
   Q[24][0] = 0.15246811460272591E+01;   
   Q[24][6] = 0.21805308663147050E-01;
   Q[25][0] = 0.13028345654441567E+01;  
   Q[25][6] = 0.40449124260612632E-01;
   Q[25][5] = 0.80898248581605506E-01;
   Q[26][0] = 0.10828882671070457E+01;  
   Q[26][6] = 0.67825997049051095E-01;
   Q[27][0] = 0.86449342473348334E+00;   
   Q[27][6] = 0.10312169905150050E+00;
   Q[27][5] = 0.20624339820055582E+00;
   Q[27][4] = 0.41378033527939212E+00;
   Q[28][0] = 0.64731932201292475E+00;   
   Q[28][6] = 0.14250178980050304E+00;
   Q[29][0] = 0.43104912710906618E+00;  
   Q[29][6] = 0.17930888947286597E+00;
   Q[29][5] = 0.35861777918101589E+00;
   Q[30][0] = 0.21537608434686875E+00;   
   Q[30][6] = 0.20570798545191801E+00;
   Q[31][0] = 0.00000000000000000E+00;   
   Q[31][6] = 0.21532669835245799E+00;
   Q[31][5] = 0.43065339692206278E+00;
   Q[31][4] = 0.85965421392331531E+00;
   Q[31][3] = 0.14381023280972494E+01;
   Q[31][2] = 0.17070586364608809E+01;
   Q[31][1] = 0.17724538509055161E+01;
   Q[32][0] = -0.21537608434686828E+00;   
   Q[32][6] = 0.20570798545191729E+00;
   Q[33][0] = -0.43104912710906512E+00;   
   Q[33][6] = 0.17930888947286680E+00;
   Q[33][5] = 0.35861777918101539E+00;
   Q[34][0] = -0.64731932201292475E+00;   
   Q[34][6] = 0.14250178980050313E+00;
   Q[35][0] = -0.86449342473348223E+00;   
   Q[35][6] = 0.10312169905150040E+00;
   Q[35][5] = 0.20624339820055645E+00;
   Q[35][4] = 0.41378033527939251E+00;
   Q[36][0] = -0.10828882671070448E+01;   
   Q[36][6] = 0.67825997049051207E-01;
   Q[37][0] = -0.13028345654441558E+01;   
   Q[37][6] = 0.40449124260612965E-01;
   Q[37][5] = 0.80898248581605464E-01;
   Q[38][0] = -0.15246811460272591E+01;  
   Q[38][6] = 0.21805308663146845E-01;
   Q[39][0] = -0.17487997352598681E+01;  
   Q[39][6] = 0.10585772562599781E-01;
   Q[39][5] = 0.21171545131053765E-01;
   Q[39][4] = 0.41723625991498418E-01;
   Q[39][3] = 0.17429756834617849E+00;
   Q[40][0] = -0.19755904965155657E+01;  
   Q[40][6] = 0.46069463066549755E-02;
   Q[41][0] = -0.22054885440752834E+01;  
   Q[41][6] = 0.17875789013035808E-02;
   Q[41][5] = 0.35751578076493088E-02;
   Q[42][0] = -0.24389717319444264E+01;  
   Q[42][6] = 0.61440870419948810E-03;
   Q[43][0] = -0.26765701097522356E+01;   
   Q[43][6] = 0.18562581305924691E-03;
   Q[43][5] = 0.37125162547352206E-03;
   Q[43][4] = 0.92266892334249228E-03;
   Q[44][0] = -0.29188775713880917E+01;   
   Q[44][6] = 0.48845979884032887E-04;
   Q[45][0] = -0.31665664130326094E+01;  
   Q[45][6] = 0.11073747996883378E-04;
   Q[45][5] = 0.22147496243806206E-04;
   Q[46][0] = -0.34204057947880262E+01;  
   Q[46][6] = 0.21349274133661244E-05;
   Q[47][0] = -0.36812855105782880E+01;  
   Q[47][6] = 0.34459424533139697E-06;
   Q[47][5] = 0.68918844742359961E-06;
   Q[47][4] = -0.29591636416831481E-04;
   Q[47][3] = -0.73848821532446476E-02;
   Q[47][2] = 0.32697607222317751E-01;
   Q[48][0] = -0.39502470903230265E+01;   
   Q[48][6] = 0.45694494841133506E-07;
   Q[49][0] = -0.42285252142956615E+01;   
   Q[49][6] = 0.48645225003442858E-08;
   Q[49][5] = 0.97290510877241157E-08;
   Q[50][0] = -0.45176039332373916E+01;  
   Q[50][6] = 0.40410215118153703E-09;
   Q[51][0] = -0.48192946553014941E+01;  
   Q[51][6] = 0.25280787679865117E-10;
   Q[51][5] = 0.50561012464229638E-10;
   Q[51][4] = 0.28972232360487857E-05;
   Q[52][0] = -0.51358470189242444E+01;  
   Q[52][6] = 0.11385718036108713E-11;
   Q[53][0] = -0.54701110501540784E+01; 
   Q[53][6] = 0.34824396741846834E-13;
   Q[53][5] = 0.69680868310556964E-13;
   Q[54][0] = -0.58257823262764017E+01;  
   Q[54][6] = 0.66969107027592850E-15;
   Q[55][0] = -0.62077875657087676E+01;  
   Q[55][6] = 0.72913045294174713E-17;
   Q[55][5] = 0.13619859943025975E-16;
   Q[55][4] = -0.11839463761818998E-06;
   Q[55][3] = 0.26307521119969644E-03;
   Q[56][0] = -0.66229208492676035E+01;  
   Q[56][6] = 0.38781336628439687E-19;
   Q[57][0] = -0.70809577848465306E+01;  
   Q[57][6] = 0.81134454288887507E-22;
   Q[57][5] = 0.11699822459143598E-19;
   Q[58][0] = -0.75967611847999752E+01;  
   Q[58][6] = 0.47670840476953439E-25;
   Q[59][0] = -0.81946859372550751E+01;  
   Q[59][6] = 0.44347090008379179E-29;
   Q[59][5] = -0.26234449600364379E-22;
   Q[59][4] = 0.11046855508027685E-08;
   Q[60][0] = -0.89192128292672965E+01;  
   Q[60][6] = 0.62575145603726558E-34;
   Q[61][0] = -0.98669848158693370E+01;  
   Q[61][6] = -0.98780327011011174E-38;
   Q[61][5] = -0.16216665272216883E-25;
   Q[62][0] = -0.11332306972524087E+02;  
   Q[62][6] = 0.76469430756097134E-43;
 

 for(int i=0;i<63;i++)
  { 
   Q[i][0] = Q[i][0]*a;
   for(int j=1;j<7;j++)
      Q[i][j] = Q[i][j]*b; 
  }


for(int i=0;i<nl;i++)
  {
   x[i] = Q[m-1+i*m][0];
   w[i] = Q[m-1+i*m][l];
  }
// Clean
    for (int idel = 0; idel < 63; idel++)
    {
        delete[] Q[idel];
        Q[idel] = NULL;
    }
    delete[] Q;
    Q = NULL;  
 }
}

//-----------------------------------------------------------------------------
/*
void quadrature_prem(double* x, double* w, int n)
{
if(n>34)
 {
  cout<<" n trop grand "<<endl;
  exit(1);
 }
else
 {
  
 }

}*/
//-----------------------------------------------------------------------------

int calculer_npol(int no, int nx) // calcul du nombre de polyn�me dans la base de
//PC pour un ordre et un nombre de dimensions donn�es
{
int npol;
double a;
int b;
if(no == 0 )
   { 
	return 0;  
   }  
else if(no > nx)
   { 
     a = 1./fact(nx);          
     for(int i=no+1;i<=no+nx;i++)
     a = a*i;        
   }     
else 
   {
     a = 1./fact(no);     
     for(int i=nx+1;i<=no+nx;i++)
     a = a*i;
   }     
b = int(a);
//cout<<setprecision(25)<<b-a<<endl;
if( fabs(b-a)>1.0E-8 ) b = b+1; 
npol = b-1;
//npol = fact(no + nx)/(fact(no)*fact(nx)) -1;
return npol;
}

//-----------------------------------------------------------------------------


double InvFcn(double p) {
  double x,q,r,e,u;
cout<<"########## flag nisp ###########"<<endl;
  if( (p<0.) || ( p>1.) ) {
  cout << "Erreur dans la fonction invfcn(). L'argument doit etre dans [0.,+1] !! \n";
  exit(1);
  }

  /********* JMM 11/09/2006 ********************
     Definition de l'intervalle [borne,1-borne]
  **********************************************/
  if(p<borne) p=borne;
  else if (p>1.-borne) p=1.-borne;

  if( (0. < p) && (p < p_min) ) {
    q=sqrt(-2.*log(p));
    x=(((((c[1]*q+c[2])*q+c[3])*q+c[4])*q+c[5])*q+c[6]) / ((((d[1]*q+d[2])*q+d[3])*q+d[4])*q+1.);
  }
  else if( (p_min <= p) && (p <= p_max) ) {
    q = p - 0.5;
    r=q*q;
    x = (((((a[1]*r+a[2])*r+a[3])*r+a[4])*r+a[5])*r+a[6])*q / (((((b[1]*r+b[2])*r+b[3])*r+b[4])*r+b[5])*r+1.);
  }
  else {
    q=sqrt(-2*log(1.-p));
    x=-(((((c[1]*q+c[2])*q+c[3])*q+c[4])*q+c[5])*q+c[6]) / ((((d[1]*q+d[2])*q+d[3])*q+d[4])*q+1.);
  }

  /***** JMM : 11/09/2006 *************
     Raffinement par la fonction erfc()
     precision absolue : 1.e-16;
  *************************************/
  e=erfc(-x/sqrt(2.))/2. - p;
// boucle infinie possible (ou très longue)  
//   do {
//     u=e*sqrt(2.*M_PI)*exp(x*x/2.);
//     x=x-u/(1+x*u/2.);
//     e=erfc(-x/sqrt(2.))/2. - p;
//   } while(fabs(e)>precision);
  for(int i=0;i<10;i++) 
    {
        u = e * sqrt(2. * M_PI) * exp(x * x / 2.);
        x = x - u / (1 + x * u / 2.);
        e = erfc(-x / sqrt(2.)) / 2. - p;
    if (fabs(e) > precision) break;
    }
  return(x);
}

//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------

}

#undef EPS
#undef PIM4
#undef MAXIT
