//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

/** ****************************************************** **
                    kNN (GPU)
 ** ****************************************************** **/


#ifndef __kNN_GPU_INCLUDE__
#define __kNN_GPU_INCLUDE__

typedef float Real;                             //!< GPU => Real (this is an implication, not a pretty arrow)


namespace URANIE
{
  namespace GPU
  {
    enum EDistance { kEuclidean, kL1, kManhattan, kChebyshev, kMahalanobis };  ///< the distance
    
    class kNN
    {
    public:            
      explicit kNN(int nx, int nk, EDistance t);

      ~kNN();
      
      /**! Size of the layer, in neurons **/
      inline int getDimension() const { return _nX; }
      /**! Size of the layer, in neurons **/
      inline int getNkNN() const { return _nK; }

      inline int getNReferencePoints() const { return _nRef; }
      inline int getNQueryPoints() const { return _nQuery; }
      inline double GetQueryOutput(int index) const { return vecQuery_host[index]; }

      void initialize();

      void setReferencePoints(int nsr, Real *vecReferencePoints, Real *vecReferenceOutput);
      void setQueryPoints(int nsq, Real *vecQueryPoints);

      void estimate();
      
      void printLog();

    public:
      Real* vecQuery_host;       //< Matrix of Query OUTPUT values
    private:
      const int _nX;               //!< Dimension
      const int _nK;               //!< the number of neighboor to take into account
      int _nRef;                   //!< Number of Reference points
      Real* matRef_host;                      //< Matrix of Reference values on host
      Real* vecRef_host;                      //< Matrix of Reference OUTPUT values on host

//       Real* matRef_device;     //< Matrix of Reference values on DEVICE

      int _nQuery;                 //!< Number of Query points
      Real* matQuery_host;         //< Matrix of Query values
      const EDistance _Distance;   // Activation function
    };

    
  } // namespace GPU
  
} // namespace URANIE

#endif // #ifndef __kNN_GPU_INCLUDE__
// End of File

