//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////


#include "nn_headers.h"

//inline __host__ __device__ float rosenbrock( const float x, const float y )

int main()
{
    namespace nn = NeuralNetworkLevenbergMarquardt;
    namespace pp = nn::Preprocessing;
    using std::cout;
    using std::endl;
    using nn::TwoLayersPerceptron;
    
    const size_t nbSamples = 10000;

    try
    {
      // Common initialization
      nn::GPU::Engine::Init();

//      std::cout << std::endl << "# --------- Testing with XOR ---------" << std::endl;
//      nn::Testing::testWithXOR( 1e-20, 1, 10 );
      std::cout << std::endl << "# --------- Testing with Rosenbrock ---------" << std::endl;
      nn::Testing::testWithRosenbrock( nbSamples, 1e-10, 5, 5000 );
//      std::cout << std::endl << "# --------- Testing with reference Rosenbrock ---------" << std::endl;
//      nn::Testing::testWithRefFileRosenbrock( "tests/rosenbrock_reference.dat", "tests/rosenbrock_output.dat", 1e-10, 1000 );
//      std::cout << std::endl << "# --------- Testing with ANDRA case ---------" << std::endl;
//      nn::Testing::testCaseANDRA( "tests/andra/", "AndraSampling_ANDRALaunching_pretraitment.dat", "__gpunnoutput_", 1e-10, 5000, 100, 1000 );

      // Shutdown engine.
      nn::GPU::Engine::Shutdown();
    }
    catch( std::exception* e )
    {
        std::cerr << e->what() << endl;
        delete e;
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}
