//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestTAnisp.h
 \author Thierry Crestaux (CS)
 \date   29/04/2013
 
 \brief Interface of the class TestTAnisp
 
 */
#ifndef CPPUNITEST_TESTTANISP_H
#define CPPUNITEST_TESTTANISP_H

// cppunit
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>
//   TDataServer* tdsIshigami = new URANIE::DataServer::TDataServer("tdsIshigami", "TDS for Ishigami Use Case");
//   tdsIshigami->addAttribute( new URANIE::DataServer::TUniformDistribution("x1", -1. * TMath::Pi(), TMath::Pi()));
//   tdsIshigami->addAttribute( new URANIE::DataServer::TUniformDistribution("x2", -1. * TMath::Pi(), TMath::Pi()));
//   tdsIshigami->addAttribute( new URANIE::DataServer::TUniformDistribution("x3", -1. * TMath::Pi(), TMath::Pi()));
//   
//   TDataServer *tdstmp = new TDataServer("tdsIshigami", "TDS for Ishigami Use Case");
//   tdstmp->fileDataRead("_exportDataIshigami_.dat");
//   
//   TString LaunchNumber = TString("__uranie__anisp__");
//   TAttribute * att = new TAttribute(LaunchNumber);
//   att->setOrigin(URANIE::DataServer::TAttribute::kInternal);
//   tdsIshigami->addAttribute(att);
//   tdsIshigami->setTuple(tdstmp->getTuple());
//   
//   URANIE::Modeler::TAnisp * anisp = new URANIE::Modeler::TAnisp(tdsIshigami, _p2fIshigami,"x1:x2:x3", "");
//   anisp->setRootFilename(str);
//   anisp->setKMin(2);
//   anisp->setNumberMaxOfSimulations(3000);
//   anisp->setTolerance(0.0000000000000000001);
//   anisp->setlog(kTRUE);
//   anisp->restartAnisp();
//   tdsIshigami->exportData("_exportDataIshigami2_.dat",_anisp->getVariablesName());
//   URANIE::Modeler::TPolynomialChaos P2(anisp->getTPolynomialChaos());
//   pc = &P2; 
//    
//   CPPUNIT_ASSERT_DOUBLES_EQUAL( 3.5000000000000008882  , pc->getCoefficient(1,0), DBL_PRECISION_COMPARE);
//   CPPUNIT_ASSERT_DOUBLES_EQUAL( -2.7141838864088682929e-06  , pc->getCoefficient(1,694), DBL_PRECISION_COMPARE);
//    
//   CPPUNIT_ASSERT_EQUAL(1511,_tdsIshigami->getNPatterns());
// ROOT
// #include "TMath.h"

// uranie
#include "TAnisp.h"
#include "TDataServer.h"
#include "UExceptions.h"
#include "TOutputFileDataServer.h"
#include "TCode.h"
#include "TSampling.h"

using namespace URANIE::DataServer;
using namespace URANIE::Exceptions;
using namespace URANIE::Launcher;
using namespace URANIE::Sampler;

/**

 * @class TAnispTest
 *
 * @brief Description of the class TAnispTest.
 *
 *  The class TAnispTest is the unit test class of the class TAnisp. 
 *  
 *
 */

class TAnispTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TAnispTest);
    CPPUNIT_TEST (testAnispDefaultValues);
    CPPUNIT_TEST (testgetKMin);
    CPPUNIT_TEST (testsetKMin);
    CPPUNIT_TEST (testgetTolerance);
    CPPUNIT_TEST (throwExceptionsetKMinMethod);
    CPPUNIT_TEST (throwExceptionTCodeNull);
    CPPUNIT_TEST (throwExceptionTCodeSeveralOutputs);
    CPPUNIT_TEST (throwExceptionFunctionNull);
    CPPUNIT_TEST (throwExceptionNonSupportedLaw);
    CPPUNIT_TEST (throwExceptionConstructorInputAttribute);
    CPPUNIT_TEST (testsetTolerance);
    CPPUNIT_TEST (throwExceptionsetToleranceMethod);
    CPPUNIT_TEST (testgetNumberMaxOfSimulations);
    CPPUNIT_TEST (testsetNumberMaxOfSimulations);
    CPPUNIT_TEST (throwExceptionsetNumberMaxOfSimulationsMethod);
    CPPUNIT_TEST (testgetGreaterIndice);
    CPPUNIT_TEST (testsetGreaterIndice);
    CPPUNIT_TEST (throwExceptionsetGreaterIndiceMethod);
    CPPUNIT_TEST (testgetMaxIndices);
    CPPUNIT_TEST (testsetMaxIndices);
    CPPUNIT_TEST (throwExceptionsetMaxIndicesMethod);
    CPPUNIT_TEST (testgetDegreeMax);
    CPPUNIT_TEST (testsetDegreeMax);
    CPPUNIT_TEST (throwExceptionsetDegreeMaxMethod);
    CPPUNIT_TEST (testgetCible);
    CPPUNIT_TEST (testgetIterationNumber);
    CPPUNIT_TEST (testgetRootFilename);
    CPPUNIT_TEST (testsetRootFilename);
    CPPUNIT_TEST (throwExceptionsetRootFilenameMethod);
    CPPUNIT_TEST (testgetDataAnispFile);
    CPPUNIT_TEST (testgetResultFile);
    CPPUNIT_TEST (testgetStochasticFile);
    CPPUNIT_TEST (testgetErrorIndicatorFile);
    CPPUNIT_TEST (testgetIndicesFile);
    CPPUNIT_TEST (testsetlog);
    CPPUNIT_TEST (testsetAllAnispParameters);
    CPPUNIT_TEST (testsetAnispParameters);
    CPPUNIT_TEST (testsetLightAnispParameters);
    CPPUNIT_TEST (throwExceptionConstructorInputAttribute);
    CPPUNIT_TEST (throwExceptiongetTPolynomialChaos);
    CPPUNIT_TEST (testchangeOfVariable);
    CPPUNIT_TEST (testgetTPolynomialChaos);
    CPPUNIT_TEST (testrunAnisp);
    CPPUNIT_TEST (testrestartAnisp);
    
    CPPUNIT_TEST (throwExceptionrestartAnisp);
    CPPUNIT_TEST (testgetErrorIndicator); //3
    CPPUNIT_TEST (testgetIntegrationIndices);
    CPPUNIT_TEST (throwExceptionchangeOfVariable);
    CPPUNIT_TEST (testexportModelCplusplus); //3
    CPPUNIT_TEST(throwExceptionExportCplusplus);
   
    CPPUNIT_TEST (throwExceptionConstructorNonAnispPatterns);
    CPPUNIT_TEST (testgetVariablesName);
    CPPUNIT_TEST (throwExceptionConstructorTdsDifferentInTCode);
    
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TAnispTest object.
     */
    TAnispTest();
    /*! Destructor
     */
    virtual ~TAnispTest();

    //---------------------------------------------
    /** @name setUp and Tear Down methods
     **/
    //@{
    //! Build a object TAnisp from a tds object
    void setUp();
    //! ! Delete the object tds and nips
    void tearDown();
    //@}

    //---------------------------TMATH::------------------
///////////////////////////////////////////////////
    /** @name Test Unitary
     **/
    //@{
    // Unit test of the method TAnisp::getNPatterns()  
//   void testgetNPatterns();
    //! Unit test of the TAnisp constructors
    /*!
     \brief test the TAnisp constructors
     */
    void testAnispDefaultValues();

    //! Unit test of the method TAnisp::getKMin()
    /*!
     \brief test the method TAnisp::getKMin()
     */
    void testgetKMin();

    //! Unit test of the method TAnisp::setKMin()
    /*!
     \brief test the method TAnisp::setKMin()
     */
    void testsetKMin();

    //! Unit test of the method TAnisp::getTolerance()
    /*!
     \brief test the method TAnisp::getTolerance()
     */
    void testgetTolerance();

    //! Unit test of the method TAnisp::setTolerance()
    /*!
     \brief test the method TAnisp::setTolerance()
     */
    void testsetTolerance();

    //! Unit test of the method TAnisp::getNumberMaxOfSimulations()
    /*!
     \brief test the method TAnisp::getNumberMaxOfSimulations()
     */
    void testgetNumberMaxOfSimulations();

    //! Unit test of the method TAnisp::setNumberMaxOfSimulations()
    /*!
     \brief test the method TAnisp::setNumberMaxOfSimulations()
     */
    void testsetNumberMaxOfSimulations();

    //! Unit test of the method TAnisp::getGreaterIndice()
    /*!
     \brief test the method TAnisp::getGreaterIndice()
     */
    void testgetGreaterIndice();

    //! Unit test of the method TAnisp::setGreaterIndice()
    /*!
     \brief test the method TAnisp::setGreaterIndice()
     */
    void testsetGreaterIndice();

    //! Unit test of the method TAnisp::getMaxIndices()
    /*!
     \brief test the method TAnisp::getMaxIndices()
     */
    void testgetMaxIndices();

    //! Unit test of the method TAnisp::setMaxIndices()
    /*!
     \brief test the method TAnisp::setMaxIndices()
     */
    void testsetMaxIndices();

    //! Unit test of the method TAnisp::getDegreeMax()
    /*!
     \brief test the method TAnisp::getDegreeMax()
     */
    void testgetDegreeMax();

    //! Unit test of the method TAnisp::setDegreeMax()
    /*!
     \brief test the method TAnisp::setDegreeMax()
     */
    void testsetDegreeMax();

    //! Unit test of the method TAnisp::getCible()
    /*!
     \brief test the method TAnisp::getCible()
     */
    void testgetCible();

    //! Unit test of the method TAnisp::getIterationNumber()
    /*!
     \brief test the method TAnisp::getIterationNumber()
     */
    void testgetIterationNumber();

    //! Unit test of the method TAnisp::getRootFilename()
    /*!
     \brief test the method TAnisp::getRootFilename()
     */
    void testgetRootFilename();

    //! Unit test of the method TAnisp::setRootFilename()
    /*!
     \brief test the method TAnisp::setRootFilename()
     */
    void testsetRootFilename();

    //! Unit test of the method TAnisp::getDataAnispFile()
    /*!
     \brief test the method TAnisp::getDataAnispFile()
     */
    void testgetDataAnispFile();

    //! Unit test of the method TAnisp::getResultFile()
    /*!
     \brief test the method TAnisp::getResultFile()
     */
    void testgetResultFile();

    //! Unit test of the method TAnisp::getStochasticFile()
    /*!
     \brief test the method TAnisp::getStochasticFile()
     */
    void testgetStochasticFile();

    //! Unit test of the method TAnisp::getErrorIndicatorFile()
    /*!
     \brief test the method TAnisp::getErrorIndicatorFile()
     */
    void testgetErrorIndicatorFile();

    //! Unit test of the method TAnisp::getIndicesFile()
    /*!
     \brief test the method TAnisp::getIndicesFile()
     */
    void testgetIndicesFile();

    //! Unit test of the method TAnisp::setlog()
    /*!
     \brief test the method TAnisp::setlog
     */
    void testsetlog();

    //! Unit test of the method TAnisp::setAllAnispParameters()
    /*!
     \brief test the method TAnisp::setAllAnispParameters()
     */
    void testsetAllAnispParameters();

    //! Unit test of the method TAnisp::setAnispParameters()
    /*!
     \brief test the method TAnisp::setAnispParameters()
     */
    void testsetAnispParameters();

    //! Unit test of the method TAnisp::setLightAnispParameters()
    /*!
     \brief test the method TAnisp::setLightAnispParameters()
     */
    void testsetLightAnispParameters();

    //! Unit test of the method TAnisp::changeOfVariable()
    /*!
     \brief test the method TAnisp::changeOfVariable()
     */
    void testchangeOfVariable();

    //! Unit test of the method TAnisp::getTPolynomialChaos()
    /*!
     \brief test the method TAnisp::getTPolynomialChaos()
     */
    void testgetTPolynomialChaos();

    //! Unit test of the method TAnisp::runAnisp()
    /*!
     \brief test the method TAnisp::runAnisp()
     */
    void testrunAnisp();

    //! Unit test of the method TAnisp::restartAnisp()
    /*!
     \brief test the method TAnisp::restartAnisp()
     */
    void testrestartAnisp();

    //! Unit test of the method TAnisp::getErrorIndicator()
    /*!
     \brief test the method TAnisp::getErrorIndicator()
     */
    void testgetErrorIndicator();

    //! Unit test of the method TAnisp::getIntegrationIndices()
    /*!
     \brief test the method TAnisp::getIntegrationIndices()
     */
    void testgetIntegrationIndices();

    //! Unit test of the method TAnisp::exportModelCplusplus()
    /*!
     \brief test the method TAnisp::exportModelCplusplus()
     */
    void testexportModelCplusplus();

    //! Unit test of the method TAnisp::getVariablesName()
    /*!
     \brief test the method TAnisp::getVariablesNames()
     */
    void testgetVariablesName();

    //@}

/////////////////////////////////////////////////  
/////////////////////////////////////////////////
/////////////////////////////////////////////////
/////////////////////////////////////////////////

    /** @name Throw Exception 
     **/
    //@{
    //! Unit test of the constructor TAnisp
    /*!
     \brief Throw Exception if one of the random variable is not supported
     */
    void throwExceptionNonSupportedLaw();

    //! Unit test of the constructor TAnisp
    /*!
     \brief Throw Exception if the analytical function is NULL
     */
    void throwExceptionFunctionNull();

    //! Unit test of the constructor TAnisp
    /*!
     \brief Throw Exception if the TCode is NULL
     */
    void throwExceptionTCodeNull();

    //! Unit test of the constructor TAnisp
    /*!
     \brief Throw Exception if the TCode has several outputs
     */
    void throwExceptionTCodeSeveralOutputs();

    //! Unit test of the method TAnisp::setKMin()
    /*!
     \brief Throw Exception if the parameter is <=0
     */
    void throwExceptionsetKMinMethod();

    //! Unit test of the method TAnisp::setTolerance()
    /*!
     \brief Throw Exception if the parameter is <=0
     */
    void throwExceptionsetToleranceMethod();

    //! Unit test of the method TAnisp::setNumberMaxOfSimulations()
    /*!
     \brief Throw Exception if the parameter is <=0
     */
    void throwExceptionsetNumberMaxOfSimulationsMethod();

    //! Unit test of the method TAnisp::setGreaterIndice()
    /*!
     \brief Throw Exception if the parameter is <=0 or > 6 and there is a normal variable
     */
    void throwExceptionsetGreaterIndiceMethod();

    //! Unit test of the method TAnisp::setMaxIndices()
    /*!
     \brief Throw Exception if the parameter is <=0 
     */
    void throwExceptionsetMaxIndicesMethod();

    //! Unit test of the method TAnisp::setDegreeMax()
    /*!
     \brief Throw Exception if the parameter is < 0 
     */
    void throwExceptionsetDegreeMaxMethod();

    //! Unit test of the method TAnisp::setRootFilename()
    /*!
     \brief Throw Exception if the parameter is empty or white 
     */
    void throwExceptionsetRootFilenameMethod();

    //!Unit test of the constructor with TCode and wrong input attributes
    /*!
     \brief Throw Exception if there is input attributes which doesn't belong to the TCode and isn't a TFormula.
     */
    void throwExceptionConstructorInputAttribute();

    //! Unit test of the method TAnisp::restartAnisp()
    /*!
     \brief Throw Exception if runAnisp hasn't been previously used.
     */
    void throwExceptionrestartAnisp();

    //! Unit test of the method TAnisp::getTPolynomialChaos()
    /*!
     \brief Throw Exception if runAnisp hasn't been previously used and so there is not a TPolynomialChaos computed.
     */
    void throwExceptiongetTPolynomialChaos();

    //! Unit test of the method TAnisp::changeOfVariable()
    /*!
     \brief Throw Exception if there is an input variable which is nor uniform neither normal.
     */
    void throwExceptionchangeOfVariable();

    //! Unit test of the method TAnisp::exportModelCplusplus()
    /*!
     \brief Throw Exception if there is no result file to read.
     */
    void throwExceptionExportCplusplus();

    //!Unit test of the constructor with TDataServer containing non Anisp patterns
    /*!
     \brief Throw Exception if the TDataServer contains patterns which don't come from a previous run of the ANISP method
     */
    void throwExceptionConstructorNonAnispPatterns();
    
    //!Unit test of the constructor with TCode containing a different TDataServer
    /*!
     \brief Throw Exception if the TCode contains a TDataServer which is different that the one given in argument to the constructor
     */
    void throwExceptionConstructorTdsDifferentInTCode();

    //@}   

private:
    URANIE::DataServer::TDataServer *_tdsIshigami;
    URANIE::DataServer::TDataServer *_tdsFlowrate;
    URANIE::DataServer::TDataServer *_tdsLinear;
    URANIE::DataServer::TDataServer *_tdsVenise;
    URANIE::DataServer::TDataServer *_tdsNULL;
    URANIE::Modeler::TAnisp *_anisp;
    URANIE::Modeler::TAnisp *_anisp2;
    URANIE::Modeler::TAnisp *_anisp3;
    URANIE::Modeler::TAnisp *_anisp4;
    URANIE::Launcher::TOutputFileDataServer *_foutDataServer;
    URANIE::Launcher::TCode *_codeFlowrate;

    void * _p2fIshigami;	///< Fct Ishigami
    void * _p2fLinear;	///< Fct Linear
    void * _p2fVenise;	///< Fct Venise

};

#endif  // CPPUNITEST_TESTTANISP_H
