//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTGPBuilder.h
 \author Nicolas Gilardi <nicolas.gilardi@cea.fr>
 \date   $Date$
 
 \brief  Test suite for class URANIE::Modeler::TGPBuilder.
 


 */
#ifndef CPPUNITEST_TGPBUILDER_H
#define CPPUNITEST_TGPBUILDER_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TDataServer.h"
#include "TKriging.h"

/*! \class TGPBuilderTest
 * \brief Unit tests for TGPBuilderTest.
 */

class TGPBuilderTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TGPBuilderTest);
    CPPUNIT_TEST(testConstruction);
    CPPUNIT_TEST(testTrend);
    CPPUNIT_TEST(testBayesian);
    CPPUNIT_TEST(testAlpha);
    CPPUNIT_TEST(testVMes);
    CPPUNIT_TEST(testSetMeasurementErrorCovMatrix);
    CPPUNIT_TEST(testFindOptimalParam);   
    CPPUNIT_TEST(testExportModelData);
    CPPUNIT_TEST(testUpdateObservations);
    CPPUNIT_TEST(testSetSeed);
    CPPUNIT_TEST(testsetTolerance);
    CPPUNIT_TEST(testFindOptimalParamBetweenBounds);
    CPPUNIT_TEST(testsetCorrFunction);
    CPPUNIT_TEST(throwExceptionSetCorrFunction);
    CPPUNIT_TEST(throwExceptionConstructorIncompatibleDataServer);
    CPPUNIT_TEST(throwExceptionUnknownOptimisationAlgorithm);
    CPPUNIT_TEST(throwExceptionSetUsePrior);
    CPPUNIT_TEST(throwExceptionUnsupportedCriterion);
    CPPUNIT_TEST(throwExceptionReMLCriterionWithoutTrend);
    CPPUNIT_TEST(throwExceptionMLCriterionWithPriorData);
    CPPUNIT_TEST(throwExceptionConstructorError);
    CPPUNIT_TEST(throwExceptionsetLengthRange);
    CPPUNIT_TEST(throwExceptionsetLengthMaxScreeningSF);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TGPBuilderTest object.
     */
    TGPBuilderTest();

    /// Destructor.
    virtual ~TGPBuilderTest();

    URANIE::DataServer::TDataServer *tdsObs;

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void testConstruction();
    void testTrend();
    void testBayesian();
    void testAlpha();
    void testVMes();
    void testSetMeasurementErrorCovMatrix();
    void testFindOptimalParam();
    void testExportModelData();
    void testUpdateObservations();
    void testSetSeed();
    void testFindOptimalParamBetweenBounds();
    void testsetCorrFunction();
    void testsetTolerance();
    void throwExceptionSetCorrFunction();
    void throwExceptionConstructorIncompatibleDataServer();
    void throwExceptionUnknownOptimisationAlgorithm();
    void throwExceptionSetUsePrior();
    void throwExceptionUnsupportedCriterion();
    void throwExceptionReMLCriterionWithoutTrend();
    void throwExceptionMLCriterionWithPriorData();
    void throwExceptionConstructorError();
    void throwExceptionsetLengthRange();
    void throwExceptionsetLengthMaxScreeningSF();
    //@}

private:
    TString _workingTestPath;
};


#endif  //CPPUNITEST_TGPBUILDER_H

