//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTKriging.h
 \author Nicolas Gilardi <nicolas.gilardi@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Modeler::TKriging.
 
 Cr�e le : 2012.11.16



 */
#ifndef CPPUNITEST_TKRIGINGTEST_H
#define CPPUNITEST_TKRIGINGTEST_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TDataServer.h"
#include "TAttribute.h"

/*! \class TKrigingTest                             
 * \brief Unit tests for TKrigingTest.
 */

class TKrigingTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TKrigingTest);
#ifdef  WITH_LIBXML2
    CPPUNIT_TEST (TestGaussianProcessComputation);
    CPPUNIT_TEST (TestGaussianProcessCheck);
    CPPUNIT_TEST (TestGaussianProcessCheckTrend);
#endif
    CPPUNIT_TEST (testConstructionFromGPBuilder);
    CPPUNIT_TEST (testEval);
    CPPUNIT_TEST (testestimWithCov);
    CPPUNIT_TEST (testestimWithCovLinearTrend);
    CPPUNIT_TEST (testThrowExceptionEstimWithCov);
    CPPUNIT_TEST (testThrowExceptionEvalWrongAttributes);
    CPPUNIT_TEST (testCleanInputsOutputs);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TKrigingTest object.
     */
    TKrigingTest();

    /// Destructor.
    virtual ~TKrigingTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void testConstructionFromGPBuilder();
    void testEval();
    void testCleanInputsOutputs();
    void testestimWithCov();
    void testestimWithCovLinearTrend();
    void testThrowExceptionEstimWithCov();
    void testThrowExceptionEvalWrongAttributes();
#ifdef  WITH_LIBXML2
    void TestGaussianProcessComputationChoix(int choix);
    void TestGaussianProcessComputation(void);
    void TestGaussianProcessCheck(void);
    void TestGaussianProcessCheckTrend(void);
#endif
    //@}

private:
    // variables
    URANIE::DataServer::TDataServer *_tdsObs;


};
#endif  //CPPUNITEST_TKRIGINGTEST_H

