//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTPolynomialMultiDimFit.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TPolynomialMultiDim
 
 Created the : 2012/08/24 15:45:24
 */

#define DBL_PRECISION_COMPARE 1.e-10
// Data Header
#include "TestTPolynomialMultiDimFit.h"
#include "TString.h"
#include <iostream>
#include <ctime>
#include "uranie_commands.h"
using namespace std;

TPolynomialMultiDimFitTest::TPolynomialMultiDimFitTest():
    tds1(NULL), tds2(NULL)
{
}

TPolynomialMultiDimFitTest::~TPolynomialMultiDimFitTest()
{
    UranieTests::removeFile(gSystem->pwd(),"filename.C");
    UranieTests::removeFile(gSystem->pwd(),"filename.f");
    UranieTests::removeFile(gSystem->pwd(),"filename.py");    
}

void TPolynomialMultiDimFitTest::setUp()
{
    tds1 = new URANIE::DataServer::TDataServer();
    tds1->unsetLog();
    tds1->fileDataRead("steamplant.dat");
    

    tds2 = new URANIE::DataServer::TDataServer();
    tds2->unsetLog();
    tds2->fileDataRead("ishigami.dat");

}

void TPolynomialMultiDimFitTest::tearDown()
{
    delete tds1;
    tds1 = NULL;
    delete tds2;
    tds2 = NULL;

    gDirectory->Delete("x_*_orig;9999");
    gDirectory->Delete("d_orig;9999");
    gDirectory->Delete("x_*_norm;9999");
    gDirectory->Delete("d_shifted;9999");
    gDirectory->Delete("res_x_*;9999");
    gDirectory->Delete("res_d;9999");
    gDirectory->Delete("res_train;9999");
    gDirectory->Delete("res_test;9999");
}

////////////////////////////////
///                          ///
///         Tests            ///
///                          ///
////////////////////////////////

void TPolynomialMultiDimFitTest::testMDFIshigami()
{

    URANIE::Modeler::TPolynomialMultiDimFit *tm =
            new URANIE::Modeler::TPolynomialMultiDimFit(tds2, "x1:x2:x3", "y",
                    TMultiDimFit::kMonomials, "v");
    tm->unsetLog();

    Int_t mPowers[] =
    { 6, 6, 6 };

    // Fit Multi Dim Fit
    tm->setMinAngle(10);
    tm->setMaxAngle(10);
    tm->setMaxFunctions(2000);
    tm->setMaxStudy(2000);
    tm->setMaxTerms(7);
    tm->setMaxPowers(mPowers);
    tm->setMinRelativeError(.01);
    tm->setPowerLimit(1);

    CPPUNIT_ASSERT(!strcmp(tm->getInputName(0), "x1"));
    CPPUNIT_ASSERT(!strcmp(tm->getInputName(1), "x2"));
    CPPUNIT_ASSERT(!strcmp(tm->getInputName(2), "x3"));

    tm->estimate();

    CPPUNIT_ASSERT(tds2->isAttribute("yhat"));

    CPPUNIT_ASSERT_EQUAL(1000, tds2->getNPatterns());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.01499840187098, tds2->getValue("yhat", 0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.60139284901202, tds2->getValue("yhat", 1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.90522347200207, tds2->getValue("yhat", 500),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.24698478781525, tds2->getValue("yhat", 998),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.81012614023027, tds2->getValue("yhat", 999),
            DBL_PRECISION_COMPARE);

    TVectorD dparam = tm->getParameters();

    CPPUNIT_ASSERT_EQUAL(7, dparam.GetNrows());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(11478.3120244233,
            tm->getMultiDimFit()->GetResidualSumSq(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-3.2081436920,
            tm->getMultiDimFit()->GetResidualMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0959329499,
            tm->getMultiDimFit()->GetResidualMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.5442130751936, dparam[0],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8632308591233, dparam[1],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(14.1661921570099, dparam[2],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-23.6158243030627, dparam[3],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-25.6303297793122, dparam[4],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(19.9496073996856, dparam[5],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.2677228079862, dparam[6],
            DBL_PRECISION_COMPARE);

    // Clean
    delete tm;
}

void TPolynomialMultiDimFitTest::testConstructorUsingArchitecture()
{

    URANIE::Modeler::TPolynomialMultiDimFit *tm =
            new URANIE::Modeler::TPolynomialMultiDimFit(tds2, "x1:x2:x3,y",
                    TMultiDimFit::kMonomials, "v");
    tm->unsetLog();

    Int_t mPowers[] =
    { 6, 6, 6 };

    // Fit Multi Dim Fit
    tm->setMinAngle(10);
    tm->setMaxAngle(10);
    tm->setMaxFunctions(2000);
    tm->setMaxStudy(2000);
    tm->setMaxTerms(7);
    tm->setMaxPowers(mPowers);
    tm->setMinRelativeError(.01);
    tm->setPowerLimit(1);

    CPPUNIT_ASSERT(!strcmp(tm->getInputName(0), "x1"));
    CPPUNIT_ASSERT(!strcmp(tm->getInputName(1), "x2"));
    CPPUNIT_ASSERT(!strcmp(tm->getInputName(2), "x3"));

    tm->estimate();

    CPPUNIT_ASSERT(tds2->isAttribute("yhat"));

    CPPUNIT_ASSERT_EQUAL(1000, tds2->getNPatterns());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.01499840187098, tds2->getValue("yhat", 0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.60139284901202, tds2->getValue("yhat", 1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.90522347200207, tds2->getValue("yhat", 500),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.24698478781525, tds2->getValue("yhat", 998),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.81012614023027, tds2->getValue("yhat", 999),
            DBL_PRECISION_COMPARE);

    TVectorD dparam = tm->getParameters();

    CPPUNIT_ASSERT_EQUAL(7, dparam.GetNrows());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(11478.3120244233,
            tm->getMultiDimFit()->GetResidualSumSq(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-3.2081436920,
            tm->getMultiDimFit()->GetResidualMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0959329499,
            tm->getMultiDimFit()->GetResidualMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.5442130751936, dparam[0],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8632308591233, dparam[1],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(14.1661921570099, dparam[2],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-23.6158243030627, dparam[3],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-25.6303297793122, dparam[4],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(19.9496073996856, dparam[5],
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.2677228079862, dparam[6],
            DBL_PRECISION_COMPARE);

    // Clean
    delete tm;
}



////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TPolynomialMultiDimFitTest::throwExceptionTPolynomialMultiDimFitConstructors()
{
    cout << endl;
    URANIE::DataServer::TDataServer *tdsnull = NULL;
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TPolynomialMultiDimFit(tdsnull, "x6:x8", "x1",
                    TMultiDimFit::kMonomials, "v"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TPolynomialMultiDimFit(tdsnull, "x6:x8,x1",
                    TMultiDimFit::kMonomials, "v"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TPolynomialMultiDimFit(tds1, "x6:x8", "toto",
                    TMultiDimFit::kMonomials, "v"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TPolynomialMultiDimFit(tds1, "x6:x8,toto",
                    TMultiDimFit::kMonomials, "v"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TPolynomialMultiDimFit(tds1, "x6:toto:x8",
                    "x1", TMultiDimFit::kMonomials, "v"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TPolynomialMultiDimFit(tds1, "x6:toto:x8,x1",
                    TMultiDimFit::kMonomials, "v"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TPolynomialMultiDimFit(tds1, "x6:x8", "x1:x2",
                    TMultiDimFit::kMonomials, "v"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TPolynomialMultiDimFit(tds1, "x6:x8,x1:x2",
                    TMultiDimFit::kMonomials, "v"),
            URANIE::Exceptions::UErrorExceptions);
}

void TPolynomialMultiDimFitTest::throwExceptionTPolynomialMultiDimFitSetterMethods()
{
    cout << endl;
    URANIE::Modeler::TPolynomialMultiDimFit *tm =
            new URANIE::Modeler::TPolynomialMultiDimFit(tds2, "x1:x2:x3", "y",
                    TMultiDimFit::kMonomials, "v");
    tm->unsetLog();
    CPPUNIT_ASSERT_THROW(tm->setMinAngle(-10.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(tm->setMinAngle(100.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(tm->setMaxAngle(-10.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(tm->setMaxAngle(100.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(tm->setMaxFunctions(-2),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(tm->setMaxStudy(-1),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(tm->setMaxPowers(NULL),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(tm->setMinRelativeError(-0.01),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(tm->setPowerLimit(-0.02),
            URANIE::Exceptions::UErrorExceptions);

    // Clean
    delete tm;
}

void TPolynomialMultiDimFitTest::throwExceptionExportFunction()
{
    cout << endl;
    URANIE::Modeler::TPolynomialMultiDimFit *tm =
            new URANIE::Modeler::TPolynomialMultiDimFit(tds2, "x1:x2:x3", "y",
                    TMultiDimFit::kMonomials, "v");
    tm->unsetLog();

    Int_t mPowers[] =
    { 6, 6, 6 };

    // Fit Multi Dim Fit
    tm->setMinAngle(10);
    tm->setMaxAngle(10);
    tm->setMaxFunctions(2000);
    tm->setMaxStudy(2000);
    tm->setMaxTerms(7);
    tm->setMaxPowers(mPowers);
    tm->setMinRelativeError(.01);
    tm->setPowerLimit(1);

    tm->estimate();

    CPPUNIT_ASSERT_THROW(
            tm->exportFunction("toto", "filename", "function_name"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            tm->exportFunction("pmml", "filename", "function_name"),
            URANIE::Exceptions::UWarningExceptions);
    CPPUNIT_ASSERT_THROW(
            tm->exportFunction("fortran", "filename", "function_name"),
            URANIE::Exceptions::UWarningExceptions);
    CPPUNIT_ASSERT_THROW(
            tm->exportFunction("python", "filename", "function_name"),
            URANIE::Exceptions::UWarningExceptions);
    CPPUNIT_ASSERT_THROW(
            tm->exportFunction("c++", "filename", "function_name"),
            URANIE::Exceptions::UWarningExceptions);

    // Clean
    delete tm;
}
