//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTKNN.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   Wed Jul  8 22:41:16 2015

 \brief  Fichier de test de la classe URANIE::DataServer::TkNN

 */
#ifndef CPPUNITEST_TESTTKNN_H
#define CPPUNITEST_TESTTKNN_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TkNN.h"

/*! \class TkNNTest
 * \brief Unit tests for TkNN.
 */
class TkNNTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TkNNTest);
    CPPUNIT_TEST (testTkNNConstructor);
    CPPUNIT_TEST (testTkNNEstimateCPU);
    CPPUNIT_TEST (testTkNNEstimateFlowrate);


#ifdef WITH_GPUANN
    CPPUNIT_TEST (testTkNNEstimateGPU);
#endif


    CPPUNIT_TEST (testTkNNExportModelCplusplus);

    CPPUNIT_TEST (throwExceptionTkNNAttributesNotExistQuery);
    CPPUNIT_TEST (throwExceptionTkNNExportFunction);
    CPPUNIT_TEST (throwExceptionTkNNSeveralOutput);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TkNNTest object.
     */
    TkNNTest();

    /// Destructor.
    virtual ~TkNNTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void testTkNNConstructor();

    void testTkNNEstimateCPU();

    void testTkNNEstimateFlowrate();

#ifdef WITH_GPUANN
    void testTkNNEstimateGPU();
#endif

    //! Unit test of exportModelCplusplus
    void testTkNNExportModelCplusplus();

    //@}

    //---------------------------------------------
    /** @name throw Exception Tests
     The list of unit test of type throwException
     **/
    //@{
    //! Throw an exectpion when attributes (input or output) do not exist in the query TDS
    void throwExceptionTkNNAttributesNotExistQuery();

  //! Throw an exectpion with setter attributs
    void throwExceptionTkNNExportFunction();

    //! Throw an exception if there is too much output variables
    void throwExceptionTkNNSeveralOutput();
    //@}

private:
    TString _workingTestPath;
    URANIE::DataServer::TDataServer *_tdsR, *_tdsQ;
    TString _sX, _sY;
};

#endif  // CPPUNITEST_TESTTKNN_H
