//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestUMLP.h
 \author Maud Salvoldi (OCC)
 \date   18/10/2011
 
 \brief Interface of the class TestUMLP
 
 */
#ifndef CPPUNITEST_TESTUMLP_H
#define CPPUNITEST_TESTUMLP_H

// cppunit
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// uranie
#include "UMLP.h"
#include "TDataServer.h"
#include "UExceptions.h"

/**

 * @class TestUMLP
 *
 * @brief Description of the class TestUMLP.
 *
 *  The class TestUMLP is the unit test class of the class UMLP. 
 *  
 *
 */

class UMLPTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (UMLPTest);
    CPPUNIT_TEST (testUMLPFirstConstructor);
    CPPUNIT_TEST (testUMLPSecondConstructor);
    CPPUNIT_TEST (testUMLPsetWeights);
    CPPUNIT_TEST (testUMLPexportWeights);
    CPPUNIT_TEST (testUMLPinitWeights);
    CPPUNIT_TEST (testUMLPloadWeights);
    CPPUNIT_TEST (testUMLPsetTDS);
    CPPUNIT_TEST (testUMLPsetSigmaHatANDgetSigmaHat);
    CPPUNIT_TEST (testUMLPsetAlphaICANDgetAlphaIC);
    CPPUNIT_TEST (testUMLPsetMinMaxValueANDdumpWeights);
    CPPUNIT_TEST (testUMLPgetActivationFunction);
    CPPUNIT_TEST (testUMLPgetEcart);
    CPPUNIT_TEST (testUMLPgetWeightsDerivate);
    CPPUNIT_TEST (testUMLPevaluateWeightsJacobianANDgetPartialWeightsDerivate);
    CPPUNIT_TEST (testUMLPExportFunction);
    
    CPPUNIT_TEST (throwExceptionUMLPgetWeightsDerivate);
    CPPUNIT_TEST (throwExceptionUMLPgetPartialWeightsDerivate);

    CPPUNIT_TEST (throwExceptionUMLPFalseNiput);
    CPPUNIT_TEST (throwExceptionUMLPFalseNidden);
    CPPUNIT_TEST (throwExceptionUMLPFalseFileName);
    CPPUNIT_TEST (throwExceptionUMLPFalseFileData);

    CPPUNIT_TEST_SUITE_END();

public:
    //! Constructor
    UMLPTest();
    //! Destructor
    virtual ~UMLPTest();

    //---------------------------------------------
    /** @name setUp and Tear Down methods
     **/
    //@{
    //! Build a object
    void setUp();
    //! ! Delete the object
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name unit test methods
     **/
    //@{
    //!
    void testUMLPFirstConstructor();
    //!
    void testUMLPSecondConstructor();
    //!
    void testUMLPsetWeights();
    //! Unit test of the method exportWeights.
    void testUMLPexportWeights();
    //!
    void testUMLPinitWeights();
    //!
    void testUMLPloadWeights();
    //!
    void testUMLPsetTDS();
    //!
    void testUMLPsetSigmaHatANDgetSigmaHat();
    //!
    void testUMLPsetAlphaICANDgetAlphaIC();
    //!
    void testUMLPsetMinMaxValueANDdumpWeights();
    //!
    void testUMLPgetActivationFunction();
    //!
    void testUMLPgetEcart();
    //!
    void testUMLPgetWeightsDerivate();
    //!
    void testUMLPevaluateWeightsJacobianANDgetPartialWeightsDerivate();
    //!
    void testUMLPExportFunction();
    //@}

    //---------------------------------------------
    /** @name throwException methods
     **/
    //@{
    //!
    void throwExceptionUMLPFalseNiput();
    //!
    void throwExceptionUMLPFalseNidden();
    //!
    void throwExceptionUMLPFalseFileName();
    //!
    void throwExceptionUMLPFalseFileData();
    //!
    void throwExceptionUMLPgetWeightsDerivate();
    //!
    void throwExceptionUMLPgetPartialWeightsDerivate();

    //@}

private:
    TString _workingTestPath;
    URANIE::DataServer::TDataServer *_tds;
    URANIE::Modeler::UMLP *_network;

};

#endif  // CPPUNITEST_TESTUMLP_H
