//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////

/*!
 \file   TFCNCode.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date     
 \brief Implementation of the class URANIE::Optimizer::TFCNCode

 \par Modifications
 \li M. Salvoldi (Areva/OCC) 2011/09/28
 - add a IFDEF WIN32 to replace rm -fr by del /Q on Windows platform
 
 */

// URANIE
#include "TFCNCode.h"
#include "UExceptions.h"

ClassImp (URANIE::Optimizer::TFCNCode)

namespace URANIE
{
namespace Optimizer
{
void TFCNCode::init()
{
    try
    {
        if (_tds != NULL)
        {
            if (_tds->getTuple() == NULL)
            {
                _tds->createTuple();
            }
        }
        _ninput = _tds->getNInputAttributes();
        _noutput = _tds->getNOutputAttributes();

        if (_noutput == 0)
            throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                    Form(
                            "TFCNCode::init [%s] ERROR\n the Number of output is not equal to 1",
                            _tds->GetName(), _noutput));

        Int_t ncpt = 1;
        Int_t nvar = _tds->getNAttributes();
        for (Int_t ivar = 0; ivar < nvar; ivar++)
        {
            if (_tds->getAttribute(ivar)->isOutput())
            {
                _soutput = _tds->getAttribute(ivar)->GetName();
                if (_soutput == _sCost)
                    _nCost = ncpt;
                ncpt++;
            }
        }
        cout << " ** TFCNCode::init _sCost[" << _sCost << "] _nCost[ " << _nCost
                << "]" << endl;

        // The size is increased by one for the iterator
        _dinputValue = new double[_ninput + _noutput + 1];
        _doutputValue = new double[_noutput + 1];

        clean();

        //_code->init();

        // Go to the working directory
        gSystem->cd(_code->getWorkingDirectory());
        // Make the Results directory for distribution
        TString sresultsdirectory = TString("UranieResults");
        if (!gSystem->OpenDirectory(sresultsdirectory))
            if (gSystem->mkdir(sresultsdirectory, kTRUE) == -1)
                throw URANIE::Exceptions::UExceptions(__FILE__,
                __LINE__,
                        Form(
                                "TLauncher::init ERROR : the subdirectory \"%s\" can not be created in the workindirectory \"%s\"",
                                sresultsdirectory.Data(),
                                _code->getWorkingDirectory().Data()));
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
        exit(-1);
    }

    catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }

}

void TFCNCode::clean()
{
    TString sworkindirectory = _code->getWorkingDirectory();

    // Place in the good directory
    if (sworkindirectory.Length() != 0)
        gSystem->cd(sworkindirectory);

    // Clean the subdirectory "UranieLauncher_*"
#ifdef WIN32
    gSystem->Exec("rd /s /q UranieLauncher_*");
#else
    gSystem->Exec("rm -fr UranieLauncher_*");
#endif
}
} // Fin du namespace Optimizer
} // Fin du namespace URANIE

