//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TObjectiveL1.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   fri jul 6 13:48:06 CEST 2007
 
 \brief Implementation of the class URANIE::Optimizer::TObjectiveL1
 
 */

// ROOT
#include "TLeaf.h"

// URANIE
#include "TObjectiveL1.h"

using std::cout;
using std::endl;

ClassImp (URANIE::Optimizer::TObjectiveL1)

namespace URANIE
{
namespace Optimizer
{
TObjectiveL1::TObjectiveL1(TString name) :
        TObjective(name)
{
    if (_blog)
        cout << "Builds  TObjectiveL1 [" << name << "]" << endl;
    SetNameTitle(name, Form("objectiveL1 function [%s]", name.Data()));
}

TObjectiveL1::~TObjectiveL1()
{
    if (_blog)
        cout << "detruit TObjectiveL1" << endl;
}

//////////////////////////////////////////////////////////
////  compute Objective
//////////////////////////////////////////////////////////

Double_t TObjectiveL1::getObjective(Double_t *dvalues, Int_t nrow)
{
    Int_t npat = _tds->getNPatterns();

    if (npat != nrow)
    {
        _dvalue = 0.0;
        return 0.0;
    }

    Double_t dres = 0;
    Double_t dvalstar, dvalhat, decart;

    TLeaf *leafstar = _tds->getTuple()->GetLeaf(_systar);

    for (Int_t ipat = 0; ipat < npat; ipat++)
    {
        _tds->getTuple()->GetEntry(ipat);

        dvalstar = leafstar->GetValue(0);
        dvalhat = dvalues[ipat];
        //decart = ( dvalstar - dvalhat ) / dvalstar ;

        decart = TMath::Abs(dvalstar - dvalhat);
        dres += decart;

        if (_blog)
        {
            cout << ipat << " - *[" << dvalstar << "]  ^[" << dvalhat << "]";
            cout << " e[" << decart << "]";
            cout << " res[" << dres << "] " << endl;
        }

    }

    _dvalue = dres;
    return _dweight * _dvalue;
}
void TObjectiveL1::printLog(Option_t *option)
{
    cout << "TObjectiveL1::printLog[" << option << "]" << endl;
    TObjective::printLog();
}

} // Fin du namespace Optimizer
} // Fin du namespace URANIE

