//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TObjectiveUncertainty.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   fri jul 6 13:48:06 CEST 2007
 
 \brief Implementation of the class URANIE::Optimizer::TObjectiveUncertainty
 
 */
#include <iostream>
using std::cout;
using std::endl;

// ROOT
#include "TLeaf.h"

// URANIE
#include "TObjectiveUncertainty.h"

ClassImp (URANIE::Optimizer::TObjectiveUncertainty)

namespace URANIE
{
namespace Optimizer
{
TObjectiveUncertainty::TObjectiveUncertainty(TString name) :
        TObjective(name), _systaruncert("")
{
    if (_blog)
        cout << "Builds  TObjectiveUncertainty [" << name << "]" << endl;
    SetNameTitle(name, Form("ObjectiveUncertainty function [%s]", name.Data()));
}

TObjectiveUncertainty::~TObjectiveUncertainty()
{
    if (_blog)
        cout << "detruit TObjectiveUncertainty" << endl;
}

//////////////////////////////////////////////////////////
////  compute Objective
//////////////////////////////////////////////////////////

Double_t TObjectiveUncertainty::getObjective(Double_t *dvalues, Int_t nrow)
{
    Int_t npat = _tds->getNPatterns();

    if (npat != nrow)
    {
        _dvalue = 0.0;
        return 0.0;
    }

    Double_t dres = 0;
    Double_t dvalstar, dvalstaruncert, dvalhat, decart;

    TLeaf *leafstar = _tds->getTuple()->GetLeaf(_systar);
    TLeaf *leafstaruncert = _tds->getTuple()->GetLeaf(_systaruncert);

    for (Int_t ipat = 0; ipat < npat; ipat++)
    {
        _tds->getTuple()->GetEntry(ipat);

        dvalstar = leafstar->GetValue(0);
        dvalstaruncert = leafstaruncert->GetValue(0);
        dvalhat = dvalues[ipat];
        //decart = ( dvalstar - dvalhat ) / dvalstar ;

        decart = (dvalstar - dvalhat);
        dres += decart * decart / TMath::Abs(dvalstaruncert);

        if (_blog)
        {
            cout << ipat << " - *[" << dvalstar << "] uncert[" << dvalstaruncert
                    << "]  ^[" << dvalhat << "]";
            cout << " e[" << decart << "]";
            cout << " res[" << dres << "] " << endl;
        }

    }

    _dvalue = dres;
    return _dweight * _dvalue;
}
void TObjectiveUncertainty::printLog(Option_t *option)
{
    cout << "TObjectiveUncertainty::printLog[" << option << "]" << endl;
    TObjective::printLog();
    cout << " _systaruncert [" << _systaruncert << "]" << endl;
}

} // Fin du namespace Optimizer
} // Fin du namespace URANIE

