//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizir_function.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2012/11/16 09:05:24
 
 \brief undefined :-(
 
 */

#include <stdlib.h>
#include <stdio.h>

// URANIE
#include "TVizir.h"
#include "TVizir_function.h"
#include "TCode.h"

// ROOT
#include "TMethodCall.h"

/**** fonction d'évaluation spécifique ****/
/** fonction analytique interprété **/
struct vzrcint
{
    URANIE::Optimizer::TVizir *viz;
    TMethodCall *decode, *hard, *soft;
};

static int vzrcint_eval(void *info, double *in, double *f)
{
    struct vzrcint *d = (struct vzrcint *) info;

    return d->viz->evaluer(in, f, d->decode, d->hard, d->soft);

}

static void vzrcint_free(vzrfitness_t *info)
{
    struct vzrcint *d = (struct vzrcint *) info->data;
    if (d->decode)
        delete d->decode;
    if (d->hard)
        delete d->hard;
    if (d->soft)
        delete d->soft;
    free(info->data);
    free(info);
}

static vzrfitness_t * vzrcint_objet(void *info)
{
    vzrfitness_t *ret;
    struct vzrcint *d;
    URANIE::Optimizer::TVizir *viz = (URANIE::Optimizer::TVizir *) info;

    // data
    d = (struct vzrcint *) malloc(sizeof(struct vzrcint));
    d->viz = viz;
    d->decode = viz->copyDecode();
    d->hard = viz->copyHardConstraint();
    d->soft = viz->copyCost();

    // protocole
    ret = (vzrfitness_t*) malloc(sizeof(vzrfitness_t));
    ret->data = (void *) d;
    ret->compute = &vzrcint_eval;
    ret->dealloc = &vzrcint_free;

    return ret;
}

vzrfitnessbuild_t *vzrcint_class(URANIE::Optimizer::TVizir *viz)
{
    vzrfitnessbuild_t *ret;

    ret = (vzrfitnessbuild_t *) malloc(sizeof(vzrfitnessbuild_t));
    ret->data = (void *) viz;
    ret->create = &vzrcint_objet;
    ret->dealloc = (void (*)(vzrfitnessbuild_t *)) &free;return
ret    ;
}

/** appel TCode **/
struct vzrprog
{
    URANIE::Optimizer::TVizir *viz;
    TMethodCall *decode, *hard;
    URANIE::Launcher::TCode *soft;
};

static int vzrprog_eval(void *info, double *in, double *f)
{
    struct vzrprog *d = (struct vzrprog *) info;

    return d->viz->evaluer(in, f, d->decode, d->hard, d->soft);
}

static void vzrprog_free(vzrfitness_t *info)
{
    struct vzrprog *d = (struct vzrprog *) info->data;
    if (d->decode)
        delete d->decode;
    if (d->hard)
        delete d->hard;
    if (d->soft)
        delete d->soft;
    free(info->data);
    free(info);
}

static vzrfitness_t * vzrprog_objet(void *info)
{
    vzrfitness_t *ret;
    struct vzrprog *d;
    URANIE::Optimizer::TVizir *viz = (URANIE::Optimizer::TVizir *) info;

    // data
    d = (struct vzrprog *) malloc(sizeof(struct vzrprog));
    d->viz = viz;
    d->decode = viz->copyDecode();
    d->hard = viz->copyHardConstraint();
    d->soft = viz->copyCode();

    // protocole
    ret = (vzrfitness_t*) malloc(sizeof(vzrfitness_t));
    ret->data = (void *) d;
    ret->compute = &vzrprog_eval;
    ret->dealloc = &vzrprog_free;

    return ret;
}

vzrfitnessbuild_t *vzrprog_class(URANIE::Optimizer::TVizir *viz)
{
    vzrfitnessbuild_t *ret;

    ret = (vzrfitnessbuild_t *) malloc(sizeof(vzrfitnessbuild_t));
    ret->data = (void *) viz;
    ret->create = &vzrprog_objet;
    ret->dealloc = (void (*)(vzrfitnessbuild_t *)) &free;return
ret    ;
}

/**** evenement generation ****/
/** save all **/
static void vzrsaveall_do(void *info, vzr_t *bis)
{
    UNUSED(bis); // ? UNUSED ?
    URANIE::Optimizer::TVizir *viz = (URANIE::Optimizer::TVizir *) info;

    viz->Results();
}

vzrevent_t *vzrsaveall(URANIE::Optimizer::TVizir *viz)
{
    vzrevent_t *ret;

    ret = (vzrevent_t *) malloc(sizeof(vzrevent_t));
    ret->data = (void *) viz;
    ret->react = &vzrsaveall_do;
    ret->dealloc = (void (*)(vzrevent_t *)) &free;return
ret    ;
}
