//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTestTMultiGenAnalyticalFunction.cxx
 \author Maud Salvoldi (OCC/Areva group)
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Optimizer::TestTMultiGenAnalyticalFunction
 
 Cree le : jeu d�c 22 11:27:34 CET 2011
 
 */

#define DBL_PRECISION_COMPARE 1.e-12
#define EPSILON 0.01 

// Uranie
#include "TestTMultiGenAnalyticalFunction.h"
#include "UDisableTest.h"

// ROOT
#include "TString.h"
#include "TROOT.h"
#include "TSystem.h" 
//fga 2015/01/12 #include "Api.h"

#include <iostream>
using namespace std;

TMultiGenAnalyticalFunctionTest::TMultiGenAnalyticalFunctionTest():
  _p2fRosenbrock(NULL), _p2fUnknown(NULL)
{
  gROOT->LoadMacro("TestUserFunctions.C");
//fga 2015/01/12     // Get the void * from the "rosenbrock" analytical function
//fga 2015/01/12     G__ClassInfo globalscope;
//fga 2015/01/12     G__MethodInfo method;
//fga 2015/01/12     long dummy;
//fga 2015/01/12     // resolve function overloading
//fga 2015/01/12     method = globalscope.GetMethod("rosenbrock", "double*,double*", &dummy);
//fga 2015/01/12     _p2fRosenbrock = method.PointerToFunc();

//fga 2015/01/12     // resolve function overloading
//fga 2015/01/12     method = globalscope.GetMethod("AnalyticalFunctionUnknwon",
//fga 2015/01/12             "double*,double*", &dummy);
//fga 2015/01/12     _p2fUnknown = method.PointerToFunc();

}

TMultiGenAnalyticalFunctionTest::~TMultiGenAnalyticalFunctionTest()
{

}

void TMultiGenAnalyticalFunctionTest::setUp()
{
    // Attributes of the study
    URANIE::DataServer::TAttribute *x = new URANIE::DataServer::TAttribute("x",
            -1.5, 1.5);
    x->setDefaultValue(-1.2);
    x->setStepValue(0.01);

    URANIE::DataServer::TAttribute *y = new URANIE::DataServer::TAttribute("y",
            -1.5, 1.5);
    y->setDefaultValue(1.);
    y->setStepValue(0.01);

    // Collection of attributes
    _tds = new URANIE::DataServer::TDataServer("tdsRosenbrock",
            "Optimize Code externe Rosenbrock via TDataServer");
    _tds->addAttribute(x);
    _tds->addAttribute(y);
    _tds->getAttribute("x")->delShare();
    _tds->getAttribute("y")->delShare();

}

void TMultiGenAnalyticalFunctionTest::tearDown()
{
    delete _tds;
    _tds = NULL;
}

////////////////////////////////
///                          ///
///         test             ///
///                          ///
////////////////////////////////
void TMultiGenAnalyticalFunctionTest::testTMultiGenAnalytiqueFunction()
{
    _tds->addAttribute("out");
    _tds->getAttribute("out")->setOutput();
    _tds->addAttribute("titi");
    _tds->getAttribute("titi")->setOutput();
    _tds->addAttribute("tata");
    _tds->getAttribute("tata")->setOutput();

    URANIE::Optimizer::TMultiGenAnalyticalFunction* topt1 =
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "out:titi:tata", "titi");
    topt1->unsetLog();    
    Int_t dim1 = (Int_t)(topt1->NDim());
    CPPUNIT_ASSERT_EQUAL(2, dim1);

    URANIE::Optimizer::TMultiGenAnalyticalFunction* topt2 =
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "out:titi:tata");
    topt2->unsetLog();       
    Int_t dim2 = (Int_t)(topt2->NDim());
    CPPUNIT_ASSERT_EQUAL(2, dim2);

    URANIE::Optimizer::TMultiGenAnalyticalFunction* topt3 =
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "out");
    topt3->unsetLog();        
    Int_t dim3 = (Int_t)(topt3->NDim());
    CPPUNIT_ASSERT_EQUAL(2, dim3);

    delete topt1;
    topt1 = NULL;
    delete topt2;
    topt2 = NULL;
    delete topt3;
    topt3 = NULL;
}

void TMultiGenAnalyticalFunctionTest::testTMultiGenAnalytiqueFunctionClone()
{
    _tds->addAttribute("out");
    _tds->getAttribute("out")->setOutput();
    _tds->addAttribute("titi");
    _tds->getAttribute("titi")->setOutput();

    URANIE::Optimizer::TMultiGenAnalyticalFunction* topt =
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "out:titi", "titi");
    topt->unsetLog();        
    Int_t dim1, dim2;
    dim1 = (Int_t)(topt->NDim());
    CPPUNIT_ASSERT_EQUAL(2, dim1);

    URANIE::Optimizer::TMultiGenAnalyticalFunction* topt2 = topt->Clone();
    dim2 = topt2->NDim();

    CPPUNIT_ASSERT_EQUAL(dim1, dim2);

    delete topt;
    topt = NULL;
    delete topt2;
    topt2 = NULL;
}

void TMultiGenAnalyticalFunctionTest::testTMultiGenAnalyticalFunctionSetLog()
{
    _tds->addAttribute("out");
    _tds->getAttribute("out")->setOutput();
    _tds->addAttribute("titi");
    _tds->getAttribute("titi")->setOutput();

    URANIE::Optimizer::TMultiGenAnalyticalFunction* topt =
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "out:titi", "titi");
    topt->unsetLog();
    CPPUNIT_ASSERT(!topt->getLog());
    
    topt->setLog();    
    CPPUNIT_ASSERT(topt->getLog());
    
    topt->changeLog();    
    CPPUNIT_ASSERT(!topt->getLog());
    
    topt->changeLog();    
    CPPUNIT_ASSERT(topt->getLog());
    
    delete topt;
    topt = NULL;
    
    
}


////////////////////////////////
///                          ///
///     throwexception       ///
///                          ///
////////////////////////////////
void TMultiGenAnalyticalFunctionTest::throwExceptionTMultiGenAnalyticalFunctionConstructorTdsNull()
{
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(NULL,
                    "rosenbrock", "x:y", "out", ""),
            URANIE::Exceptions::UErrorExceptions);
}

void TMultiGenAnalyticalFunctionTest::throwExceptionTMultiGenAnalyticalFunctionConstructorFalseInput()
{
    _tds->addAttribute("out");
    _tds->getAttribute("out")->setOutput();

    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:toto:y", "out"),
            URANIE::Exceptions::UErrorExceptions);
}

void TMultiGenAnalyticalFunctionTest::throwExceptionTMultiGenAnalyticalFunctionConstructorFalseOutput()
{
    _tds->addAttribute("out");
    _tds->getAttribute("out")->setOutput();

    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "toto", "out"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "y", "out"),
            URANIE::Exceptions::UErrorExceptions);
}
void TMultiGenAnalyticalFunctionTest::throwExceptionTMultiGenAnalyticalFunctionConstructorFalseCost()
{
    _tds->addAttribute("out");
    _tds->getAttribute("out")->setOutput();
    _tds->addAttribute("titi");
    _tds->getAttribute("titi")->setOutput();
    _tds->addAttribute("tata");
    _tds->getAttribute("tata")->setOutput();

    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "out:titi", "tata"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "out:titi:tata", "toto"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "out:titi:tata", "y"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "rosenbrock", "x:y", "out:titi:tata", "out:titi"),
            URANIE::Exceptions::UErrorExceptions);
}

void TMultiGenAnalyticalFunctionTest::throwExceptionTMultiGenAnalyticalFunctionConstructorUnknowFunction()
{
    _tds->addAttribute("toto");
    _tds->getAttribute("toto")->setOutput();
    _tds->addAttribute("titi");
    _tds->getAttribute("titi")->setOutput();

    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenAnalyticalFunction(_tds,
                    "functionUnknown", "x:y", "toto:titi"),
            URANIE::Exceptions::UErrorExceptions);
}
