//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTOptimizer.cxx
 \author Cedric Forgues <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Optimizer::TOptimizer
 
 Cree le : mer nov 21 07:59:24 2007

 /par Modifications
 /li F. Gaudier 2012-01-30 14:03:42
 - accept now to pass the void * to an analytical function
 - change position of the CPPUNIT_ASSERT_EQUAL(target, value, precision) tests.
 /li M. Salvoldi (Areva/OCC) 2011/12/23
 - add uits tests.
 /li M. Salvoldi (Areva/OCC) 2011/09/28
 - Add a IFDEF WIN32 to access to ROSENBROCK code in testRosenbrockCodeInputFileRecreateOutputDataser().
 - Add a IFDEF WIN32 to replace rm by rmdir/del on Windows platform.
 /li C. Forgues (2010-12-21 10:45:59)
 - Correction des tests  testRosenbrockCodeInputFileRecreateOutputDataser et testRosenbrockCodeInputsKeyAndRowRecreate: recherche en relatif de rosenbrock. testRosenbrockCodeInputFileRecreateOutputDataser fait maintenant correctement appel � rosenbrock -r et non -k.
 - Correction des valeurs de r�f�rence pour les tests
 /li F. Gaudier (2010-12-14 10:45:58)
 - Activate the CLUB_FOUND option to pass the Define WITH_CLUB
 - adapt all the TU for the new rosenbrock code (Option -v -k -r) and name of input and output files
 - add the cleanWorkingDirectories method and put it in the two methods setUp and tearDown
 - ADD two TU
 1. \b testRosenbrockCodeInputsKeyAndRowRecreate for DI113.
 A attribute with several input files in the optimization process
 2. \b testRosenbrockCodeInputFileRecreateOutputDataser
 A attribute with the TInputFileRecreate input file
 /li Fabrice Gaudier (2010-04-13 09:35:32)
 Change the name of the test testRosenbrockCode to testRosenbrockCodeOutputKey, and test the size of the returned TDS
 Add the same test when the output file is a TOutputFileRow
 
 */

#define DBL_PRECISION_COMPARE 1.e-12
#define DBL_PRECISION_COMPARE_LOW 1.e-4
#define EPSILON 0.01 

// Uranie
#include "TestTOptimizer.h"
#include "TOutputFileKey.h"
#include "TOutputFileRow.h"
#include "TOutputFileColumn.h"
#include "TOutputFileDataServer.h"
#include "TObjective.h"
#include "TOptimizer.h"
#include "UDisableTest.h"
#include "code_paths.h"
#include "uranie_commands.h"

// ROOT
#include "TString.h"
#include "TROOT.h"
#include "TSystem.h" 
//fga 2015/01/12 #include "Api.h"

#include <iostream>
using namespace std;

using namespace URANIE::DataServer;
using namespace URANIE::Launcher;
using namespace URANIE::Optimizer;

TOptimizerTest::TOptimizerTest() :
        _tds1(NULL), _foutKey(NULL), _foutRow(NULL), _foutDataServer(NULL), _foutCol(NULL), _tcode(NULL), _topt(NULL), _pX(NULL), _pY(NULL)
{
    _workingTestPath = gSystem->pwd();
    gROOT->LoadMacro("TestUserFunctions.C");

    _sReferenceDirectory = gSystem->pwd();
#ifdef WIN32
    _sReferenceDirectory += "\\";
#else
    _sReferenceDirectory += "/";
#endif  
    //cout << " *** _sReferenceDirectory[" << _sReferenceDirectory << "]" << endl;

//fga 2015/01/12     // Get the void * from the "rosenbrock" analytical function
//fga 2015/01/12     G__ClassInfo globalscope;
//fga 2015/01/12     G__MethodInfo method;
//fga 2015/01/12     long dummy;
//fga 2015/01/12     // resolve function overloading
//fga 2015/01/12     method = globalscope.GetMethod("rosenbrock", "double*,double*", &dummy);
//fga 2015/01/12     _p2fRosenbrock = method.PointerToFunc();

//fga 2015/01/12     // Get the void * from the "rosenbrock" analytical function
//fga 2015/01/12     G__ClassInfo globalscope2;
//fga 2015/01/12     G__MethodInfo method2;
//fga 2015/01/12     long dummy2;
//fga 2015/01/12     // resolve function overloading
//fga 2015/01/12     method2 = globalscope2.GetMethod("rosenbrock2", "double*,double*", &dummy2);
//fga 2015/01/12     _p2fRosenbrock2 = method2.PointerToFunc();
}

TOptimizerTest::~TOptimizerTest()
{
}

void TOptimizerTest::setUp()
{
    cout << endl;
    cout << " ################## setUp #################" << endl;
    // Create the DataServer
    _tds1 = new URANIE::DataServer::TDataServer("testOptimizer",
            "test of classes");

    _foutKey = NULL;
    _foutRow = NULL;
    _foutDataServer = NULL;
    _foutCol = NULL;
    _topt = NULL;
    _tcode = NULL;

    _pX = new URANIE::DataServer::TAttribute("x", -1.5, 1.5);
    _pX->setDefaultValue(-1.2);
    _pX->setStepValue(0.1);

    _pY = new URANIE::DataServer::TAttribute("y", -1.5, 1.5);
    _pY->setDefaultValue(1.);
    _pY->setStepValue(0.1);

    gSystem->cd(_workingTestPath);
//   cleanWorkingDirectories("setUp");
    
    // Backup of the file input_rosenbrock_with_keys.dat
    gSystem->CopyFile("input_rosenbrock_with_keys.dat","input_rosenbrock_with_keys.dat_bkp",kTRUE);
    
    cout << " ############## End Of SetUp ##############" << endl;
    cout << " ##########################################" << endl;
}

void TOptimizerTest::tearDown()
{
    cout << " ################ tearDown ################" << endl;
    if (_foutKey != NULL)
    {
        delete _foutKey;
        _foutKey = NULL;
    }
    if (_foutRow != NULL)
    {
        delete _foutRow;
        _foutRow = NULL;
    }
    if (_foutDataServer != NULL)
    {
        delete _foutDataServer;
        _foutDataServer = NULL;
    }
    if (_foutCol != NULL)
    {
        delete _foutCol;
        _foutCol = NULL;
    }

    if (_tcode != NULL)
    {
        delete _tcode;
        _tcode = NULL;
    }

    if (_topt != NULL)
    {
        delete _topt;
        _topt = NULL;
    }

    if (_tds1 != NULL)
    {
        delete _tds1;
        _tds1 = NULL;
    }

    if (_pX != NULL)
    {
        delete _pX;
        _pX = NULL;
    }
    if (_pY != NULL)
    {
        delete _pY;
        _pY = NULL;
    }

    gSystem->cd(_workingTestPath);
    gSystem->CopyFile("input_rosenbrock_with_keys.dat_bkp","input_rosenbrock_with_keys.dat",kTRUE);
    cleanWorkingDirectories("tearDown");    
    
    cout << " ############# End Of TearDown ############" << endl;
    cout << " ##########################################" << endl;
}

void TOptimizerTest::cleanWorkingDirectories(const char *option)
{
    cout << " ** TOptimizerTest::cleanWorkingDirectories option[" << option
	 << "] _workingTestPath[" << _workingTestPath << "]" << endl;
			
			
    UranieTests::removeFolder(gSystem->pwd(),"URANIE");
    UranieTests::removeFolder(gSystem->pwd(),"tmpLanceurUranie");
    UranieTests::removeFile(gSystem->pwd(),"uranieoptimizer.dat");
    UranieTests::removeFile(gSystem->pwd(),"testOptimizer.out");
    UranieTests::removeFile(gSystem->pwd(),"testRosenbrockFunction.out");
    UranieTests::removeFile(gSystem->pwd(),"_tds_rosenbrock_.dat_bkp");
    UranieTests::removeFile(gSystem->pwd(),"input_rosenbrock_with_keys.dat_bkp");
    UranieTests::removeFile(gSystem->pwd(),"_output_rosenbrock_withDataServer_.dat");
    UranieTests::removeFile(gSystem->pwd(),"_output_rosenbrock_with_keys_.dat");
    UranieTests::removeFile(gSystem->pwd(),"_output_rosenbrock_with_values_cols_.dat");
    UranieTests::removeFile(gSystem->pwd(),"_output_rosenbrock_with_values_rows_.dat");
    UranieTests::removeFile(gSystem->pwd(),"_Save_testOptimizer_Time_20161012114207_Iteration_9_.dat");
    UranieTests::removeFile(gSystem->pwd(),"_Save_testOptimizer_Time_20161012123849_Iteration_9_.dat");
    UranieTests::removeFile(gSystem->pwd(),"tds_rosenbrock_sampling_.root");
    UranieTests::removeFile(gSystem->pwd(),"_dataserver_.root");

}


////////////////////////////////
///                          ///
///         Tests            ///
///                          ///
////////////////////////////////

void TOptimizerTest::testRosenbrockCodeOutputKey()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();
    
    // Study Attributes
    _pX->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pY->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, -1.2, DBL_PRECISION_COMPARE);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1., DBL_PRECISION_COMPARE);

    // Output file
    _foutKey = new URANIE::Launcher::TOutputFileKey(
            "_output_rosenbrock_with_keys_.dat");
    _foutKey->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -k");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));
    _tcode->addOutputFile(_foutKey);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->setTolerance(0.001);
    _topt->setMaxIterations(3000);
    _topt->optimize("nointermed");

    //no significative test : depend on OS tested
    //CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257); 
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.998121828756797, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.995908405941382, dcrt, 0.01);
    _tds1->getAttribute("fval")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockCodeOutputKeyNewInput()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();
    
    // Study Attributes
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    _tds1->addAttribute("xshift","x-0.1");
    _tds1->getAttribute("xshift")->setFileKey(_sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"),"x");
    _tds1->addAttribute("yshift","y+0.2");
    _tds1->getAttribute("yshift")->setFileKey(_sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"),"y");
    
    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, -1.2, DBL_PRECISION_COMPARE);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1., DBL_PRECISION_COMPARE);

    // Output file
    _foutKey = new URANIE::Launcher::TOutputFileKey(
            "_output_rosenbrock_with_keys_.dat");
    _foutKey->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -k");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));
    _tcode->addOutputFile(_foutKey);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(3000);
    _topt->optimize("nointermed");

    //no significative test : depend on OS tested
    //CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257); 
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.1, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8, dcrt, 0.01);
    _tds1->getAttribute("fval")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockCodeOutputKeyNewInputAndOutput()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();
    
    // Study Attributes
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    _tds1->addAttribute("xshift","x-0.1");
    _tds1->getAttribute("xshift")->setFileKey(_sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"),"x");
    _tds1->addAttribute("yshift","y+0.2");
    _tds1->getAttribute("yshift")->setFileKey(_sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"),"y");
    
    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, -1.2, DBL_PRECISION_COMPARE);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1., DBL_PRECISION_COMPARE);

    // Output file
    _foutKey = new URANIE::Launcher::TOutputFileKey(
            "_output_rosenbrock_with_keys_.dat");
    _foutKey->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -k");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));
    _tcode->addOutputFile(_foutKey);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->addOutputVariable("fval+1:fval*fval:fval*3");
    _topt->selectCost("fval+1");
    _topt->setTolerance(0.001);
    _topt->setMaxFunctionCalls(3000);
    _topt->optimize("nointermed");

    //no significative test : depend on OS tested
    //CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257); 
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.1, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8, dcrt, 0.01);
    _tds1->getAttribute("fval+1")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockCodeOutputKeyNewOutputSum()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();
    
    // Study Attributes
    _pX->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pY->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, -1.2, DBL_PRECISION_COMPARE);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1., DBL_PRECISION_COMPARE);

    // Output file
    _foutKey = new URANIE::Launcher::TOutputFileKey(
            "_output_rosenbrock_with_keys_.dat");
    _foutKey->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -k");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));
    _tcode->addOutputFile(_foutKey);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->addOutputVariable("fval+1:fval*fval:fval*3");
    _topt->selectCost("fval+1");
    _topt->setTolerance(0.0001);
    _topt->setMaxFunctionCalls(3000);
    _topt->optimize("nointermed");

    //no significative test : depend on OS tested
    //CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257); 
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.9999, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.995908405941382, dcrt, 0.01);
    _tds1->getAttribute("fval+1")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}


void TOptimizerTest::testRosenbrockCodeOutputKeyNewOutputMult()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();
    
    // Study Attributes
    _pX->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pY->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, -1.2, DBL_PRECISION_COMPARE);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1., DBL_PRECISION_COMPARE);

    // Output file
    _foutKey = new URANIE::Launcher::TOutputFileKey(
            "_output_rosenbrock_with_keys_.dat");
    _foutKey->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -k");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));
    _tcode->addOutputFile(_foutKey);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->addOutputVariable("fval+1:fval*fval:fval*3");
    _topt->selectCost("fval*3");
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(3000);
    _topt->optimize("nointermed");

    //no significative test : depend on OS tested
    //CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257); 
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.998121828756797, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.995908405941382, dcrt, 0.01);
    _tds1->getAttribute("fval*3")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}


void TOptimizerTest::testRosenbrockCodeOutputKeyNewOutputSquared()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();
    
    // Study Attributes
    _pX->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pY->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, -1.2, DBL_PRECISION_COMPARE);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1., DBL_PRECISION_COMPARE);

    // Output file
    _foutKey = new URANIE::Launcher::TOutputFileKey(
            "_output_rosenbrock_with_keys_.dat");
    _foutKey->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -k");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));
    _tcode->addOutputFile(_foutKey);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->addOutputVariable("fval+1:fval*fval:fval*3");
    _topt->selectCost("fval*fval");
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(3000);
    _topt->optimize("nointermed");

    //no significative test : depend on OS tested
    //CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257); 
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.998121828756797, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.995908405941382, dcrt, 0.01);
    _tds1->getAttribute("fval*fval")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockCodeOutputRow()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // Study Attributes
    _pX->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pY->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, -1.2, DBL_PRECISION_COMPARE);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1., DBL_PRECISION_COMPARE);

    // Output file
    _foutRow = new URANIE::Launcher::TOutputFileRow(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutRow->addAttribute("fval", 3);

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -k");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));

    _tcode->addOutputFile(_foutRow);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->setTolerance(0.001);
    _topt->setMaxIterations(3000);
    _topt->optimize("nointermed");

    //  no significative test : depend on OS tested
    //   CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257);
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 0.998121828756797, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 0.995908405941382, 0.01);
    _tds1->getAttribute("fval")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockCodeInputsKeyAndRowRecreate()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use case
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -k");
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->optimize("same");

    Int_t npatkey = _tds1->getNPatterns();
    Double_t dcrtxkey = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrtxkey);
    Double_t dcrtykey = 0.0;
    _tds1->getAttribute("y")->getDefaultValue(dcrtykey);
    Double_t dcrtfvalkey = 0.0;
    _tds1->getAttribute("fval")->getDefaultValue(dcrtfvalkey);

    // no-significative TU which depend on OS
    //   CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 83);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.994630330308481, dcrtxkey, 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.983759407864929, dcrtykey, 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrtfvalkey, DBL_PRECISION_COMPARE_LOW);

    tearDown();
    setUp();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use case
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -r");
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->optimize("same,nointermed");

    CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), npatkey);
    Double_t dcrtxrowrecreate = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrtxrowrecreate);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrtxrowrecreate, dcrtxkey,
            DBL_PRECISION_COMPARE);
    Double_t dcrtyrowrecreate = 0.0;
    _tds1->getAttribute("y")->getDefaultValue(dcrtyrowrecreate);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrtyrowrecreate, dcrtykey,
            DBL_PRECISION_COMPARE);
    Double_t dcrtvalrowrecreate = 0.0;
    _tds1->getAttribute("fval")->getDefaultValue(dcrtvalrowrecreate);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrtvalrowrecreate, dcrtfvalkey,
            DBL_PRECISION_COMPARE);

}

void TOptimizerTest::testRosenbrockCodeInputKeyOutputColumn()

{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // Study Attributes
    _pX->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pY->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, -1.2, DBL_PRECISION_COMPARE);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1., DBL_PRECISION_COMPARE);

// Output file
    _foutCol = new URANIE::Launcher::TOutputFileColumn(
            "_output_rosenbrock_with_values_cols_.dat");
    _foutCol->addAttribute("fval", 3);
    _foutCol->unsetLog();

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -k");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));
    _tcode->unsetLog();

    _tcode->addOutputFile(_foutCol);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->setTolerance(0.001);
    _topt->setMaxIterations(3000);
    _topt->optimize("nointermed");

    //no significative test : depend on OS tested
    //CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257); 
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.998121828756797, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.995908405941382, dcrt, 0.01);
    _tds1->getAttribute("fval")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockCodeInputFileRecreateOutputDataser()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -r");
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->unsetLog();
    _topt->optimize("same,nointermed");

    Double_t dcrt = 0.0;

// no significative test : depend on OS tested
//   CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 83);
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.998121828756797, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.983759407864929, dcrt, 0.01);
    _tds1->getAttribute("fval")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockCodeInputFileRecreateOutputKey()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutKey = new URANIE::Launcher::TOutputFileKey(
            "_output_rosenbrock_with_keys_.dat");
    _foutKey->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -r");
    _tcode->addOutputFile(_foutKey);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->optimize("same,nointermed");

    Double_t dcrt = 0.0;

    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 0.999586305048755, EPSILON);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 0.999586305048755, EPSILON);

}

void TOptimizerTest::testRosenbrockCodeInputFileRecreateOutputRow()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutRow = new URANIE::Launcher::TOutputFileRow(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutRow->addAttribute("fval", 3);

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -r");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));

    _tcode->addOutputFile(_foutRow);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->setTolerance(0.001);
    _topt->setMaxIterations(3000);
    _topt->optimize("");

    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 0.998121828756797, EPSILON);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1.01302705754572, EPSILON);

}

void TOptimizerTest::testRosenbrockCodeInputFileRecreateOutputColumn()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file 
    _foutCol = new URANIE::Launcher::TOutputFileColumn(
            "_output_rosenbrock_with_values_cols_.dat");
    _foutCol->addAttribute("fval", 3);

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -r");
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));

    _tcode->addOutputFile(_foutCol);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);

    _topt->optimize("nointermed");

    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.998121828756797, dcrt, EPSILON);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.983759407864929, dcrt, EPSILON);
    _tds1->getAttribute("fval")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockFunctionSimplex()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->optimize("same");

    Double_t dvalx, dvaly, dvalc;
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out")->getDefaultValue(dvalc);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.00112713046e+00, dvalx,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.001912376519e+00, dvaly,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dvalc, DBL_PRECISION_COMPARE_LOW);
}

void TOptimizerTest::rosenbrockP(double *x, Double_t *y)
{
  Double_t dx1 = x[0], dx2 = x[1];
  Double_t da=10.0, db=1.0;
  
  y[0] = da * (dx2-dx1*dx1)*(dx2-dx1*dx1) + db*(1.0-dx1)*(1.0-dx1);
  
}

void TOptimizerTest::testRosenbrockFunctionPointerSimplex()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, TOptimizerTest::rosenbrockP, "x:y", "out");
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->optimize("same");

    Double_t dvalx, dvaly, dvalc;
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out")->getDefaultValue(dvalc);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.00112713046e+00, dvalx,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.001912376519e+00, dvaly,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dvalc, DBL_PRECISION_COMPARE_LOW);
}

void TOptimizerTest::testRosenbrockFunctionSimplexNewInput()
{

    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);
    
    // Study Attributes
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    _tds1->addAttribute("xshift","x-0.1");
    _tds1->addAttribute("yshift","y+0.2");

    // Optimizer process
#ifdef ROOT5
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _p2fRosenbrock, "xshift:yshift", "out");
#else
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "xshift:yshift", "out");
#endif
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->optimize("same");
    
    Double_t dcrt = 0.0;
    //no significative test : depend on OS tested
    //CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257); 
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.1, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8, dcrt, 0.01);
    _tds1->getAttribute("out")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockFunctionSimplexNewInputAndOutput()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);
    
    // Study Attributes
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    _tds1->addAttribute("xshift","x-0.1");
    _tds1->addAttribute("yshift","y+0.2");

    // Optimizer process
#ifdef ROOT5
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _p2fRosenbrock, "xshift:yshift", "out");
#else
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "xshift:yshift", "out");
#endif
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->addOutputVariable("out+1:out*out:out*3");
    _topt->selectCost("out+1");
    _topt->optimize("same");   //Directory where we can find references input files

    Double_t dcrt = 0.0;
    //no significative test : depend on OS tested
    //CPPUNIT_ASSERT_EQUAL(_tds1->getNPatterns(), 257); 
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.1, dcrt, 0.01);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8, dcrt, 0.01);
    _tds1->getAttribute("out+1")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, dcrt, DBL_PRECISION_COMPARE_LOW);

}

void TOptimizerTest::testRosenbrockFunctionSimplexNewOutputSum()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
#ifdef ROOT5
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _p2fRosenbrock, "", "out");
#else
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");
#endif
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->addOutputVariable("out+1:out*out:out*3");
    _topt->selectCost("out+1");
    _topt->optimize("same");

    Double_t dvalx, dvaly, dvalc;
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out+1")->getDefaultValue(dvalc);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.00112713046e+00, dvalx,
            0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.001912376519e+00, dvaly,
            0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, dvalc, DBL_PRECISION_COMPARE_LOW);
}

void TOptimizerTest::testRosenbrockFunctionSimplexNewOutputMult()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
#ifdef ROOT5
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _p2fRosenbrock, "", "out");
#else
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");
#endif
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->addOutputVariable("out+1:out*out:out*3");
    _topt->selectCost("out*3");
    _topt->optimize("same");

    Double_t dvalx, dvaly, dvalc;
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out*3")->getDefaultValue(dvalc);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.00166150660036, dvalx,
            0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.001912376519e+00, dvaly,
            0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dvalc, DBL_PRECISION_COMPARE_LOW);
}

void TOptimizerTest::testRosenbrockFunctionSimplexNewOutputSquared()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
#ifdef ROOT5
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _p2fRosenbrock, "", "out");
#else
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");
#endif
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->addOutputVariable("out+1:out*out:out*3");
    _topt->selectCost("out*out");
    _topt->optimize("same");

    Double_t dvalx, dvaly, dvalc;
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out*out")->getDefaultValue(dvalc);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.00112713046e+00, dvalx,
            0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.997070906564473, dvaly,
            0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dvalc, DBL_PRECISION_COMPARE_LOW);
}

void TOptimizerTest::testRosenbrockFunctionMigrad()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kMigrad);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->optimize("same");

    Double_t dvalx, dvaly, dvalc;
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out")->getDefaultValue(dvalc);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.999635010309e-01, dvalx,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.99927902442e-01, dvaly,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dvalc, DBL_PRECISION_COMPARE_LOW);
}

void TOptimizerTest::testRosenbrockFunctionCombined()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kCombined);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->optimize("same");

    Double_t dvalx, dvaly, dvalc;
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out")->getDefaultValue(dvalc);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.999635010309e-01, dvalx,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.99927902442e-01, dvaly,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dvalc, DBL_PRECISION_COMPARE_LOW);
}

void TOptimizerTest::testRosenbrockFunctionScn()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kScan);
    _topt->setTolerance(0.0001);
    _topt->setMaxIterations(300);
    _topt->optimize("same");

    Double_t dvalx, dvaly, dvalc;
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out")->getDefaultValue(dvalc);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.179997804098, dvalx,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.020000640123, dvaly,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(6.13916466062575, dvalc,
            DBL_PRECISION_COMPARE_LOW);
}

void TOptimizerTest::testOptimizeRosenbrockFunctionWithOneConstantVariable()
{
    _pX->setStepValue(0.01);
    _pY->setStepValue(0.01);
    Double_t dValRef = 0.25;
    Double_t dExpectedXValue = -0.5;
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock2", "x:y", "out:out2");
    _topt->unsetLog();
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->setTolerance(0.000001);
    _topt->setMaxIterations(300);
    _topt->selectCost("out2");
    _pY->setDefaultValue(dValRef);
    _topt->fixParameter("y");
    _topt->optimize("same");

    Double_t dvalx, dvaly, dvalc;
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out2")->getDefaultValue(dvalc);
// cout<<endl<<"##################    "<<dvalx<<" : "<<dvaly<<" : "<<dvalc<<endl;
// _tds1->exportData("_optimizer_rosenbrock_constant_.dat","x:y:out:out2");
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dExpectedXValue, dvalx,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dValRef, dvaly,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dvalc, DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_EQUAL(4,_tds1->getNAttributes());
    _topt->invertParameters();
    _topt->optimize("same");
    
    _pX->getDefaultValue(dvalx);
    _pY->getDefaultValue(dvaly);
    _tds1->getAttribute("out2")->getDefaultValue(dvalc);
// cout<<endl<<"##################    "<<dvalx<<" : "<<dvaly<<" : "<<dvalc<<endl;
// _tds1->exportData("_optimizer_rosenbrock_constant_.dat","x:y:out:out2");
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dExpectedXValue, dvalx,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dValRef, dvaly,
            DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dvalc, DBL_PRECISION_COMPARE_LOW);
    CPPUNIT_ASSERT_EQUAL(4,_tds1->getNAttributes());
    
}


void TOptimizerTest::testOptimizersetToleranceAndgetTolerance()
{
    Double_t tol1, tol2;

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");

    tol1 = _topt->getTolerance();
    _topt->setTolerance(0.001);
    tol2 = _topt->getTolerance();

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.01, tol1, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.001, tol2, DBL_PRECISION_COMPARE);

}

void TOptimizerTest::testOptimizersetMaxIterationsAndgetMaxIterations()
{
    Int_t niter1, niter2;

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");

    niter1 = _topt->getMaxIterations();
    _topt->setMaxIterations(3000);
    niter2 = _topt->getMaxIterations();

    CPPUNIT_ASSERT_EQUAL(50, niter1);
    CPPUNIT_ASSERT_EQUAL(3000, niter2);
}

void TOptimizerTest::testOptimizersetMaxFunctionCallsAndgetMaxFunctionCalls()
{
    Int_t nfunction1, nfunction2;

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");

    nfunction1 = _topt->getMaxFunctionCalls();
    _topt->setMaxFunctionCalls(30000);
    nfunction2 = _topt->getMaxFunctionCalls();

    CPPUNIT_ASSERT_EQUAL(10000, nfunction1);
    CPPUNIT_ASSERT_EQUAL(30000, nfunction2);
}

void TOptimizerTest::testOptimizersetMethodAndgetMethod()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");

    CPPUNIT_ASSERT_EQUAL(URANIE::Optimizer::TOptimizer::kMigrad,
            _topt->getMethod());

    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    CPPUNIT_ASSERT_EQUAL(URANIE::Optimizer::TOptimizer::kSimplex,
            _topt->getMethod());

    _topt->setMethod(URANIE::Optimizer::TOptimizer::kCombined);
    CPPUNIT_ASSERT_EQUAL(URANIE::Optimizer::TOptimizer::kCombined,
            _topt->getMethod());

    _topt->setMethod(URANIE::Optimizer::TOptimizer::kScan);
    CPPUNIT_ASSERT_EQUAL(URANIE::Optimizer::TOptimizer::kScan,
            _topt->getMethod());

    _topt->setMethod(URANIE::Optimizer::TOptimizer::kMigrad);
    CPPUNIT_ASSERT_EQUAL(URANIE::Optimizer::TOptimizer::kMigrad,
            _topt->getMethod());
}

void TOptimizerTest::testOptimizersetPrintLevelAndgetLevel()
{
    Int_t level1, level2;

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");

    level1 = _topt->getPrintLevel();
    _topt->setPrintLevel(4);
    level2 = _topt->getPrintLevel();

    CPPUNIT_ASSERT_EQUAL(0, level1);
    CPPUNIT_ASSERT_EQUAL(4, level2);
}

void TOptimizerTest::testTOptimizerSetLog()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Optimizer process
    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "", "out");
    _topt->unsetLog();    
    CPPUNIT_ASSERT(!_topt->getLog());
    
    _topt->setLog();    
    CPPUNIT_ASSERT(_topt->getLog());
    
    _topt->changeLog();    
    CPPUNIT_ASSERT(!_topt->getLog());
    
    _topt->changeLog();    
    CPPUNIT_ASSERT(_topt->getLog());
    
}


void TOptimizerTest::testOptimizeObjective()
{

    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();
    
    TString inputFileEvalCode =  "input_rosenbrock_with_keys.dat";
        
    // The a attribute of the use case
    TAttribute *a = new TAttribute("a", 9.0,11.0);
    a->setDefaultValue(9.9);
    a->setStepValue(0.01);
    a->setFileKey(inputFileEvalCode);
    
    // The b attribute of the use case
    TAttribute *b = new TAttribute("b", 0.0,2.0);
    b->setDefaultValue(1.1);
    b->setStepValue(0.01);
    b->setFileKey(inputFileEvalCode);
    
    // The c attribute of the use case
    TAttribute *c = new TAttribute("c", -1.0,1.0);
    c->setDefaultValue(0.1);
    c->setStepValue(0.01);
    c->setFileKey(inputFileEvalCode);
    

    delete _tds1;
    // Define the DataServer and add the attributes
    _tds1 = new TDataServer("tds", "Optimize Code externe Rosenbrock via TDataServer");
    _tds1->addAttribute(a);
    _tds1->addAttribute(b);
    _tds1->addAttribute(c);

    // The output file of the code where values are stored in 
    _foutDataServer = new TOutputFileDataServer("_output_rosenbrock_withDataServer_.dat");
   _foutDataServer->addAttribute("fval");
   
    // Create an TCode object for my code
    _tcode = new TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -o");
    
    // Add _tcode output file
    _tcode->addOutputFile( _foutDataServer);
    
    
    TDataServer *tdsref = new TDataServer("TDSRef", "Objectives values");
    tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
    
    // Create an TOptimizer object from TDataServer and TCode objects
    _topt = new TOptimizer(_tds1, _tcode);  
    _topt->setMethod(TOptimizer::kSimplex);
    _topt->setTolerance(1e-3);

    // add an objective   
    _topt->addObjective("obj1",
                tdsref,
                "fstar",
                _foutDataServer,
                "fval");
    
    gSystem->CopyFile("_tds_rosenbrock_.dat","_tds_rosenbrock_.dat_bkp",kTRUE);
    
    _topt->optimize();
    _tds1->exportData("_tds_rosenbrock_.dat");
    
    Double_t dcrta;
    _tds1->getAttribute("a")->getDefaultValue(dcrta);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10.0, dcrta, 2*1e-2);
    Double_t dcrtb;
    _tds1->getAttribute("b")->getDefaultValue(dcrtb);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, dcrtb, 2*1e-2);
    Double_t dcrtc;
    _tds1->getAttribute("c")->getDefaultValue(dcrtc);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrtc, 2*1e-2);
    
    a->delShare();
    b->delShare();
    c->delShare();
 
    delete tdsref;
    gSystem->CopyFile("_tds_rosenbrock_.dat_bkp","_tds_rosenbrock_.dat",kTRUE);
}

void TOptimizerTest::testOptimizeSeveralObjectives()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    URANIE::DataServer::TDataServer * tds =
            new URANIE::DataServer::TDataServer();

    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();
    // Definition of the attributes
    TString sJDDReference = gSystem->pwd();
#ifdef WIN32
    sJDDReference += "\\";
#else
    sJDDReference += "/";
#endif
    sJDDReference += "input_rosenbrock_with_keys.dat";

    tds->addAttribute("a", 5.0, 15.0);
    tds->getAttribute("a")->setFileKey(sJDDReference, "a");
    tds->getAttribute("a")->setDefaultValue(6.0);
    tds->getAttribute("a")->setStepValue(0.1);

    tds->addAttribute("b", 0.0, 2.0);
    tds->getAttribute("b")->setFileKey(sJDDReference, "b");
    tds->getAttribute("b")->setDefaultValue(1.5 );
    tds->getAttribute("b")->setStepValue(0.1);

    tds->addAttribute("c", -1.0, 2.0);
    tds->getAttribute("c")->setFileKey(sJDDReference, "c");
    tds->getAttribute("c")->setDefaultValue(1.0);
    tds->getAttribute("c")->setStepValue(0.1);
    
    // Definition of the output file of the code
    URANIE::Launcher::TOutputFileDataServer *fout =
            new URANIE::Launcher::TOutputFileDataServer(
                    "_output_rosenbrock_withDataServer_.dat");
    fout->addAttribute("fval");
    fout->addAttribute("fval2");
    fout->unsetLog();


    // Definition of the code
    URANIE::Launcher::TCode *mycode =
            new URANIE::Launcher::TCode(tds, UranieTests::code_path("rosenbrock") + " -v -o");
    mycode->addOutputFile(fout);
    mycode->unsetLog();

    // Definition of the reference data server
    URANIE::DataServer::TDataServer *tdsref1 =
            new URANIE::DataServer::TDataServer("tdsref1", "Objectives values");
    tdsref1->fileDataRead("rosenbrokReferences1.dat");

    URANIE::DataServer::TDataServer *tdsref2 =
            new URANIE::DataServer::TDataServer("tdsref2", "Objectives values");
    tdsref2->fileDataRead("rosenbrokReferences2.dat");

    // Definition of the optimizer
    _topt = new URANIE::Optimizer::TOptimizer(tds, mycode);
    _topt->addObjective("obj1", tdsref1, "y1", fout, "fval");
    _topt->addObjective("obj2", tdsref2, "y2", fout, "fval2");
    _topt->unsetLog();
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    //_topt->setSave();    _topt->setClean();
    _topt->optimize();

    Double_t dcrta;
    tds->getAttribute("a")->getDefaultValue(dcrta);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10.0, dcrta, 1e-2);
    Double_t dcrtb;
    tds->getAttribute("b")->getDefaultValue(dcrtb);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, dcrtb, 1e-2);
    Double_t dcrtc;
    tds->getAttribute("c")->getDefaultValue(dcrtc);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrtc, 1e-2);

    // Clean
    delete tdsref1;
    delete tdsref2;
    delete fout;
    delete mycode;
    delete tds;
}

void TOptimizerTest::testOptimizeSeveralObjectivesWithUnactive()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    URANIE::DataServer::TDataServer * tds =
            new URANIE::DataServer::TDataServer();

    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();
    // Definition of the attributes
    TString sJDDReference = gSystem->pwd();
#ifdef WIN32
    sJDDReference += "\\";
#else
    sJDDReference += "/";
#endif  
    sJDDReference += "input_rosenbrock_with_keys.dat";

    tds->addAttribute("a", 5.0, 15.0);
    tds->getAttribute("a")->setFileKey(sJDDReference, "a");
    tds->getAttribute("a")->setDefaultValue(6.0);
    tds->getAttribute("a")->setStepValue(0.1);

    tds->addAttribute("b", 0.0, 2.0);
    tds->getAttribute("b")->setFileKey(sJDDReference, "b");
    tds->getAttribute("b")->setDefaultValue(1.5 );
    tds->getAttribute("b")->setStepValue(0.1);

    tds->addAttribute("c", -1.0, 2.0);
    tds->getAttribute("c")->setFileKey(sJDDReference, "c");
    tds->getAttribute("c")->setDefaultValue(1.0);
    tds->getAttribute("c")->setStepValue(0.1);

    // Definition of the output file of the code
    URANIE::Launcher::TOutputFileDataServer *fout =
            new URANIE::Launcher::TOutputFileDataServer(
                    "_output_rosenbrock_withDataServer_.dat");
    fout->addAttribute("fval");
    fout->addAttribute("fval2");
    fout->unsetLog();

    // Definition of the code
    URANIE::Launcher::TCode *mycode =
            new URANIE::Launcher::TCode(tds, UranieTests::code_path("rosenbrock") + " -v -o");
    mycode->addOutputFile(fout);
    mycode->unsetLog();

    // Definition of the reference data server
    URANIE::DataServer::TDataServer *tdsref1 =
            new URANIE::DataServer::TDataServer("tdsref1", "Objectives values");
    tdsref1->fileDataRead("rosenbrokReferences1.dat");

    URANIE::DataServer::TDataServer *tdsref2 =
            new URANIE::DataServer::TDataServer("tdsref2", "Objectives values");
    tdsref2->fileDataRead("rosenbrokReferences2.dat");

    // Definition of the optimizer
    _topt = new URANIE::Optimizer::TOptimizer(tds, mycode);
    _topt->addObjective("obj1", tdsref1, "y1", fout, "fval");
    _topt->addObjective("obj2", tdsref2, "y2", fout, "fval2");
    _topt->unsetLog();
    _topt->setMethod(URANIE::Optimizer::TOptimizer::kSimplex);
    _topt->unactiveObjective("obj1");
    _topt->optimize();

    Double_t dcrta;
    tds->getAttribute("a")->getDefaultValue(dcrta);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10.0, dcrta, 1e-2);
    Double_t dcrtb;
    tds->getAttribute("b")->getDefaultValue(dcrtb);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, dcrtb, 1e-2);
    Double_t dcrtc;
    tds->getAttribute("c")->getDefaultValue(dcrtc);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, dcrtc, 1e-2);

    // Clean
    delete tdsref1;
    delete tdsref2;
    delete fout;
    delete mycode;
    delete tds;
}

void TOptimizerTest::testOptimizerAddObjective()
{
    
  // input file of evalCode.C, it contains the value of the parameters
  // in one column
  TString inputFileEvalCode = "evalCode_params.in";
  // output file of evalCode.C
  TString outputFileEvalCode = "_output_evalCode_.dat";  
    
  // The a attribute of the use case
  TAttribute *a = new TAttribute("a", 9.0,11.0);
  a->setDefaultValue(9.9);
  a->setStepValue(0.01);
  a->setFileFlag(inputFileEvalCode,"@a@");
  
  // The b attribute of the use case
  TAttribute *b = new TAttribute("b", 0.0,2.0);
  b->setDefaultValue(1.1);
  b->setStepValue(0.01);
  b->setFileFlag(inputFileEvalCode,"@b@");
  
  // The c attribute of the use case
  TAttribute *c = new TAttribute("c", -1.0,1.0);
  c->setDefaultValue(0.1);
  c->setStepValue(0.01);
  c->setFileFlag(inputFileEvalCode,"@c@");
  
  // The d attribute of the use case (not influent)
  TAttribute *d = new TAttribute("d", -1.0,1.0);
  d->setDefaultValue(0.7);
  d->setStepValue(0.01);
  d->setFileFlag(inputFileEvalCode,"@d@");

  // Add the attributes
  _tds1->addAttribute(a);
  _tds1->addAttribute(b);
  _tds1->addAttribute(c);
  _tds1->addAttribute(d);
  
  // The output file of the code where values are stored in (key = value) format
  _foutRow = new TOutputFileRow(outputFileEvalCode);
  _foutRow->addAttribute("fval");
  
  // Create an TCode object for my code
  _tcode = new TCode(_tds1, "root -l -q evalCode.C");
  // Add mycode output file
  _tcode->addOutputFile( _foutRow );
  // Add file needed to run mycode /////////////////////
  _tcode->addInputFile("evalCode.C");
  _tcode->addInputFile("tds_rosenbrock_sampling_.dat");
  _tcode->addInputFile("input_rosenbrock_with_keys.dat");
  //////////////////////////////////////////////////////:
  
  TDataServer *tdsref = new TDataServer("TDSRef", "Objectives values");
  tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
  
  // Create an TOptimizer object from TDataServer and TCode objects
  _topt = new TOptimizer(_tds1, _tcode);  
  _topt->setMethod(TOptimizer::kSimplex);
  _topt->setTolerance(1e-3);

  // add an objective
  _topt->addObjective("obj1",
             tdsref,
             "fstar",
             _foutRow,
             "fval");
  
  TObjective* obj1 = (_topt->getObjectives()).back();
  
  CPPUNIT_ASSERT_EQUAL(TOptimizer::kSumOfSquare,_topt->getOptimType());
  CPPUNIT_ASSERT(TString("fval") == (obj1->_syhat));
  CPPUNIT_ASSERT(TString("fstar") == (obj1->_systar));
  CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0,obj1->_dweight,DBL_PRECISION_COMPARE);
  CPPUNIT_ASSERT(TString("TDSRef") == TString(obj1->_tds->GetName()));
  cout<< (obj1->_outfile->getFileName()).Data()<<endl;
  TString tmpstr = (obj1->_outfile->getFileName()).Data();
  CPPUNIT_ASSERT( tmpstr.EqualTo("_output_evalCode_.dat"));
  
  a->delShare();
  b->delShare();
  c->delShare();
  d->delShare();
  delete tdsref;
}

void TOptimizerTest::testOptimizerAddObjectiveUncertaintySumSquareError()
{
  // input file of evalCode.C, it contains the value of the parameters
  // in one column
  TString inputFileEvalCode = "evalCode_params.in";
  // output file of evalCode.C
  TString outputFileEvalCode = "_output_evalCode_.dat";  
    
  // The a attribute of the use case
  TAttribute *a = new TAttribute("a", 9.0,11.0);
  a->setDefaultValue(9.9);
  a->setStepValue(0.01);
  a->setFileFlag(inputFileEvalCode,"@a@");
  
  // The b attribute of the use case
  TAttribute *b = new TAttribute("b", 0.0,2.0);
  b->setDefaultValue(1.1);
  b->setStepValue(0.01);
  b->setFileFlag(inputFileEvalCode,"@b@");
  
  // The c attribute of the use case
  TAttribute *c = new TAttribute("c", -1.0,1.0);
  c->setDefaultValue(0.1);
  c->setStepValue(0.01);
  c->setFileFlag(inputFileEvalCode,"@c@");
  
  // The d attribute of the use case (not influent)
  TAttribute *d = new TAttribute("d", -1.0,1.0);
  d->setDefaultValue(0.7);
  d->setStepValue(0.01);
  d->setFileFlag(inputFileEvalCode,"@d@");

  // Define the DataServer and add the attributes
//   TDataServer *tds = new TDataServer("tds", "Optimize Code externe Rosenbrock via TDataServer");
  _tds1->addAttribute(a);
  _tds1->addAttribute(b);
  _tds1->addAttribute(c);
  _tds1->addAttribute(d);
  
  // The output file of the code where values are stored in (key = value) format
  _foutRow = new TOutputFileRow(outputFileEvalCode);
  _foutRow->addAttribute("fval");
  
  // Create an TCode object for my code
  _tcode = new TCode(_tds1, "root -l -q evalCode.C");
  // Add mycode output file
  _tcode->addOutputFile( _foutRow );
  // Add file needed to run mycode /////////////////////
  _tcode->addInputFile("evalCode.C");
  _tcode->addInputFile("tds_rosenbrock_sampling_.dat");
  _tcode->addInputFile("input_rosenbrock_with_keys.dat");
  //////////////////////////////////////////////////////:
  
  TDataServer *tdsref = new TDataServer("TDSRef", "Objectives values");
  tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
  
  // Create an TOptimizer object from TDataServer and TCode objects
  _topt = new TOptimizer(_tds1, _tcode);  
  _topt->setMethod(TOptimizer::kSimplex);
  _topt->setTolerance(1e-3);

  // add an objective
//   Bool_t bWeighted = kTRUE;
  //Bool_t bWeighted = kFALSE;
  
  Int_t nOldNumberOfObjectives = _topt->getObjectives().size();
  
  _topt->addObjective("obj1",
             tdsref,
             "fstar",
             "w",
             _foutRow,
             "fval");
  
  TObjectiveUncertainty* obj1 = (TObjectiveUncertainty*) (_topt->getObjectives()).back();
  
  CPPUNIT_ASSERT_EQUAL(TOptimizer::kSumOfSquare,_topt->getOptimType());
  CPPUNIT_ASSERT(TString("fval") == (obj1->_syhat));
  CPPUNIT_ASSERT(TString("fstar") == (obj1->_systar));
  CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0,obj1->_dweight,DBL_PRECISION_COMPARE);
  CPPUNIT_ASSERT(TString("TDSRef") == TString(obj1->_tds->GetName()));
  cout<< (obj1->_outfile->getFileName()).Data()<<endl;
  TString tmpstr = (obj1->_outfile->getFileName()).Data();
  CPPUNIT_ASSERT( tmpstr.EqualTo("_output_evalCode_.dat"));
  tmpstr = obj1->getSigmaName();
  CPPUNIT_ASSERT( tmpstr.EqualTo("w"));
  Int_t nNumberOfObjectives = _topt->getObjectives().size();
  CPPUNIT_ASSERT_EQUAL(nOldNumberOfObjectives+1,nNumberOfObjectives);
  
  a->delShare();
  b->delShare();
  c->delShare();
  d->delShare();
  delete tdsref;
    
}

void TOptimizerTest::testOptimizerUnactiveObjective()
{
    // input file of evalCode.C, it contains the value of the parameters
  // in one column
  TString inputFileEvalCode = "evalCode_params.in";
  // output file of evalCode.C
  TString outputFileEvalCode = "_output_evalCode_.dat";  
    
  // The a attribute of the use case
  TAttribute *a = new TAttribute("a", 9.0,11.0);
  a->setDefaultValue(9.9);
  a->setStepValue(0.01);
  a->setFileFlag(inputFileEvalCode,"@a@");
  
  // The b attribute of the use case
  TAttribute *b = new TAttribute("b", 0.0,2.0);
  b->setDefaultValue(1.1);
  b->setStepValue(0.01);
  b->setFileFlag(inputFileEvalCode,"@b@");
  
  // The c attribute of the use case
  TAttribute *c = new TAttribute("c", -1.0,1.0);
  c->setDefaultValue(0.1);
  c->setStepValue(0.01);
  c->setFileFlag(inputFileEvalCode,"@c@");
  
  // The d attribute of the use case (not influent)
  TAttribute *d = new TAttribute("d", -1.0,1.0);
  d->setDefaultValue(0.7);
  d->setStepValue(0.01);
  d->setFileFlag(inputFileEvalCode,"@d@");

  // Define the DataServer and add the attributes
  _tds1->addAttribute(a);
  _tds1->addAttribute(b);
  _tds1->addAttribute(c);
  _tds1->addAttribute(d);
  
  // The output file of the code where values are stored in (key = value) format
  _foutRow = new TOutputFileRow(outputFileEvalCode);
  _foutRow->addAttribute("fval");
  
  // Create an TCode object for my code
  _tcode = new TCode(_tds1, "root -l -q evalCode.C");
  // Add mycode output file
  _tcode->addOutputFile( _foutRow );
  // Add file needed to run mycode /////////////////////
  _tcode->addInputFile("evalCode.C");
  _tcode->addInputFile("tds_rosenbrock_sampling_.dat");
  _tcode->addInputFile("input_rosenbrock_with_keys.dat");
  //////////////////////////////////////////////////////:
  
  TDataServer *tdsref = new TDataServer("TDSRef", "Objectives values");
  tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
  
  // Create an TOptimizer object from TDataServer and TCode objects
  _topt = new TOptimizer(_tds1, _tcode);  
  _topt->setMethod(TOptimizer::kSimplex);
  _topt->setTolerance(1e-3);

  // add an objective
  _topt->addObjective("obj1",
             tdsref,
             "fstar",
             _foutRow,
             "fval");
  
  TObjective* obj1 = (_topt->getObjectives()).back();
  
  CPPUNIT_ASSERT(obj1->isActive());
  _topt->unactiveObjective("obj1");
  CPPUNIT_ASSERT(!obj1->isActive());
  Double_t dvalues[10];
    dvalues[0] = 6.301019000e+00;
    dvalues[1] = 7.039946000e+01;
    dvalues[2] = 2.608487000e+01;
    dvalues[3] = 4.678512000e+00;
    dvalues[4] = 2.201640000e+00;
    dvalues[5] = 2.162794000e+01;
    dvalues[6] = 1.334784000e+01;
    dvalues[7] = 0.0;//1.367355000e+00;
    dvalues[8] = 9.676354000e+00;
    dvalues[9] = 4.532999000e+01;
  CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,obj1->getObjective(dvalues,10),DBL_PRECISION_COMPARE);
  
  a->delShare();
  b->delShare();
  c->delShare();
  d->delShare();
  delete tdsref;
    

}

void TOptimizerTest::testOptimizerActiveObjective()
{
    
  // input file of evalCode.C, it contains the value of the parameters
  // in one column
  TString inputFileEvalCode = "evalCode_params.in";
  // output file of evalCode.C
  TString outputFileEvalCode = "_output_evalCode_.dat";  
    
  // The a attribute of the use case
  TAttribute *a = new TAttribute("a", 9.0,11.0);
  a->setDefaultValue(9.9);
  a->setStepValue(0.01);
  a->setFileFlag(inputFileEvalCode,"@a@");
  
  // The b attribute of the use case
  TAttribute *b = new TAttribute("b", 0.0,2.0);
  b->setDefaultValue(1.1);
  b->setStepValue(0.01);
  b->setFileFlag(inputFileEvalCode,"@b@");
  
  // The c attribute of the use case
  TAttribute *c = new TAttribute("c", -1.0,1.0);
  c->setDefaultValue(0.1);
  c->setStepValue(0.01);
  c->setFileFlag(inputFileEvalCode,"@c@");
  
  // The d attribute of the use case (not influent)
  TAttribute *d = new TAttribute("d", -1.0,1.0);
  d->setDefaultValue(0.7);
  d->setStepValue(0.01);
  d->setFileFlag(inputFileEvalCode,"@d@");

  // Define the DataServer and add the attributes
  _tds1->addAttribute(a);
  _tds1->addAttribute(b);
  _tds1->addAttribute(c);
  _tds1->addAttribute(d);
  
  // The output file of the code where values are stored in (key = value) format
  _foutRow = new TOutputFileRow(outputFileEvalCode);
  _foutRow->addAttribute("fval");
  
  // Create an TCode object for my code
  _tcode = new TCode(_tds1, "root -l -q evalCode.C");
  // Add mycode output file
  _tcode->addOutputFile( _foutRow );
  // Add file needed to run mycode /////////////////////
  _tcode->addInputFile("evalCode.C");
  _tcode->addInputFile("tds_rosenbrock_sampling_.dat");
  _tcode->addInputFile("input_rosenbrock_with_keys.dat");
  //////////////////////////////////////////////////////:
  
  TDataServer *tdsref = new TDataServer("TDSRef", "Objectives values");
  tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
  
  // Create an TOptimizer object from TDataServer and TCode objects
  _topt = new TOptimizer(_tds1, _tcode);  
  _topt->setMethod(TOptimizer::kSimplex);
  _topt->setTolerance(1e-3);

  // add an objective
  _topt->addObjective("obj1",
             tdsref,
             "fstar",
             _foutRow,
             "fval");
  
  TObjective* obj1 = (_topt->getObjectives()).back();
   
  _topt->unactiveObjective("obj1");
  CPPUNIT_ASSERT(!obj1->isActive());
  _topt->activeObjective("obj1");
  CPPUNIT_ASSERT(obj1->isActive());
  
  Double_t dvalues[10];
    dvalues[0] = 6.301019000e+00;
    dvalues[1] = 7.039946000e+01;
    dvalues[2] = 2.608487000e+01;
    dvalues[3] = 4.678512000e+00;
    dvalues[4] = 2.201640000e+00;
    dvalues[5] = 2.162794000e+01;
    dvalues[6] = 1.334784000e+01;
    dvalues[7] = 0.0;//1.367355000e+00;
    dvalues[8] = 9.676354000e+00;
    dvalues[9] = 4.532999000e+01;
  CPPUNIT_ASSERT_DOUBLES_EQUAL(1.367355000e+00*1.367355000e+0/10.0,obj1->getObjective(dvalues,10),DBL_PRECISION_COMPARE);
  
  a->delShare();
  b->delShare();
  c->delShare();
  d->delShare();
  delete tdsref;

}

void TOptimizerTest::testOptimizerInvertObjective()
{
    
  // input file of evalCode.C, it contains the value of the parameters
  // in one column
  TString inputFileEvalCode = "evalCode_params.in";
  // output file of evalCode.C
  TString outputFileEvalCode = "_output_evalCode_.dat";  
    
  // The a attribute of the use case
  TAttribute *a = new TAttribute("a", 9.0,11.0);
  a->setDefaultValue(9.9);
  a->setStepValue(0.01);
  a->setFileFlag(inputFileEvalCode,"@a@");
  
  // The b attribute of the use case
  TAttribute *b = new TAttribute("b", 0.0,2.0);
  b->setDefaultValue(1.1);
  b->setStepValue(0.01);
  b->setFileFlag(inputFileEvalCode,"@b@");
  
  // The c attribute of the use case
  TAttribute *c = new TAttribute("c", -1.0,1.0);
  c->setDefaultValue(0.1);
  c->setStepValue(0.01);
  c->setFileFlag(inputFileEvalCode,"@c@");
  
  // The d attribute of the use case (not influent)
  TAttribute *d = new TAttribute("d", -1.0,1.0);
  d->setDefaultValue(0.7);
  d->setStepValue(0.01);
  d->setFileFlag(inputFileEvalCode,"@d@");

  // Add the attributes
  _tds1->addAttribute(a);
  _tds1->addAttribute(b);
  _tds1->addAttribute(c);
  _tds1->addAttribute(d);
  
  // The output file of the code where values are stored in (key = value) format
  _foutRow = new TOutputFileRow(outputFileEvalCode);
  _foutRow->addAttribute("fval");
  
  // Create an TCode object for my code
  _tcode = new TCode(_tds1, "root -l -q evalCode.C");
  // Add mycode output file
  _tcode->addOutputFile( _foutRow );
  // Add file needed to run mycode /////////////////////
  _tcode->addInputFile("evalCode.C");
  _tcode->addInputFile("tds_rosenbrock_sampling_.dat");
  _tcode->addInputFile("input_rosenbrock_with_keys.dat");
  //////////////////////////////////////////////////////:
  
  TDataServer *tdsref = new TDataServer("TDSRef", "Objectives values");
  tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
  
  // Create an TOptimizer object from TDataServer and TCode objects
  _topt = new TOptimizer(_tds1, _tcode);  
  _topt->setMethod(TOptimizer::kSimplex);
  _topt->setTolerance(1e-3);

  // add an objective
  _topt->addObjective("obj1",
             tdsref,
             "fstar",
             _foutRow,
             "fval");
  
  TObjective* obj1 = (_topt->getObjectives()).back();
  
  // add an objective
  _topt->addObjective("obj2",
             tdsref,
             "fstar",
             _foutRow,
             "fval");
  
  TObjective* obj2 = (_topt->getObjectives()).back();
   
  _topt->unactiveObjective("obj1");
  CPPUNIT_ASSERT(!obj1->isActive());
  CPPUNIT_ASSERT(obj2->isActive());
  
  _topt->invertObjectives();
  CPPUNIT_ASSERT(obj1->isActive());
  CPPUNIT_ASSERT(!obj2->isActive());
  
  _topt->invertObjectives();
  CPPUNIT_ASSERT(!obj1->isActive());
  CPPUNIT_ASSERT(obj2->isActive());
  
  a->delShare();
  b->delShare();
  c->delShare();
  d->delShare();
  delete tdsref;

}

void TOptimizerTest::testOptimizerselectCost()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

    _tcode = new URANIE::Launcher::TCode(_tds1, UranieTests::code_path("rosenbrock") + " -v -r");
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->selectCost("fval");

    TString scost = "fval";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(scost.Data(), _topt->getSelectedCost()));
    _topt->optimize();

}

void TOptimizerTest::testOptimizerFixParameter()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",
                    sRepertoireReference.Data()));
#endif
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();

    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kAttribute,
            _tds1->getAttribute("x")->getOrigin());
    _topt->fixParameter("x");
    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kConstant,
            _tds1->getAttribute("x")->getOrigin());

}

void TOptimizerTest::testOptimizerUnfixParameter()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",
                    sRepertoireReference.Data()));
#endif
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();

    _topt->fixParameter("x");
    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kConstant,
            _tds1->getAttribute("x")->getOrigin());
    _topt->unfixParameter("x");
    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kAttribute,
            _tds1->getAttribute("x")->getOrigin());

}

void TOptimizerTest::testOptimizerinvertParameters()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",
                    sRepertoireReference.Data()));
#endif
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();

    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kAttribute,
            _tds1->getAttribute("x")->getOrigin());
    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kAttribute,
            _tds1->getAttribute("y")->getOrigin());

    _topt->fixParameter("x");

    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kConstant,
            _tds1->getAttribute("x")->getOrigin());
    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kAttribute,
            _tds1->getAttribute("y")->getOrigin());

    _topt->invertParameters();
    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kAttribute,
            _tds1->getAttribute("x")->getOrigin());
    CPPUNIT_ASSERT_EQUAL(URANIE::DataServer::TAttribute::kConstant,
            _tds1->getAttribute("y")->getOrigin());

}

void TOptimizerTest::testOptimizerConstructorInput()
{

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "y:x",
            "out");
    CPPUNIT_ASSERT_EQUAL(1, _tds1->getNOutputAttributes());
    CPPUNIT_ASSERT(_tds1->isAttribute("out"));
    TString tmpstr = "out";
    CPPUNIT_ASSERT(tmpstr.EqualTo(_topt->getSelectedCost()));
    
    URANIE::Optimizer::TOptimizer *topt2 = new URANIE::Optimizer::TOptimizer(
            _tds1, "rosenbrock", "x:y", "");
    
    CPPUNIT_ASSERT_EQUAL(2, _tds1->getNOutputAttributes());
    CPPUNIT_ASSERT(_tds1->isAttribute("rosenbrock"));
    tmpstr = "Uranie_Optimizer_Function_rosenbrock";
    CPPUNIT_ASSERT(tmpstr.EqualTo(topt2->GetName()));
    tmpstr = "Optimize Function[rosenbrock] with TDS[testOptimizer]";
    CPPUNIT_ASSERT(tmpstr.EqualTo(topt2->GetTitle()));
    CPPUNIT_ASSERT_EQUAL(10000,topt2->getMaxFunctionCalls());
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.01,topt2->getTolerance(),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_EQUAL(0,topt2->getPrintLevel());
    CPPUNIT_ASSERT_EQUAL(50,topt2->getMaxIterations());
    CPPUNIT_ASSERT_EQUAL(URANIE::Optimizer::TOptimizer::kMinimizeFunction,topt2->getOptimType());
    CPPUNIT_ASSERT_EQUAL(URANIE::Optimizer::TOptimizer::kMigrad,topt2->getMethod());
    tmpstr = "rosenbrock";
    CPPUNIT_ASSERT(tmpstr.EqualTo(topt2->getSelectedCost()));
    
    delete topt2;
    
    

}

void TOptimizerTest::testOptimizerConstructorMoreOneOutput()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock2", "",
            "out:out2");

    CPPUNIT_ASSERT_EQUAL(2, _tds1->getNOutputAttributes());

}


void TOptimizerTest::testOptimizerClean()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",
                    sRepertoireReference.Data()));
#endif
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    
    
    TString sworkindirectory = gSystem->UnixPathName(sRepertoireReference+TString("/URANIE"));
    _tcode->setWorkingDirectory(sworkindirectory);
    TString sSubDirectory1 = gSystem->UnixPathName(sworkindirectory + TString("/UranieLauncher_1"));
    TString sSubDirectory18 = gSystem->UnixPathName(sworkindirectory + TString("/UranieLauncher_18"));
    // create directory
    gSystem->mkdir(sSubDirectory1.Data(),kTRUE);
    gSystem->mkdir(sSubDirectory18.Data(),kTRUE);
    
    CPPUNIT_ASSERT(gSystem->cd(sSubDirectory1.Data()));
    CPPUNIT_ASSERT(gSystem->cd(sSubDirectory18.Data()));
    
    // clean working directory
    _topt->clean();
   
    CPPUNIT_ASSERT(!gSystem->cd(sSubDirectory1.Data()));
    CPPUNIT_ASSERT(!gSystem->cd(sSubDirectory18.Data()));
    
    //    _foutDataServer->getAttribute(0)->delShare();
    
}


////////////////////////////////
///                          ///                            
///     throwexception       ///
///                          ///
////////////////////////////////
void TOptimizerTest::throwExceptionOptimizerConstructorTdsNull()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Create the TDataServer object
    URANIE::DataServer::TDataServer *tdsnull = NULL;

    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TOptimizer(tdsnull, "rosenbrock", "",
                    "out"), URANIE::Exceptions::UErrorExceptions);

    tdsnull = new URANIE::DataServer::TDataServer("tdsnull", "my tds null");

#ifdef ROOT5
	CPPUNIT_ASSERT_THROW(new URANIE::Optimizer::TOptimizer(tdsnull, _p2fRosenbrock, "", "out"), URANIE::Exceptions::UErrorExceptions);
#else
	CPPUNIT_ASSERT_THROW(new URANIE::Optimizer::TOptimizer(tdsnull, "rosenbrock", "", "out"), URANIE::Exceptions::UErrorExceptions);
#endif

    delete tdsnull;
}

void TOptimizerTest::throwExceptionOptimizerConstructorFalseInput()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_IGNORED_TEST()

    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TOptimizer(_tds1, "rosenbrock", "toto",
                    "out"), URANIE::Exceptions::UErrorExceptions);
}

void TOptimizerTest::throwExceptionOptimizerConstructorUnknowFunction()
{
//fga 2015/01/12     _tds1->addAttribute(_pX);
//fga 2015/01/12     _tds1->addAttribute(_pY);
//fga 2015/01/12     G__ClassInfo globalscope;
//fga 2015/01/12     G__MethodInfo method;
//fga 2015/01/12     long dummy;
//fga 2015/01/12     // resolve function overloading
//fga 2015/01/12     method = globalscope.GetMethod("toto", "double*,double*", &dummy);
//fga 2015/01/12    void * p2ftoto = method.PointerToFunc();
    
   CPPUNIT_ASSERT_THROW(new URANIE::Optimizer::TOptimizer(_tds1, "p2ftoto","","out"),URANIE::Exceptions::UErrorExceptions);
}

void TOptimizerTest::throwExceptionOptimizerConstructorUnknowTCode()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    URANIE::Launcher::TCode *mycode = NULL;

    CPPUNIT_ASSERT_THROW(new URANIE::Optimizer::TOptimizer(_tds1, mycode),
            URANIE::Exceptions::UErrorExceptions);
}

//void TOptimizerTest::throwExceptionOptimizersetMaxIterations()
//{
//  // The x attribute of the use case
//  _pX->setFileKey(_sReferenceDirectory+TString("input_rosenbrock_with_values_rows.dat"),"x","",URANIE::DataServer::TAttributeFileKey::kNewColumn);
//
//  // The y attribute of the use cases
//  _pY->setFileKey(_sReferenceDirectory+TString("input_rosenbrock_with_values_rows.dat"),"y","",URANIE::DataServer::TAttributeFileKey::kNewColumn);
//
//  _tds1->addAttribute( _pX );
//  _tds1->addAttribute( _pY );
//
//  // THIS TEST HAS BEEN DISABLED
//  // TODO: fix me
//  URANIE_IGNORED_TEST();
//
//  //Directory where we can find references input files
//  TString sRepertoireReference = gSystem->pwd();
//
//  // Output file
//  _foutDataServer = new URANIE::Launcher::TOutputFileDataServer("_output_rosenbrock_with_values_rows_.dat");
//  _foutDataServer->addAttribute("fval");
//
//#ifdef WIN32
//  _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
//#else
//  _tcode = new URANIE::Launcher::TCode(_tds1, Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",sRepertoireReference.Data()));
//#endif
//  _tcode->addOutputFile( _foutDataServer );
//
//  _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
//  _topt->unsetLog();
//
//  CPPUNIT_ASSERT_THROW(_topt->setMaxIterations(10.5),URANIE::Exceptions::UErrorExceptions);
//}

//void TOptimizerTest::throwExceptionOptimizersetMaxFunctionCalls()
//{
//  // The x attribute of the use case
//  _pX->setFileKey(_sReferenceDirectory+TString("input_rosenbrock_with_values_rows.dat"),"x","",URANIE::DataServer::TAttributeFileKey::kNewColumn);
//
//  // The y attribute of the use cases
//  _pY->setFileKey(_sReferenceDirectory+TString("input_rosenbrock_with_values_rows.dat"),"y","",URANIE::DataServer::TAttributeFileKey::kNewColumn);
//
//  _tds1->addAttribute( _pX );
//  _tds1->addAttribute( _pY );
//
//  // THIS TEST HAS BEEN DISABLED
//  // TODO: fix me
//  URANIE_IGNORED_TEST();
//
//  //Directory where we can find references input files
//  TString sRepertoireReference = gSystem->pwd();
//
//  // Output file
//  _foutDataServer = new URANIE::Launcher::TOutputFileDataServer("_output_rosenbrock_with_values_rows_.dat");
//  _foutDataServer->addAttribute("fval");
//
//#ifdef WIN32
//  _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
//#else
//  _tcode = new URANIE::Launcher::TCode(_tds1, Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",sRepertoireReference.Data()));
//#endif
//  _tcode->addOutputFile( _foutDataServer );
//
//  _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
//  _topt->unsetLog();
//
//  CPPUNIT_ASSERT_THROW(_topt->setMaxFunctionCalls(10.5),URANIE::Exceptions::UErrorExceptions);
//}

//void TOptimizerTest::throwExceptionOptimizersetPrintLevelNoInteger()
//{
//  // The x attribute of the use case
//  _pX->setFileKey(_sReferenceDirectory+TString("input_rosenbrock_with_values_rows.dat"),"x","",URANIE::DataServer::TAttributeFileKey::kNewColumn);
//
//  // The y attribute of the use cases
//  _pY->setFileKey(_sReferenceDirectory+TString("input_rosenbrock_with_values_rows.dat"),"y","",URANIE::DataServer::TAttributeFileKey::kNewColumn);
//
//  _tds1->addAttribute( _pX );
//  _tds1->addAttribute( _pY );
//
//  // THIS TEST HAS BEEN DISABLED
//  // TODO: fix me
//  URANIE_IGNORED_TEST();
//
//  //Directory where we can find references input files
//  TString sRepertoireReference = gSystem->pwd();
//
//  // Output file
//  _foutDataServer = new URANIE::Launcher::TOutputFileDataServer("_output_rosenbrock_with_values_rows_.dat");
//  _foutDataServer->addAttribute("fval");
//
//#ifdef WIN32
//  _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
//#else
//  _tcode = new URANIE::Launcher::TCode(_tds1, Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",sRepertoireReference.Data()));
//#endif
//  _tcode->addOutputFile( _foutDataServer );
//
//  _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
//  _topt->unsetLog();
//
//  CPPUNIT_ASSERT_THROW(_topt->setPrintLevel(1.5),URANIE::Exceptions::UErrorExceptions);
//}

void TOptimizerTest::throwExceptionOptimizersetPrintLevelInf1()
{
    // The x attribute of the use case
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_IGNORED_TEST()
    ;

    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",
                    sRepertoireReference.Data()));
#endif
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();

    CPPUNIT_ASSERT_THROW(_topt->setPrintLevel(0),
            URANIE::Exceptions::UErrorExceptions);
}

void TOptimizerTest::throwExceptionOptimizersetPrintLevelSup5()
{
    // The x attribute of the use case
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_IGNORED_TEST()
    ;

    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",
                    sRepertoireReference.Data()));
#endif
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();

    CPPUNIT_ASSERT_THROW(_topt->setPrintLevel(6),
            URANIE::Exceptions::UErrorExceptions);
    
    
}

void TOptimizerTest::throwExceptionOptimizerSelectCost()
{
     // The x attribute of the use case
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);
    
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",
                    sRepertoireReference.Data()));
#endif
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();

    CPPUNIT_ASSERT_THROW(_topt->selectCost("fval:fval"),
            URANIE::Exceptions::UErrorExceptions);
    
    CPPUNIT_ASSERT_THROW(_topt->selectCost("toto"),
            URANIE::Exceptions::UErrorExceptions);
    
    CPPUNIT_ASSERT_THROW(_topt->selectCost("y"),
            URANIE::Exceptions::UErrorExceptions);
    
}





void TOptimizerTest::throwExceptionOptimizerFixParameter()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",
                    sRepertoireReference.Data()));
#endif
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();

    CPPUNIT_ASSERT_THROW(_topt->fixParameter("toto"),
            URANIE::Exceptions::UErrorExceptions);
    
    _pY->setOrigin(URANIE::DataServer::TAttribute::kConstant);
    
    CPPUNIT_ASSERT_THROW(_topt->fixParameter("y"),
            URANIE::Exceptions::UWarningExceptions);
    
    _pY->setOrigin(URANIE::DataServer::TAttribute::kDeleted);
    
    CPPUNIT_ASSERT_THROW(_topt->fixParameter("y"),
            URANIE::Exceptions::UErrorExceptions);
    
    _pY->setOrigin(URANIE::DataServer::TAttribute::kIterator);
    
    CPPUNIT_ASSERT_THROW(_topt->fixParameter("y"),
            URANIE::Exceptions::UErrorExceptions);
    
    _pY->setOrigin(URANIE::DataServer::TAttribute::kInternal);
    
    CPPUNIT_ASSERT_THROW(_topt->fixParameter("y"),
            URANIE::Exceptions::UErrorExceptions);
    

}

void TOptimizerTest::throwExceptionOptimizerUnfixParameter()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // The x attribute of the use case
    _pX->setStepValue(0.01);
    _pX->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "x", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    // The y attribute of the use cases
    _pY->setStepValue(0.01);
    _pY->setFileKey(
            _sReferenceDirectory
                    + TString("input_rosenbrock_with_values_rows.dat"), "y", "",
            URANIE::DataServer::TAttributeFileKey::kNewColumn);

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    // Output file
    _foutDataServer = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_values_rows_.dat");
    _foutDataServer->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1, "rosenbrock.exe -v -r 1>NUL");
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -r>>/dev/null",
                    sRepertoireReference.Data()));
#endif
    _tcode->addOutputFile(_foutDataServer);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();

    CPPUNIT_ASSERT_THROW(_topt->unfixParameter("toto"),
            URANIE::Exceptions::UErrorExceptions);
    
    CPPUNIT_ASSERT_THROW(_topt->unfixParameter("y"),
            URANIE::Exceptions::UWarningExceptions);
    
    _pY->setOrigin(URANIE::DataServer::TAttribute::kDeleted);
    
    CPPUNIT_ASSERT_THROW(_topt->unfixParameter("y"),
            URANIE::Exceptions::UErrorExceptions);
    
    _pY->setOrigin(URANIE::DataServer::TAttribute::kIterator);
    
    CPPUNIT_ASSERT_THROW(_topt->unfixParameter("y"),
            URANIE::Exceptions::UErrorExceptions);
    
    _pY->setOrigin(URANIE::DataServer::TAttribute::kInternal);
    
    CPPUNIT_ASSERT_THROW(_topt->unfixParameter("y"),
            URANIE::Exceptions::UErrorExceptions);

}

void TOptimizerTest::throwExceptionOptimizeraddObjectiveTdsNull()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    URANIE::DataServer::TDataServer * tds =
            new URANIE::DataServer::TDataServer();

    tds->addAttribute("a", -5.0, 5.0);
    tds->addAttribute("b", -1.0, 1.0);
    tds->addAttribute("c", -5.0, 5.0);
    // Definition of the attributes
    TString sJDDReference = gSystem->pwd();
#ifdef WIN32
    sJDDReference += "\\";
#else
    sJDDReference += "/";
#endif
    sJDDReference += "input_rosenbrock_with_keys.dat";
    tds->getAttribute("a")->setFileKey(sJDDReference, "a");
    tds->getAttribute("b")->setFileKey(sJDDReference, "b");
    tds->getAttribute("c")->setFileKey(sJDDReference, "c");
    tds->getAttribute("a")->setDefaultValue(0.200);
    tds->getAttribute("a")->setStepValue(0.01);
    tds->getAttribute("b")->setDefaultValue(0.200);
    tds->getAttribute("b")->setStepValue(0.01);
    tds->getAttribute("c")->setDefaultValue(0.200);
    tds->getAttribute("c")->setStepValue(0.01);

    URANIE::Launcher::TOutputFileRow *fout =
            new URANIE::Launcher::TOutputFileRow(
                    "_function_venise_output1_.dat");
    fout->addAttribute("yhat");

    URANIE::Launcher::TCode *mycode = new URANIE::Launcher::TCode(tds,
            "codeFunctionVenise >> /dev/null");
    mycode->addOutputFile(fout);

    _topt = new URANIE::Optimizer::TOptimizer(tds, mycode);

    URANIE::DataServer::TDataServer *tdsnull = NULL;
            ///new URANIE::DataServer::TDataServer();


    CPPUNIT_ASSERT_THROW(
            _topt->addObjective("obj1", tdsnull, "y2", fout, "yhat"),
            URANIE::Exceptions::UErrorExceptions);

    tdsnull = new URANIE::DataServer::TDataServer();
    CPPUNIT_ASSERT_THROW(
            _topt->addObjective("obj1", tdsnull, "y", fout, "yhat"),
            URANIE::Exceptions::UErrorExceptions);

    delete fout;
    delete mycode;
    delete tdsnull;
    delete tds;
}

void TOptimizerTest::throwExceptionOptimizeraddObjectiveYStarFalse()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    URANIE::DataServer::TDataServer * tds =
            new URANIE::DataServer::TDataServer();
    tds->addAttribute("a", -5.0, 5.0);
    tds->addAttribute("b", -1.0, 1.0);
    tds->addAttribute("c", -5.0, 5.0);
    // Definition of the attributes
    TString sJDDReference = gSystem->pwd();
#ifdef WIN32
    sJDDReference += "\\";
#else
    sJDDReference += "/";
#endif
    sJDDReference += "input_rosenbrock_with_keys.dat";
    tds->getAttribute("a")->setFileKey(sJDDReference, "a");
    tds->getAttribute("b")->setFileKey(sJDDReference, "b");
    tds->getAttribute("c")->setFileKey(sJDDReference, "c");
    tds->getAttribute("a")->setDefaultValue(0.200);
    tds->getAttribute("a")->setStepValue(0.01);
    tds->getAttribute("b")->setDefaultValue(0.200);
    tds->getAttribute("b")->setStepValue(0.01);
    tds->getAttribute("c")->setDefaultValue(0.200);
    tds->getAttribute("c")->setStepValue(0.01);

    URANIE::Launcher::TOutputFileRow *fout =
            new URANIE::Launcher::TOutputFileRow(
                    "_function_venise_output1_.dat");
    fout->addAttribute("yhat");

    URANIE::Launcher::TCode *mycode = new URANIE::Launcher::TCode(tds,
            "codeFunctionVenise >> /dev/null");
    mycode->addOutputFile(fout);

    URANIE::DataServer::TDataServer *tdsref =
            new URANIE::DataServer::TDataServer("TDSRef", "Objectives values");
    tdsref->fileDataRead("rosenbrokReferences1.dat");

    _topt = new URANIE::Optimizer::TOptimizer(tds, mycode);

    CPPUNIT_ASSERT_THROW(
            _topt->addObjective("obj1", tdsref, "toto", fout, "yhat"),
            URANIE::Exceptions::UErrorExceptions);

    delete fout;
    delete mycode;
    delete tdsref;
    delete tds;
}

void TOptimizerTest::throwExceptionOptimizeraddObjectiveYHatFalse()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    URANIE::DataServer::TDataServer * tds =
            new URANIE::DataServer::TDataServer();
    tds->addAttribute("a", -5.0, 5.0);
    tds->addAttribute("b", -1.0, 1.0);
    tds->addAttribute("c", -5.0, 5.0);
    // Definition of the attributes
    TString sJDDReference = gSystem->pwd();
#ifdef WIN32
    sJDDReference += "\\";
#else
    sJDDReference += "/";
#endif
    sJDDReference += "input_rosenbrock_with_keys.dat";
    tds->getAttribute("a")->setFileKey(sJDDReference, "a");
    tds->getAttribute("b")->setFileKey(sJDDReference, "b");
    tds->getAttribute("c")->setFileKey(sJDDReference, "c");
    tds->getAttribute("a")->setDefaultValue(0.200);
    tds->getAttribute("a")->setStepValue(0.01);
    tds->getAttribute("b")->setDefaultValue(0.200);
    tds->getAttribute("b")->setStepValue(0.01);
    tds->getAttribute("c")->setDefaultValue(0.200);
    tds->getAttribute("c")->setStepValue(0.01);

    URANIE::Launcher::TOutputFileRow *fout =
            new URANIE::Launcher::TOutputFileRow(
                    "_function_venise_output1_.dat");
    fout->addAttribute("yhat");
    fout->setLog();

    URANIE::Launcher::TCode *mycode = new URANIE::Launcher::TCode(tds,
            "codeFunctionVenise >> /dev/null");
    mycode->addOutputFile(fout);

    URANIE::DataServer::TDataServer *tdsnull =
            new URANIE::DataServer::TDataServer();
    URANIE::DataServer::TDataServer *tdsref =
            new URANIE::DataServer::TDataServer("TDSRef", "Objectives values");
    tdsref->fileDataRead("rosenbrokReferences1.dat");

    _topt = new URANIE::Optimizer::TOptimizer(tds, mycode);

    CPPUNIT_ASSERT_THROW(_topt->addObjective("obj1", tdsref, "y", fout, "toto"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(_topt->addObjective("obj1", tdsref, "tata", fout, "yhat"),
            URANIE::Exceptions::UErrorExceptions);

    delete fout;
    delete mycode;
    delete tdsref;
    delete tdsnull;
    delete tds;

}

void TOptimizerTest::throwExceptionOptimizeraddObjectiveOutFileNull()
{
    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    URANIE::DataServer::TDataServer * tds =
            new URANIE::DataServer::TDataServer();
    tds->addAttribute("a", -5.0, 5.0);
    tds->addAttribute("b", -1.0, 1.0);
    tds->addAttribute("c", -5.0, 5.0);
    // Definition of the attributes
    TString sJDDReference = gSystem->pwd();
#ifdef WIN32
    sJDDReference += "\\";
#else
    sJDDReference += "/";
#endif
    sJDDReference += "input_rosenbrock_with_keys.dat";
    tds->getAttribute("a")->setFileKey(sJDDReference, "a");
    tds->getAttribute("b")->setFileKey(sJDDReference, "b");
    tds->getAttribute("c")->setFileKey(sJDDReference, "c");
    tds->getAttribute("a")->setDefaultValue(0.200);
    tds->getAttribute("a")->setStepValue(0.01);
    tds->getAttribute("b")->setDefaultValue(0.200);
    tds->getAttribute("b")->setStepValue(0.01);
    tds->getAttribute("c")->setDefaultValue(0.200);
    tds->getAttribute("c")->setStepValue(0.01);

    URANIE::Launcher::TOutputFileRow *fout =
            new URANIE::Launcher::TOutputFileRow(
                    "_function_venise_output1_.dat");
    fout->addAttribute("yhat");

    URANIE::Launcher::TCode *mycode = new URANIE::Launcher::TCode(tds,
            "codeFunctionVenise >> /dev/null");
    mycode->addOutputFile(fout);

    URANIE::DataServer::TDataServer *tdsref =
            new URANIE::DataServer::TDataServer("TDSRef", "Objectives values");
    tdsref->fileDataRead("rosenbrokReferences1.dat");

    _topt = new URANIE::Optimizer::TOptimizer(tds, mycode);

    URANIE::Launcher::TOutputFileRow *toto = NULL;
    CPPUNIT_ASSERT_THROW(_topt->addObjective("obj1", tdsref, "y1", toto, "yhat"),
            URANIE::Exceptions::UErrorExceptions);

    delete fout;
    delete mycode;
    delete tdsref;
    delete tds;
}

void TOptimizerTest::throwExceptionOptimizerOptimize()
{
    //Directory where we can find references input files
    TString sRepertoireReference = gSystem->pwd();

    // Study Attributes
    _pX->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));
    _pY->setFileKey(
            _sReferenceDirectory + TString("input_rosenbrock_with_keys.dat"));

    _tds1->addAttribute(_pX);
    _tds1->addAttribute(_pY);

    Double_t dcrt = 0.0;
    _tds1->getAttribute("x")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, -1.2, DBL_PRECISION_COMPARE);
    _tds1->getAttribute("y")->getDefaultValue(dcrt);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dcrt, 1., DBL_PRECISION_COMPARE);

    // Output file
    _foutKey = new URANIE::Launcher::TOutputFileKey(
            "_output_rosenbrock_with_keys_.dat");
    _foutKey->addAttribute("fval");

#ifdef WIN32
    _tcode = new URANIE::Launcher::TCode(_tds1,"rosenbrock.exe -v -k");
    _tcode->setWorkingDirectory(gSystem->pwd() + TString("\\tmpLanceurUranie\\OptimizerUranie\\rosenbrock"));
#else
    _tcode = new URANIE::Launcher::TCode(_tds1,
            Form("%s/../../teST/coDE/rosENBROK/rosenbrock -v -k",
                    sRepertoireReference.Data()));
    _tcode->setWorkingDirectory(
            gSystem->pwd()
                    + TString("/tmpLanceurUranie/OptimizerUranie/rosenbrock"));
#endif
    _tcode->addOutputFile(_foutKey);

    _topt = new URANIE::Optimizer::TOptimizer(_tds1, _tcode);
    _topt->unsetLog();
    _topt->setTolerance(0.001);
    _topt->setMaxIterations(3000);
    _topt->setMaxFunctionCalls(5);
    CPPUNIT_ASSERT_THROW(_topt->optimize(""),URANIE::Exceptions::UErrorExceptions);
    
}

void TOptimizerTest::throwExceptionOptimizerUnactiveObjective()
{
    // input file of evalCode.C, it contains the value of the parameters
  // in one column
  TString inputFileEvalCode = "evalCode_params.in";
  // output file of evalCode.C
  TString outputFileEvalCode = "_output_evalCode_.dat";  
    
  // The a attribute of the use case
  TAttribute *a = new TAttribute("a", 9.0,11.0);
  a->setDefaultValue(9.9);
  a->setStepValue(0.01);
  a->setFileFlag(inputFileEvalCode,"@a@");
  
  // The b attribute of the use case
  TAttribute *b = new TAttribute("b", 0.0,2.0);
  b->setDefaultValue(1.1);
  b->setStepValue(0.01);
  b->setFileFlag(inputFileEvalCode,"@b@");
  
  // The c attribute of the use case
  TAttribute *c = new TAttribute("c", -1.0,1.0);
  c->setDefaultValue(0.1);
  c->setStepValue(0.01);
  c->setFileFlag(inputFileEvalCode,"@c@");
  
  // The d attribute of the use case (not influent)
  TAttribute *d = new TAttribute("d", -1.0,1.0);
  d->setDefaultValue(0.7);
  d->setStepValue(0.01);
  d->setFileFlag(inputFileEvalCode,"@d@");

  // Define the DataServer and add the attributes
  _tds1->addAttribute(a);
  _tds1->addAttribute(b);
  _tds1->addAttribute(c);
  _tds1->addAttribute(d);
  
  // The output file of the code where values are stored in (key = value) format
  _foutRow = new TOutputFileRow(outputFileEvalCode);
  _foutRow->addAttribute("fval");
  
  // Create an TCode object for my code
  _tcode = new TCode(_tds1, "root -l -q evalCode.C");
  // Add mycode output file
  _tcode->addOutputFile( _foutRow );
  // Add file needed to run mycode /////////////////////
  _tcode->addInputFile("evalCode.C");
  _tcode->addInputFile("tds_rosenbrock_sampling_.dat");
  _tcode->addInputFile("input_rosenbrock_with_keys.dat");
  //////////////////////////////////////////////////////:
  
  TDataServer *tdsref = new TDataServer("TDSRef", "Objectives values");
  tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
  
  // Create an TOptimizer object from TDataServer and TCode objects
  _topt = new TOptimizer(_tds1, _tcode);  
  _topt->setMethod(TOptimizer::kSimplex);
  _topt->setTolerance(1e-3);

  // add an objective
  _topt->addObjective("obj1",
             tdsref,
             "fstar",
             _foutRow,
             "fval");
  
  //TObjective* obj1 = (_topt->getObjectives()).back();
  
  CPPUNIT_ASSERT_THROW(_topt->unactiveObjective("toto"),URANIE::Exceptions::UErrorExceptions);
  
  a->delShare();
  b->delShare();
  c->delShare();
  d->delShare();
  delete tdsref;
    
}


void TOptimizerTest::throwExceptionOptimizerActiveObjective()
{
    // input file of evalCode.C, it contains the value of the parameters
  // in one column
  TString inputFileEvalCode = "evalCode_params.in";
  // output file of evalCode.C
  TString outputFileEvalCode = "_output_evalCode_.dat";  
    
  // The a attribute of the use case
  TAttribute *a = new TAttribute("a", 9.0,11.0);
  a->setDefaultValue(9.9);
  a->setStepValue(0.01);
  a->setFileFlag(inputFileEvalCode,"@a@");
  
  // The b attribute of the use case
  TAttribute *b = new TAttribute("b", 0.0,2.0);
  b->setDefaultValue(1.1);
  b->setStepValue(0.01);
  b->setFileFlag(inputFileEvalCode,"@b@");
  
  // The c attribute of the use case
  TAttribute *c = new TAttribute("c", -1.0,1.0);
  c->setDefaultValue(0.1);
  c->setStepValue(0.01);
  c->setFileFlag(inputFileEvalCode,"@c@");
  
  // The d attribute of the use case (not influent)
  TAttribute *d = new TAttribute("d", -1.0,1.0);
  d->setDefaultValue(0.7);
  d->setStepValue(0.01);
  d->setFileFlag(inputFileEvalCode,"@d@");

  // Define the DataServer and add the attributes
  _tds1->addAttribute(a);
  _tds1->addAttribute(b);
  _tds1->addAttribute(c);
  _tds1->addAttribute(d);
  
  // The output file of the code where values are stored in (key = value) format
  _foutRow = new TOutputFileRow(outputFileEvalCode);
  _foutRow->addAttribute("fval");
  
  // Create an TCode object for my code
  _tcode = new TCode(_tds1, "root -l -q evalCode.C");
  // Add mycode output file
  _tcode->addOutputFile( _foutRow );
  // Add file needed to run mycode /////////////////////
  _tcode->addInputFile("evalCode.C");
  _tcode->addInputFile("tds_rosenbrock_sampling_.dat");
  _tcode->addInputFile("input_rosenbrock_with_keys.dat");
  //////////////////////////////////////////////////////:
  
  TDataServer *tdsref = new TDataServer("TDSRef", "Objectives values");
  tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
  
  // Create an TOptimizer object from TDataServer and TCode objects
  _topt = new TOptimizer(_tds1, _tcode);  
  _topt->setMethod(TOptimizer::kSimplex);
  _topt->setTolerance(1e-3);

  // add an objective
  _topt->addObjective("obj1",
             tdsref,
             "fstar",
             _foutRow,
             "fval");
  
  //TObjective* obj1 = (_topt->getObjectives()).back();
  
  CPPUNIT_ASSERT_THROW(_topt->activeObjective("toto"),URANIE::Exceptions::UErrorExceptions);
  
  a->delShare();
  b->delShare();
  c->delShare();
  d->delShare();
  delete tdsref;

}



