{
    
  using namespace URANIE::DataServer;
  using namespace URANIE::Launcher;
  
  // To adapt for each code
  // file containing the datas used for the calibration
  TString dataFile = "tds_rosenbrock_sampling_.dat";
  Int_t numberOfDatas = 10;
  // command used to launche the code
  TString commandCode = "rosenbrock -v -k";
  // input file of the code
  TString inputFileCode = "input_rosenbrock_with_keys.dat";
  // output file of the code
  TString outputFileCode = "_output_rosenbrock_with_keys_.dat";
  // variables of the code
  TString inputsVariables[2] = {"x","y"};
  Int_t nvar = 2;
  // parameters of the code to calibrate
  TString inputsParameters[3] = {"a","b","c"};
  Int_t npar = 3;
  // output variable of the code
  TString outputVarible = "fval";
  
  // input file of this script, it contains the value of the parameters 
  // in one column
  TString inputFileEvalCode = "evalCode_params.in";
  // output file of this script, it contains the output values of the code
  TString outputFileEvalCode = "_output_evalCode_.dat";
    
  // Define the DataServer and add the attributes
  TDataServer *tds = new TDataServer("tds", "Optimize Code externe via TDataServer");
  // load the datas file
  tds->fileDataRead(dataFile);
  Int_t iter;
  // set the key file for each variables
  for(iter = 0; iter<nvar; iter++)
      tds->getAttribute(inputsVariables[iter])->setFileKey(inputFileCode);
  
  // create the array containing the value of the parameters  
  Double_t* parametersValues = new Double_t[npar];
  for(iter=0; iter<npar; iter++)
      parametersValues[iter] = 0;
  
  // obtain the value of each parameter from the script input file
  ifstream file(inputFileEvalCode);
  iter = 0;
  while(file.good() && iter<npar)
  {
      file >> parametersValues[iter];
      iter++;
  }
  file.close();
  
  // set the parameters attributes
  for(Int_t itt=0; itt<npar; itt++)
  {
      // add an attribute for each parameters
      tds->addAttribute( new TAttribute(inputsParameters[itt]));
      // set the key file
      tds->getAttribute(inputsParameters[itt])->setFileKey(inputFileCode);
      // set the value of the parameter at the computation
      tds->fillAttribute(inputsParameters[itt], Form("%e",parametersValues[itt]));
  }

  // The output file of the code where values are stored in (key = value) format
  TOutputFileKey *fOutputFile = new TOutputFileKey(outputFileCode);
  fOutputFile->addAttribute(new TAttribute(outputVarible));
  
  // Create an TCode object for my code
  TCode *mycode = new TCode(tds, commandCode);
  // Add the output file
  mycode->addOutputFile( fOutputFile );
  
  // Create a launcher and launch the code
  TLauncher *lanceur = new TLauncher(tds, mycode);
  lanceur->run();
  
  // fill the output file of the evalCode script
  ofstream exportFile(outputFileEvalCode);
  for(Int_t ittt=0; ittt<numberOfDatas;ittt++)
      exportFile << tds->getValue(outputVarible,ittt)<<endl;
  
  
  
  delete mycode;
  delete fOutputFile;
  delete lanceur;
  delete parametersValues;
  delete tds;

}