#include <iostream>
using namespace URANIE::DataServer;
using namespace URANIE::Launcher;
using namespace URANIE::Optimizer;

void rosenbrock(Double_t *x, Double_t *y)
{
    Double_t dx1 = x[0], dx2 = x[1];
    Double_t da = 10.0, db = 1.0;

    y[0] = da * (dx2 - dx1 * dx1) * (dx2 - dx1 * dx1)
            + db * (1.0 - dx1) * (1.0 - dx1);

}

void testOptimizer()
{
    // Attributes of the study
    TAttribute *x = new TAttribute("x", -1.5, 1.5);
    x->setDefaultValue(-1.2);
    x->setStepValue(0.01);

    TAttribute *y = new TAttribute("y", -1.5, 1.5);
    y->setDefaultValue(1.);
    y->setStepValue(0.01);

    // Collection of attributes
    TDataServer *tdsRosenbrock = new TDataServer("tdsRosenbrock",
            "Optimize Code externe Rosenbrock via TDataServer");
    tdsRosenbrock->addAttribute(x);
    tdsRosenbrock->addAttribute(y);

    // Optimizer process
    TOptimizer * topt = new TOptimizer(tdsRosenbrock, rosenbrock, "", "out");

    ofstream sortie;
    sortie.open("testOptimizer.out", ios::out);
    sortie << topt->getTolerance() << "\n";
    topt->setTolerance(0.001);
    sortie << topt->getTolerance() << "\n";

    sortie << topt->getMaxIterations() << "\n";
    topt->setMaxIterations(3000);
    sortie << topt->getMaxIterations() << "\n";

    sortie << topt->getMaxFunctionCalls() << "\n";
    topt->setMaxFunctionCalls(30000);
    sortie << topt->getMaxFunctionCalls() << "\n";

    sortie << topt->getMethod() << "\n";
    topt->setMethod(TOptimizer::kSimplex);
    sortie << topt->getMethod() << "\n";
    topt->setMethod(TOptimizer::kCombined);
    sortie << topt->getMethod() << "\n";
    topt->setMethod(TOptimizer::kScan);
    sortie << topt->getMethod() << "\n";
    topt->setMethod(TOptimizer::kMigrad);
    sortie << topt->getMethod() << "\n";

    sortie << topt->getPrintLevel() << "\n";
    topt->setPrintLevel(4);
    sortie << topt->getPrintLevel() << "\n";

    sortie.close();

}
