//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.4 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   vzr_struct.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2003-12-11.16.08.18

 \brief structure pour vizir

 */

#ifndef __VZR_STRUCT__
#define __VZR_STRUCT__

#include <pthread.h>
//extern int pthread_yield(void);

/* #define COMBINATOIRE
 * */

#ifdef COMBINATOIRE
typedef int vzrgene_t;
#define MPI_GENE MPI_INT
#else
typedef double vzrgene_t;
#define MPI_GENE MPI_DOUBLE
#endif
#define MPI_UNSAFE

#ifdef WIN32
#include "libVizir_EXPORT.h"
#else
#define libVizir_EXPORT
#endif

/* predefinition */
struct libVizir_EXPORT vizir;
struct libVizir_EXPORT vzrwork;
//typedef struct vzrwork vzrwork_t;

/* protocole */
struct libVizir_EXPORT vzrfitness
{
    void *data;
    int (*compute)(void *, vzrgene_t*, double*);
    void (*dealloc)(struct vzrfitness*);
};

struct libVizir_EXPORT vzrfitnessbuild
{
    void *data;
    struct vzrfitness *(*create)(void *);
    void (*dealloc)(struct vzrfitnessbuild*);
};

struct libVizir_EXPORT vzrcrossover
{
    void *data;
    void (*sample)(void*, vzrgene_t**);
    void (*allele)(void*, vzrgene_t**, vzrgene_t*);
    void (*mutation)(void*, vzrgene_t**);
    void (*homozygote)(void*, vzrgene_t**);
    void (*heterozygote)(void*, vzrgene_t**);
    void (*dealloc)(struct vzrcrossover *);
};

struct libVizir_EXPORT vzrcrossoverbuild
{
    void *data;
    struct vzrcrossover *(*create)(void *);
    void (*dealloc)(struct vzrcrossoverbuild *);
};

struct libVizir_EXPORT vzrclasse
{ 
    void *data;
    int rank_field_size;
    double (*rank)(void *, struct vizir*, struct vzrwork *);
    void (*rerank)(void *, struct vizir*);
    int (*elite_size)(void *, struct vizir*);
    void (*dealloc)(struct vzrclasse *);
};

struct libVizir_EXPORT vzrevent
{
    void *data;
    void (*react)(void *, struct vizir*);
    void (*dealloc)(struct vzrevent *);
};

/* structure */
struct libVizir_EXPORT vizir
{
    char *id; // identifiant (pour differentier les ilots)
    /* parametre resolution*/
    int nbgene, // nombre de gene
            nbfun, // nombre de critere
            nbsoft, // nombre de contrainte molle
            nbhide,  // sorties cach�es
            rangdomi, // indice du rang de domination
            nbcrit, // nbsoft + nbfun + nbhide + rang
            maxevalu, // seuil nombre maximum d'evaluation
            itaille, // taille population theorique
            N, // taille de la population pratique
            genedisc, // nombre de gene discret
            consang; // taille risque consanguinite
    double alpha, // taux de survie
            phomo, // taux d'homozigote cree
            pmut, // probabilite de mutation
            bias, // probabilite de biaisage des naissances (pere elite)
            prop, // facteur de surpopulation
            prec, // precision (mono)
            pmut1g; // probabilite de mutation un gene
    /* valeur d'execution */
    int ipop, // indice de l'individu courant
            nbevalu, // nombre d'evaluation
            nbsurv, // nombre de survivant
            nbelite, // taille de l'elite
            reeval, // indice de r��valuation des individus
            niter, // nombre de generation
            slave, // nombre d'executant
            imigr, // nombre d'individu imigre
            emigr, // nombre d'individu emigre
            nadmis; // nombre d'evaluation utile (contrainte hard)
    double fitseuil; // seuil des morts-n�s
    /* population */
    vzrgene_t ***population, // population actuelle et nouvelle
            ***pile_migrant; // fifo des migrants
    double **pop_val, // fitness de la population (anciens individus)
            **descendant_val; // fitness des nouveaux individus
    /* operateur */
    struct vzrfitnessbuild *ffun; // fabrique d'evaluateur
    struct vzrcrossoverbuild *foper; // fabrique d'operateurs de croisement
    struct vzrclasse *darwin; // fonction de synthese des fitness
    struct vzrevent *generation; // evenement nouvelle generation
    /* distribution */
    int bgstate; // etat de la resolution
    pthread_mutex_t mutex, // population courante
            mutim, // pile des imigrations
#ifdef MPI_UNSAFE
            mutmpi, // appel mpi (si mpi non thread-safe)
#endif
            mutpop; // population survivante
    /* fifo des migrants */
    int baspile, // index du bas de pile
            sommetpile, // index du haut de la pile
            taillepile; // taille de la pile

};

struct libVizir_EXPORT vzrwork
{
    vzrgene_t *pere[2], *mere[2], *descendant[2]; //genotype de travail
    double *val; //fitness individu courant
    /* operateur */
    struct vzrfitness *fun; // fermeture d'evaluation
    struct vzrcrossover *oper; // operateurs de croisement
};

#endif

