//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////

#include "gramschmitd.h"

#include <math.h>

static void GramSchmidt(int size, double *rot)
{
    int i, j, k;
    double proj, norm;

    /* gram-schmidt orthonormalisation */
    for (j = 0; j < size; j++)
    {
        for (i = 0; i < j; i++)
        {
            /* projection on vi */
            proj = 0.;
            //denom = 0.;
            for (k = 0; k < size; k++)
            {
                proj += rot[j * size + k] * rot[i * size + k];
                //denom += rot[i*size+k]*rot[i*size+k];
            }
            //proj /= denom;
            /* remove component in direction vi */
            for (k = 0; k < size; k++)
            {
                rot[j * size + k] -= proj * rot[i * size + k];
            }
        }
        /* normalisation */
        norm = 0;
        for (i = 0; i < size; i++)
        {
            norm += rot[j * size + i] * rot[j * size + i];
        }
        norm = sqrt(norm);
        for (i = 0; i < size; i++)
        {
            rot[j * size + i] /= norm;
        }
    }
}

void sorm_gramschmidt(int size, double *vec, double *rot)
{
    int i, j, big, lig;

    /* choix de la composante à retirer */
    big = 0;
    for (i = 1; i < size; i++)
    {
        if (fabs(vec[i]) > fabs(vec[big]))
            big = i;
    }

    /* copie des composantes */
    lig = 1;
    for (i = 0; i < size; i++)
    {
        /* nouvelle composante */
        rot[i] = vec[i];
        /* autre composante */
        if (i == big)
            continue;
        for (j = 0; j < size; j++)
        {
            rot[lig * size + j] = (j == i) ? 1. : 0.;
        }
        lig++;
    }

    GramSchmidt(size, rot);

}

void sorm_rotation(int size, double *rot, double *in, double *out)
{
    int i, j;

    for (i = 0; i < size; i++)
    {
        out[i] = 0.;
        for (j = 0; j < size; j++)
            out[i] += rot[j * size + i] * in[j];
    }
}

