//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TFitGetter.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-07-26.14.47.36

 \brief   recuperation de sortie transformées
 */

#include "TFitGetter.h"

namespace URANIE
{
namespace Reoptimizer
{

// TFitGetter
TFitGetter::TFitGetter(URANIE::DataServer::TAttribute *ref, TObjArray *all,
        TOptimFit *fit) :
        TObject(), _ref(ref), _grad(NULL), _fit(fit)
{
    int i;
    int size = all->GetEntries();

    // TODO test Double type ?
    _rang = -1;
    for (i = 0; i < size; i++)
        if (ref == (URANIE::DataServer::TAttribute*) all->At(i))
        {
            _rang = i;
            break;
        }
    if (_rang == -1)
        throw AttributeNotFound();
    _rangg = -1;
}

TFitGetter::TFitGetter(URANIE::DataServer::TAttribute *ref, 
    URANIE::DataServer::TAttribute *grd, TObjArray *all, TOptimFit *fit) :
    TObject(), _ref(ref), _grad(grd), _fit(fit)
{
    int i;
    int size = all->GetEntries();

    // TODO test Double type ?
    _rang = _rangg = -1;
    for (i = 0; i < size; i++) {
        if (ref == (URANIE::DataServer::TAttribute*) all->At(i)) _rang = i;
        if (grd == (URANIE::DataServer::TAttribute*) all->At(i)) _rangg = i;
    }
    if (_rang == -1 || _rangg == -1)
        throw AttributeNotFound();
}

TFitGetter::~TFitGetter()
{
}

double TFitGetter::from(double *From)
{
    if (_fit)
        return (*_fit)(From[_rang]);
    else
        return From[_rang];
}

double TFitGetter::from(vector<URANIE::DataServer::UEntry *> *From)
{
    if (_fit)
        return (*_fit)(*From->at(_rang)->val.dou);
    else
        return *From->at(_rang)->val.dou;
}

double TFitGetter::from(int size, double *From, double *grad, double *to)
{
    double ret;
    int i;

    if (_fit)
        ret = (*_fit)(size, From[_rang], grad + _rang * size, to);
    else
    {
        ret = From[_rang];
        for (i = 0; i < size; i++)
            to[i] = grad[_rang * size + i];
    }
    return ret;
}

double TFitGetter::from(int size, 
    vector<URANIE::DataServer::UEntry*> *from, 
    vector<URANIE::DataServer::UEntry*> *grad, double *to)
{
    double ret;
    vector<double> *grd;
    int i;
    
    ret = *from->at(_rang)->val.dou;
    grd = grad->at(_rang)->val.vec;
    
    if (_fit) ret = (*_fit)(size, ret, &(*grd)[0], to);
    else {
        for (i = 0; i < size; i++) to[i] = (*grd)[i];
    }
    return ret;
}

double TFitGetter::from(int size, 
    vector<URANIE::DataServer::UEntry*> *from, double *to)
{
    double ret;
    vector<double> *grd;
    int i;
    
    ret = *from->at(_rang)->val.dou;
    grd = from->at(_rangg)->val.vec;
    
    if (_fit) ret = (*_fit)(size, ret, &(*grd)[0], to);
    else {
        for (i = 0; i < size; i++) to[i] = (*grd)[i];
    }
    return ret;
}

}
} //namespace
