//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TOptimFit.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-11-28.15.27.23

 \brief objectifs et contraintes standards
 */

#ifndef __VIZIRFIT__
#define __VIZIRFIT__

//ROOT
#include "Rtypes.h"
#include "TNamed.h"

// Uranie
#include "Reoptimizer.h"

namespace URANIE
{
namespace Reoptimizer
{

class TOptimFit: public TObject
{
public:
    virtual double operator()(double) = 0;
    virtual double operator()(int, double, double*, double*) = 0;

    ClassDef(URANIE::Reoptimizer::TOptimFit, ID_REOPTIMIZER)
};

class TMaximizeFit: public TOptimFit
{
public:
    TMaximizeFit()
    {
    }
    virtual ~TMaximizeFit()
    {
    }

    virtual double operator()(double in)
    {
        return -in;
    }
    virtual double operator()(int, double, double*, double*);

    ClassDef(URANIE::Reoptimizer::TMaximizeFit, ID_REOPTIMIZER)
};

class TTargetFit: public TOptimFit
{
protected:
    double _ref;

public:
    TTargetFit(double val)
    {
        _ref = val;
    }
    virtual ~TTargetFit()
    {
    }

    virtual double operator()(double in);
    virtual double operator()(int, double, double*, double*);

    ClassDef(URANIE::Reoptimizer::TTargetFit, ID_REOPTIMIZER)
};

class TLesserFit: public TOptimFit
{
protected:
    double _ref;

public:
    TLesserFit(double val)
    {
        _ref = val;
    }
    virtual ~TLesserFit()
    {
    }

    virtual double operator()(double in)
    {
        return in - _ref;
    }
    virtual double operator()(int, double, double*, double*);

    ClassDef(URANIE::Reoptimizer::TLesserFit, ID_REOPTIMIZER)
};

class TGreaterFit: public TMaximizeFit
{
protected:
    double _ref;

public:
    TGreaterFit(double val)
    {
        _ref = val;
    }
    virtual ~TGreaterFit()
    {
    }

    virtual double operator()(double in)
    {
        return _ref - in;
    }
    virtual double operator()(int, double, double*, double*);

    ClassDef(URANIE::Reoptimizer::TGreaterFit, ID_REOPTIMIZER)
};

class TInsideFit: public TOptimFit
{
protected:
    double _low, _top;

public:
    TInsideFit(double, double);
    virtual ~TInsideFit()
    {
    }

    virtual double operator()(double in);
    virtual double operator()(int, double, double*, double*);

    ClassDef(URANIE::Reoptimizer::TInsideFit, ID_REOPTIMIZER)
};

extern TMaximizeFit kMaximizeFit;

}
} //namespace

#endif

