//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.4 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TOptimShare.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-03-20.11.12.53

 \brief methodes communes aux algo d optimisation
 */

#ifndef __TOPTIMSHARE__
#define __TOPTIMSHARE__

//ROOT
#include "Rtypes.h"
#include "TNamed.h"

// Uranie
#include "Reoptimizer.h"
#include "TMaster.h"

#include "TDataServer.h"
#include "TEval.h"
#include "TOptimFit.h"

namespace URANIE
{
namespace Reoptimizer
{

class TOptimShare: public URANIE::Relauncher::TMaster
{
private:
    void create();

protected:
    //int _gene, _hide;
    double _prec;
    TObjArray _const;
    URANIE::DataServer::TDataServer *_tdstrc;
    int _idtrc;
    Bool_t _blog;

    void initTrace();

public:
    TOptimShare(URANIE::DataServer::TDataServer *tds,
            URANIE::Relauncher::TRun *run);
    TOptimShare(URANIE::DataServer::TDataServer *tds,
            URANIE::Relauncher::TEval *fun);
    virtual ~TOptimShare();

    // initialisation
    virtual void addObjective(URANIE::DataServer::TAttribute *, TOptimFit *f =
            NULL) = 0;
    void addConstraint(URANIE::DataServer::TAttribute *, TOptimFit *f = NULL);
    //! Set the mono precision stop threshold
    void setTolerance(double prec)
    {
        _prec = prec;
    }
    double getTolerance()
    {
        return _prec;
    }
    
    void setTrace(URANIE::DataServer::TDataServer *tds) { _tdstrc = tds; }

    /** @name TMaster protocole **/
    //virtual void solverLoop(URANIE::Relauncher::TRun *) = 0;
    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    //@}

    ClassDef(URANIE::Reoptimizer::TOptimShare, ID_REOPTIMIZER)
};

}
} //namespace

#endif
