//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizirSimplex.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-02-10.06.52.54

 \brief   algorithme inspiré du simplex de Nelder Mead
 */

#include "TVizirSimplex.h"

#include "simplex/nms.h"
#include "vizir/vzr_macro.h"

ClassImp (URANIE::Reoptimizer::TVizirSimplex);

namespace URANIE
{
namespace Reoptimizer
{

// TVizirSimplex
/* constructeur */
TVizirSimplex::TVizirSimplex()
{
    defaultShare();
}

/* parameter value */

/* initialisation */
struct vzragent *TVizirSimplex::build(int gene, int obj, int cont, double prec)
{
    struct nms *self;

    self = nms_init(gene, obj, cont, prec);
    buildShare(&(self->base));
    
    if (_doresume) {
        nms_resume(self, _pop, _resume) ;
        vzrpop_set_again(&(self->base.pop));
    }
    else nms_prepare(self, _step, _pop, _eval);

    return vzragent_nms(self);
}

}
} //namespace
