//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestMain.cxx
 \author Gilles Arnaud <gilles.arnaud@cea.fr>
 \date   mar mai 21 14:41:19 2013
 
 \brief Programme principal pour les tests unitaires sur la librairie Reoptimizer
 
 */

// Liste des Tests Unitaires d'uranie
#include "TDirectory.h"

#ifdef WITH_NLOPT
#include "TestTNlopt.h"

CPPUNIT_TEST_SUITE_REGISTRATION( TNloptTest );
#endif

#ifdef WITH_VIZIR
#include "TestTTangram.h"
#include "TestTVizir.h"

CPPUNIT_TEST_SUITE_REGISTRATION( TTangramTest );
CPPUNIT_TEST_SUITE_REGISTRATION( TVizirJitTest );
CPPUNIT_TEST_SUITE_REGISTRATION( TVizirJitTest2 );
CPPUNIT_TEST_SUITE_REGISTRATION( TVizirIntTest );
#ifdef WITH_PYTHON
CPPUNIT_TEST_SUITE_REGISTRATION( TVizirExeTest );
#endif
#endif

bool uranie_test_suite(const char *module, const char *xml, const char *xsl);

int main()
{
    bool wasSuccessful;
#ifdef WIN32
    wasSuccessful = uranie_test_suite("Reoptimizer",
                                      "testReoptimizerResults.xml", "..\\..\\..\\..\\testReport.xsl");
#else
    wasSuccessful = uranie_test_suite("Reoptimizer",
                                      "testReoptimizerResults.xml", "../../../../testReport.xsl");
#endif

    gDirectory->ls();

    // ---  Return error code 1 if the one of test failed.
    return wasSuccessful ? 0 : 1;
}
