//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#ifndef __TESTTANGRAM__
#define __TESTTANGRAM__

// CppUnit
#include <cppunit/extensions/HelperMacros.h>
// #include <cppunit/Test.h>

// URANIE
#include "TDataServer.h"
#include "TVizirSolver.h"

class TTangramTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TTangramTest);

    CPPUNIT_TEST (swarmSequential);
    CPPUNIT_TEST (simplexThreaded);
#ifdef WITH_MPI
    CPPUNIT_TEST( geneticMpi );
    CPPUNIT_TEST( geneticBiMpi );
#ifdef MPI_IS_SHARED
    CPPUNIT_TEST( geneticUserMpi );
#endif
    CPPUNIT_TEST( geneticRobust1Mpi );
    CPPUNIT_TEST( geneticRobust2Mpi );
#ifdef WITH_PYTHON
    CPPUNIT_TEST( geneticRobustPyMpi );
#endif
#endif

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     *      **/
    //@{
    //! Default constructor
    TTangramTest();

    //! Default destructor
    virtual ~TTangramTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void tearDown();

    //@}
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void swarmSequential();
    void simplexThreaded();
#ifdef WITH_MPI
    void geneticMpi();
    void geneticBiMpi();
    void geneticUserMpi();
    void geneticRobust1Mpi();
    void geneticRobust2Mpi();
    void geneticRobustPyMpi();
#endif
    //@}

private:
    void validateTangram(URANIE::DataServer::TDataServer &tds, double prec);
    void validateTangram2(URANIE::DataServer::TDataServer &tds, double prec);
    void validateBi(URANIE::DataServer::TDataServer &tds);
    void validateRobustBi(URANIE::DataServer::TDataServer &tds);

};

#endif

