/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzr_agent.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-02-18
 
 \brief  agent island pour les fourmis
*/

#include <stdlib.h>

#include "vzr_agent.h"

#include "vizir/vzrfifo_api.h"
#include "vzr_core.h"
#include "vzr_query.h"

#define FIRST 3
#define STEP 100

static int vzr_emigrate(struct vizir *viz, vzrgene_t *migrant)
{
    /* cf vzr_getIndividu de vzr.i */
    int i, ret;
    struct vzrpop *pop = &viz->base.pop;

    pthread_mutex_lock(&(viz->base.lock));
    if (pop->level == pop->stop &&
            pop->cur != pop->level+pop->step
        ) {
        for(i=0; i<pop->gene; i++) {
            migrant[i] = pop->item[pop->rg[pop->cur]].dip.domin[i];
        }
        ret = 1;
    }
    else 
        ret = 0;
    pthread_mutex_unlock(&(viz->base.lock));
    return ret;
}

struct vzragent *vzragent_vzr(struct vizir *self)
{
    struct vzragent  *ret;

    /* agent */
    ret = (struct vzragent *) malloc(sizeof(struct vzragent));
    ret->data = (void *) self;
    ret->next = ret->first = FIRST * self->base.pop.stop;
    ret->step = STEP;
    ret->immigr = ret->emigr = 0;
    vzrfifo_init(&(ret->migr), self->base.pop.gene);
    vzrfifo_build(&(ret->migr), self->base.pop.stop);
    ret->core = (struct vzrcore *(*)(void*)) vzrcore_vzr;
    ret->emigrate = (int(*)(void*,vzrgene_t*)) vzr_emigrate ;
    ret->dealloc = (void(*)(void*)) vzr_dealloc ;

    self->base.migr = &(ret->migr);

    return ret;
}

