/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   nms_query.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-04-13
 
 \brief  simplex user interface
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "nms_query.h"

#include "vizir/vzrbase_api.h"
#include "vizir/vzrfifo_api.h"
#include "vizir/vzrpop_api.h"
#include "vizir/vzr_alloc.h"
#include "vizir/vzr_aleas.h"
#include "vizir/vzr_macro.h"
#include "vizir/vzritem.h"


struct nms *nms_init(int nbin, int nbout, int nbcont, double prec)
{
    struct nms *ret;

    ret = (struct nms *) malloc(sizeof(struct nms));
    vzrpop_init(&(ret->base.pop), nbin, nbcont, nbout, prec);

    /* working value */
    mt_init(0);
    vzrbase_init(&(ret->base));
    ret->base.migr = NULL;

    return ret;
}

int nms_prepare(struct nms *ret, int ant, int stop, int cyc)
{
    /* working value */
    vzrbase_build(&(ret->base), cyc);
    vzrpop_aploid_build(&(ret->base.pop), stop, ant);

    return 1;
}

int nms_build(struct nms *ret, int ant, int cyc, int stop)
{
    printf("nms_build is deprecated, use nms_prepare instead\n");
    return nms_prepare(ret, ant, stop, cyc);
}

int nms_resume(struct nms *ret, int maxeval, char *nfile)
{
    /* working value */
    vzrbase_build(&(ret->base), maxeval);
    vzrpop_resume(&(ret->base.pop), APLOID, nfile);

    return 1;
}

void nms_dump(struct nms *self, char *nfile)
{
    vzrpop_dump(&(self->base.pop), nfile);
}

void nms_dealloc(struct nms *self)
{
    vzrbase_dealloc(&(self->base));
    vzrpop_aploid_dealloc(&(self->base.pop));
    free(self);
}
