/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2003-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzr_alloc.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2003-12-10
 
 \brief  macro d'allocation de vecteur, matrice, 3DMatrice
*/


#ifndef __VZRALLOC__
#define __VZRALLOC__

#define NEWvecteur(type,vec,nb) vec = (type *) calloc((size_t) nb, sizeof(type))
#define NEWmatrice(type,mat,nl,nc) \
{\
    mat = (type **) calloc((size_t) nl, sizeof(type*));\
    mat[0] = (type *) calloc((size_t) (nc)*(nl), sizeof(type));\
    if (mat[0])\
        for(i=1;i<(nl);i++) mat[i]=mat[i-1] + (nc);\
    else {\
        free(mat);\
        mat = (type **) NULL;\
    }\
}
#define NEWdiploid(type,subtype,mat,nl,nc) \
{\
    mat = (type*) calloc((size_t) nl, sizeof(type));\
    mat[0][0] = (subtype *) calloc((size_t) (nc*nl*2), sizeof(subtype));\
    if (mat[0][0]) {\
        for(i=1;i<nl;i++) mat[i][0] = mat[i-1][0] + 2*nc;\
        for(i=0;i<nl;i++) mat[i][1] = mat[i][0] + nc;\
    } else {\
        free(mat);\
        mat = (type*) NULL;\
    }\
}
#define NEWmatrice3D(type,mat,nl,nc,n) \
{\
    mat = (type ***) calloc((size_t) nl, sizeof(type**));\
    mat[0] = (type **) calloc((size_t) (nc*nl), sizeof(type*));\
    mat[0][0] = (type *) calloc((size_t) (nc*nl*n), sizeof(type));\
    if (mat[0]) for(i=1;i<nl;i++) mat[i]= mat[i-1] + nc;\
    else {\
        free(mat);\
        mat = (type ***) NULL;\
    }\
    if(mat[0][0]) {\
        for(j=1;j<nc;j++) mat[0][j] = mat[0][j-1]+n;\
        for(i=1;i<nl;i++) {\
            mat[i][0] = mat[i-1][nc-1]+n;\
            for(j=1;j<nc;j++) mat[i][j] = mat[i][j-1]+n;\
        }\
    }\
    else {\
        free(mat[0]);\
        free(mat);\
        mat = (type ***) NULL;\
    }\
}

#define DELvecteur(vec) free(vec)
#define DELmatrice(mat) {free(mat[0]); free(mat);}
#define DELdiploid(mat) {free(mat[0][0]); free(mat);}
#define DELmatrice3D(mat) \
{ free(mat[0][0]); free(mat[0]); free(mat);}

#endif

