/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzr_macro.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-02-17
 
 \brief  macro pour un usage courant
*/

#ifndef __VZR_MACRO__
#define __VZR_MACRO__

#include "vizir/vzr_config.h"

/* pareto dominance 0-7 %1 */
#define VZR_DEFLT_POLICY 0
#define VZR_SOFT_POLICY 1
#define VZR_STRICT_POLICY 2
#define VZR_POLICY_MASK 7

/* survie 8-120 %8 */
#define VZR_PARETO_CONT 8

/* setter */
#define VZR_CONSTRAINT(self, val) vzrpop_set_policy(&((self)->base.pop), VZR_POLICY_MASK, val)
#define VZR_CONTINUE(self, val) vzrpop_set_policy(&((self)->base.pop), VZR_PARETO_CONT, val)
#define VZR_FITNESS(self, fun) (self)->base.fit = fun
#define VZR_PRECIS(self, prec) (self)->base.pop.precis = prec
#define VZR_SURV(self, val) (self)->base.pop.sort = val
#define VZR_HIDEN(self, nb) (self)->base.pop.hiden = nb
#define VZR_DELDATA(self, fun) (self)->base.pop.dealloc_data = fun
#define VZR_LABEL(self, lab) (self)->base.id = lab
#define VZR_QUIET(self) (self)->base.quiet = 1

#define VZR_PARETO_DEFAULT(self) VZR_CONSTRAINT(self, VZR_DEFLT_POLICY)
#define VZR_PARETO_SOFT(self) VZR_CONSTRAINT(self, VZR_SOFT_POLICY)
#define VZR_PARETO_STRICT(self) VZR_CONSTRAINT(self, VZR_STRICT_POLICY)

/* protocole */
#define PRO_ASK(obj, meth) obj->meth(obj->data)
#define PRO_CALL(obj, meth, ...) obj->meth(obj->data, __VA_ARGS__)
#define PRO_CREATE(obj) obj->create(obj->data)
#define PRO_DEALLOC(obj) obj->dealloc(obj)
#define PRO_DEALLOC2(obj, arg) obj->dealloc(obj, arg)

/*loop*/
#define EACH(id,size) (id=0;id<(size);id++)
#define FROMEACH(id,deb,fin) (id=(deb);id<(fin);id++)

/* matrice */
#define VECTEUR(typ, var, siz) typ var[siz]
#define MATRICE(typ, var, lsiz, csiz) typ *var[lsiz];\
    typ var ## _[lsiz*csiz];\
    { for(i=0;i<lsiz;i++) var[i]=var ## _+i*csiz; }

#endif
