/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzragent_api.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-12-02
 
 \brief  gestion des agents autonomes
*/

#ifndef __VZRAGENT_API__
#define __VZRAGENT_API__

/* dependance */
#include "vizir/vzr_config.h"
#include "vizir/vzrfifo_api.h"

/* predef */
struct vzrcore;

/* protocole */
struct vzrnetwork {
    void *data;
    int (*sample)(void*, int);
    void (*dealloc)(struct vzrnetwork *);
};

typedef struct vzrnetwork vzrnetwork_t;

/* structure */
typedef struct vzragent {
    void *data;
    int first, step, next;
    int immigr, emigr; // info suivi 
    struct vzrfifo migr;
    struct vzrcore *(*core)(void *);
    //void (*save)(void*, char*);
    int (*emigrate)(void *self, vzrgene_t *mig);
    void (*dealloc) (void *);
} vzragent_t;


/* private */
int vzragent_emigrate(struct vzragent *self, vzrgene_t *emig);
int vzragent_immigrate(struct vzragent *self, vzrgene_t *mig);
//int vzragent_immigration(struct vzragent *self, vzrgene_t *ind);
int vzragent_booting(struct vzragent *self);
int vzragent_running(struct vzragent *self);

/* public */
#define VZRAGENT_SET(self, boot, pas) (self)->next=(self)->first=boot; (self)->step=pas;
//#define VZRAGENT_SAVE(self, sauv) (self)->save = (void(*)(void*,char*)) sauv;

#ifdef __cplusplus
extern "C" {
#endif

static INLINE struct vzrbase *vzragent_base(struct vzragent *self){
    return (struct vzrbase *) self->data;
}

VZRLIB_EXPORT void vzragent_reset(struct vzragent *self);
VZRLIB_EXPORT struct vzrcore *vzragent_core(struct vzragent *self);
VZRLIB_EXPORT void vzragent_run(struct vzragent *self);
VZRLIB_EXPORT void vzragent_start(struct vzragent *self);
VZRLIB_EXPORT void vzragent_wait(struct vzragent *self);

VZRLIB_EXPORT int vzragent_converged(struct vzragent *self);
VZRLIB_EXPORT void vzragent_extend(struct vzragent *self, int eval);
VZRLIB_EXPORT void vzragent_fitness_changed(struct vzragent *self);

VZRLIB_EXPORT int vzragent_item(struct vzragent *self, int id, vzrgene_t **zone,
        double  **front, void **data, int *rang, int *age, int *info, double *isol);
VZRLIB_EXPORT void vzragent_dump(struct vzragent *self, char *nfile);
VZRLIB_EXPORT void vzragent_dealloc(struct vzragent *self);

#ifdef __cplusplus
} /* end of extern "C" */
#endif

#endif
