/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2009-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzrfifo_api.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2009-09-11
 
 \brief  gestion d'une fifo pour les immigrations
*/

#include <stdlib.h>

#include "vizir/vzrfifo_api.h"

#include "vizir/vzr_alloc.h"
#include "vizir/vzr_macro.h"

#define suiv(i) if (++(i) == self->maxi) i = 0;
//#define suiv(i) ((i)<self->maxi-1 ? ((i)+1) : 0)

void vzrfifo_init(struct vzrfifo *self, int nbgene)
{
    pthread_mutex_init(&(self->lock),( pthread_mutexattr_t *) NULL);
    self->bas = self->sommet = self->taille = self->maxi = 0 ;
    self->nbgene = nbgene;
    self->fifo = NULL;
}

int vzrfifo_build(struct vzrfifo *self, int taille)
{
    int     i; // need for NEWmatrice

    NEWmatrice(vzrgene_t, self->fifo, taille, self->nbgene);
    if (self->fifo) {
        self->maxi = taille;
        return 1;
    }
    else return 0;
}

int vzrfifo_put(struct vzrfifo *self, vzrgene_t *x)
{
    /* cf vzr_putIndividu de vzr.i */
    int i, ret;

    ret = 0;
    pthread_mutex_lock(&(self->lock));  
    if(self->taille < self->maxi){
        for EACH(i, self->nbgene) self->fifo[self->sommet][i]=x[i];
        suiv(self->sommet);self->taille++;
        ret = 1;
    }
    pthread_mutex_unlock(&(self->lock));
    return ret;
}

int vzrfifo_get(struct vzrfifo *self, vzrgene_t *x)
{
    int i, ret;

    pthread_mutex_lock(&(self->lock));
    ret = 0;
    if(self->taille > 0) {
        for(i=0;i<self->nbgene;i++) x[i] = self->fifo[self->bas][i];
        suiv(self->bas);self->taille--;
        ret = 1;
    }
    pthread_mutex_unlock(&(self->lock));
    return ret;
}

void vzrfifo_reset(struct vzrfifo *self)
{
    /* netoyage pour recommencer (apres synchronisation) */
    /* vide la pile des migrants */
    self->bas = self->sommet = self->taille = 0;
}

void vzrfifo_dealloc(struct vzrfifo *self)
{
    DELmatrice(self->fifo);
    pthread_mutex_destroy(&self->lock);
}
