/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzrtribe_api.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-12-15
 
 \brief  gestion des ilots via les threads
*/

#include <stdlib.h>

#include "vizir/vzrtribe_api.h"

#include "vizir/vzr_macro.h"

#include "vizir/vzr_aleas.h"
#include "vizir/vzrbase_api.h"

struct vzrtribe *vzrtribe_create(int siz, struct vzragent **il)
{
    struct vzrtribe *ret;

    ret = (struct vzrtribe *) malloc(sizeof(struct vzrtribe));
    ret->size = siz;
    ret->gene = ((struct vzrbase *) il[0]->data)->pop.gene;
    ret->ilot = il;
    ret->migr = NULL;

    return ret;
}

void vzrtribe_migre(struct vzrtribe *self, int nid)
{
    int qqn;
    struct vzragent *agent;
    VZRTAB(vzrgene_t, migre, self->gene);

    agent = self->ilot[nid];
    if (vzragent_emigrate(agent, migre)) {
        if (self->migr) 
            qqn = PRO_CALL(self->migr, sample, nid);
        else {
            do {
                qqn = (int) mt_int(self->size);
            }
            while (qqn == nid);
        }
        if (vzragent_running(self->ilot[qqn])) {
            vzragent_immigrate(self->ilot[qqn], migre);
        }
    }
    VZRTAB_OUT(migre);
}
    
void vzrtribe_run(struct vzrtribe *self)
{
    int all, nid;
    struct vzragent *agent;

    all = self->size;
    while (all) {
        all = self->size;
        for EACH(nid, self->size) {
            agent = self->ilot[nid];
            if (vzragent_running(agent)) {
                vzrtribe_migre(self, nid);
            }
            else all--;
        }
    }

}

void vzrtribe_dealloc(struct vzrtribe *self)
{
    int id;

    if (self->migr) PRO_DEALLOC(self->migr);
    for EACH(id, self->size) vzragent_dealloc(self->ilot[id]);
    free(self);
}

