#ifdef JIT
#include "TAttribute.h"
#include "TFlatScript.h"
#include "TKeyScript.h"
#include "TFlatResult.h"
#include "TCodeEval.h"
#include "TComposeEval.h"
//#include "TSequentialRun.h"
#include "TThreadedRun.h"
//#include "TMpiRun.h"
#include "TDataServer.h"
#include "TVizirGenetic.h"
#include "TVizir2.h"
#endif

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;

// user defined 
#define TOLERANCE 0.001
#define NBmaxEVAL 10000
#define TAILLEPOP 400

void test_TVizirComposeFlowrate()
{
    // Define the DataServer

    // Add the study attributes ( min, max and nominal values)
    TAttribute rw("rw", 0.05, 0.15), r("r", 100.0, 50000.0), tu("tu", 63070.0,
            115600.0), tl("tl", 63.1, 116.0), hu("hu", 990.0, 1110.0), hl("hl",
            700.0, 820.0), l("l", 1120.0, 1680.0), kw("kw", 9855.0, 12045.0);
    TAttribute yh("yhat"), d("d");

    // working dir
    TString dir(
            gSystem->Getenv("HOME") + TString("/tmp/LanceurUranie/flowrate"));

    // premier code
    TKeyScript fin1("flowrate1.py");
    fin1.setInputs(8, &rw, "Rw", &r, "R", &tu, "Tu", &tl, "Tl", &hu, "Hu", &hl,
            "Hl", &l, "L", &kw, "Kw");
    TFlatResult fout1("flowrate.res");
    fout1.setOutputs(1, &yh);
#ifdef PYTHON3
    TCodeEval code1("python2 flowrate1.py > flowrate.res");
#else
    TCodeEval code1("python flowrate1.py > flowrate.res");
#endif
    code1.setWorkingDir(dir);
    code1.addInputFile(&fin1);
    code1.addOutputFile(&fout1);

    // second code
    TKeyScript fin2("flowrate2.py");
    fin2.setInputs(2, &hu, "Hu", &hl, "Hl");
    TFlatResult fout2("flowrate2.res");
    fout2.setOutputs(1, &d);
#ifdef PYTHON3
    TCodeEval code2("python2 flowrate2.py > flowrate2.res");
#else
    TCodeEval code2("python flowrate2.py > flowrate2.res");
#endif
    code2.setWorkingDir(dir);
    code2.addInputFile(&fin2);
    code2.addOutputFile(&fout2);

    // compose code
    TComposeEval mycode;
    mycode.setEvals(2, &code1, &code2);
    mycode.addConstantValue(&r, 20000);
    mycode.addConstantValue(&tu, 90000);
    mycode.addConstantValue(&tl, 90);

    //TSequentialRun run(&mycode);
    TThreadedRun run(&mycode, 14);
    //TMpiRun run(&mycode);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tdsFlowrate",
                "OATMinMax Design of experiments for Flowrate");
        tds.addAttribute(&rw);
        //tds.addAttribute(&r);
        //tds.addAttribute(&tu);
        //tds.addAttribute(&tl);
        tds.addAttribute(&hu);
        tds.addAttribute(&hl);
        tds.addAttribute(&l);
        tds.addAttribute(&kw);
        TDataServer trc("tdsTrace", "trace optimisation");

        TVizirGenetic solv;
        solv.setSize(TAILLEPOP, NBmaxEVAL);
        TVizir2 vzr(&tds, &run, &solv);
        //vzr.setConstantValue(&r, 20000);
        //vzr.setConstantValue(&tu, 90000);
        //vzr.setConstantValue(&tl, 90);
        vzr.addObjective(&yh);
        vzr.setTolerance(TOLERANCE);
        vzr.setTrace(&trc);

        /* resolution */
        vzr.solverLoop();

        /* solution */
        //tds.getTuple()->Scan("*");
        tds.exportData("_flow_.dat");
        trc.exportData("_flow_trace_.dat");
        run.stopSlave();
    }

}

