//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TArchimedianCopula.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mer oct 17 17:36:39 CEST 2012
 
 \brief Interface of the class URANIE::Sampler::TArchimedianCopula
 
 */
/**
 @class URANIE::Sampler::TArchimedianCopula
 
 @brief Description of the class TArchimedianCopula.

 Archimedian Copula are defined by (in 2D)
 \f[
 C(u,v) = \phi^{-1}\left( \phi(u) + \phi(v) \right)
 \f]

 This copula is defined with one parameter (\b _dparam ).
 
 */
#ifndef TARCHIMEDIANCOPULA_H
#define TARCHIMEDIANCOPULA_H
#include "TCopula.h"

namespace URANIE
{
namespace Sampler
{
class TArchimedianCopula: public TCopula
{
    // Attributes
public:
    Double_t _dparam;               ///< the parameter of the archimediam copula
private:
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TArchimedianCopula(URANIE::DataServer::TDataServer *tds, Double_t dparam,
            Int_t nCalcul);
    //! Default destructor
    virtual ~TArchimedianCopula();
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TArchimedianCopula, ID_SAMPLER)
    //  ArchimedianCopula Class
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
