//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*! \file TDistribution.cxx
 * \brief Implementation of the class representing some probability law (distribution) 
 *
 */

#include "TDistribution.h"

ClassImp (TDistribution)

TDistribution::TDistribution()
{

    TVectorD unit(2);
    unit.Zero();
    TMatrixD Id(2, 2);
    Id.UnitMatrix();
    initialisation(unit, Id);
}

TDistribution::TDistribution(TVectorD M, TMatrixD A)
{
    initialisation(M, A);
    TString sLabelVariable(5 * _dim - 1);
    sLabelVariable += "indice:";
    for (Int_t i = 1; i < _dim; i++)
    {
        sLabelVariable += "var";
        sLabelVariable += i;
        sLabelVariable += ":";
    }
    sLabelVariable += "var";
    sLabelVariable += _dim;
    _vect = new TNtupleD("echantillon", "echantillon2", sLabelVariable.Data());
    cout << sLabelVariable << endl;
}

void TDistribution::initialisation(TVectorD M, TMatrixD A)
{
    _taille = 0;
    _dprob = 1;
    _dim = M.GetNrows();
    _vectMean.ResizeTo(_dim);
    _vectMean = M;
    _matEcartTypeCorr.ResizeTo(_dim, _dim);
    _matEcartTypeCorr = A;
    _rdm = new TRandom3();
    _rdm->SetSeed(0);
    _rdm->Uniform(1.0);
}
TDistribution::~TDistribution()
{
    delete _vect;
    _vect = NULL;
    delete _rdm;
    _rdm = NULL;
}

void TDistribution::setTaille(Int_t n)
{
    _taille = n;
}
void TDistribution::simulation()
{

}

Int_t TDistribution::getDim()
{
    return _dim;
}

TVectorD TDistribution::getMean()
{
    return _vectMean;
}

TVectorD TDistribution::getSigma()
{
    TVectorD etype(_matEcartTypeCorr.GetNrows());
    for (Int_t i = 0; i < _matEcartTypeCorr.GetNrows(); i++)
    {
        etype(i) = _matEcartTypeCorr(i, i);
    }

    return etype;
}

TMatrixD TDistribution::getMatEcartTypeCorrelation()
{
    return _matEcartTypeCorr;
}

TMatrixD TDistribution::getMatCorrelation()
{
    TMatrixD mat(_matEcartTypeCorr.GetNrows(), _matEcartTypeCorr.GetNrows());
    mat = _matEcartTypeCorr;
    for (Int_t i = 0; i < _matEcartTypeCorr.GetNrows(); i++)
        mat(i, i) = 1;

    return mat;
}

TMatrixD TDistribution::getMatCovariance()
{
    TMatrixD M(_matEcartTypeCorr.GetNrows(), _matEcartTypeCorr.GetNrows());
    M = getMatEcartTypeCorrelation();
    for (Int_t i = 0; i < _matEcartTypeCorr.GetNrows(); i++)
        for (Int_t j = 0; j < i; j++)
        {
            M(i, j) = M(i, j) * M(i, i) * M(j, j);
            M(j, i) = M(i, j);
        }
    for (Int_t k = 0; k < _matEcartTypeCorr.GetNrows(); k++)
        M(k, k) = M(k, k) * M(k, k);

    return M;
}

void TDistribution::setMean(TVectorD M)
{

    _dim = M.GetNrows();
    _vectMean.ResizeTo(_dim);
    _matEcartTypeCorr.ResizeTo(_dim, _dim);
    _vectMean = M;
    verification();
}

void TDistribution::setMatEcartTypeCorrelation(TMatrixD A)
{
    _dim = A.GetNrows();
    _vectMean.ResizeTo(_dim);
    _matEcartTypeCorr.ResizeTo(_dim, _dim);
    _matEcartTypeCorr = A;
    verification();
}

void TDistribution::verification()
{
    if (_vectMean.GetNrows() != _matEcartTypeCorr.GetNrows())
    {
        cout << "**** pas la meme dim entre moy[" << _vectMean.GetNrows()
                << "] et varcov[" << _matEcartTypeCorr.GetNrows() << "]"
                << endl;
    }
}

