//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TGaussianCopula.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   2012-11-28 10:37:02
 
 \brief Interface of the class URANIE::Sampler::TGaussianCopula
 
 */
/**
 @class URANIE::Sampler::TGaussianCopula
 
 @brief Description of the class TGaussianCopula.
 
 
 */
#ifndef TGAUSSIANCOPULA_H
#define TGAUSSIANCOPULA_H

////// URANIE
#include "TEllipticalCopula.h"
#include "UExceptions.h"

namespace URANIE
{
namespace Sampler
{
class TGaussianCopula: public TEllipticalCopula
{
private:
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TGaussianCopula(URANIE::DataServer::TDataServer *tds, Option_t *option,
            Int_t nCalcul);
    //! Default destructor
    virtual ~TGaussianCopula();
    //@}

    //---------------------------------------------
    /** @name Generation of the sample
     **/
    //@{
    //! Generates the matrix of U
    /*!
     \param option (Option_t *) The option to pass.
     */
    void generateUMatrix(Option_t *option);
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TGaussianCopula, ID_SAMPLER)
    // TGaussianCopula Class
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
