//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
// THaltonSequence mar jui 25 13:03:02 CEST 2006
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   THaltonSequence.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   tue jul 25 13:03:02 CEST 2006
 
 \brief Implementation of the class URANIE::Sampler::THaltonSequence
 
 */
#include <assert.h>

#include "THaltonSequence.h"

ClassImp (URANIE::Sampler::THaltonSequence)

namespace URANIE
{
namespace Sampler
{
////////////////////////////////////////////////////////////
//  Constructor with size
////////////////////////////////////////////////////////////
THaltonSequence::THaltonSequence(ULong_t ndim) :
        TSequence(ndim), Radical(NULL), Base(NULL)
{
}
THaltonSequence::~THaltonSequence()
{
    if (_blog)
        cout << endl << " ** Destroy THaltonSequence" << endl;
    if (Base)
    {
        delete[] Base;
        Base = NULL;
    }
    if (Radical)
    {
        delete[] Radical;
        Radical = NULL;
    }
}

//////////////////////////////////////////////////////////
//  The init method
//////////////////////////////////////////////////////////
void THaltonSequence::init()
{
    Radical = new Double_t[_nDim];
    assert(Radical);
    FirstPrimes(_nDim);

    for (Int_t j = 0; j < (Int_t) _nDim; j++)
    {
        Radical[j] = 1.0 / (double) Base[j];
        _dValue[j] = 0.0;
    }
    setInstance(0);
}

//////////////////////////////////////////////////////////
//  The setInstance method
//////////////////////////////////////////////////////////
void THaltonSequence::setInstance(const ULong_t n)
{
    ULong_t im;
    Int_t b;
    Double_t fac;

    for (Int_t j = 0; j < (Int_t) _nDim; j++)
    {
        _dValue[j] = 0.0;
        fac = Radical[j];
        b = Base[j];

        for (im = n; im > 0; im /= b, fac *= Radical[j])
            _dValue[j] += fac * (double) (im % b);

        if (_blog)
            cout << "[" << j << "] prime[" << Base[j] << "] val0[" << _dValue[j]
                    << "]" << endl;
    }
}

void THaltonSequence::FirstPrimes(const ULong_t n)
{
    Int_t i, j, p, b;

    if (n == 0)
    {
        delete[] Base;
        Base = NULL;
        return;
    }
    Base = new Int_t[n];
    assert(Base);

    Base[0] = 2;

    for (p = 3, i = 1; i < (Int_t) n; p += 2)
    {
        Base[i] = p;

        for (j = 1; (b = Base[j] <= p / Base[j]) && (p % Base[j]); j++)
            ;

        if (b == 0)
            i++;
    }
}

//////////////////////////////////////////////////////////
//  Printing Log
//////////////////////////////////////////////////////////
void THaltonSequence::printLog(Option_t *option)
{
    cout << "THaltonSequence::printLog[" << option << "]" << endl;
}

} // Fin du namespace Sampler
} // Fin du namespace URANIE

