//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TOA.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mar apr 15 23:16:30 CEST 2008
 
 \brief Interface of the class URANIE::Sampler::TOA
 
 */
/**
 
 * @class URANIE::Sampler::TOA
 *
 * @brief Description of the class TOA.
 *
 * To be written by the developper.
 *
 */
#ifndef TOA_H
#define TOA_H

#include <iostream>
using std::cout;
using std::endl;
using std::cerr;

////// URANIE
#include "TSamplerStochastic.h"

namespace URANIE
{
namespace Sampler
{
class TOA: public TSamplerStochastic
{
private:
    Int_t _nSymbols; ///< The number of symbols
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TOA(URANIE::DataServer::TDataServer *tds, Option_t *option = "oa",
            Int_t nCalcul = 1000);
    //! Default destructor
    virtual ~TOA();
    //@}

    //---------------------------------------------
    /** @name Generation of the sample
     **/
    //@{
    //! Generates the sample
    virtual void generateSample(Option_t *option);
    //@}

    //---------------------------------------------
    /** @name Machine state
     For stochastic methods, 2 states \em init et \em terminate can be created :
     the first one can be executed just before the generation of the sample, and the second
     one just after the generation. They can be used for pre or post traitement purpose.
     **/
    //@{
    //! The pre-traitment step
    virtual void init();
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TOA, ID_SAMPLER)
    //Classe de
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
