//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TPetras.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mar apr 8 15:09:37 CEST 2008
 
 \brief Interface of the class URANIE::Sampler::TPetras
 
 */
/**
 
 * @class URANIE::Sampler::TPetras
 *
 * @brief Description of the class URANIE::Sampler::TPetras.
 *
 * To be written by the developper.
 *
 */
#ifndef TPETRAS_H
#define TPETRAS_H
#include "TSamplerDeterminist.h"

#include <iostream>
using std::cout;
using std::endl;

namespace URANIE
{
namespace Sampler
{
class TPetras: public TSamplerDeterminist
{
    // Attributes
private:
    Int_t _nQ;
    // Operations
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    /*!
     The last argument is the level required by Petras and not the size of the sample.
     */
    TPetras(URANIE::DataServer::TDataServer *tds, Option_t *option, Int_t nS);
    //! Default destructor
    virtual ~TPetras();
    //@}

    //---------------------------------------------
    /** @name Generation of the sample
     **/
    //@{
    //! Generates the sample
    virtual void generateSample(Option_t *option = "");
    //@}

    //---------------------------------------------
    Int_t getLevel()
    {
        return _nQ;   
    }
    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TPetras, ID_SAMPLER)
    //Classe de
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
// fin du fichier $RCSfile$.
