//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TSamplerStochastic.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   Wed Mar 8 09:16:13 2006

 \brief  Creation of the abstract class TSamplerStochastic


 */
#include <ctime>

////// ROOT
#include "TRandom3.h"

////// URANIE
#include "TSamplerStochastic.h"

ClassImp (URANIE::Sampler::TSamplerStochastic);

namespace URANIE
{
namespace Sampler
{
// Constructor with a dataserver
TSamplerStochastic::TSamplerStochastic(URANIE::DataServer::TDataServer *tds,
        Option_t *option, Int_t ns) :
    TSampler(tds, option, ns)
{
    setSeed();
}

// destructor
TSamplerStochastic::~TSamplerStochastic()
{
}

void TSamplerStochastic::init()
{
    // delete the TDSNtupleD if it exists
    try
    {
        if (_tds->hasTuple())
            throw URANIE::Exceptions::UInfoExceptions(__FILE__, __LINE__,
                    Form(
                            "TSamplerStochastic::init: the TDS [%s] contains data: we need to empty it ! ",
                            _tds->GetName()));
    } catch (URANIE::Exceptions::UInfoExceptions& ue)
    {
        ue.printMessage();
	_tds->deleteTuple();
    }

    // print Log information
    if (_blog)
    {
        cout << " ** _nseed[" << _nSeed << "]" << endl;
        _lstOfAttributesToSample->Print();
    }
}

void TSamplerStochastic::terminate()
{
    if (_blog)
        cout << "TSamplerStochastic::terminate[]" << endl;
}

void TSamplerStochastic::setSeed(Int_t ind)
{
    if (ind == 0)
    {
        time_t secondes;
        time(&secondes);
        _nSeed = (Int_t)(secondes);
    }
    else
    {
        _nSeed = ind;
    }
    gRandom->SetSeed(_nSeed);
}


////////////////////////////////////////////////////////////
//  Printing Log
////////////////////////////////////////////////////////////
void TSamplerStochastic::printLog(Option_t *option)
{
    cout << "**** TSamplerStochastic::printLog[" << option << "] Name["
            << GetName() << "] Title[" << GetTitle() << "] ID[" << GetID()
            << "]" << endl;
    TSampler::printLog(option);
    cout << " ** _nSeed  [" << _nSeed << "]" << endl;
    cout << "**** End Of TSamplerStochastic::PrintLog" << endl;
}

} // Fin du namespace Sampler
} // Fin du namespace URANIE

// fin du fichier $RCSfile$.
