//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
#include <assert.h>

#include "TSequence.h"

using std::cout;
using std::endl;

ClassImp (URANIE::Sampler::TSequence)

namespace URANIE
{
namespace Sampler
{

//---------------------------------------------
/** @name Static varibale
 **/
//@{
//! Counter of instance
Int_t TSequence::_seqCounter = 0;
//! Boolean of the log
Bool_t TSequence::_blog = kFALSE;
//@}
TSequence::TSequence() :
        _nDim(0)
{
    if (_blog)
        cout << " ** Build TSequence []" << endl;
    _seqCounter++;
    if (_blog)
        cout << " ** counter TSequence [" << _seqCounter << "]" << endl;

}
TSequence::TSequence(ULong_t ndim) :
        _nDim(ndim)
{
    if (_blog)
        cout << " ** Build TSequence[" << ndim << "]" << endl;
    if (_nDim < 1)
    {
        throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                Form(
                        "TSequence::TSequence constructor ERROR : the dim %lu must be positive",
                        ndim));
    }
    _dValue = new Double_t[_nDim];
    assert(_dValue);

    _seqCounter++;
    if (_blog)
        cout << "compteur TSequence [" << _seqCounter << "]" << endl;

}

TSequence::~TSequence()
{
    if (_blog)
        cout << endl << " ** Destroy TSequence" << endl;
    _seqCounter--;
    if (_blog)
        cout << " ** counter TSequence [" << _seqCounter << "]" << endl;
    if (_dValue)
    {
        delete[] _dValue;
        _dValue = NULL;
    }
}

void TSequence::printLog(Option_t *option)
{
    cout << "TSequence::printLog[" << option << "] Name[" << GetName()
            << "] Title[" << GetTitle() << "]" << endl;
    cout << "_nDim[" << _nDim << "]" << endl;
    cout << "TSequence::end of printLog[]" << endl;

}

} // Fin du namespace Sampler
} // Fin du namespace URANIE
