//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TBASICSAMPLING_H
#define CPPUNITEST_TBASICSAMPLING_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSampler.h"
#include "TBasicSampling.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TBasicSamplingTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TBasicSamplingTest);
    CPPUNIT_TEST (testTBasicSamplingConstructorSmallSize);
    CPPUNIT_TEST (testBasicSamplinggenerateSampleLHS);
    CPPUNIT_TEST (testBasicSamplinggenerateSampleSRS);
    CPPUNIT_TEST (testBasicSamplinggenerateCorrSample);
    CPPUNIT_TEST (testBasicSamplinggenerateCorrSampleSRS);
    CPPUNIT_TEST (testTBasicSamplingSetUserCorrelation);
    CPPUNIT_TEST (tesTBasicSamplingtSetTolerance);
    CPPUNIT_TEST (testBasicSamplingGenerateSampleSimilar);

    //CPPUNIT_TEST( throwExceptionBasicSamplinggenerateSample);
    CPPUNIT_TEST (throwExceptionBasicSamplingConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionsetCorrelationMatrix);
    CPPUNIT_TEST (throwExceptionsetUserCorrelationNameFalse);
    CPPUNIT_TEST (throwExceptionsetUserCorrelationValueFalse);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TBasicSamplingTest();

    //! Default destructor
    virtual ~TBasicSamplingTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! 
    void testBasicSamplinggenerateSampleLHS();
    //!
    void testBasicSamplinggenerateSampleSRS();
    //!
    void testBasicSamplinggenerateCorrSample();
    //!
    void testBasicSamplinggenerateCorrSampleSRS();
    //!
    void testTBasicSamplingConstructorSmallSize();
    //!
    void testTBasicSamplingSetUserCorrelation();
    //!
    void tesTBasicSamplingtSetTolerance();
    //!
    void testBasicSamplingGenerateSampleSimilar();
    //!
    void throwExceptionBasicSamplingConstructorTdsNull();
    //!
    void throwExceptionsetCorrelationMatrix();
    //!
    void throwExceptionsetUserCorrelationNameFalse();
    //!
    void throwExceptionsetUserCorrelationValueFalse();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds, *_tds2;
    URANIE::Sampler::TBasicSampling *_bspl;

};

#endif  // CPPUNITEST_TBASICSAMPLING_H
