//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TCOPULA_H
#define CPPUNITEST_TCOPULA_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSampler.h"
#include "TCopula.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TCopulaTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TCopulaTest);

    CPPUNIT_TEST (testTCopulaClayton);
    CPPUNIT_TEST (testTCopulaFrank);
    CPPUNIT_TEST (testTCopulaPlackett);
    CPPUNIT_TEST (testTCopulaPlackettThetaUn);
    CPPUNIT_TEST (testTCopulaAliMikhailHaq);
    CPPUNIT_TEST (testTCopulaElliptical);
    CPPUNIT_TEST (testTCopulaGaussian);
    CPPUNIT_TEST (testTCopulaGaussianWithCorrelation);
    CPPUNIT_TEST (testTCopulaGaussianTAttribut);

    CPPUNIT_TEST (throwExceptionTCopulaClaytonThetaNegatif);
    CPPUNIT_TEST (throwExceptionTCopulaPlackettThetaNegatif);
    CPPUNIT_TEST (throwExceptionTCopulaAliMikhailHaqThetaSupUn);
    CPPUNIT_TEST (throwExceptionTCopulaAliMikhailHaqThetaInfMoinsUn);
    CPPUNIT_TEST (throwExceptionTCopulaConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionTEllipticalCopulasetUserCorrelationAttributeValueFalse);
    CPPUNIT_TEST (throwExceptionTEllipticalCopulasetUserCorrelationSameIndex);
    CPPUNIT_TEST (throwExceptionTEllipticalCopulasetUserCorrelationIndexNegative);
    CPPUNIT_TEST (throwExceptionTEllipticalCopulasetUserCorrelationIndexSup);
    CPPUNIT_TEST (throwExceptionTCopulaGaussianCorrelationNegative);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TCopulaTest();

    //! Default destructor
    virtual ~TCopulaTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //!
    void testTCopulaClayton();
    //!
    void testTCopulaFrank();
    //!
    void testTCopulaPlackett();
    //!
    void testTCopulaPlackettThetaUn();
    //!
    void testTCopulaAliMikhailHaq();
    //!
    void testTCopulaElliptical();
     //!
    void testTCopulaGaussian();
     //!
    void testTCopulaGaussianWithCorrelation();
    //!
    void testTCopulaGaussianTAttribut();
   //@}
    //---------------------------------------------
    /** @name Unit Tests
     The list of throw Exception
     **/
    //@{ 
    //!
    void throwExceptionTCopulaClaytonThetaNegatif();
    //!
    void throwExceptionTCopulaPlackettThetaNegatif();
    //!
    void throwExceptionTCopulaAliMikhailHaqThetaSupUn();
    //!
    void throwExceptionTCopulaAliMikhailHaqThetaInfMoinsUn();
    //!
    void throwExceptionTCopulaConstructorTdsNull();
    //!
    void throwExceptionTEllipticalCopulasetUserCorrelationAttributeValueFalse();
    //!
    void throwExceptionTEllipticalCopulasetUserCorrelationSameIndex();
    //!
    void throwExceptionTEllipticalCopulasetUserCorrelationIndexNegative(); 
    //!
    void throwExceptionTEllipticalCopulasetUserCorrelationIndexSup();
    //!
    void throwExceptionTCopulaGaussianCorrelationNegative();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds;
    URANIE::Sampler::TCopula *_tcop;
    Int_t _nSize;

};

#endif  // CPPUNITEST_TCOPULA_H
