//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TGAUSSIANSAMPLING_H
#define CPPUNITEST_TGAUSSIANSAMPLING_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TGaussianSampling.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TGaussianSamplingTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TGaussianSamplingTest);
    CPPUNIT_TEST (testGaussianSampling);
    CPPUNIT_TEST (testGaussianSamplingSRS);
    CPPUNIT_TEST (testGaussianSamplingWithCorrelation);
    CPPUNIT_TEST (testGaussianSamplingWithSeed);
    CPPUNIT_TEST (testGaussianSamplingWithFixedSumCovariance);

    CPPUNIT_TEST (throwExceptionWithOtherLaw);
    CPPUNIT_TEST (throwExceptionTGaussianSamplinggenerateSampleMatCorrNegative);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingConstrutorTdsNull);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationSameIndex);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationIndexINegative);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationIndexJNegative);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationIndexValueFalse);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationNameValueFalse);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationAttributeValueFalse);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationIndexISup);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationIndexJSup);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationFalseNameI);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationFalseNameJ);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationFalseAttributI);
    CPPUNIT_TEST (throwExceptionTGaussianSamplingsetUserCorrelationFalseAttributJ);CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TGaussianSamplingTest();

    //! Default destructor
    virtual ~TGaussianSamplingTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test of the GaussianSampling with LHS option
    void testGaussianSampling();
    //! Test of the GaussianSampling with SRS option
    void testGaussianSamplingSRS();
    //! Test of the GaussianSampling with correlation seed
    void testGaussianSamplingWithCorrelation();
    //! Test of the GaussianSampling with the seed
    void testGaussianSamplingWithSeed();
    //! Test of the GaussianSampling with fixed sum of covariance
    void testGaussianSamplingWithFixedSumCovariance();
    //@}

    //---------------------------------------------
    /** @name Throw exceptions
     **/
    //@{ 
    //! Throw an exception when an stochastic attribute is not a TNormalDistribution
    void throwExceptionWithOtherLaw();
    //!
    void throwExceptionTGaussianSamplingConstrutorTdsNull();
    //!
    void throwExceptionTGaussianSamplinggenerateSampleMatCorrNegative();
    /* //! */
    /* void throwExceptionTGaussianSamplinggenerateSampleByDakota99991(); */
    /* //!  */
    /* void throwExceptionTGaussianSamplinggenerateSampleByDakotaLawNoKnown(); */
    /* //!  */
    void throwExceptionTGaussianSamplinggenSample();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationSameIndex();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationIndexINegative();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationIndexJNegative();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationIndexValueFalse();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationNameValueFalse();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationAttributeValueFalse();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationIndexISup();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationIndexJSup();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationFalseNameI();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationFalseNameJ();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationFalseAttributI();
    //! 
    void throwExceptionTGaussianSamplingsetUserCorrelationFalseAttributJ();

    //@}

private:
    URANIE::DataServer::TDataServer *_tdsAllNormal;
    URANIE::DataServer::TDataServer *_tdsANormalAndUniform;

};

#endif  // CPPUNITEST_TDATASERVER_H
