//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestTImportanceSampling.h
 \author Maud Salvoldi <AREVA/OCC>
 \date   $Date$
 
 \brief  Fichier de test de la classe  URANIE::DataServer::TImportanceSampling
 
 Created le : lun jui 18 09:41:03 CEST 2011
 
 */
#ifndef CPPUNITEST_TIMPORTANCESAMPLING_H
#define CPPUNITEST_TIMPORTANCESAMPLING_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TDataServer.h"
#include "TUniformDistribution.h"
#include "TNormalDistribution.h"
#include "UExceptions.h"
#include "TImportanceSampling.h"
#include "TSampling.h"
#include "TSampler.h"

class TImportanceSamplingTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TImportanceSamplingTest);

    CPPUNIT_TEST (throwExceptionTImportanceSamplingTdsNull);
    CPPUNIT_TEST (throwExceptionTImportanceSamplingNameUnknow);
    CPPUNIT_TEST (testgetTDS);
    CPPUNIT_TEST (testgenerateSample);
    CPPUNIT_TEST (throwExceptioncomputeMeanFalseName);
    CPPUNIT_TEST (testcomputeMean);
    CPPUNIT_TEST (throwExceptioncomputeStdFalseName);
    CPPUNIT_TEST (throwExceptionTestcomputeStdEmptySample);
    CPPUNIT_TEST (testcomputeStd);
    CPPUNIT_TEST (throwExceptioncomputeThresholdFalseName);
    CPPUNIT_TEST (testcomputeThreshold);
    CPPUNIT_TEST (throwExceptionComputeQuantileFalseName);
    CPPUNIT_TEST (throwExceptionComputeQuantileFalseAlpha);
    CPPUNIT_TEST (throwExceptionComputeQuantileFalseMethod);
    CPPUNIT_TEST (testComputeQuantile);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TImportanceSamplingTest();

    //! Default destructor
    virtual ~TImportanceSamplingTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and Tear Down methods      
     **/
    //@{
    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     
     **/
    //@{
    //! Unit test to check the parameter tds
    /*!
     \brief This test checks that the code returns an error message if the parameter tds is empty.
     */
    void throwExceptionTImportanceSamplingTdsNull();
    /*!
     \brief This test checks that the code returns an error message if the parameter name is unknwon of the DataServer.
     */
    void throwExceptionTImportanceSamplingNameUnknow();
    /*!
     \brief This test checks that the code returns the new DataServer built par the constructor.
     */
    void testgetTDS();
    //! Test the generateSample method
    void testgenerateSample();
    /*!
     \brief This test checks that the code returns an error message if the parameter name is unknown of the DataServer.
     */
    void throwExceptioncomputeMeanFalseName();
    /*!
     \brief This test checks that the code compute correctly the mean of the attribute given in parameter.
     */
    void testcomputeMean();
    /*!
     \brief This test checks that the code returns an error message if the parameter name is unknown of the DataServer.
     */
    void throwExceptioncomputeStdFalseName();
    /*!
     \brief This test checks that the code returns an error message if the sample is empty .
     */
    void throwExceptionTestcomputeStdEmptySample();
    /*!
     \brief This test checks that the code compute correctly the std of the attribute given in parameter.
     */
    void testcomputeStd();
    /*!
     \brief This test checks that the code returns an error message if the parameter name is unknown of the DataServer.
     */
    void throwExceptioncomputeThresholdFalseName();
    /*!
     \brief This test checks that the code compute correctly the alpha quantile of the attribute given in parameter.
     */
    void testcomputeThreshold();
    /*!
     \brief This test checks that the code returns an error message if the parameter name is unknown of the DataServer.
     */
    void throwExceptionComputeQuantileFalseName();
    /*!
     \brief This test checks that the code returns an error message if the parameter alpha is not between 0 and 1.0.
     */
    void throwExceptionComputeQuantileFalseAlpha();
    /*!
     \brief This test checks that the code returns an error message if the parameter nMethod is not between one of kQuantile1, kQuantile2.
     */
    void throwExceptionComputeQuantileFalseMethod();
    /*!
     \brief This test checks that the code compute correctly the alpha quantile of the attribute given in parameter.
     */
    void testComputeQuantile();

    //@}
private:
    URANIE::DataServer::TDataServer *_tds;
    URANIE::DataServer::TDataServer *_tdstmp;
    URANIE::DataServer::TDataServer *_tdsis;
    URANIE::DataServer::TDSNtupleD * _tuple;
    URANIE::DataServer::TNormalDistribution *_att;
    URANIE::Sampler::TImportanceSampling *_tis;

    Double_t _seuil;
    Int_t _nS;
};

#endif
