//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TMAXIMINLHS_H
#define CPPUNITEST_TMAXIMINLHS_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TMaxiMinLHS.h"

class TMaxiMinLHSTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TMaxiMinLHSTest);

    CPPUNIT_TEST (testMaxiMinLHS);
    CPPUNIT_TEST (testMaxiMinLHSFilledTDS);

    CPPUNIT_TEST (throwExceptionTMaxiMinLHSConstrutorTdsNull);
    CPPUNIT_TEST (throwExceptionTMaxiMinLHSWrongTemperature);
    CPPUNIT_TEST (throwExceptionTMaxiMinLHSWrongTemperatureCoeff);
    CPPUNIT_TEST (throwExceptionTMaxiMinLHSWrongNIter);
    CPPUNIT_TEST (throwExceptionTMaxiMinLHSWrongNinnerIter);
    CPPUNIT_TEST (throwExceptionTMaxiMinLHSFilledTDSWrongNS);
    
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TMaxiMinLHSTest();

    //! Default destructor
    virtual ~TMaxiMinLHSTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test of the names
    void testMaxiMinLHS();
    void testMaxiMinLHSFilledTDS();
    //@}

    //---------------------------------------------
    /** @name throwException methods
     **/
    //@{
    //!
    void throwExceptionTMaxiMinLHSConstrutorTdsNull();
    //!
    void throwExceptionTMaxiMinLHSWrongTemperature();
    //!
    void throwExceptionTMaxiMinLHSWrongTemperatureCoeff();
    //!
    void throwExceptionTMaxiMinLHSWrongNIter();
    //!
    void throwExceptionTMaxiMinLHSWrongNinnerIter();
    //!
    void throwExceptionTMaxiMinLHSFilledTDSWrongNS();
    //@}

private:
    TString _workingTestPath;

    URANIE::DataServer::TDataServer *_tds;
    URANIE::Sampler::TMaxiMinLHS *_sampler;
    Int_t _nS;
};

#endif  // CPPUNITEST_TSAMPLING_H

