//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TSPACEFILLING_H
#define CPPUNITEST_TSPACEFILLING_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSpaceFilling.h"
#include "TSampler.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TSpaceFillingTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TSpaceFillingTest);

    CPPUNIT_TEST (testTSpaceFillingConstructor);
    CPPUNIT_TEST (throwExceptionTSpaceFillingConstructorTdsNull);
    CPPUNIT_TEST (testTSpaceFillingsetTransformFunctionANDgetTransformFunction);
    CPPUNIT_TEST (testTSpaceFillingsetPermutationANDgetPermutation);
    CPPUNIT_TEST (throwExceptionTSpaceFillingsetPermutationBadValue);
    CPPUNIT_TEST (testTSpaceFillingsetFrequenciesANDgetFrequency);
    CPPUNIT_TEST (throwExceptionTSpaceFillinggetFrequencyHighIndex);
    CPPUNIT_TEST (throwExceptionTSpaceFillingsetFrequencyHighIndex);
    CPPUNIT_TEST (throwExceptionTSpaceFillingsetFrequencyLessIndex);
    CPPUNIT_TEST (throwExceptionTSpaceFillingsetFrequencyBadValue);
    CPPUNIT_TEST (testTSpaceFillingsetSize);
    CPPUNIT_TEST (testTSpaceFillingenerateSample);
    CPPUNIT_TEST (testTSpaceFillinggenerateSampleCukier);
    CPPUNIT_TEST (testTSpaceFillinggenerateSampleSaltelliB);
    CPPUNIT_TEST (testTSpaceFillinggenerateSampleNormalLaw);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TSpaceFillingTest();

    //! Default destructor
    virtual ~TSpaceFillingTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //!
    void testTSpaceFillingConstructor();
    //!
    void testTSpaceFillingsetTransformFunctionANDgetTransformFunction();
    //!
    void testTSpaceFillingsetPermutationANDgetPermutation();
    //!
    void testTSpaceFillingsetFrequenciesANDgetFrequency();
    //!
    void testTSpaceFillingenerateSample();
    //!
    void testTSpaceFillinggenerateSampleCukier();
    //!
    void testTSpaceFillinggenerateSampleSaltelliB();
    //!
    void testTSpaceFillinggenerateSampleNormalLaw();
    //!
    void testTSpaceFillingsetSize();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of throw exception
     **/
    //@{ 
    //!
    void throwExceptionTSpaceFillingConstructorTdsNull();
    //!
    void throwExceptionTSpaceFillingsetPermutationBadValue();
    //!
    void throwExceptionTSpaceFillinggetFrequencyHighIndex();
    //!
    void throwExceptionTSpaceFillingsetFrequencyHighIndex();
    //!
    void throwExceptionTSpaceFillingsetFrequencyLessIndex();
    //!
    void throwExceptionTSpaceFillingsetFrequencyBadValue();
    //@}

private:
    Int_t _nXDefault, _nXCukier, _nXSaltelliB;
    URANIE::DataServer::TDataServer *_tdsDefault;
    URANIE::DataServer::TDataServer *_tdsCukier;
    URANIE::DataServer::TDataServer *_tdsSaltelliB;
    URANIE::Sampler::TSpaceFilling *_samplerDefault;
    URANIE::Sampler::TSpaceFilling *_samplerCukier;
    URANIE::Sampler::TSpaceFilling *_samplerSaltelliB;
};

#endif  // CPPUNITEST_TSPACEFILLING_H
