//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TRBD.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mer jun 9 15:03:08 CEST 2010
 
 \brief Interface of classe URANIE::Sensitivity::TRBD
 
 */
/**
 
 * @class URANIE::Sensitivity::TRBD
 *
 * @brief Description of the class TRBD.
 *
 * To write by the developper.
 *
 */
#ifndef RBD_H
#define RBD_H
#include "TFast.h"

namespace URANIE
{
namespace Sensitivity
{
class TRBD: public TFast
{
private:
    //! constructor helpers
    void stdInit();
    
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor with a function
#ifdef ROOT5
    TRBD(URANIE::DataServer::TDataServer *tds, void* fcn, Int_t ns = 100,
            const char *varexpinput = "", const char *varexpoutput = "");
#else
    TRBD(URANIE::DataServer::TDataServer *tds, void (*fcn)(Double_t*,Double_t*),
            const char *varexpinput, const char *varexpoutput, Int_t ns = 100);
#endif
    //! Default constructor with a function name
    TRBD(URANIE::DataServer::TDataServer *tds, const char* fcn, Int_t ns = 100,
            const char *varexpinput = "", const char *varexpoutput = "");
    //! Default constructor with a tcode
    TRBD(URANIE::DataServer::TDataServer *tds, URANIE::Launcher::TCode *fcode,
            Int_t ns);
    //! Default constructor with a trun
    TRBD(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TRun *run,
            Int_t ns);
    //! Default destructor
    virtual ~TRBD();
    //@}

    //---------------------------------------------
    /** @name Functions to the frequencies
     The RBD method is to choose the same  frequency to each uncertainty attribute
     **/
    //@{
    //! Sets the frequencies array
    void setFrequencies(Int_t nfreq = 0, Int_t valfreq = 1);
    //@}

    //---------------------------------------------
    /** @name Generate the sample
     **/
    //@{
    virtual void generateSample(Option_t * option = "");
    //@}

    //---------------------------------------------
    /** @name Computes the indexes
     **/
    //@{
    //! Evaluates the index from the dataserver
    virtual void evaluateIndexes(Option_t * option = "");
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sensitivity::TRBD, ID_SENSITIVITY)
    //Classe de
};

} // Fin du namespace Sensitivity
} // Fin du namespace URANIE
#endif
