//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef TESTTFAST_H
#define TESTTFAST_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSensitivity.h"
#include "TOutputFileRow.h"
#include "TFast.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TFastTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TFastTest);

    CPPUNIT_TEST (testTFastwithCode);
#ifdef WITH_LEVELE
    CPPUNIT_TEST (testLevelEFast);
    CPPUNIT_TEST (testLevelEFastOutputRestrOption);
#endif
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputFunction);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputCode);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputRun);
    CPPUNIT_TEST (throwExceptionTFastConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionTFastConstructorFalseCode);
    CPPUNIT_TEST (throwExceptionTFastConstructorFalseSize);
    CPPUNIT_TEST (testTFastsetFrequencies);
    CPPUNIT_TEST (throwExceptionTFastsetFrequencies);
    CPPUNIT_TEST (testIshigamiFAST);
    CPPUNIT_TEST (throwExceptionTFastConstructorFalseFnct);
    CPPUNIT_TEST (throwExceptionTFastConstructorFalseInput);
    CPPUNIT_TEST (throwExceptionTFastConstructorToMoreInput);
    CPPUNIT_TEST (testTFastsetTransformFunction);
    CPPUNIT_TEST (testTFastsetM);

    CPPUNIT_TEST (testIshigamiFastComputeIndexesWithTLauncherFunctionName);
    CPPUNIT_TEST (testIshigamiFastComputeIndexesWithTLauncherFunction);
    CPPUNIT_TEST (testIshigamiFastComputeIndexesWithThreadedandNot);
    CPPUNIT_TEST (testIshigamiFastComputeIndexesWithTLauncher2);
    CPPUNIT_TEST (testIshigamiFastComputeIndexesGenerateSampleWithTLauncherFunction);
    CPPUNIT_TEST (testIshigamiFastComputeIndexesGenerateSampleCreateLauncherWithTLauncherFunction);

    CPPUNIT_TEST (testFlowrateFastComputeIndexesWithTLauncherCode);
    CPPUNIT_TEST (testFlowrateFastComputeIndexesGenerateSampleWithTLauncherCode);
    CPPUNIT_TEST (testFlowrateFastComputeIndexesGenerateSampleCreateLauncherWithTLauncherCode);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TFastTest();

    //! Default destructor
    virtual ~TFastTest();

    void setUp();
    void setUp1();
    void setUp10();
    void setUp2();
    void tearDown();
	
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test the FAST method for the Ishigami function
    void testTFastwithCode();
#ifdef WITH_LEVELE
    void setUpLevelE();
    //! Test of the Fast for both SRC and SRRC using LevelE
    void testLevelEFast();
    //! Test the Sobol method for the Levele function restraining output vector in option
    void testLevelEFastOutputRestrOption();
    //! Check the results of the levelE sobol estrimation
    void checkLevelEResults(TTree *ntresu, vector<int> *elmts = NULL);
#endif
    void throwExceptionConstructorTooLongOutputFunction();
    void throwExceptionConstructorTooLongOutputCode();
    void throwExceptionConstructorTooLongOutputRun();
    void throwExceptionTFastConstructorTdsNull();
    void throwExceptionTFastConstructorFalseCode();
    void throwExceptionTFastConstructorFalseSize();
    void testTFastsetFrequencies();
    void throwExceptionTFastsetFrequencies();
    void testIshigamiFAST();
    void throwExceptionTFastConstructorFalseFnct();
    void throwExceptionTFastConstructorFalseInput();
    void throwExceptionTFastConstructorToMoreInput();
    void testTFastsetTransformFunction();
    void testTFastsetM();

    //---------------------------------------------

    /** @name Validation of the sample values for Ishigami
     **/
    //@{
    void validateValuesIshigami();
    //@}

    /** @name Validation of the sample values for Flowrate
     **/
    //@{
    void validateValuesFlowrate();
    //@}

    //---------------------------------------------

    //! Test the ability to launch computeIndexes alone with a TLauncherFunction called only with the name
    void testIshigamiFastComputeIndexesWithTLauncherFunctionName();
    //! Test the ability to launch computeIndexes alone with a TLauncherFunction
    void testIshigamiFastComputeIndexesWithTLauncherFunction();
    //! Test the ability to launch computeIndexes alone with a TLauncher2
    void testIshigamiFastComputeIndexesWithThreadedandNot();
    //! Test the ability to launch computeIndexes alone with a TLauncher2
    void testIshigamiFastComputeIndexesWithTLauncher2();
    //! Test the ability to launch computeIndexes after generateSample with a TLauncherFunction
    void testIshigamiFastComputeIndexesGenerateSampleWithTLauncherFunction();
    //! Test the ability to launch computeIndexes after generateSample a code creation with a TLauncherFunction
    void testIshigamiFastComputeIndexesGenerateSampleCreateLauncherWithTLauncherFunction();
    //! Test the ability to launch computeIndexes alone with a TLauncher
    void testFlowrateFastComputeIndexesWithTLauncherCode();
    //! Test the ability to launch computeIndexes after generateSample with a TLauncher
    void testFlowrateFastComputeIndexesGenerateSampleWithTLauncherCode();
    //! Test the ability to launch computeIndexes after generateSample a code creation with a TLauncher
    void testFlowrateFastComputeIndexesGenerateSampleCreateLauncherWithTLauncherCode();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds, *_tds2, *_tds3;
    URANIE::Sensitivity::TFast *_tfast;
    Int_t ns;
    URANIE::Launcher::TCode *_mycode;
    void * _p2fIshigami, *_p2fFlowratemodel;
    URANIE::Launcher::TOutputFileRow *_fout;
};

#endif
