//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TJOHNSONRW_H
#define CPPUNITEST_TJOHNSONRW_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSensitivity.h"
#include "TJohnsonRW.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TJohnsonRWTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TJohnsonRWTest);

    CPPUNIT_TEST (testJohnsonRWFromPaper);
    CPPUNIT_TEST (testJohnsonRWFromDatabase);
    CPPUNIT_TEST (testJohnsonRWFromPaperTwoOutputs);
    // CPPUNIT_TEST (testJohnsonRWFromPaperTwoOutputsAsVector);

#ifdef WITH_LEVELE
    CPPUNIT_TEST (testLevelEJohnsonRW);
    CPPUNIT_TEST (testLevelEJohnsonRWOutputRestrVarexp);
    CPPUNIT_TEST (testLevelEJohnsonRWOutputRestrOption);
    CPPUNIT_TEST (testLevelEJohnsonRWOutputRestrBoth);
#endif

    CPPUNIT_TEST (testJohnsonRWIshigamiFunctionLauncher);
    CPPUNIT_TEST (testJohnsonRWIshigamiFunctionRelauncher);
    CPPUNIT_TEST (testJohnsonRWIshigamiFunctionThreadedandNot);
    
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputFunctionFirst);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputFunctionSecond);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputCode);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputRun);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputFromData);
    CPPUNIT_TEST (throwExceptionTJohnsonRWConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionTJohnsonRWConstructorFalseInput);
    CPPUNIT_TEST (throwExceptionTJohnsonRWConstructorFalseOutput);
    CPPUNIT_TEST (throwExceptionTJohnsonRWCorrelationMatrix);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TJohnsonRWTest();

    //! Default destructor
    virtual ~TJohnsonRWTest();

    void setUp();
    TMatrixD setUpIshi();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Test of the JohnsonRW with numerical example from Paper
    void testJohnsonRWFromPaper();
    //! Test of the JohnsonRW with numerical example from Paper
    void testJohnsonRWFromPaperTwoOutputs();
    //! Test of the JohnsonRW with numerical example from Paper
    void testJohnsonRWFromPaperTwoOutputsAsVector();    
    //! Test of the JohnsonRW from flowrate
    void testJohnsonRWFromDatabase();


#ifdef WITH_LEVELE
    //! Test of the JohnsonRW for both SRC and SRRC using LevelE
    void testLevelEJohnsonRW();
    //! Test the Sobol method for the Levele function restraining output vector in constructor
    void testLevelEJohnsonRWOutputRestrVarexp();
    //! Test the Sobol method for the Levele function restraining output vector in option
    void testLevelEJohnsonRWOutputRestrOption();
    //! Test the Sobol method for the Levele function restraining output vector in both
    void testLevelEJohnsonRWOutputRestrBoth();
    //! Check the results of the levelE sobol estrimation
    void checkLevelEResults(TTree *ntresu, vector<int> *elmts = NULL);
#endif

    void testJohnsonRWIshigamiFunctionLauncher();
    void testJohnsonRWIshigamiFunctionRelauncher();
    void testJohnsonRWIshigamiFunctionThreadedandNot();
    void checkIshigamiResults(TTree *res);
    
//     //! Test the SRC/SRRC JohnsonRW for the Ishigami function
    void throwExceptionConstructorTooLongOutputFunctionFirst();
    void throwExceptionConstructorTooLongOutputFunctionSecond();
    void throwExceptionConstructorTooLongOutputCode();
    void throwExceptionConstructorTooLongOutputRun();
    void throwExceptionConstructorTooLongOutputFromData();
    void throwExceptionTJohnsonRWConstructorTdsNull();
    void throwExceptionTJohnsonRWConstructorFalseInput();
    void throwExceptionTJohnsonRWConstructorFalseOutput();
    void throwExceptionTJohnsonRWCorrelationMatrix();
    //@}

private:
    URANIE::DataServer::TDataServer *tds1, *tds;
    URANIE::Sensitivity::TJohnsonRW *tjrw;
    URANIE::Sensitivity::TJohnsonRW *tss1;
};

#endif  // CPPUNITEST_TJOHNSONRW_H
