//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTRBD.cxx
 \author maud Salvoldi (OCC/Areva group)
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Sensitivity::TRBD
 
 Cree le : jeu dec 22 12:30:15 CET 2011
 
 */

#define EPSILON1 0.01
#define EPSILON2 0.02
#define LVLE_EPSILON 0.001
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTSensitivity.h"
#include "TestTRBD.h"
#include "TRBD.h"
#include "UDisableTest.h"
#include "TUniformDistribution.h"
#include "TLogUniformDistribution.h"
#include "TOutputFileRow.h"
#include "TOutputFileRow.h"
#include "TCode.h"
#include "TLauncher.h"
#include "TLauncherFunction.h"
#include "TKeyScript.h"
#include "TFlatResult.h"
#include "TCodeEval.h"
#include "TCJitEval.h"
#include "TSequentialRun.h"
#include "TThreadedRun.h"
#include "TLauncher2.h"
//#include "TThreadedRun.h"
#include "TestUserFunctions.h"
#include "code_paths.h"
#include "uranie_commands.h"

// ROOT
#include "TString.h"
#include "TMath.h"
#include "TTreePlayer.h"
#include "TROOT.h"
#ifdef ROOT5
#include "Api.h"
#endif

#include <iostream> 
#include <iomanip> 
using namespace std;

TRBDTest::TRBDTest() :
  _tds(NULL), _tds2(NULL), _tds3(NULL), _trbd(NULL), _mycode(NULL), _p2fIshigami(NULL), _fout(NULL)
{
#ifdef ROOT5
    //   gROOT->LoadMacro("UserFunctions.C");
    G__ClassInfo globalscope;
    G__MethodInfo method;
    long dummy;
    method = globalscope.GetMethod("Ishigami", "double*,double*", &dummy);
    _p2fIshigami = method.PointerToFunc();
#endif
}


TRBDTest::~TRBDTest()
{
    UranieTests::removeFile(gSystem->pwd(),"testRBDFlowrate.out");
    UranieTests::removeFile(gSystem->pwd(),"_rbd_original_sample.dat");
    UranieTests::removeFile(gSystem->pwd(),"_rbd_final_sample.dat");
    UranieTests::removeFile(gSystem->pwd(),"_rbd_launching_.dat");
    UranieTests::removeFile(gSystem->pwd(),"outpuT.dat");
    UranieTests::removeFile(gSystem->pwd(),"outpuS.dat");
}

void TRBDTest::setUp()
{
    _tds = new URANIE::DataServer::TDataServer("tdsflowreate",
            "DataBase flowreate");
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("rw", 0.05, 0.15));
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("r", 100.0, 50000.0));
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("tu", 63070.0,
                    115600.0));
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("tl", 63.1, 116.0));
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("hu", 990.0, 1110.0));
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("hl", 700.0, 820.0));
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("l", 1120.0, 1680.0));
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("kw", 9855.0,
                    12045.0));
    _tds->getAttribute("rw")->delShare();
    _tds->getAttribute("r")->delShare();
    _tds->getAttribute("tu")->delShare();
    _tds->getAttribute("tl")->delShare();
    _tds->getAttribute("hu")->delShare();
    _tds->getAttribute("hl")->delShare();
    _tds->getAttribute("l")->delShare();
    _tds->getAttribute("kw")->delShare();
       
    _tds2 = new URANIE::DataServer::TDataServer("tds",
            "TDS for Ishigami function");
    _tds2->addAttribute(
            new URANIE::DataServer::TUniformDistribution("x_{1}",
                    -1.0 * TMath::Pi(), TMath::Pi()));
    _tds2->addAttribute(
            new URANIE::DataServer::TUniformDistribution("x_{2}",
                    -1.0 * TMath::Pi(), TMath::Pi()));
    _tds2->addAttribute(
            new URANIE::DataServer::TUniformDistribution("x_{3}",
                    -1.0 * TMath::Pi(), TMath::Pi()));
    _tds2->getAttribute("x_{1}")->delShare();
    _tds2->getAttribute("x_{2}")->delShare();
    _tds2->getAttribute("x_{3}")->delShare();
    
    _ns = 10000;

    _tds3 = NULL;
    _fout = NULL;
}

void TRBDTest::tearDown()
{
    if (_fout)
    {
        delete _fout;
        _fout = NULL;

    }
 
    if (_mycode != NULL)
    {
        delete _mycode;
        _mycode = NULL;
    }

    delete _trbd;
    _trbd = NULL;   
  
    delete _tds2;
    _tds2 = NULL;
    
    delete _tds;
    _tds = NULL;

    if (_tds3)
      {
        delete _tds3;
        _tds3 = NULL;
      }

}
//////////////////////////
//                      //
//    test              //
//                      //
/////////////////////////
void TRBDTest::setUp1()
{
#ifdef ROOT5
    _trbd = new URANIE::Sensitivity::TRBD(_tds2, _p2fIshigami, 1000);
#else
    _trbd = new URANIE::Sensitivity::TRBD(_tds2, &Ishigami, "", "Ishigami", 1000);
#endif
    _trbd->setDrawProgressBar(kFALSE);
    _trbd->setSeed(123);
}

void TRBDTest::setUp2()
{
    TString sJDDReference = gSystem->pwd();
    sJDDReference += TString("/flowrate_input_with_values_rows.in");
    _tds->getAttribute("rw")->setFileKey(sJDDReference, "Rw", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    _tds->getAttribute("r")->setFileKey(sJDDReference, "R", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    _tds->getAttribute("tu")->setFileKey(sJDDReference, "Tu", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    _tds->getAttribute("tl")->setFileKey(sJDDReference, "Tl", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    _tds->getAttribute("hu")->setFileKey(sJDDReference, "Hu", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    _tds->getAttribute("hl")->setFileKey(sJDDReference, "Hl", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    _tds->getAttribute("l")->setFileKey(sJDDReference, "L", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    _tds->getAttribute("kw")->setFileKey(sJDDReference, "Kw", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);

    _fout = new URANIE::Launcher::TOutputFileRow("_output_flowrate_withRow_.dat");
    _fout->addAttribute("yhat");

    _mycode = new URANIE::Launcher::TCode(_tds, UranieTests::code_path("flowrate") + " -s -r");
    _mycode->addOutputFile(_fout);

    _ns = 1000;
    _trbd = new URANIE::Sensitivity::TRBD(_tds, _mycode, _ns);
    _trbd->setDrawProgressBar(kFALSE);
    _trbd->setSeed(123);
}

void TRBDTest::testTRBDwithFunction()
{
    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_IGNORED_TEST()
    ;

    gROOT->Macro("testRBDFlowrate.C");

    ifstream outFile("testRBDFlowrate.out", ios::in);

    Int_t nS;
    outFile >> nS;

    Double_t *dval = new Double_t[nS];
    for (Int_t i = 0; i < nS; i++)
        outFile >> dval[i];

    //Close the file
    outFile.close();

    CPPUNIT_ASSERT_EQUAL(16, nS);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.747765, dval[0], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.747765, dval[1], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00172775, dval[2], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00172775, dval[3], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00292971, dval[4], EPSILON1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00292971, dval[5], EPSILON1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00210481, dval[6], EPSILON1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00210481, dval[7], EPSILON1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0350558, dval[8], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0350558, dval[9], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0373881, dval[10], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0373881, dval[11], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0414046, dval[12], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0414046, dval[13], EPSILON2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0117342, dval[14], EPSILON1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0117342, dval[15], EPSILON1);

    delete[] dval;
    dval = NULL;
}

void TRBDTest::testTRBDwithCode()
{

    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_IGNORED_TEST()
    ;

    TString sJDDReference = gSystem->pwd();
    sJDDReference += TString("/flowrate_input_with_keys.in");
    _tds->getAttribute("rw")->setFileKey(sJDDReference, "Rw");
    _tds->getAttribute("r")->setFileKey(sJDDReference, "R");
    _tds->getAttribute("tu")->setFileKey(sJDDReference, "Tu");
    _tds->getAttribute("tl")->setFileKey(sJDDReference, "Tl");
    _tds->getAttribute("hu")->setFileKey(sJDDReference, "Hu");
    _tds->getAttribute("hl")->setFileKey(sJDDReference, "Hl");
    _tds->getAttribute("l")->setFileKey(sJDDReference, "L");
    _tds->getAttribute("kw")->setFileKey(sJDDReference, "Kw");

    // The output file of the code
    URANIE::Launcher::TOutputFileRow *fout =
            new URANIE::Launcher::TOutputFileRow("_output_flowrate_withRow_.dat");
    // The attribute in the output file
    fout->addAttribute("yhat");

    // Instanciation de mon code  
    TString sRepertoireReference = gSystem->pwd();
#ifdef WIN32
    URANIE::Launcher::TCode *mycode = new URANIE::Launcher::TCode(_tds, "flowrate.exe -k 1>NUL");
#else
    URANIE::Launcher::TCode *mycode =
            new URANIE::Launcher::TCode(_tds,
                    Form(
                            "%s/../../../../launCHER/teST/coDE/flowREATE/flowrate  -k>>/dev/null",
                            sRepertoireReference.Data()));
#endif
    mycode->addOutputFile(fout);

    Int_t ns = 500;
    _trbd = new URANIE::Sensitivity::TRBD(_tds, mycode, ns);
    _trbd->setDrawProgressBar(kFALSE);
    _trbd->computeIndexes("nointermed");

    Double_t sobol;
    TTree *tree = _trbd->getResultTuple();
    tree->SetBranchAddress("Sobol", &sobol);

    CPPUNIT_ASSERT_EQUAL(16, (Int_t)(tree->GetEntries()));

    delete fout;
    delete mycode;
}

#ifdef WITH_LEVELE
// ====================================
//            LevelE
// ====================================
void TRBDTest::setUpLevelE()
{
  _tds3 = new URANIE::DataServer::TDataServer("tdsLevelE","levele");
  _tds3->addAttribute( new URANIE::DataServer::TUniformDistribution("t", 100, 1000) );
  _tds3->addAttribute( new URANIE::DataServer::TLogUniformDistribution("kl", 0.001, 0.01) );
  _tds3->addAttribute( new URANIE::DataServer::TLogUniformDistribution("kc", 0.000001, 0.00001) );
  _tds3->addAttribute( new URANIE::DataServer::TLogUniformDistribution("v1", 0.001, 0.1) );
  _tds3->addAttribute( new URANIE::DataServer::TUniformDistribution("l1", 100, 500) );
  _tds3->addAttribute( new URANIE::DataServer::TUniformDistribution("r1", 1, 5) );
  _tds3->addAttribute( new URANIE::DataServer::TUniformDistribution("rc1", 3, 30) );
  _tds3->addAttribute( new URANIE::DataServer::TLogUniformDistribution("v2", 0.01, 0.1) );
  _tds3->addAttribute( new URANIE::DataServer::TUniformDistribution("l2", 50, 200) );
  _tds3->addAttribute( new URANIE::DataServer::TUniformDistribution("r2", 1, 5) );
  _tds3->addAttribute( new URANIE::DataServer::TUniformDistribution("rc2", 3, 30) );
  _tds3->addAttribute( new URANIE::DataServer::TLogUniformDistribution("w", 100000, 10000000) );

  TString name="levelE_input_with_values_rows.in";
  _tds3->getAttribute("t")->setFileKey(name, "t", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("kl")->setFileKey(name, "kl", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("kc")->setFileKey(name, "kc", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("v1")->setFileKey(name, "v1", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("l1")->setFileKey(name, "l1", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("r1")->setFileKey(name, "r1", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("rc1")->setFileKey(name, "rc1", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("v2")->setFileKey(name, "v2", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("l2")->setFileKey(name, "l2", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("r2")->setFileKey(name, "r2", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("rc2")->setFileKey(name, "rc2", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);
  _tds3->getAttribute("w")->setFileKey(name, "w", "%e", URANIE::DataServer::TAttributeFileKey::kNewRow);

  _tds3->getAttribute( "t" )->delShare();
  _tds3->getAttribute( "kl" )->delShare();
  _tds3->getAttribute( "kc" )->delShare();
  _tds3->getAttribute( "v1" )->delShare();
  _tds3->getAttribute( "l1" )->delShare();
  _tds3->getAttribute( "r1" )->delShare();
  _tds3->getAttribute( "rc1" )->delShare();
  _tds3->getAttribute( "v2" )->delShare();
  _tds3->getAttribute( "l2" )->delShare();
  _tds3->getAttribute( "r2" )->delShare();
  _tds3->getAttribute( "rc2" )->delShare();
  _tds3->getAttribute( "w" )->delShare      ();

  _fout = new URANIE::Launcher::TOutputFileRow("_output_levelE_withRow_.dat");
  _fout->addAttribute(new URANIE::DataServer::TAttribute("y", URANIE::DataServer::TAttribute::kVector), 2 );
  _fout->getAttribute(0)->delShare();

#ifdef WIN32
  _mycode = new URANIE::Launcher::TCode(_tds3, "levele.exe");
#else
  _mycode = new URANIE::Launcher::TCode(_tds3, Form("%s/%s",gSystem->pwd(),"../../../../launCHER/teST/coDE/levelE/levele 2> /dev/null "));
#endif
  _mycode->addOutputFile(_fout);  
  
}

void TRBDTest::checkLevelEResults(TTree *ntresu, vector<int> *elmts)
{

  int npts = ntresu->Draw("Sobol"," Inp==\"w\" && Algo==\"--first--\" ","goff");
  double *first=ntresu->GetV1();

  double firref[26]={0.0655585704567,0.108223395282,0.164623431882,0.182772633833,0.159900229516,0.121798158293,0.0987812179497,0.0960098708989,0.112918212722,
		     0.0383260843544,0.0225496315861,0.0237385871304,0.026223654309,0.0289017551991,0.0310425804372,0.0321912398824,0.0325218748084,0.0323933040047,
		     0.0285130562869,0.0376153696445,0.053746332772,0.0569021317247,0.063796507647,0.0733773704197,0.0844623853536,0.0958758098835};
 
  for(int ival=0; ival<npts; ival++)
    CPPUNIT_ASSERT_DOUBLES_EQUAL(firref[ ((elmts==NULL) ? ival : elmts->at(ival)) ], first[ival], LVLE_EPSILON);

}

void TRBDTest::testLevelERBD()
{
  setUpLevelE();

  Int_t ns = 500;
  _trbd = new URANIE::Sensitivity::TRBD(_tds3, _mycode, ns);
  _trbd->setSeed(1234);
  _trbd->setDrawProgressBar(false);
  _trbd->computeIndexes("nointermed");

  TTree *ntresu = (TTree*)_trbd->getResultTuple();
  checkLevelEResults(ntresu);
  
}

void TRBDTest::testLevelERBDOutputRestrOption()
{
  setUpLevelE();

  Int_t ns = 500;
  _trbd = new URANIE::Sensitivity::TRBD(_tds3, _mycode, ns);
  _trbd->setSeed(1234);
  _trbd->setDrawProgressBar(false);
  _trbd->computeIndexes("nointermed,output=y[4]:y[12]:y[21]");
  
  TTree *ntresu = (TTree*)_trbd->getResultTuple();
  checkLevelEResults(ntresu, _trbd->getAttributeElements(string("y")));

}
#endif

void TRBDTest::testTRBDsetFrequencies()
{
    TString sJDDReference = gSystem->pwd();
    sJDDReference += TString("/flowrate_input_with_keys.in");
    _tds->getAttribute("rw")->setFileKey(sJDDReference, "Rw");
    _tds->getAttribute("r")->setFileKey(sJDDReference, "R");
    _tds->getAttribute("tu")->setFileKey(sJDDReference, "Tu");
    _tds->getAttribute("tl")->setFileKey(sJDDReference, "Tl");
    _tds->getAttribute("hu")->setFileKey(sJDDReference, "Hu");
    _tds->getAttribute("hl")->setFileKey(sJDDReference, "Hl");
    _tds->getAttribute("l")->setFileKey(sJDDReference, "L");
    _tds->getAttribute("kw")->setFileKey(sJDDReference, "Kw");

    // The output file of the code
    _fout = new URANIE::Launcher::TOutputFileRow("_output1_.dat");
    // The attribute in the output file
    _fout->addAttribute("yhat");

    // Instanciation de mon code
    TString sRepertoireReference = gSystem->pwd();
#ifdef WIN32
    _mycode = new URANIE::Launcher::TCode(_tds, "flowrate.exe -k 1>NUL");
#else
    _mycode =
            new URANIE::Launcher::TCode(_tds,
                    Form(
                            "%s/../../../../launCHER/teST/coDE/flowREATE/flowrate  -k>>/dev/null",
                            sRepertoireReference.Data()));
#endif
    _mycode->addOutputFile(_fout);

    Int_t ns = 500;
    _trbd = new URANIE::Sensitivity::TRBD(_tds, _mycode, ns);
    _trbd->setDrawProgressBar(kFALSE);
    
    Int_t nff = 2;
    CPPUNIT_ASSERT_NO_THROW(_trbd->setFrequencies(8, nff));
}

void TRBDTest::throwExceptionTRBDsetFrequencies()
{
    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_IGNORED_TEST()
    ;

    TString sJDDReference = gSystem->pwd();
    sJDDReference += TString("/flowrate_input_with_keys.in");
    _tds->getAttribute("rw")->setFileKey(sJDDReference, "Rw");
    _tds->getAttribute("r")->setFileKey(sJDDReference, "R");
    _tds->getAttribute("tu")->setFileKey(sJDDReference, "Tu");
    _tds->getAttribute("tl")->setFileKey(sJDDReference, "Tl");
    _tds->getAttribute("hu")->setFileKey(sJDDReference, "Hu");
    _tds->getAttribute("hl")->setFileKey(sJDDReference, "Hl");
    _tds->getAttribute("l")->setFileKey(sJDDReference, "L");
    _tds->getAttribute("kw")->setFileKey(sJDDReference, "Kw");

    // The output file of the code
    URANIE::Launcher::TOutputFileRow *fout =
            new URANIE::Launcher::TOutputFileRow("_output1_.dat");
    // The attribute in the output file
    fout->addAttribute("yhat");

    // Instanciation de mon code
    TString sRepertoireReference = gSystem->pwd();
#ifdef WIN32
    URANIE::Launcher::TCode *mycode = new URANIE::Launcher::TCode(_tds, "flowrate.exe -k 1>NUL");
#else
    URANIE::Launcher::TCode *mycode =
            new URANIE::Launcher::TCode(_tds,
                    Form(
                            "%s/../../../../launCHER/teST/coDE/flowREATE/flowrate  -k>>/dev/null",
                            sRepertoireReference.Data()));
#endif
    mycode->addOutputFile(fout);

    Int_t ns = 500;
    _trbd = new URANIE::Sensitivity::TRBD(_tds, mycode, ns);
    _trbd->setDrawProgressBar(kFALSE);
    
    Int_t nff = 2;

    CPPUNIT_ASSERT_THROW(_trbd->setFrequencies(3, nff),
            URANIE::Exceptions::UErrorExceptions);

    delete fout;
    delete mycode;
}

void TRBDTest::validateValuesIshigami()
{
    Double_t sobol;
    TTree *tree = _trbd->getResultTuple();
    tree->SetBranchAddress("Sobol", &sobol);
    CPPUNIT_ASSERT_EQUAL(6, (Int_t)(tree->GetEntries()));

    // Reference values
    Double_t dval[3] =
    { 0.2829795, 0.4155328, 0.0050800 };

    for (Int_t i = 0; i < 3; i++)
    {
        tree->GetEntry(2 * i);
        CPPUNIT_ASSERT_DOUBLES_EQUAL(dval[i], sobol, 0.001);
    }

}

void TRBDTest::testIshigamiRBDComputeIndexesWithTLauncherFunction()
{
    setUp1();
    _trbd->computeIndexes();
    validateValuesIshigami();
    // _tds2->getAttribute("Rk_x_{1}")->delShare();
    // _tds2->getAttribute("Rk_x_{2}")->delShare();
    // _tds2->getAttribute("Rk_x_{3}")->delShare();
    // _tds2->getAttribute("rbd__n__iter__tds")->delShare();
    // _tds2->getAttribute("Ishigami")->delShare();
    // _tds2->getAttribute("Rk_rbd__n__iter__tds")->delShare();
    // _tds2->getAttribute("Rk_Ishigami")->delShare();
}



void TRBDTest::testIshigamiRBDComputeIndexesWithThreadedandNot()
{
    URANIE::DataServer::TAttribute *out = new URANIE::DataServer::TAttribute("TestIshigami");
    out->delShare();
    
    URANIE::Relauncher::TCJitEval eval(&Ishigami);
    eval.addInput(_tds2->getAttribute("x_{1}"));
    eval.addInput(_tds2->getAttribute("x_{2}"));
    eval.addInput(_tds2->getAttribute("x_{3}"));
    eval.addOutput(out);
    
    URANIE::Relauncher::TSequentialRun run(&eval);
    run.startSlave();
    if( run.onMaster() )
    {
        _trbd = new URANIE::Sensitivity::TRBD(_tds2, &run, 50);
        _trbd->setDrawProgressBar(kFALSE);
        _trbd->setSeed(123);
        _trbd->computeIndexes();
    
        run.stopSlave();
    }

    TTree *T = _trbd->getResultTuple();    
    ((TTreePlayer*)(T->GetPlayer()))->SetScanRedirect(kTRUE);
    ((TTreePlayer*)(T->GetPlayer()))->SetScanFileName("outpuS.txt");
    T->Scan("*");
    
    _tds2->deleteTuple();
    _tds2->delAttribute("TestIshigami");

    URANIE::DataServer::TAttribute *outT = new URANIE::DataServer::TAttribute("TestIshigamiT");
    outT->delShare();
    
    URANIE::Relauncher::TCJitEval eval2(&Ishigami);
    eval2.addInput(_tds2->getAttribute("x_{1}"));
    eval2.addInput(_tds2->getAttribute("x_{2}"));
    eval2.addInput(_tds2->getAttribute("x_{3}"));
    eval2.addOutput(outT);
    
    URANIE::Relauncher::TThreadedRun runT(&eval2,4);
    runT.startSlave();
    if( runT.onMaster() )
    {
        delete _trbd;

        _trbd = new URANIE::Sensitivity::TRBD(_tds2, &runT, 50);
        _trbd->setDrawProgressBar(kFALSE);
        _trbd->setSeed(123);
        _trbd->computeIndexes();

        runT.stopSlave();
    }

    T = _trbd->getResultTuple();    
    ((TTreePlayer*)(T->GetPlayer()))->SetScanRedirect(kTRUE);
    ((TTreePlayer*)(T->GetPlayer()))->SetScanFileName("outpuT.txt");
    T->Scan("*");

    Int_t nret = UranieTests::areFilesEqual(TString("outpuT.dat"),
                                            TString("outpuS.dat"));

    CPPUNIT_ASSERT_EQUAL(nret, 0);        
}

        
void TRBDTest::testIshigamiRBDComputeIndexesWithTLauncher2()
{
    URANIE::DataServer::TAttribute *out = new URANIE::DataServer::TAttribute("Ishigami");
    out->delShare();
    
    URANIE::Relauncher::TCJitEval eval(&Ishigami);
    eval.addInput(_tds2->getAttribute("x_{1}"));
    eval.addInput(_tds2->getAttribute("x_{2}"));
    eval.addInput(_tds2->getAttribute("x_{3}"));
    eval.addOutput(out);
    
    URANIE::Relauncher::TSequentialRun run(&eval);
    run.startSlave();
    if( run.onMaster() )
    {
        int ns = 1000;
        _trbd = new URANIE::Sensitivity::TRBD(_tds2, &run, ns);
        _trbd->setDrawProgressBar(kFALSE);
        _trbd->setSeed(123);
        _trbd->computeIndexes();
        validateValuesIshigami();

        run.stopSlave();
    }

}


void TRBDTest::testIshigamiRBDComputeIndexesGenerateSampleWithTLauncherFunction()
{
    setUp1();
    _trbd->generateSample();
    _trbd->computeIndexes();
    validateValuesIshigami();
    // _tds2->getAttribute("Rk_x_{1}")->delShare();
    // _tds2->getAttribute("Rk_x_{2}")->delShare();
    // _tds2->getAttribute("Rk_x_{3}")->delShare();
    // _tds2->getAttribute("rbd__n__iter__tds")->delShare();
    // _tds2->getAttribute("Ishigami")->delShare();
    // _tds2->getAttribute("Rk_rbd__n__iter__tds")->delShare();
    // _tds2->getAttribute("Rk_Ishigami")->delShare();
}

void TRBDTest::testIshigamiRBDComputeIndexesGenerateSampleCreateLauncherWithTLauncherFunction()
{
    setUp1();
    _trbd->generateSample();
    URANIE::Launcher::TLauncherFunction *tlf =
#ifdef ROOT5
            new URANIE::Launcher::TLauncherFunction(_tds2, _p2fIshigami,
#else
            new URANIE::Launcher::TLauncherFunction(_tds2, &Ishigami,
#endif
                    "", "p2fIshigami");
    tlf->setDrawProgressBar(kFALSE);
    tlf->run();
    _trbd->computeIndexes();
    validateValuesIshigami();
    // _tds2->getAttribute("Rk_x_{1}")->delShare();
    // _tds2->getAttribute("Rk_x_{2}")->delShare();
    // _tds2->getAttribute("Rk_x_{3}")->delShare();
    // _tds2->getAttribute("rbd__n__iter__tds")->delShare();
    // _tds2->getAttribute("p2fIshigami")->delShare();
    // _tds2->getAttribute("Ishigami")->delShare();
    // _tds2->getAttribute("Rk_rbd__n__iter__tds")->delShare();
    // _tds2->getAttribute("Rk_p2fIshigami")->delShare();
    // _tds2->getAttribute("Rk_Ishigami")->delShare();
    delete tlf;
}

void TRBDTest::validateValuesFlowrate()
{
    Double_t sobol;
    TTree *tree = _trbd->getResultTuple();
    tree->SetBranchAddress("Sobol", &sobol);

    Double_t dval[8]={0.7408388,0.0165918,0.0129145,0.0093455,0.0452954,0.0632714,0.0426520, 0.0197094};
    CPPUNIT_ASSERT_EQUAL(16, (Int_t)(tree->GetEntries()));

    for (Int_t i = 0; i < 8; i++)
    {
        tree->GetEntry(2* i);
        CPPUNIT_ASSERT_DOUBLES_EQUAL(dval[i], sobol, 0.0001);
    }

}

void TRBDTest::testFlowrateRBDComputeIndexesWithTLauncherCode()
{
    setUp2();
    _trbd->computeIndexes("nointermed");
    validateValuesFlowrate();
    // _tds->getAttribute("Rk_rw")->delShare();
    // _tds->getAttribute("Rk_r")->delShare();
    // _tds->getAttribute("Rk_tu")->delShare();
    // _tds->getAttribute("Rk_tl")->delShare();
    // _tds->getAttribute("Rk_hu")->delShare();
    // _tds->getAttribute("Rk_hl")->delShare();
    // _tds->getAttribute("Rk_l")->delShare();
    // _tds->getAttribute("Rk_kw")->delShare();
    // _tds->getAttribute("Rk_rbd__n__iter__tdsflowreate")->delShare();
    // _tds->getAttribute("Rk_yhat")->delShare();
    // _tds->getAttribute("rbd__n__iter__tdsflowreate")->delShare();
}

void TRBDTest::testFlowrateRBDComputeIndexesWithTCodeEval()
{

    URANIE::Relauncher::TKeyScript fin("flowrate_input_with_keys.in");
    fin.addInput(_tds->getAttribute("rw"), "Rw");
    fin.addInput(_tds->getAttribute("r"), "R");
    fin.addInput(_tds->getAttribute("tu"), "Tu");
    fin.addInput(_tds->getAttribute("tl"), "Tl");
    fin.addInput(_tds->getAttribute("hu"),  "Hu");
    fin.addInput(_tds->getAttribute("hl"), "Hl");
    fin.addInput(_tds->getAttribute("l"), "L");
    fin.addInput(_tds->getAttribute("kw"), "Kw");

    URANIE::Relauncher::TFlatResult fout("_output_flowrate_withRow_.dat");
    URANIE::DataServer::TAttribute *yhat = new URANIE::DataServer::TAttribute("yhat");
    yhat->delShare();
    URANIE::DataServer::TAttribute *d = new URANIE::DataServer::TAttribute("d");
    d->delShare();
    fout.addOutput(yhat);
    fout.addOutput(d);
    
    TString dirReference = gSystem->pwd();
    URANIE::Relauncher::TCodeEval code((UranieTests::code_path("flowrate") + " -s -k").data());
    code.addInputFile(&fin);
    code.addOutputFile(&fout);
    code.setOldTmpDir();

    URANIE::Relauncher::TSequentialRun run(&code);
    //URANIE::Relauncher::TThreadedRun run(&code, 4);
    run.startSlave();
    if (run.onMaster()) {
        _ns = 1000;
        _trbd = new URANIE::Sensitivity::TRBD(_tds, &run, _ns);
        _trbd->setDrawProgressBar(kFALSE);
        _trbd->setSeed(123);
        _trbd->computeIndexes("output=yhat");
        validateValuesFlowrate();
        
        run.stopSlave();
    }
}

void TRBDTest::testFlowrateRBDComputeIndexesGenerateSampleWithTLauncherCode()
{
    setUp2();
    _trbd->generateSample();
    _trbd->computeIndexes("nointermed");
    validateValuesFlowrate();
    // _tds->getAttribute("Rk_rw")->delShare();
    // _tds->getAttribute("Rk_r")->delShare();
    // _tds->getAttribute("Rk_tu")->delShare();
    // _tds->getAttribute("Rk_tl")->delShare();
    // _tds->getAttribute("Rk_hu")->delShare();
    // _tds->getAttribute("Rk_hl")->delShare();
    // _tds->getAttribute("Rk_l")->delShare();
    // _tds->getAttribute("Rk_kw")->delShare();
    // _tds->getAttribute("Rk_rbd__n__iter__tdsflowreate")->delShare();
    // _tds->getAttribute("Rk_yhat")->delShare();
    // _tds->getAttribute("rbd__n__iter__tdsflowreate")->delShare();
}

void TRBDTest::testFlowrateRBDComputeIndexesGenerateSampleCreateLauncherWithTLauncherCode()
{
    setUp2();
    _trbd->generateSample();
    URANIE::Launcher::TLauncher *tl = new URANIE::Launcher::TLauncher(_tds,
            _mycode);
    tl->setDrawProgressBar(kFALSE);
    tl->run("nointermed");
    _trbd->computeIndexes("nointermed");
    validateValuesFlowrate();
    // _tds->getAttribute("Rk_rw")->delShare();
    // _tds->getAttribute("Rk_r")->delShare();
    // _tds->getAttribute("Rk_tu")->delShare();
    // _tds->getAttribute("Rk_tl")->delShare();
    // _tds->getAttribute("Rk_hu")->delShare();
    // _tds->getAttribute("Rk_hl")->delShare();
    // _tds->getAttribute("Rk_l")->delShare();
    // _tds->getAttribute("Rk_kw")->delShare();
    // _tds->getAttribute("Rk_rbd__n__iter__tdsflowreate")->delShare();
    // _tds->getAttribute("Rk_yhat")->delShare();
    // _tds->getAttribute("rbd__n__iter__tdsflowreate")->delShare();
    delete tl;
}

void TRBDTest::throwExceptionConstructorTooLongOutputFunction() {
    vector<URANIE::DataServer::TUniformDistribution> inputs;
    TestTSensitivity::add_flowrate_attributes(inputs);

    URANIE::DataServer::TDataServer tds("tds", "");
    for (auto& input: inputs)
        tds.addAttribute(&input);

    const string outputs_1 = "voila_un_nom_beaucoup_trop_long_pour_un_seul_attribut:encore_un_p*tain_de_nom_beaucoup_trop_long_pour_un_seul_attribut";
    TestTSensitivity::constructor_assert_throw<URANIE::Sensitivity::TRBD>(tds, &flowrateModel,
        outputs_1);

    const string outputs_2 = "voila_un_nom_beaucoup_trop_long_pour_un_attribut:encore_un_p*tain_de_nom_beaucoup_trop_long_pour_un_seul_attribut";
    TestTSensitivity::constructor_assert_throw<URANIE::Sensitivity::TRBD>(tds, &flowrateModel,
        outputs_2);
}

void TRBDTest::throwExceptionConstructorTooLongOutputCode() {
    vector<URANIE::DataServer::TUniformDistribution> inputs;
    TestTSensitivity::add_flowrate_attributes(inputs);

    URANIE::DataServer::TDataServer tds("tds", "");
    for (auto& input: inputs)
        tds.addAttribute(&input);

    TString input_file = "flowrate_input_with_keys.in";
    TestTSensitivity::set_file_key_to_upper_first(tds, input_file);

    URANIE::Launcher::TOutputFileRow output_file("_output_flowrate_withRow_.dat");
    output_file.addAttribute("voila_un_nom_beaucoup_trop_long_pour_un_seul_attribut");

    URANIE::Launcher::TCode code(&tds, "");
    code.addOutputFile(&output_file);
    const size_t n_simulations = 50;
    TestTSensitivity::constructor_assert_throw<URANIE::Sensitivity::TRBD>(tds, code,
        n_simulations);
}

void TRBDTest::throwExceptionConstructorTooLongOutputRun() {
    vector<URANIE::DataServer::TUniformDistribution> inputs;
    TestTSensitivity::add_flowrate_attributes(inputs);
    URANIE::DataServer::TAttribute output("voila_un_nom_beaucoup_trop_long_pour_un_seul_attribut_de_runner");

    URANIE::Relauncher::TCJitEval eval(&flowrateModel);
    for(auto& input: inputs)
        eval.addInput(&input);
    eval.addOutput(&output);

    URANIE::Relauncher::TSequentialRun run(&eval);
    run.startSlave();
    if (run.onMaster()) {
        URANIE::DataServer::TDataServer tds("tds", "");
        for (auto& input: inputs)
            tds.addAttribute(&input);

        const size_t n_simulations = 50;
        TestTSensitivity::constructor_assert_throw<URANIE::Sensitivity::TRBD>(tds, run,
            n_simulations);
        run.stopSlave();
    }
}
