//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef TESTTSENSITIVITY_H
#define TESTTSENSITIVITY_H

#include "TSequentialRun.h"
#include "TCode.h"
#include "TUniformDistribution.h"
#include <cppunit/extensions/HelperMacros.h>
#include <vector>
#include <string>

namespace TestTSensitivity {
    void add_flowrate_attributes(vector<URANIE::DataServer::TUniformDistribution>&
        inputs, bool default_values=false);

    void set_file_key_to_upper_first(URANIE::DataServer::TDataServer& tds, TString& input_file);

    template <typename T>
    void constructor_assert_throw(URANIE::DataServer::TDataServer& tds,
        void (*function)(double*, double*), const string& outputs) {
        CPPUNIT_ASSERT_THROW(T fast(&tds, function, "", outputs.data()),
            URANIE::Exceptions::UErrorExceptions
        );
    }

    template <typename T>
    void constructor_assert_throw(URANIE::DataServer::TDataServer& tds,
        URANIE::Launcher::TCode& code, size_t n_simulations) {
        CPPUNIT_ASSERT_THROW(T fast(&tds, &code, n_simulations),
            URANIE::Exceptions::UErrorExceptions);
    }

    template <typename T>
    void constructor_assert_throw(URANIE::DataServer::TDataServer& tds,
        URANIE::Relauncher::TSequentialRun& run, size_t n_simulations) {
        CPPUNIT_ASSERT_THROW(T fast(&tds, &run, n_simulations),
            URANIE::Exceptions::UErrorExceptions);
    }
}

#endif
