//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestTShapley.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   2020/09/29

 \brief Header of the unit tests of the URANIE::Sensitivity::TShapley class
*/
#ifndef CPPUNITEST_TShapley_H
#define CPPUNITEST_TShapley_H

#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSensitivity.h"
#include "TShapley.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TShapleyTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TShapleyTest);

    CPPUNIT_TEST (testconstructorShapley);

    CPPUNIT_TEST (testExceptionTShapleyComputeIndexes);
    CPPUNIT_TEST (testExceptionTShapleyComputeIndexesFromTDS);
    CPPUNIT_TEST (throwExceptionTShapleyConstructor);
    CPPUNIT_TEST (throwExceptionsGenerateSample);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor for the CPPUnit tests
     **/
    //@{
    //! Default constructor
    TShapleyTest();

    //! Default destructor
    virtual ~TShapleyTest();

    //! the setUp method
    void setUp();
    //! the tearDown method
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Validation methods for the use cases
     **/
    //@{
    //!   Validation on the Ishigami function
    void validateValuesIshigami();
    //! Validation method on the Flowrate function
    void validateValuesFlowrate();
    //@}


    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Test the Shapley constyructor
    void testconstructorShapley();

    /*!
     * \brief Tests and Exceptions for the \b computeIndexes method
     */
    void testExceptionTShapleyComputeIndexes();

    /*!
     * \brief Tests and Exceptions for the \b computeIndexes from TDS method
     */
    void testExceptionTShapleyComputeIndexesFromTDS();

    //@}

    //---------------------------------------------
    /** @name Throw Exceptions
     The list of Throw Exceptions
     **/
    //@{
    //! Throw Exceptions for the constructor
    void throwExceptionTShapleyConstructor();

    //! Throw exceptions for the generateSample method
    void throwExceptionsGenerateSample();
    //@}

private:
    URANIE::DataServer::TDataServer *_tdsA, *_tdsD;
    URANIE::Sensitivity::TShapley * _TShapley;
};

#endif  // CPPUNITEST_TShapley_H
