//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TSOBOL_H
#define CPPUNITEST_TSOBOL_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSensitivity.h"
#include "TSobol.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TSobolTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TSobolTest);

#ifdef WITH_LEVELE
    CPPUNIT_TEST (testLevelESobol);
    CPPUNIT_TEST (testLevelESobolOutputRestrVarexp);
    CPPUNIT_TEST (testLevelESobolOutputRestrOption);
    CPPUNIT_TEST (testLevelESobolOutputRestrBoth);
#endif
    CPPUNIT_TEST (testIshigamiSobol);
    CPPUNIT_TEST (testIshigamiSobolWithDrawings);
    CPPUNIT_TEST (testIshigamiSobolWithData);
    CPPUNIT_TEST (testIshigamiSobolLoadingData);
    CPPUNIT_TEST (testIshigamiSobolLoadingDataExtraAtt);
    CPPUNIT_TEST (testIshigamiSobolFromExistingFiles);
    CPPUNIT_TEST (testTSobolConstructorwithTCode);
    CPPUNIT_TEST (throwExceptionTSobolConstructorFalseCode);
    CPPUNIT_TEST (throwExceptionTSobolConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionTSobolConstructorFalseFnct);
    CPPUNIT_TEST (throwExceptionTSobolConstructorFalseInput);
    CPPUNIT_TEST (throwExceptionTSobolConstructorFalseOutput);
    CPPUNIT_TEST (throwExceptionTSobolConstructorTooLongOutput);
//  CPPUNIT_TEST( throwExceptionTSobolConstructorFalseSize);
    CPPUNIT_TEST (throwExceptionTSobolConstructorFalseOption);
    CPPUNIT_TEST (throwExceptionTSobolConstructorNoFunction);
    CPPUNIT_TEST (throwExceptionTSobolConstructorWithData);
    CPPUNIT_TEST (throwExceptionTSobolConstructorLoadingData);
    CPPUNIT_TEST (throwExceptionTSobolConstructorFromExistingFile);
    CPPUNIT_TEST (testTSobolConstructorLhs);
    CPPUNIT_TEST (testTSobolConstructorQMCSobol);
    CPPUNIT_TEST (testTSobolConstructorQMCHalton);
    CPPUNIT_TEST (testTSobolConstructorSobol);
    CPPUNIT_TEST (testTSobolConstructorSaltelli);

    CPPUNIT_TEST (testIshigamiSobolDecaled);
    CPPUNIT_TEST (testIshigamiSobolComputeIndexesWithTLauncherFunction);
    CPPUNIT_TEST (testIshigamiSobolComputeIndexesWithTLauncher2);
    CPPUNIT_TEST (testSobolComputeIndexesWithThreadedandNot);
    CPPUNIT_TEST (testIshigamiSobolComputeIndexesGenerateSampleWithTLauncherFunction);
    CPPUNIT_TEST (testIshigamiSobolComputeIndexesGenerateSampleCreateLauncherWithTLauncherFunction);
    CPPUNIT_TEST (testFlowrateSobolCompute300Outputs);
    CPPUNIT_TEST (testFlowrateSobolComputeIndexesWithTLauncherCode);    
    CPPUNIT_TEST (testFlowrateSobolComputeIndexesGenerateSampleWithTLauncherCode);
    CPPUNIT_TEST (testFlowrateSobolComputeIndexesGenerateSampleCreateLauncherWithTLauncherCode);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TSobolTest();

    //! Default destructor
    virtual ~TSobolTest();

    void setUp();
    void setUpIshi();
    void setUp1();
    void setUp2(bool bad=false);
    void setUp300();
    void tearDown();
    //@}

    //---------------------------------------------

    /** @name Validation of the sample values for Ishigami
     **/
    //@{
    void validateValuesIshigami(bool rerun=false);
    //@}

    /** @name Validation of the sample values for Flowrate
     **/
    //@{
    void validateValuesFlowrate(bool plenty=false);
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
#ifdef WITH_LEVELE
    //! Test the Sobol method for the Levele function
    void testLevelESobol();
    //! Test the Sobol method for the Levele function restraining output vector in constructor
    void testLevelESobolOutputRestrVarexp();
    //! Test the Sobol method for the Levele function restraining output vector in option
    void testLevelESobolOutputRestrOption();
    //! Test the Sobol method for the Levele function restraining output vector in both
    void testLevelESobolOutputRestrBoth();
    //! Check the results of the levelE sobol estrimation
    void checkLevelEResults(TTree *ntresu, vector<int> *elmts=NULL);
#endif
    //! Test the Sobol method for the Ishigami function
    void testIshigamiSobol();
    //! Test the Sobol method for the Ishigami function
    void testIshigamiSobolWithDrawings();
    //! Test the Sobol method for the Ishigami function with data
    void testIshigamiSobolWithData();
    //! Test the Sobol method for the Ishigami function with data
    void testIshigamiSobolLoadingData();
    //! Test the Sobol method for the Ishigami function with data and more att
    void testIshigamiSobolLoadingDataExtraAtt();
    //! Test the Sobol method for the Ishigami function from existing files
    void testIshigamiSobolFromExistingFiles();
    //! Test the Sobol method for the Ishigami function plus a constant (1e4)
    void testIshigamiSobolDecaled();
    //! Test the ability to launch computeIndexes alone with a TLauncherFunction
    void testIshigamiSobolComputeIndexesWithTLauncherFunction();
    //! Test the ability to launch computeIndexes alone with a TLauncher2
    void testIshigamiSobolComputeIndexesWithTLauncher2();
    //! Test the ability to launch Threaded and Sequential
    void testSobolComputeIndexesWithThreadedandNot();
    //! Test the ability to launch computeIndexes after generateSample with a TLauncherFunction
    void testIshigamiSobolComputeIndexesGenerateSampleWithTLauncherFunction();
    //! Test the ability to launch computeIndexes after generateSample a code creation with a TLauncherFunction
    void testIshigamiSobolComputeIndexesGenerateSampleCreateLauncherWithTLauncherFunction();
    //! Test the ability to launch sobol with 300 outputs
    void testFlowrateSobolCompute300Outputs();
    //! Test the ability to launch computeIndexes alone with a TLauncher
    void testFlowrateSobolComputeIndexesWithTLauncherCode();
    //! Test the ability to launch computeIndexes after generateSample with a TLauncher
    void testFlowrateSobolComputeIndexesGenerateSampleWithTLauncherCode();
    //! Test the ability to launch computeIndexes after generateSample a code creation with a TLauncher
    void testFlowrateSobolComputeIndexesGenerateSampleCreateLauncherWithTLauncherCode();

    void throwExceptionTSobolConstructorTdsNull();
    void throwExceptionTSobolConstructorFalseFnct();
    void testTSobolConstructorwithTCode();
    void throwExceptionTSobolConstructorFalseCode();
    void throwExceptionTSobolConstructorFalseInput();
    void throwExceptionTSobolConstructorFalseOutput();
    void throwExceptionTSobolConstructorTooLongOutput();
    void throwExceptionTSobolConstructorFalseSize();
    void throwExceptionTSobolConstructorFalseOption();
    void throwExceptionTSobolConstructorNoFunction();
    void throwExceptionTSobolConstructorWithData();
    void throwExceptionTSobolConstructorLoadingData();
    void throwExceptionTSobolConstructorFromExistingFile();
    void testTSobolConstructorLhs();
    void testTSobolConstructorQMCSobol();
    void testTSobolConstructorQMCHalton();
    void testTSobolConstructorSobol();
    void testTSobolConstructorSaltelli();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds1, *_tds, *_tds2;
    URANIE::Sensitivity::TSobol * _tsobol;
    URANIE::Launcher::TCode *_mycode;
    void * _p2fIshigami;
    void * _p2fIshigamiDecaled;
    URANIE::Launcher::TOutputFile *_fout;
};

#endif  // CPPUNITEST_TSOBOL_H
