//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTTestAndersonDarling.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$

 \brief  Test File of the class URANIE::DataServer::TTestAndersonDarling

 Created: wen nov 21 07:52:44 2007
 \par Modifications
 \li M. Salvoldi 18/01/2012
 - add 8 news unit tests
 */
#ifndef CPPUNITEST_TTESTTTESTANDERSONDARLING_H
#define CPPUNITEST_TTESTTTESTANDERSONDARLING_H

// CPPUNIT
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

//ROOT

//URANIE
#include "TTestAndersonDarling.h"
#include "TDataServer.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TTestAndersonDarlingTest
 * \brief Unit tests for TTestAndersonDarling.
 */
class TTestAndersonDarlingTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TTestAndersonDarlingTest);

    CPPUNIT_TEST (testA2blissCase3);
    CPPUNIT_TEST (testA2BrisbaneCase3);
    CPPUNIT_TEST (testA2ForearmsCase3);
    CPPUNIT_TEST (testA2blissCase0);
    CPPUNIT_TEST (testUniformValues);
    CPPUNIT_TEST (throwExceptionTestAndersonDarlingConstructorTdsNullEmpty);
    CPPUNIT_TEST (throwExceptionTestAndersonDarlingConstructorMoreAttribut);
    CPPUNIT_TEST (throwExceptionTestAndersonDarlingConstructorFalseAttribut);
    CPPUNIT_TEST (throwExceptionTestAndersonDarlingcomputeScoreNoLaw);
    CPPUNIT_TEST (throwExceptionTestAndersonDarlingcomputeScoreBadLaw);
    CPPUNIT_TEST (throwExceptionTestAndersonDarlingcomputeScoreNormalBadDefined);
    CPPUNIT_TEST (throwExceptionTestAndersonDarlingcomputeScoreUniformBadDefined);
    CPPUNIT_TEST (throwExceptionTestAndersonDarlingcomputeScoreLogNormalBadDefined);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor, Destructor, methods with the setUp and tearDown
     **/
    //@{
    //! Constructs a TTestAndersonDarlingTest object.
    TTestAndersonDarlingTest();

    //! Destructor.
    virtual ~TTestAndersonDarlingTest();

    //! the setUp method
    void setUp();

    //! the tearDown method
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Computes the A2 statistic on the bliss dataset
    void testA2blissCase3();
    /*!
     * \brief Computes the A2 statistic on the Brisbane dataset
     *
     * from :https://www.spcforexcel.com/knowledge/basic-statistics/anderson-darling-test-for-normality
     */
    void testA2BrisbaneCase3();
    /*!
     * \brief Computes the A2 statistic on the Forearms dataset
     *
     * from :https://www.spcforexcel.com/knowledge/basic-statistics/anderson-darling-test-for-normality
     */
    void testA2ForearmsCase3();
    //! Computes the A2 statistic on the bliss dataset when specified mean and standard deviation for the normal law
    void testA2blissCase0();
    //@}
    //! Test uniform law
    void testUniformValues();
    //---------------------------------------------
    /** @name Throw Exceptions
     The list of throw exceptions
     **/
    //@{
    //! Verfiy that the exception is thrown if the tds is NULL or Empty
    void throwExceptionTestAndersonDarlingConstructorTdsNullEmpty();
    //! Verfiy that the exception is thrown if more than 2 attributs are used
    void throwExceptionTestAndersonDarlingConstructorMoreAttribut();
    //! Verfiy that the exception is thrown if the inputattribut doens't exist in the tds
    void throwExceptionTestAndersonDarlingConstructorFalseAttribut();
    //! Verfiy that the exception is thrown if the tds doesn't contained law
    void throwExceptionTestAndersonDarlingcomputeScoreNoLaw();
    //! Verfiy that the exception is thrown if a law no accepted is used
    void throwExceptionTestAndersonDarlingcomputeScoreBadLaw();
    //! Verfiy that the exception is thrown if the normal isn't well defined
    void throwExceptionTestAndersonDarlingcomputeScoreNormalBadDefined();
    //! Verfiy that the exception is thrown if the uniform isn't well defined
    void throwExceptionTestAndersonDarlingcomputeScoreUniformBadDefined();
    //! Verfiy that the exception is thrown if the lognormal isn't well defined
    void throwExceptionTestAndersonDarlingcomputeScoreLogNormalBadDefined();
    //@}

private:
    URANIE::DataServer::TDataServer *_tdsBliss;
    URANIE::DataServer::TDataServer *_tdsBrisbane;
    URANIE::DataServer::TDataServer *_tdsForearms;
    URANIE::UncertModeler::TTestAndersonDarling *_tad;
};

#endif  // CPPUNITEST_TATTRIBUTE_H
