//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTTestKolmogorovSmirnov.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test File of the class URANIE::DataServer::TTestKolmogorovSmirnov
 
 Created: wen nov 21 07:52:44 2007

 \par Modifications
 \li M. Salvoldi 18/01/2012
 - add 8 news unit tests  
 */
#ifndef CPPUNITEST_TATTRIBUTE_H
#define CPPUNITEST_TATTRIBUTE_H

// CPPUNIT
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

//ROOT

//URANIE
#include "TTestKolmogorovSmirnov.h"
#include "TDataServer.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TTestKolmogorovSmirnovTest
 * \brief Unitary tests for TTestKolmogorovSmirnov.
 */
class TTestKolmogorovSmirnovTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TTestKolmogorovSmirnovTest);

    CPPUNIT_TEST (testKSBlissCase3);
    CPPUNIT_TEST (testUniformValues);
    CPPUNIT_TEST (testKSBlissCase0);
    CPPUNIT_TEST (testKSYoungsModulusNormalCase3Case0);
    CPPUNIT_TEST (testKSYoungsModulusLogNormalCase3Case0);

    CPPUNIT_TEST (throwExceptionTestKolmogorovSmirnovConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionTestKolmogorovSmirnovConstructorTdsEmpty);
    CPPUNIT_TEST (throwExceptionTestKolmogorovSmirnovConstructorMoreAttribut);
    CPPUNIT_TEST (throwExceptionTestKolmogorovSmirnovConstructorFalseAttribut);
    CPPUNIT_TEST (throwExceptionTestKolmogorovSmirnovcomputeScoreNoLaw);
    CPPUNIT_TEST (throwExceptionTestKolmogorovSmirnovcomputeScoreBadLaw);
    CPPUNIT_TEST (throwExceptionTestKolmogorovSmirnovcomputeScoreNormalBadDefined);
    CPPUNIT_TEST (throwExceptionTestKolmogorovSmirnovcomputeScoreLogNormalBadDefined);
    CPPUNIT_TEST (throwExceptionTestKolmogorovSmirnovcomputeScoreUniformBadDefined);
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor, Destructor, methods with the setUp and tearDown
     **/
    //@{
    //! Constructs a TTestKolmogorovSmirnovTest object.
    TTestKolmogorovSmirnovTest();

    //! Destructor.
    virtual ~TTestKolmogorovSmirnovTest();

    //! the setUp method
    void setUp();

    //! the tearDown method
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Computes the KS statistic on the bliss dataset
    void testKSBlissCase3();
    //! Computes the KS statistic on the bliss dataset when specified mean and standard deviation for the normal law
    void testKSBlissCase0();
    //! Test uniform law
    void testUniformValues();
    //! Computes the KS statistic on the youngsmodulus dataset normal
    void testKSYoungsModulusNormalCase3Case0();
    //! Computes the KS statistic on the youngsmodulus dataset LogNormal law
    void testKSYoungsModulusLogNormalCase3Case0();
    //@}

    //---------------------------------------------
    /** @name Throw Exceptions
     The list of throw exceptions
     **/
    //@{
    //! Verfiy that the exception is thrown if the tds is NULL
    void throwExceptionTestKolmogorovSmirnovConstructorTdsNull();
    //! Verfiy that the exception is thrown if the tds is empty
    void throwExceptionTestKolmogorovSmirnovConstructorTdsEmpty();
    //! Verfiy that the exception is thrown if more than 2 attributs are used
    void throwExceptionTestKolmogorovSmirnovConstructorMoreAttribut();
    //! Verfiy that the exception is thrown if the inputattribut doens't exist in the tds
    void throwExceptionTestKolmogorovSmirnovConstructorFalseAttribut();
    //! Verfiy that the exception is thrown if the tds doesn't contained law
    void throwExceptionTestKolmogorovSmirnovcomputeScoreNoLaw();
    //! Verfiy that the exception is thrown if a law no accepted is used 
    void throwExceptionTestKolmogorovSmirnovcomputeScoreBadLaw();
    //! Verfiy that the exception is thrown if the normal isn't well defined
    void throwExceptionTestKolmogorovSmirnovcomputeScoreNormalBadDefined();
    //! Verfiy that the exception is thrown if the Uniform isn't well defined
    void throwExceptionTestKolmogorovSmirnovcomputeScoreUniformBadDefined();
    //! Verfiy that the exception is thrown if the lognormal isn't well defined
    void throwExceptionTestKolmogorovSmirnovcomputeScoreLogNormalBadDefined();
    //@}

private:
    URANIE::DataServer::TDataServer *_tdsBliss, *_tdsYoungModulus;
    URANIE::UncertModeler::TTestKolmogorovSmirnov *_tks;
};

#endif  // CPPUNITEST_TATTRIBUTE_H
