import argparse, json, pathlib, logging

parser = argparse.ArgumentParser(description='Write Uranie types aliases')
parser.add_argument('install_dir', help='installation directory')
parser.add_argument('completion', help='json completion file')
parser.add_argument('--log', choices=['DEBUG', 'INFO'])
args = parser.parse_args()

with open(args.completion) as file:
    uranie_completion = json.load(file)

if args.log:
    logging.basicConfig(level=getattr(logging, args.log))

uranie_types_tree = {}
uranie_completion = [line.split('::') for line in uranie_completion]
for uranie_namespace in [line for line in uranie_completion if len(line)==1]:
    print('uranie_namespace:', uranie_namespace)
    uranie_types_tree = {uranie_namespace[0]: {}}
for uranie_module in [line for line in uranie_completion if len(line)==2]:
    print('uranie_module:', uranie_module)
    uranie_types_tree[uranie_module[0]][uranie_module[1]] = []
for line in [line for line in uranie_completion if len(line)==3]:
    print('line:', line)
    uranie_types_tree[line[0]][line[1]].append(line[2])

for uranie_namespace, uranie_modules in uranie_types_tree.items():
    uranie_dir = pathlib.Path(args.install_dir + '/' + uranie_namespace)
    logging.info('uranie_dir:' + str(uranie_dir))
    uranie_dir.mkdir(parents=True, exist_ok=True)
    init = uranie_dir / '__init__.py'
    logging.debug('init:' + str(init))
    init.touch()
    for uranie_module, uranie_types in uranie_modules.items():
        module_path = uranie_dir / (uranie_module + '.py')
        logging.debug('module_path:' + str(module_path))
        with open(str(module_path), 'w') as module_file:
            print('import ROOT\n', file=module_file)
            for uranie_type in uranie_types:
                full_type = ['ROOT', uranie_namespace, uranie_module, uranie_type]
                alias = uranie_type + ' = ' + '.'.join(full_type)
                print(alias, file=module_file)
