//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TBiMpiRun.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-08-23.10.18.58

 \brief version mpi a deux niveaux de la distribution
 */

#ifndef __BIMPIRUN__
#define __BIMPIRUN__

#include "TMpiRun.h"

namespace URANIE
{
namespace MpiRelauncher
{

class TBiMpiRun: public TMpiRun
{
protected:
    static MPI_Comm __intra;
    static bool __isBiMpi;

    int _lowsz, _lowid, _topsz, _topid;

    void forward_header(int *buf, int siz);
    void forward_value(double *buf, int siz);
    void leader_loop(int grp);
    void calcul_loop(int grp);
    //TMpiEval *mpifun(TEval *f) {return dynamic_cast<TMpiEval*>(f);}
    void calculSize(int siz);

public:
    // exceptions
    class UncorrectSize {};
    class NotInAMpiContext {};

    TBiMpiRun(URANIE::Relauncher::TEval *r, int sz, Bool_t init=kTRUE);
    virtual ~TBiMpiRun(void);

    static MPI_Comm getCalculMpiComm();
    static long getCalculMpiCommPy();
    static bool isBiMpi();

    virtual void startSlave(void);
    virtual Bool_t onMaster(void);
    virtual void stopSlave(void);

    ClassDef(URANIE::MpiRelauncher::TBiMpiRun, ID_RELAUNCHER)
};

}
}
#endif
