//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TCIntEval.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-04-04.15.34.57

 \brief   fonction C interprete
 */

#include "TCIntEval.h"

//ROOT
#ifdef ROOT5
#include "Api.h"
#endif

ClassImp (URANIE::Relauncher::TCIntEval);

namespace URANIE
{
namespace Relauncher
{

TCIntEval::TCIntEval(TString fun, Bool_t h):
    TDoubleEval()
{
    _fun = (TMethodCall *) NULL;
    _arg = NULL;
    _ret = h;
    if (!fun)
        return;

    // creation TMethodCall
    _fun = new TMethodCall();
    _fun->InitWithPrototype(fun.Data(), "Double_t*,Double_t*");
    
    if (_fun->IsValid() == kFALSE)
    {
        delete _fun; _fun = NULL;
        throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__, 
                                                   Form("TCIntEval ERROR : the Function[%s] was not found", fun.Data())
            );
    }
}

#ifdef ROOT5
TCIntEval::TCIntEval(void *fun, Bool_t h):
    TDoubleEval()
{
    _fun = (TMethodCall *) NULL;
    _arg = NULL;
    _ret = h;
    if (!fun)
        return;
    char *funcname = G__p2f2funcname(fun);
    if (!funcname)
        return;

    // creation TMethodCall
    _fun = new TMethodCall();
    _fun->InitWithPrototype(funcname, "Double_t*,Double_t*");

    if (_fun->IsValid() == kFALSE)
    {
        delete _fun; _fun=NULL;
        throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__, 
            Form("TCIntEval ERROR : the Function[%s] was not found", funcname)
        );
    }
}
#endif

TCIntEval::TCIntEval(TString fun, void *arg):
    TDoubleEval()
{
    _fun = (TMethodCall *) NULL;
    _arg = arg;
    _ret = kTRUE;
    if (!fun)
        return;

    // creation TMethodCall
    _fun = new TMethodCall();
    _fun->InitWithPrototype(fun.Data(), "Double_t*,Double_t*,void*");

    if (_fun->IsValid() == kFALSE)
    {
        delete _fun; _fun = NULL;
        throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__, 
                                                   Form("TCIntEval ERROR : the Function[%s] was not found", fun.Data())
            );
    }
}

#ifdef ROOT5
TCIntEval::TCIntEval(void *fun, void *arg):
    TDoubleEval()
{
    _fun = (TMethodCall *) NULL;
    _arg = arg;
    _ret = kTRUE;
    if (!fun)
        return;
    char *funcname = G__p2f2funcname(fun);
    if (!funcname)
        return;

    // creation TMethodCall
    _fun = new TMethodCall();
    _fun->InitWithPrototype(funcname, "Double_t*,Double_t*,void*");

    if (_fun->IsValid() == kFALSE)
    {
        delete _fun; _fun = NULL;
        throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__, 
                                                   Form("TCIntEval ERROR : the Function[%s] was not found", fun.Data())
            );
    }
}
#endif

TCIntEval::~TCIntEval()
{
    if (_fun)
        delete _fun;
}

Int_t TCIntEval::eval(Double_t *in, Double_t *out, int ncalc)
{
    UNUSED(ncalc);
    
    Int_t admis;
    void *args[3];
    Longptr_t result;

    admis = 1;
    if (_fun)
    {
        args[0] = (void *) in;
        args[1] = (void *) out;
        if (_arg) {
            args[2] = (void *) _arg;
            _fun->SetParamPtrs(args, 3);
        } 
        else _fun->SetParamPtrs(args, 2);
        _fun->Execute(result);
        if (_ret && !result)
            admis = 0;
    }
    else
        admis = 0; // throw Error
    
    return ((admis==0)?admis:_output.GetEntries());;
}


}
}

