//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TCJitEval.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-09-22.08.50.27

 \brief version compilée de macros
 */

#ifndef __TCJITEVAL__
#define __TCJITEVAL__

#include <functional>

//ROOT
#include "Rtypes.h"

//URANIE
#include "Relauncher.h"
#include "TStandardEval.h"

namespace URANIE
{
namespace Relauncher
{

class TCJitEval: public TSimpleEval
{
protected:
  enum {proc, func, ferm, hard, fonc, harf} _typ;
    union {
        int (*func)(double*, double*);
        void (*proc)(double*, double*);
        int (*ferm)(double*, double*, void*);
        int (*hard)(vector<URANIE::DataServer::UEntry*> *,vector<URANIE::DataServer::UEntry*> *);
        std::function<int(double*, double*)> *fonc;
        std::function<int(vector<URANIE::DataServer::UEntry*>*,vector<URANIE::DataServer::UEntry*>*)> *harf;
    } _p;
    void *_extra;

public:
    TCJitEval(int (*)(double *, double*));
    TCJitEval(void (*)(double *, double*));
    TCJitEval(int (*)(double*, double*, void*), void*);
    TCJitEval(int (*)(vector<URANIE::DataServer::UEntry*> *, vector<URANIE::DataServer::UEntry*>*));
    TCJitEval(std::function<int(double*, double*)> *);
    TCJitEval(std::function<int(vector<URANIE::DataServer::UEntry*>*,vector<URANIE::DataServer::UEntry*>*)> *);
    virtual ~TCJitEval();
    int eval(std::vector<URANIE::DataServer::UEntry*>*, std::vector<URANIE::DataServer::UEntry*>*, int =0 );
    
    ClassDef(URANIE::Relauncher::TCJitEval, ID_RELAUNCHER)
};


}
} //namespace

#endif

