//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.4 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TCodeEval.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-04-04.15.35.59

 \brief encapsulation dun TCode
 */

#ifndef __TCODEEVAL__
#define __TCODEEVAL__

#include <sstream>

//ROOT
#include "Rtypes.h"
#include "TString.h"
#include "TList.h"

// Uranie
#include "Relauncher.h"
#include "TShareEval.h"

#include "TDataServer.h"

namespace URANIE
{
namespace Relauncher
{

class TCodeScript: public TObject
{
public:
    virtual void build(TString &dir, TObjArray *in) = 0;
    virtual void put(TString &ref, TString &tmp, Double_t *in) = 0;
    virtual void put(TString &ref, TString &tmp, vector<URANIE::DataServer::UEntry*> *in) = 0;

//    std::stringstream buffer;
    
    ClassDef(URANIE::Relauncher::TCodeScript, ID_RELAUNCHER)
};

class TCodeResult: public TObject
{
public:
    virtual void build(TObjArray *) = 0;
    virtual int get(TString &, Double_t*) = 0;
    //    virtual int get(TString &, Double_t**) = 0;
    virtual int get(TString &work, vector<URANIE::DataServer::UEntry*> *sout) = 0;

    virtual void clearFile(TString tmpdir) = 0;
    
//    std::stringstream buffer;
    
    ClassDef(URANIE::Relauncher::TCodeResult, ID_RELAUNCHER)
};

class TCodeBase
{
protected:
    TString _cmd;
    TString _origcmd;
    TString _initialdir, _localdir;
    Bool_t _isnumered;
    Bool_t _keepAllFolders;
    TString _workdir, _tmpdir;
    TList *_scripts;
    TCodeResult *_result;

    void buildDir(void);
    int putAndRun(Double_t *, int ncalc=-1);
    int putAndRun(vector<URANIE::DataServer::UEntry*>*, int ncalc=-1);

public:
    //TCodeEval();
    //TCodeEval(const TString &, const TString &);
    TCodeBase(const char *scmd);
    virtual ~TCodeBase(void);

    void addInputFile(TCodeScript *fil);
    void addOutputFile(TCodeResult *fil);
    void setReference(TString dir);
    void setWorkingDir(TString dir);
    void setOldTmpDir();
    void keepAllFolders();

    virtual void setTmpDir(int id);
    
    ClassDef(URANIE::Relauncher::TCodeBase, ID_RELAUNCHER)
};

class TCodeEval: public TShareEval, public TCodeBase
{
public:
    //TCodeEval();
    //TCodeEval(const TString &, const TString &);
    TCodeEval(const char *scmd);
    virtual ~TCodeEval(void);

    virtual void freeze(void);
    virtual TEval *rebuild(int rank, Bool_t chef = kTRUE);
    virtual Bool_t unbuild();
    virtual int eval(vector<URANIE::DataServer::UEntry*>*, vector<URANIE::DataServer::UEntry*>*, int =0 );

    ClassDef(URANIE::Relauncher::TCodeEval, ID_RELAUNCHER)
};

}
}

#endif

