// --- C++ ---
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TComponentRun.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2014-10-16
 
 \brief  pseudo runner for salome component
*/

#include "TComponentRun.h"

ClassImp (URANIE::Relauncher::TComponentRun);

namespace URANIE
{
namespace Relauncher
{

TComponentRun::TComponentRun(TEval *fun, coroutine<void>::push_type *back) :
        TSimpleRun(fun)
{
    _yield = back;
    _ncpus = 1;
    _workfun = NULL;
}

TComponentRun::~TComponentRun(void)
{
    stopSlave();
}

void TComponentRun::startSlave(void)
{
    _workfun = _fun->rebuild(0);
    (*_yield)();
}

void TComponentRun::stopSlave(void)
{
    if (!_workfun)
        return;

    if (_workfun->unbuild())
        delete _workfun;
    _workfun = NULL;

}

int TComponentRun::get(long &id)
{
    struct element tuple;
    // int res;

    tuple = _qin.front();
    _qin.pop();
    id = tuple.id;
    return 0;
}

int TComponentRun::eval(vector<URANIE::DataServer::UEntry*> *in, 
    vector<URANIE::DataServer::UEntry*> *out, int ncalc)
{
    return _workfun->eval(in, out, ncalc);
}

}} //namespace

