//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TLineResult.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-09-09.14.51.09

 \brief   fichier de résultat du TCodeEval
 */

#include "TLineResult.h"

#include <iostream>
#include <fstream>

//ROOT
//URANIE

ClassImp (URANIE::Relauncher::TLineResult);

namespace URANIE
{
namespace Relauncher
{

/* Constructeur destructeur */
TLineResult::TLineResult(const char *path) :
    _path(path)
{
    _svecbeg=TString(""); _svecend=TString(""); _svecdel=TString(" ");
    _sstrbeg=TString("");  _sstrend=TString("");
}

TLineResult::~TLineResult()
{
}

/* definition */
void TLineResult::addOutput(URANIE::DataServer::TAttribute *att)
{
    _vars.Add(att);
}

void TLineResult::setOutputs(int size, URANIE::DataServer::TAttribute *att, ...)
{
    int i;
    va_list ap;
    URANIE::DataServer::TAttribute *attr;

    _vars.Add(att);
    va_start(ap, att);
    for (i = 1; i < size; i++)
    {
        attr = va_arg(ap, URANIE::DataServer::TAttribute *);
        _vars.Add(attr);
    }
    va_end(ap);
}

/* usage */
void TLineResult::build(TObjArray *out)
{
    TIter next(&_vars);
    URANIE::DataServer::TAttribute *tmp;

    while ((tmp = (URANIE::DataServer::TAttribute *) next()))
        out->Add(tmp);
}

int TLineResult::get(TString &work, Double_t*res)
{    
    int i, deb, len, dec;
    TObject *tmp;
    TIter next(&_vars);
    TString line;
    char* path = new char[work.Length() + _path.Length() + 2];
#ifdef WIN32
    sprintf(path, "%s\\%s", work.Data(), _path.Data());
#else
    sprintf(path, "%s/%s", work.Data(), _path.Data());
#endif	
    ifstream fin(path, std::ios::in);
    if (!fin)
    {
        std::cout << "Cannot open :: " << path << std::endl;
        delete[] path;
        return 0;
    }  
  
    i=0; string test;
    while (line.ReadLine(fin))
    {
	if (line[0] == '#' || URANIE::DataServer::UEntry::isWhiteLine( string(line) ) )
	    continue;
        
        res[i++] = line.Atof();
        if ( i == _vars.GetEntries() )
            break;
    }
    fin.close();

    int ret=((i<_vars.GetEntries()) ? 0 : _vars.GetEntries());
    if(ret==0)
        std::cout << "TLineResults("<<_path<<"): only "<<i<<" values found out of the "<< _vars.GetEntries()<<" expected \n";
    
    delete[] path;    
    return ret;//_vars.GetEntries();
}

void TLineResult::setVectorProperties( TString beg, TString delim, TString end)
{
    _svecbeg=beg; _svecend=end; _svecdel=delim;
}

void TLineResult::setStringProperties( TString beg, TString end)
{
    _sstrbeg=beg; _sstrend=end; 
}
  
int TLineResult::get(TString &work, vector<URANIE::DataServer::UEntry*> *sout)
{
    int i;
    
    TIter next(&_vars);
    TString line;
    char* path = new char[work.Length() + _path.Length() + 2];
#ifdef WIN32
    sprintf(path, "%s\\%s", work.Data(), _path.Data());
#else
    sprintf(path, "%s/%s", work.Data(), _path.Data());
#endif	
    ifstream fin(path, std::ios::in);
    if (!fin)
    {
	std::cout << "Cannot open :: " << path << std::endl;
        delete[] path;
	return 0;
    }

    i=0; string test;
    while (line.ReadLine(fin))
    {
	if (line[0] == '#' || URANIE::DataServer::UEntry::isWhiteLine( string(line) ) )
	    continue;

        bool isString = sout->at(i)->tag==URANIE::DataServer::UEntry::String;
	sout->at(i)->fillEntry( line, ((isString) ? "" : _svecdel), ((isString) ? _sstrbeg : _svecbeg), ((isString) ? _sstrend :_svecend ));
        i++;
        if ( i == _vars.GetEntries() )
            break;

    }
    fin.close();
    
    int ret=((i<_vars.GetEntries()) ? 0 : _vars.GetEntries());
    if(ret==0)
        std::cout << "TLineResults("<<_path<<"): only "<<i<<" values found out of the "<< _vars.GetEntries()<<" expected \n";
    
    delete[] path;    
    return ret;
}

void TLineResult::clearFile(TString tmpdir)
{
#ifdef WIN32
    TString fullPath = tmpdir+"\\"+_path;
    gSystem->Exec( Form("if exist %s del /f %s", fullPath.Data(), fullPath.Data()) ); 
#else
    TString fullPath = tmpdir+"/"+_path;
    gSystem->Exec( Form("rm -f %s", fullPath.Data()) );
#endif    
}

}
}
