//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TLineResult.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-09-09.14.51.03

 \brief fichier de résultat du TCodeEval
 */

#ifndef __TLINERESULT__
#define __TLINERESULT__

//ROOT
#include "TString.h"
#include "TList.h"
#include "TObjArray.h"
#include "TRegexp.h"

//Uranie
#include "Relauncher.h"
#include "TAttribute.h"
#include "TCodeEval.h"

namespace URANIE
{
namespace Relauncher
{

class TLineResult: public TCodeResult
{
protected:
    static TRegexp _num;
    TString _path;
    TObjArray _vars;
    TString _svecbeg, _svecend, _svecdel, _sstrbeg, _sstrend;
    
public:
    TLineResult(const char *);
    virtual ~TLineResult(void);

    void addOutput(URANIE::DataServer::TAttribute *at);
    void setOutputs(int, URANIE::DataServer::TAttribute *at, ...);
    void build(TObjArray *);
    int get(TString &, Double_t*);
    int get(TString &work, vector<URANIE::DataServer::UEntry*> *sout);

    void clearFile(TString tmpdir);
    void setVectorProperties( TString beg, TString delim, TString end);
    void setStringProperties( TString beg, TString end);

    ClassDef(URANIE::Relauncher::TLineResult, ID_RELAUNCHER)

};

}
} //namespace

#endif
