// --- C++ ---
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TSocketRun.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2017-11-23
 
 \brief  independant process communicating by socket
*/


#include "TSeparateRun.h"

#include <iostream>
#include <queue>
#include <vector>

//ROOT
#include "TROOT.h"

#include "loop.h"

ClassImp (URANIE::Relauncher::TSeparateRun);

namespace URANIE
{
namespace Relauncher
{

TSeparateRun::TSeparateRun(URANIE::Relauncher::TEval *func) :
        TShareRun(func)
{
    _ncpus = 0;
}

TSeparateRun::~TSeparateRun() { }

// utilitaires
void TSeparateRun::send_uentry(int canal, int ncalc, int size, vector<URANIE::DataServer::UEntry*> *in)
{
    int i, j, k, si, sd, insz;
    std::vector<int> head;
    std::vector<double> corps;
    static int one = 0;

    // send header info
    head.resize(size+2);
    //head[0] = tag;
    head[0] = ncalc;
    head[1] = size;
    if (one) std::cout << head[0] << " " << head[1] << std::endl;
    insz = 0;
    for (i=0; i<size; i++) {
        switch (in->at(i)->tag) {
        case URANIE::DataServer::UEntry::Vector :
            head[i+2] = in->at(i)->getDim();
            break;
        case URANIE::DataServer::UEntry::String :
            head[i+2] = (in->at(i)->val.str->length()*sizeof(char) + sizeof(double)) 
                    / sizeof(double);
            break;
        case URANIE::DataServer::UEntry::Double :
            head[i+2] = 1;
            break;
        default :
            std::cout << "NotImplemented\n";
        }
        if (one) std::cout << in->at(i)->tag << " " << head[i+2] << std::endl;
        insz += head[i+2];
    }
    if (one) std::cout << "HEAD.." << insz << " " << head[1] << std::endl;
    si = send_header(canal, &(head[0]), size+2);
    // send info
    corps.resize(insz);
    for (i=j=0; i<size; i++) {
        switch (in->at(i)->tag) {
        case URANIE::DataServer::UEntry::Vector :
            for EACH(k, head[i+2]) {
                corps[j] = (*(in->at(i)->val.vec))[k];
                j++;
            }
            break;
        case URANIE::DataServer::UEntry::String :
            strncpy((char *) &corps[j], in->at(i)->val.str->c_str(), 
                    head[i+2]*sizeof(double)/sizeof(char) ) ;
            j += head[i+2];
            break;
        case URANIE::DataServer::UEntry::Double :
            corps[j] = in->at(i)->val.dou[0];
            j++;
            break;
        default :
            std::cout << "NotImplemented\n";
        }
    }
    sd = send_value(canal, &(corps[0]), insz);
    if (one) std::cout << "HEAD..." << corps[0] << " " << corps[1] << std::endl;
    one = 0;
}

void TSeparateRun::to_uentry(int *isz, double *vals, 
        std::vector<URANIE::DataServer::UEntry*> &vec)
{
    int i, iat, icel;
    vector<URANIE::DataServer::UEntry*>::iterator it;
    string *tmps;
    static int one = 0;
    if (one) std::cout << "ToUEntry.\n";
    
    iat = 2; icel = 0;
    for ( it=vec.begin(); it != vec.end(); it++) {
        if (one) std::cout << "ToUEntry..\n";
        //(*it)->clearEntry();
        switch ((*it)->tag) {
        case URANIE::DataServer::UEntry::Vector :
            if (one) std::cout << "ToUEntry V " << iat-2 << icel << " " << vals[icel] << std::endl;
            for EACH(i, isz[iat])
                (*it)->pushEntry(vals[icel+i]);
            break;
        case URANIE::DataServer::UEntry::String :
            if (one) std::cout << "ToUEntry S " << iat-2 << icel << " " << vals[icel] << std::endl;
            tmps = new string((char*) &vals[icel]);
            (*it)->pushEntry(tmps);
            break;
        default :
            if (one) std::cout << "ToUEntry D " << iat-2 << icel << " " << vals[icel] << std::endl;
            (*it)->pushEntry(vals[icel]);
            break;
        }
        icel += isz[iat];
        iat++;
    }
    if (one) std::cout << "ToUEntry " << iat << " " << icel << std::endl;
    one = 0;
}

void TSeparateRun::empty_uentry(TIter &next, std::vector<URANIE::DataServer::UEntry*> &ret)
{
    URANIE::DataServer::TAttribute *att;
    static int one=0;
    
    while ((att = (URANIE::DataServer::TAttribute*) next() )) {
        switch (att->getDataType()) {
        case URANIE::DataServer::TAttribute::kVector :
            if (one) std::cout << "V";
            ret.push_back(new URANIE::DataServer::UEntry
                    (URANIE::DataServer::UEntry::Vector, string(att->GetName()))
                ); 
            break;
        case URANIE::DataServer::TAttribute::kString :
            if (one) std::cout << "S";
            ret.push_back(new URANIE::DataServer::UEntry
                    (URANIE::DataServer::UEntry::String, string(att->GetName()))
                ); 
            break;
        default :
            if (one) std::cout << "D";
            ret.push_back(new URANIE::DataServer::UEntry
                    (URANIE::DataServer::UEntry::Double, string(att->GetName()))
                );
            break;
        }
    }
    if (one) std::cout << "\n";
    one = 0;
}

void TSeparateRun::calcul_loop()
{
    int i, ret, size, si, sd;
    double *inbuf, *outbuf, *gradbuf;
    std::vector<URANIE::DataServer::UEntry*>::iterator it;
    static int one=0;

    //TSeparate *sock = _sclient[0];
    int *headbuf = new int[_sizein+2];
    si = receive_header(0, headbuf, _sizein+2);
    URANIE::Relauncher::TEval *fun = _fun->rebuild(headbuf[0]);
    TIter next(_fun->getInputList());
    std::vector<URANIE::DataServer::UEntry*> invec;
    empty_uentry(next, invec);
    TIter nexto(_fun->getOutputList());
    std::vector<URANIE::DataServer::UEntry*> outvec;
    empty_uentry(nexto, outvec);
    
    
    
    while (headbuf[1] != 0) {
        // read input
        size = 0;
        for (i=0; i<_sizein; i++) size+=headbuf[i+2];
        if (one) std::cout << "SIZ IN " << size << std::endl;
        inbuf = new double[size];
        sd = receive_value(0, inbuf, size);
        if (one) std::cout << si << sd << "IN " << inbuf[0] << " " << inbuf[1] << std::endl;
        // convert to unetry
        to_uentry(headbuf, inbuf, invec);
        // prepare output recept
        if (one) std::cout << "UE prepare\n";
        // eval
        if (one) std::cout << "UE eval\n";
        ret = fun->eval(&invec, &outvec, headbuf[0]);
        if (one) std::cout << "OUT " << ret << " " << *(outvec[0]->val.dou) << std::endl;
        // send output
        if (ret) { 
            send_uentry(0, ret, _sizeout, &outvec);
        }
        else {
            int *nores = new int[_sizeout+2];
            for EACH(i, _sizeout+2) nores[i] = 0;
            sd = send_header(0, nores, _sizeout+2);
            delete[] nores;
        }
        // cleanup
        for (it=invec.begin(); it != invec.end(); it++)
            (*it)->clearEntry();
        for (it=outvec.begin(); it != outvec.end(); it++)
            (*it)->clearEntry();
        delete[] inbuf;
        one = 0;
        si = receive_header(0, headbuf, _sizein+2);
        if (si == -1) {
            std::cout << "Separate Breaked\n";
            break;
        }
    }
    
    delete[] headbuf;
    if (fun->unbuild()) delete fun;
}

// protocole
void TSeparateRun::startSlave()
{
    _sizein = _fun->getInputList()->GetEntries();
    _sizeout = _fun->getOutputList()->GetEntries();
}

void TSeparateRun::stopSlave()
{
    
    if (onMaster()) {
        int i;
        std::vector<int> tab;

        tab.resize(_sizein+2);
        for (i=0; i<_sizein+2; i++) tab[i] = 0;
        for (i=1; i<_ncpus; i++) {
            send_header(i, &(tab[0]), _sizein+2);
        }
    }
}

void TSeparateRun::put(long id, std::vector<URANIE::DataServer::UEntry*> *in,
        std::vector<URANIE::DataServer::UEntry*> *out, int ncalc)
{
    int i, sid;
    struct element tupl = {id, ncalc, in, out };
    
    if (_sleep.empty()) {
        _pend.push(tupl);
    }
    else {
        // reservation resource
        sid = _sleep.front();
        _sleep.pop();
        _run[sid] = tupl;
        // send header info
        send_uentry(sid, ncalc, _sizein, in);
    }
}

int TSeparateRun::get(long &idc)
{
    int i, id, ret, sd;
    struct element ref, next;
    static int one=0;
    
    if (one) std::cout << "Get wait\n" ;
    
    id = wait_service();
    ref = _run[id];
    int size;
    // read header
    if (one) std::cout << "Get UEntry\n" ;
    int *headbuf = new int[_sizeout+2];
    sd = receive_header(id, headbuf, _sizeout+2);
    if (one) std::cout << "Get UEntry." << headbuf[0] << headbuf[1] << std::endl ;
    ret = headbuf[0];
    if (ret) {
        size = 0;
        for (i=0; i<_sizeout; i++) size+=headbuf[i+2];
        // read input
        double *buf = new double[size];
        sd = receive_value(id, buf, size);
        // convert to unetry
        to_uentry(headbuf, buf, *ref.outv);
        // cleanup
        delete[] buf;
    }
    delete[] headbuf;
    
    if (_pend.empty()) _sleep.push(id);
    else {
        std::vector<int> head;
        // send new calcul
        next = _pend.front();
        _pend.pop();
        _run[id] = next;
        send_uentry(id, next.calcnb, _sizein, next.inv);
    }
    idc = ref.id;
    one = 0;
    return ret;
}

}} //namespace
