/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TSeparateRun.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2017-12-18
 
 \brief  separate process communicating by stream
*/

#ifndef __TSEPARATERUN__
#define __TSEPARATERUN__

#include "TShareRun.h"

#include <queue>
#include <vector>

#include "TSocket.h"
#include "TServerSocket.h"
#include "TMonitor.h"

namespace URANIE
{
namespace Relauncher
{

class TSeparateRun : public TShareRun
{
protected:
    int _sizein, _sizeout, _nbress;
    std::queue<element> _pend;
    std::queue<int> _sleep;
    std::vector<struct element> _run;


    virtual int wait_service() = 0;
    virtual int send_header(int canal, int *head, int size) = 0;
    virtual int send_value(int canal, double *mesg, int size) = 0;
    virtual int receive_header(int canal, int *head, int size) = 0;
    virtual int receive_value(int canal, double *mesg, int size) = 0;
        
    //void send_double(int canal, int tag, int ncalc, int size, double *in);
    void send_uentry(int canal, int ncalc, int size, 
            vector<URANIE::DataServer::UEntry*> *in);
    void to_uentry(int *isz, double *vals, 
            std::vector<URANIE::DataServer::UEntry*> &dest);
    void empty_uentry(TIter &next, 
            std::vector<URANIE::DataServer::UEntry*> &dest);
    void calcul_loop();
    
public:
    
    TSeparateRun(URANIE::Relauncher::TEval *r);
    virtual ~TSeparateRun();
    
    virtual void startSlave();
    virtual void stopSlave();
    
    virtual void put(long, vector<URANIE::DataServer::UEntry*> *in,
            vector<URANIE::DataServer::UEntry*> *out, int =0);
    virtual int get(long &);
    
    ClassDef(URANIE::Relauncher::TSeparateRun, ID_RELAUNCHER)
};


}} //namespace
#endif
