//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.4 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TShareRun.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-07-18.09.03.44

 \brief TRun base class
 */

#ifndef __SHARE_RUN__
#define __SHARE_RUN__

//ROOT
#include "Rtypes.h"

// Uranie
#include "TRun.h"
#include "TEval.h"

#include <queue>

namespace URANIE
{
namespace Relauncher
{


#define SINGL 11
#define GRAD 12
#define UENTR 13

struct element
{
    long id, calcnb;
    vector<URANIE::DataServer::UEntry*> *inv, *outv;
};
struct relement
{
    long id;
    int stat;
};

class TShareRun: public TRun
{
protected:
    TEval *_fun;

public:
    TShareRun(TEval *fun);
    virtual ~TShareRun();

    virtual TEval *getEval()
    {
        return _fun;
    }
    virtual void put(long, vector<URANIE::DataServer::UEntry*> *in, vector<URANIE::DataServer::UEntry*> *out, int icalc)=0;
    
    ClassDef(URANIE::Relauncher::TShareRun, ID_RELAUNCHER)
};

class TSimpleRun: public TShareRun
{
protected:

    std::queue<struct element> _qin;


public:
    TSimpleRun(TEval *fun);
    virtual ~TSimpleRun();

    virtual Bool_t onMaster(void);

    virtual void put(long, vector<URANIE::DataServer::UEntry*> *in, vector<URANIE::DataServer::UEntry*> *out, int icalc=0);

    virtual Bool_t readyToGet();
        
    ClassDef(URANIE::Relauncher::TSimpleRun, ID_RELAUNCHER)
};

class TDecoratorRun: public TRun
{
protected:
    TRun *_subrun;

public:
    TDecoratorRun(TRun *run);
    virtual ~TDecoratorRun();

    virtual TEval *getEval();
    virtual void startSlave();
    virtual Bool_t onMaster();
    virtual void stopSlave();    
    
    ClassDef(URANIE::Relauncher::TDecoratorRun, ID_RELAUNCHER)
};

}
} //namespace

#endif
