// --- C++ ---
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TStandardEval.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2014-07-07
 
 \brief  simple eval shared class
*/

#include "TStandardEval.h"

ClassImp (URANIE::Relauncher::TStandardEval);
ClassImp (URANIE::Relauncher::TSimpleEval);
ClassImp (URANIE::Relauncher::TDoubleEval);

namespace URANIE
{
namespace Relauncher
{

/* TStandardEval */
void TStandardEval::addInput(URANIE::DataServer::TAttribute *un)
{
    if (_freezed)
        throw TEval::FreezedEval();
    checkAttributeType( un );
    _input.Add((TObject *) un);
}

void TStandardEval::addOutput(URANIE::DataServer::TAttribute *un)
{
    if (_freezed)
        throw TEval::FreezedEval();
    checkAttributeType( un );
    _output.Add((TObject *) un);
}

void TStandardEval::setInputs(Int_t size, URANIE::DataServer::TAttribute *un,
        ...)
{
    va_list ap;
    int i;

    if (_freezed)
        throw TEval::FreezedEval();
    va_start(ap, un);    
    checkAttributeType( un );    
    _input.Add((TObject *) un);
    for (i = 1; i < size; i++)
    {
        _input.Add(va_arg(ap, TObject *));
    }
    va_end(ap);
}

void TStandardEval::setOutputs(Int_t size, URANIE::DataServer::TAttribute *un,
    ...)
{
    va_list ap;
    int i;

    if (_freezed)
        throw TEval::FreezedEval();
    va_start(ap, un);
    checkAttributeType( un );    
    _output.Add((TObject *) un);
    for (i = 1; i < size; i++)
    {
        _output.Add(va_arg(ap, TObject *));
    }
    va_end(ap);
}


/* TSimpleEval */


/* TDoubleEval */
int TDoubleEval::eval(vector<URANIE::DataServer::UEntry*> *in, vector<URANIE::DataServer::UEntry*> *out, int ncalc)
{
    int ret;
    double *inp, *outp;
    
    inp = new double[in->size()];
    outp = new double[out->size()];

    for (unsigned int ival=0; ival<in->size(); ival++)
        inp[ival] = *in->at(ival)->val.dou;
    
    ret = eval(inp, outp, ncalc);

    for(unsigned int ival=0; ival<out->size(); ival++)
        *out->at(ival)->val.dou = outp[ival] ;

    delete [] outp;
    delete [] inp;
    
    return ((ret!=0) ? getOutDimension(out) : ret);
}


}
} //namespace

