//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestUserFunctions.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   2012-09-14 10:45:11

 \brief Implement the user function 

 */

// ROOT
#include "TROOT.h"
#include "TMath.h"

#include "UEntry.h"

int precodeComposeScalaire(Double_t *in, Double_t *out)
{
    out[0] = (in[0] - in[1]);
    return 1;
}

int postcodeComposeScalaire(Double_t *in, Double_t *out)
{
    out[0] = 2.0 * in[0];
    return 1;
}

int precodeComposeMulti(Double_t *in, Double_t *out)
{
    out[0] = in[0] / in[1];
    return 1;
}

int postcodeComposeMulti(Double_t *in, Double_t *out)
{
    out[0] = in[0] * in[0];
    return 1;
}

int postcodeComposeMulti_hard(vector<URANIE::DataServer::UEntry*> *in, vector<URANIE::DataServer::UEntry*> *out)
{

  vector<double> *input = in->at(0)->val.vec, *output = out->at(0)->val.vec;

  for(size_t ival = 0; ival<input->size(); ival++)
      output->push_back( input->at(ival) * input->at(ival) );

  return 1;
}

int FlowrateReatmt_hard(vector<URANIE::DataServer::UEntry*> *in, vector<URANIE::DataServer::UEntry*> *out)
{

    vector<double> *v1 = in->at(0)->val.vec, *v2 = in->at(2)->val.vec;
    string *w1=in->at(1)->val.str, *w2=in->at(3)->val.str;

    double thev1=987654321, thev2=987654321;
    string words[10]={"zero","one","two","three","four","five","six","seven","eight","nine"};
  
    for(unsigned int iday=0; iday<10; iday++)
    {
	if( *w1 == words[iday] )
	    thev1 = ( (iday<v1->size()) ? v1->at(iday) : -123456789 );
      
	if( *w2 == words[iday] )
	    thev2 = ( (iday<v2->size()) ? v2->at(iday) : -123456789 );
    }

    out->at(0)->pushEntry(thev1);
    out->at(1)->pushEntry(thev2);
  
    return 1;
}

void Ishigami(Double_t *paramt, Double_t *res)
{
    Double_t a = 7.0;
    Double_t b = 0.1;

    Double_t x1 = paramt[0];
    Double_t x2 = paramt[1];
    Double_t x3 = paramt[2];

    res[0] = sin(x1);
    res[0] += a * sin(x2) * sin(x2);
    res[0] += b * x3 * x3 * x3 * x3 * sin(x1);
}

int Ishigami2(Double_t *paramt, Double_t *res, void* info)
{
    Double_t *ab = (Double_t *) info;
    Double_t a = ab[0];
    Double_t b = ab[1];

    res[0] = sin(paramt[0]);
    res[0] += a * sin(paramt[1]) * sin(paramt[1]);
    res[0] += b * paramt[2] * paramt[2] * paramt[2] * paramt[2]
            * sin(paramt[0]);
    //res[1] = 5.0 * paramt[0];
    return  1;
}

int Ishigami3(Double_t *paramt, Double_t *res)
{
    Double_t a = 7.0;
    Double_t b = 0.1;

    Double_t x1 = paramt[0];
    Double_t x2 = paramt[1];
    Double_t x3 = paramt[2];

    if (x1 < x2) return 0;
    res[0] = sin(x1);
    res[0] += a * sin(x2) * sin(x2);
    res[0] += b * x3 * x3 * x3 * x3 * sin(x1);
    return 1;
}

int FakeGrad(Double_t *paramt, Double_t *res, Double_t *grad)
{
    UNUSED(paramt, res, grad);
    return  1;
}

/****************** End of User Function **********************/
