using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::MpiRelauncher;

void testBidonMPI(int outType=0, int readempty=0, const string& python_command="python")
{
    TAttribute x("x",URANIE::DataServer::TAttribute::kVector), diff("diff",URANIE::DataServer::TAttribute::kVector);
    TAttribute delai("coef"), pente("pente"); 
    TAttribute outAtt("OUTTYPE");
    TAttribute empty("EMPTY");        
    
    // Set the reference input file and the key for each input attributes
    TFlagScript fin("bidon.py");
    fin.addInput(&delai, "START");
    fin.addInput(&pente, "PENTE");
    fin.addInput(&outAtt, "OUTTYPE");
    fin.addInput(&empty, "EMPTY");

    // The output file of the code
    URANIE::Relauncher::TCodeResult *fout=NULL;
    switch(outType)
    {
    case 0: fout = new URANIE::Relauncher::TKeyResult("bidon.res");
        ((URANIE::Relauncher::TKeyResult*)fout)->setOutputs(2, &x, "x", &diff, "diff");
        if(readempty==1)
            ((URANIE::Relauncher::TKeyResult*)fout)->setVectorProperties("[",",","]");        
        break;
    case 1: fout = new URANIE::Relauncher::TLineResult("bidon.res");
        ((URANIE::Relauncher::TLineResult*)fout)->setOutputs(2, &x, &diff);
        if(readempty==1)
            ((URANIE::Relauncher::TLineResult*)fout)->setVectorProperties("["," ","]");        
        break;
    case 2: fout = new URANIE::Relauncher::TFlatResult("bidon.res");
        ((URANIE::Relauncher::TFlatResult*)fout)->setOutputs(2, &x, &diff);
        if(readempty==1)
            ((URANIE::Relauncher::TFlatResult*)fout)->setVectorProperties("[",",","]");        
        break;
    case 3: fout = new URANIE::Relauncher::TFlatResult("bidon.res");
        ((URANIE::Relauncher::TFlatResult*)fout)->setOutputs(2, &x, &diff);
        ((URANIE::Relauncher::TFlatResult*)fout)->isMultiLine();
        break;
    }

    // Instanciation de mon code
    const string command = python_command + " bidon.py > bidon.res";
    std::cout << "command: [" << command << "]" << std::endl;
    TCodeEval mycode(command.data());
    //mycode.setOldTmpDir();
    mycode.addInputFile(&fin);
    mycode.addOutputFile(fout);

    TMpiRun run(&mycode);
    run.startSlave();
    if (run.onMaster())
    {
	TDataServer tds("tdsBidon", "Design of experiments for Bidon");
	tds.addAttribute(&delai);
	tds.addAttribute(&pente);
	
	tds.fileDataRead("bidonUniformDesign.dat", kFALSE, kTRUE);

        TLauncher2 lanceur(&tds, &run);
        URANIE::DataServer::TDataServer _errBidon("errbidon",
                                                  "Design of experiments for Bidon not working");
        lanceur.setSaveError(&_errBidon);
        lanceur.addConstantValue(&outAtt, (double)outType);
        lanceur.addConstantValue(&empty, (double)readempty);        
        // resolution
        lanceur.solverLoop();

        tds.exportData("_output_testBidonMPI_.dat");
        if(readempty==0)
            _errBidon.exportData("_output_testBidonMPI_err_.dat");
	
        run.stopSlave();
    }
    delete fout;
}
