using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::MpiRelauncher;

void testFlowrateMPI(int err=0)
{
    TAttribute *yhat = new TAttribute("yhat"); yhat->delShare();
    TAttribute *d = new TAttribute("d"); d->delShare();

    TAttribute *rw = new TAttribute("rw"); rw->delShare();
    TAttribute *r = new TAttribute("r"); r->delShare();
    TAttribute *tu = new TAttribute("tu"); tu->delShare();
    TAttribute *tl = new TAttribute("tl"); tl->delShare();
    TAttribute *hu = new TAttribute("hu"); hu->delShare();
    TAttribute *hl = new TAttribute("hl"); hl->delShare();
    TAttribute *l = new TAttribute("l"); l->delShare();
    TAttribute *kw = new TAttribute("kw"); kw->delShare();

    // Set the reference input file and the key for each input attributes
    TFlatScript fin("flowrate_input_with_values_rows.in");
    fin.setInputs(8, rw, r, tu, tl, hu, hl, l, kw);

    // The output file of the code
    TFlatResult fout("_output_flowrate_withRow_.dat");
    fout.setOutputs(2, yhat, d);

    // Instanciation de mon code
    TCodeEval mycode(((err==0)?"flowrate -s -r":"flowrate -s -rf"));
    //mycode.setOldTmpDir();
    mycode.addInputFile(&fin);
    mycode.addOutputFile(&fout);

    TMpiRun run(&mycode);
    run.startSlave();
    if (run.onMaster())
    {
        // Define the DataServer
        TDataServer *tds = new TDataServer("tdsflowrate", "Design of Experiments for Flowrate");
        tds->addAttribute(rw);
        tds->addAttribute(r);
        tds->addAttribute(tu);
        tds->addAttribute(tl);
        tds->addAttribute(hu);
        tds->addAttribute(hl);
        tds->addAttribute(l);
        tds->addAttribute(kw);
        tds->fileDataRead("flowrateUniformDesign.dat", kFALSE, kTRUE);

        TLauncher2 lanceur(tds, &run);

        TDataServer *_errFlowrate=NULL;
        if(err!=0)
        {
            _errFlowrate=new URANIE::DataServer::TDataServer("errFlowrate","poeut");
            lanceur.setSaveError(_errFlowrate);
        }
        // resolution
        lanceur.solverLoop();

        tds->exportData("_output_testFlowrateMPI_.dat");
        if(err!=0)
            _errFlowrate->exportData("_output_testFlowrateMPI_err_.dat");

        run.stopSlave();

        delete tds;
    }

}
