#!/usr/bin/env python

import ROOT
import math

def ishigami(x1, x2, x3) :
    """ Ishigami function """
    a = 7.0
    b = 0.1
    
    res = math.sin(x1)
    res += a * math.sin(x2) * math.sin(x2)
    res += b * x3 * x3 * x3 * x3 * math.sin(x1)

    return [res,]

# ------------------------------------------------------------------------------


# input attribute
x1 = ROOT.URANIE.DataServer.TAttribute("x_{1}")
x2 = ROOT.URANIE.DataServer.TAttribute("x_{2}")
x3 = ROOT.URANIE.DataServer.TAttribute("x_{3}")
# output attribute
ishi = ROOT.URANIE.DataServer.TAttribute("Ishigami")

# code
code = ROOT.URANIE.Relauncher.TPythonEval(ishigami)
code.addInput(x1)
code.addInput(x2)
code.addInput(x3)
code.addOutput(ishi)

# run
run = ROOT.URANIE.MpiRelauncher.TMpiRun(code)
run.startSlave()
if (run.onMaster()) :
    # tds
    tds = ROOT.URANIE.DataServer.TDataServer("tds", "tds for Python Ishigami")
    tds.addAttribute(x1)
    tds.addAttribute(x2)
    tds.addAttribute(x3)
    tds.fileDataRead("Ishigami.dat", False, True)

    # launcher
    launch = ROOT.URANIE.Relauncher.TLauncher2(tds, run)
    launch.solverLoop()

    tds.exportData("_output_testIshigamiMPIPython_.dat");

    run.stopSlave()

    ROOT.SetOwnership(run, True)
