using namespace URANIE::DataServer;
using namespace URANIE::Sampler;
using namespace URANIE::Relauncher;

int precode(Double_t *in, Double_t *out)
{
    out[0] = in[0] / in[1];
    return 1;
}

int postcode(Double_t *in, Double_t *out)
{
    out[0] = in[0] * in[0];
    return 1;
}

void test_TLauncherComposeMulti()
{
    // Define the DataServer

    // Add the study attributes ( min, max and nominal values)
    TUniformDistribution coef("coef", 0.1, 1.0), pente("pente", 0.4, 1.6);
    TAttribute delai("delai"), x("x"), diff("diff"), diff2("diff2");

    // pretraitement
#ifdef JIT
    TCJitEval pre(&precode);
#else
    TCIntEval pre(&precode);
#endif
    pre.setInputs(2, &coef, &pente);
    pre.addOutput(&delai);

    // traitement code
    // Set the reference input file and the key for each input attributes
    TFlagScript fin("bidon.py");
    fin.setInputs(2, &delai, "START", &pente, "PENTE");

    // The output file of the code
    TFlatResult fout("bidon.res");
    fout.setOutputs(2, &x, &diff);

    // Instanciation de mon code
    TCodeMulti mycode("python bidon.py > bidon.res");
    //TCodeMulti mycode("python2 bidon.py > bidon.res");
//     mycode.setWorkingDir(gSystem->Getenv("HOME") + TString("/tmp/"));
    mycode.addInputFile(&fin);
    mycode.addOutputFile(&fout);

    // postraitement
#ifdef JIT
    TCJitEval post(&postcode);
#else
    TCIntEval post(&postcode);
#endif
    post.addInput(&diff);
    post.addOutput(&diff2);

    // composition
    TComposeMulti code(&mycode);
    code.addBeforeEval(&pre);
    code.addAfterEval(&post);

    TSequentialRun run(&code);
    //TThreadedRun run(&code, 2);
    //TMpiRun run(&code);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tdsBidon", "Design of experiments for Bidon");
        tds.addAttribute(&coef);
        tds.addAttribute(&pente);

        TSampling sampling(&tds, "lhs", 50);
        sampling.generateSample();

        TLauncher2 lanceur(&tds, &run);

        /* resolution */
        lanceur.solverLoop();

        run.stopSlave();
        /* solution */
        //        tds.getTuple()->Scan("*");
        tds.exportData("_compose_m_.dat");
        tds.draw("diff:diff2");

    }

}
