#ifdef JIT
#include "TAttribute.h"
#include "TUniformDistribution.h"
#include "TCodeEval.h"
#include "TKeyScript.h"
#include "TFlatResult.h"
#include "TSequentialRun.h"
#include "TThreadedRun.h"
#include "TMpiRun.h"
#include "TDataServer.h"
#include "TSampling.h"
#include "TLauncher2.h"
#endif

using namespace URANIE::DataServer;
using namespace URANIE::Sampler;
using namespace URANIE::Relauncher;

void test_TLauncherFlowrate()
{
    // Define the DataServer

    // Add the study attributes ( min, max and nominal values)
    TUniformDistribution rw("rw", 0.05, 0.15), r("r", 100.0, 50000.0), tu("tu",
            63070.0, 115600.0), tl("tl", 63.1, 116.0), hu("hu", 990.0, 1110.0),
            hl("hl", 700.0, 820.0), l("l", 1120.0, 1680.0), kw("kw", 9855.0,
                    12045.0);
    TAttribute yh("yhat"), d("d");

    // Set the reference input file and the key for each input attributes
    //TKeyScript fin("flowrate_input_with_keys.in");
    //TFlatScript fin("flowrate_input_with_values_rows.in");
    TKeyScript fin("flowrate.py");
    fin.setInputs(8, &rw, "Rw", &r, "R", &tu, "Tu", &tl, "Tl", &hu, "Hu", &hl,
            "Hl", &l, "L", &kw, "Kw");
    //fin.setInputs(8, &rw, &r, &tu, &tl, &hu, &hl, &l, &kw);

    // The output file of the code
    //TFlatResult fout("_output_flowrate_withRow_.dat");
    //TFlatResult fout("_output_flowrate_withRow_.dat");
    TKeyResult fout("flowrate.res");
    // The attribute in the output file
    fout.setOutputs(2, &yh, "ŷ", &d, "dist");

    // Instanciation de mon code
    //TCodeEval mycode("flowrate -k >> /dev/null");
    //TCodeEval mycode("flowrate -r >> /dev/null");
#ifdef PYTHON3
    TCodeEval mycode("python2 flowrate.py > flowrate.res");
#else
    TCodeEval mycode("python flowrate.py > flowrate.res");
#endif
    mycode.setWorkingDir(
            gSystem->Getenv("HOME") + TString("/tmp/LanceurUranie/flowrate"));
    mycode.addInputFile(&fin);
    mycode.addOutputFile(&fout);

    //TSequentialRun run(&mycode);
    TThreadedRun run(&mycode, 14);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tdsFlowrate",
                "OATMinMax Design of experiments for Flowrate");
        tds.addAttribute(&rw);
        tds.addAttribute(&r);
        //tds.addAttribute(&tu);
        //tds.addAttribute(&tl);
        tds.addAttribute(&hu);
        tds.addAttribute(&hl);
        tds.addAttribute(&l);
        tds.addAttribute(&kw);

        TSampling sampling(&tds, "lhs", 1000);
        sampling.generateSample();

        TLauncher2 lanceur(&tds, &run);
        lanceur.addConstantValue(&tu, 8000);
        lanceur.addConstantValue(&tl, 80);

        /* resolution */
        lanceur.solverLoop();

        run.stopSlave();
        /* solution */
        //tds.getTuple()->Scan("*");
        tds.exportData("_flow_.dat");
    }

}
