using namespace URANIE::DataServer;
using namespace URANIE::Sampler;
using namespace URANIE::Relauncher;

void test_TLauncherFlowrateSequential()
{
    // Define the DataServer

    // Add the study attributes ( min, max and nominal values)
    TUniformDistribution rw("rw", 0.05, 0.15), r("r", 100.0, 50000.0), tu("tu",
            63070.0, 115600.0), tl("tl", 63.1, 116.0), hu("hu", 990.0, 1110.0),
            hl("hl", 700.0, 820.0), l("l", 1120.0, 1680.0), kw("kw", 9855.0,
                    12045.0);

    TAttribute yhat("yhat"), d("d");

    // Set the reference input file and the key for each input attributes
    TKeyScript fin("flowrate_input_with_keys.in");
    fin.setInputs(8, &rw, "Rw", &r, "R", &tu, "Tu", &tl, "Tl", &hu, "Hu", &hl,
            "Hl", &l, "L", &kw, "Kw");

    // The output file of the code
    TFlatResult fout("_output_flowrate_withRow_.dat");
    fout.setOutputs(2, &yhat, &d);

    // Instanciation de mon code
    TCodeEval mycode("flowrate -s -k");
//     mycode.setWorkingDir(gSystem->Getenv("HOME") + TString("/tmp/"));
    mycode.addInputFile(&fin);
    mycode.addOutputFile(&fout);

    TSequentialRun run(&mycode);
    //TThreadedRun run(&mycode, 2);
    //TMpiRun run(mycode);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tdsflowrateSeq", "Design of experiments for Flowrate");
        tds.addAttribute(&rw);
        tds.addAttribute(&r);
        tds.addAttribute(&tu);
        tds.addAttribute(&tl);
        tds.addAttribute(&hu);
        tds.addAttribute(&hl);
        tds.addAttribute(&l);
        tds.addAttribute(&kw);

        TSampling sampling(&tds, "lhs", 500);
        sampling.generateSample();

        TLauncher2 lanceur(&tds, &run);

        /* resolution */
        lanceur.solverLoop();

        /* solution */
        tds.draw("yhat:rw");

        run.stopSlave();
    }

}
