
/**

clang++ -g -c -std=c++11 -DCOMPIL test_TLauncherMultitypeSocket.C \
 -I$ROOTSYS/include -I$URANIESYS/include

clang++ -std=cxx11 -o runSockTest \
  test_TLauncherMultitypeSocket.o \
  -L$ROOTSYS/lib -L$URANIESYS/lib \
  -lUranieDataServer -lUranieSampler -lUranieRelauncher \
  `root-config --evelibs`

**/

#ifdef COMPIL
#include "TUniformDistribution.h"
#include "TAttribute.h"
#include "TFlatScript.h"
#include "TKeyResult.h"
#include "TCodeEval.h"
#include "TSocketRun.h"
#include "DataServer.h"
#include "TSampling.h"
#include "TLauncher2.h" 
#endif


using namespace URANIE::DataServer ;
using namespace URANIE::Sampler ;
using namespace URANIE::Relauncher ;

void test_TLauncherMultitypeSocket()
{
    /* inputs */
    TUniformDistribution seed("seed", 0, 100000);
    /* outputs */
    TAttribute w1("w1", TAttribute::kString);
    TAttribute w2("w2", TAttribute::kString);
    TAttribute v1("v1", TAttribute::kVector);
    TAttribute v2("v2", TAttribute::kVector);
    TAttribute f1("f1");
    
    /* code etc */
    TFlatScript inputf("multitype_input.dat");
    inputf.addInput(&seed);
    TKeyResult outputf("_output_multitype_mt_Key_.dat");
    outputf.addOutput(&w1, "w1");
    outputf.addOutput(&v1, "v1");
    outputf.addOutput(&v2, "v2");
    outputf.addOutput(&f1, "f1");
    outputf.addOutput(&w2, "w2");
    TCodeEval eval("multitype -mtKey");
    eval.addInputFile(&inputf);
    eval.addOutputFile(&outputf);
    
    /* runner */
    //TSequentialRun run(&eval);
    TSocketRun run(&eval);
    run.startSlave();
    if (run.onMaster()) {
        TDataServer tds("mttds", "Multi Type TDS");
        tds.addAttribute(&seed);
        
        TSampling sampl(&tds, "lhs", 100);
        sampl.generateSample();
        
        TLauncher2 launch(&tds, &run);
        launch.solverLoop();
        
        run.stopSlave();
        
        tds.exportData("mt.res");
    }
}

#ifdef COMPIL
int main()
{
    test_TLauncherMultitypeSocket();
}
#endif
