
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TAttribute.h"
#include "TUniformDistribution.h"
#include "TCJitEval.h"
#include "TThreadedRun.h"
#include "TDataServer.h"
#include "TSampling.h"
//#include "TIter.h"
#include "TLauncher2.h"

using namespace URANIE::DataServer;
using namespace URANIE::Sampler;
using namespace URANIE::Relauncher;

/* user */
#define SLOW 3000
#define TAILLEPOP 250 

/* constantes */
#define NIV 255.0 //niveau de gris
#define TAB 8 // nombre de régions
#define NBGENE 7 // nombre de paramètres

/* modèle */
static const char tgr[TAB * TAB] =
{
/*  0  1  2  3  4  5  6  7 */
0, 1, 1, 1, 0, 0, 1, 1, // 0
        1, 0, 1, 1, 1, 0, 0, 0, // 1
        1, 1, 0, 0, 1, 0, 0, 0, // 2
        1, 1, 0, 0, 0, 1, 1, 0, // 3
        0, 1, 1, 0, 0, 1, 0, 1, // 4
        0, 0, 0, 1, 1, 0, 1, 0, // 5
        1, 0, 0, 1, 0, 1, 0, 1, // 6
        1, 0, 0, 0, 1, 0, 1, 0  // 7
        };

static const char *tangram[TAB] =
{ &tgr[0], &tgr[TAB], &tgr[2 * TAB], &tgr[3 * TAB], &tgr[4 * TAB],
        &tgr[5 * TAB], &tgr[6 * TAB], &tgr[7 * TAB] };

double contraste(int size, double *sol, const char **connexion)
{
    int i, j;
    double min, diff;

    min = NIV;
#ifdef SLOW
    for (int k = 0; k < SLOW; k++)
#endif
        for (i = 0; i < size; i++)
        {
            for (j = i + 1; j < size; j++)
            {
                if (connexion[i][j])
                {
                    diff = fabs(sol[i] - sol[j]);
                    if (diff < min)
                        min = diff;
                }
            }
        }
    return min;
}

/* wrapper */
void mydecode(int size, double *intern, double *user)
{
    int i;

    user[0] = NIV; /* bord blanc imposé */
    for (i = 1; i < size; i++)
        user[i] = intern[i - 1];
}

int fitness(double *in, double *out)
{
    double all[TAB];
    mydecode(TAB, in, all);
    out[0] = contraste(TAB, all, tangram);
    return 1;
}

/* solver */
void test_TLauncherThreaded()
{
    /* init */
    TUniformDistribution z1("zone1", 0., 255.), z2("zone2", 0., 255.), z3(
            "zone3", 0., 255.), z4("zone4", 0., 255.), z5("zone5", 0., 255.),
            z6("zone6", 0., 255.), z7("zone7", 0., 255.);
    TAttribute ct("contrast");

    TCJitEval code(&fitness);
    code.setInputs(NBGENE, &z1, &z2, &z3, &z4, &z5, &z6, &z7);
    code.addOutput(&ct);

    TThreadedRun run(&code, 14);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tds", "tds for vizir test");
        code.addAllInputs(&tds);

        TSampling sampl(&tds, "lhs", TAILLEPOP);
        sampl.generateSample();

        TLauncher2 launch(&tds, &run);

        /* resolution */
        launch.solverLoop();

        run.stopSlave();
        //tds.getTuple()->Scan("*");
        tds.exportData("_thread_.dat");
    }

}
