#include "uranie_commands.h"
#include "uranie_config.h"

#include "TSystem.h"

#include <iostream>
#include <fstream>

std::string UranieTests::uranie_command(const std::string& command, bool redirect_output) {
    std::string scripted_command;
    std::string script_name;

    std::cout << std::endl << "uranie_command: GetWorkingDirectory: " <<
        gSystem->GetWorkingDirectory() << std::endl;

    if (!UNIX_BASH.empty()) {
        script_name = "command.bash";
        std::ofstream script(script_name);
        script << "source " + URANIE_LOADER << std::endl;
        script << command;
        if (redirect_output)
            script << " > command.out";
        script << std::endl << "echo $?";
        if (redirect_output)
            script << std::endl << "tail command.out";
        scripted_command = UNIX_BASH + " ./" + script_name;
    }

    if (!WINDOWS_POWERSHELL.empty()) {
        script_name = "command.ps1";
        std::ofstream script(script_name);
        script << ". " + URANIE_LOADER << std::endl;
        script << command;
        if (redirect_output)
            script << " > command.out";
        script << std::endl << "$?" << std::endl << "$Error";
        if (redirect_output)
            script << std::endl << "Get-Content command.out -Tail 10";
        scripted_command = WINDOWS_POWERSHELL + " ./" + script_name;
    }

    std::cout << script_name << ":" << std::endl;
    std::ifstream script(script_name);
    for(std::string line; getline(script, line);)
        std::cout << line << std::endl;
    std::cout << "scripted_command: [" << scripted_command << "]" << std::endl;

    return scripted_command;
}

std::string UranieTests::command_prefix() {
    std::string prefix;
    if (!WINDOWS_POWERSHELL.empty())
        prefix = "& ";
    return prefix;
}

void UranieTests::removeFile(const char* folder, const char* filename)
{
#ifdef WIN32
    TString fullPath = Form("%s\\%s", folder, filename);
    gSystem->Exec( Form("if exist %s del /f %s", fullPath.Data(), fullPath.Data()) );
#else
    TString fullPath = Form("%s/%s", folder, filename);
    gSystem->Exec( Form("rm -f %s", fullPath.Data()) );
#endif
}

void UranieTests::removeFolder(const char* folder, const char* filename)
{
#ifdef WIN32
    TString fullPath = Form("%s\\%s", folder, filename);
    gSystem->Exec( Form("if exist %s rmdir /S /Q %s", fullPath.Data(), fullPath.Data()) );
#else
    TString fullPath = Form("%s/%s", folder, filename);
    gSystem->Exec( Form("rm -rf %s", fullPath.Data()) );
#endif
}

int UranieTests::areFilesEqual(TString refFileName, TString otherFileName, TString ignoreLinePattern)
{

    // Get content of the files in TStrings
    TString str1, str2;

    std::ifstream ifs1(refFileName, std::ios::in);
    str1.ReadFile(ifs1);
    ifs1.close();

    std::ifstream ifs2(otherFileName, std::ios::in);
    str2.ReadFile(ifs2);
    ifs2.close();

    // Remove the lines to ignore from the strings
    if (ignoreLinePattern.CompareTo(""))
    {
        Ssiz_t i0, i1;
        i0 = str1.Index(ignoreLinePattern);
        i1 = str1.Index("\n", i0);
        str1.Remove(i0, i1 - i0 + 1);

        i0 = str2.Index(ignoreLinePattern);
        i1 = str2.Index("\n", i0);
        str2.Remove(i0, i1 - i0 + 1);
    }

    // Compare the TStrings
    // first case: no modification of the format
    Int_t cmp1 = str1.CompareTo(str2);
    // second case: windows compatibility format modification
    str2 = str2.ReplaceAll("e+0", "e+");
    str2 = str2.ReplaceAll("e-0", "e-");
    Int_t cmp2 = str1.CompareTo(str2);

    return cmp1 * cmp2;
    
}
