#ifndef RESOURCEMODEL_H
#define RESOURCEMODEL_H

#include <cppunit/TestPath.h>
#include <vector>
#include <map>
#include <stack>
#include <string>

#ifdef RESOURCE_USE_WINNTTIMER
#include "WinNtTimer.h"
typedef WinNtTimer Timer;
#else
#include "Timer.h"
#include "Memory.h"
#endif


/// Model that represents test timing.
class ResourceModel
{
public:
    /*! Constructs a ResourceModel object.
     */
    ResourceModel();

    /// Destructor.
    virtual ~ResourceModel();

    void setExpectedTestCount( int count );

    void enterTest( CPPUNIT_NS::Test *test,
                    bool isSuite );

    void exitTest( CPPUNIT_NS::Test *test,
                   bool isSuite );
    
    double totalElapsedTime() const;

    double averageTestCaseTime() const;

    double testTimeFor( CPPUNIT_NS::Test *test ) const;

    double testTimeFor( int testIndex ) const;

    double testMemoryFor( int testIndex ) const;

    static std::string timeStringFor( double time );

    bool isSuite( int testIndex ) const;

    const CPPUNIT_NS::TestPath &testPathFor( int testIndex ) const;

    // -1 is none
    int indexOf( CPPUNIT_NS::Test *test ) const;

    int childCountFor( int testIndex ) const;

    int childAtFor( int testIndex, 
                    int chidIndex ) const;

private:
    struct TestInfo
    {
        CPPUNIT_NS::TestPath m_path;
        Timer m_timer;
        Memory m_memory;
        bool m_isSuite;
        std::vector<int> m_childIndexes;
    };

    /// Prevents the use of the copy constructor.
    ResourceModel( const ResourceModel &other );

    /// Prevents the use of the copy operator.
    void operator =( const ResourceModel &other );

private:
    CPPUNIT_NS::TestPath m_currentPath;
  
    int m_testCaseCount;
    double m_totalTestCaseTime;

    typedef std::map<CPPUNIT_NS::Test *, int> TestToIndexes;
  
    TestToIndexes m_testToIndexes;
    std::stack<int> m_testIndexes;
    std::vector<TestInfo> m_tests;
};


#endif  // RESOURCEMODEL_H
