#include <cppunit/Test.h>
#include <cppunit/tools/XmlDocument.h>
#include <cppunit/tools/XmlElement.h>
#include "ResourceModel.h"
#include "ResourceXmlHook.h"


ResourceXmlHook::ResourceXmlHook( ResourceModel *model )
    : m_model( model )
{
}


ResourceXmlHook::~ResourceXmlHook()
{
}


void ResourceXmlHook::endDocument( CPPUNIT_NS::XmlDocument *document )
{   
}

void ResourceXmlHook::failTestAdded( CPPUNIT_NS::XmlDocument *document,
                               CPPUNIT_NS::XmlElement *testElement,
                               CPPUNIT_NS::Test *test,
                               CPPUNIT_NS::TestFailure *failure )
{
    successfulTestAdded( document, testElement, test );
}


void ResourceXmlHook::successfulTestAdded( CPPUNIT_NS::XmlDocument *document,
                                     CPPUNIT_NS::XmlElement *testElement,
                                     CPPUNIT_NS::Test *test )
{
    int testIndex = m_model->indexOf( test );
    double time = (testIndex >= 0) ? m_model->testTimeFor( testIndex ) : 0.0;
    double memory = (testIndex >= 0) ? m_model->testMemoryFor( testIndex ) : 0.0;
    const CPPUNIT_NS::TestPath &path = m_model->testPathFor( testIndex );
    testElement->addElement( new CPPUNIT_NS::XmlElement( "Time",
                                                         ResourceModel::timeStringFor( time ) ) );
    testElement->addElement( new CPPUNIT_NS::XmlElement( "Memory",
                                                         ResourceModel::timeStringFor( memory ) ) );
}


void ResourceXmlHook::statisticsAdded( CPPUNIT_NS::XmlDocument *document,
                                 CPPUNIT_NS::XmlElement *statisticsElement )
{
    statisticsElement->addElement( 
        new CPPUNIT_NS::XmlElement( "TotalElapsedTime",
                                    ResourceModel::timeStringFor( m_model->totalElapsedTime() ) ) );
    statisticsElement->addElement( 
        new CPPUNIT_NS::XmlElement( "AverageTestCaseTime",
                                    ResourceModel::timeStringFor( m_model->averageTestCaseTime() ) ) );
}
