#!/bin/bash -f

function uranie_add {
    if [[ $# -ne 2  ]]; then
        echo "${FUNCNAME[0]}: Please give PATHS_LIST ONE_PATH."
        return 1
    fi

    PATHS_LIST=$1
    ORIG_PATH=`echo $(eval echo "\\$$PATHS_LIST")`
    ONE_PATH=$2

    if [ -z "$ORIG_PATH" ]; then
        export ${PATHS_LIST}="${ONE_PATH}"
    else
        if [[ ${ONE_PATH} =~ ^/usr/.* ]]; then
            export ${PATHS_LIST}="${ORIG_PATH}:${ONE_PATH}"
        else
            export ${PATHS_LIST}="${ONE_PATH}:${ORIG_PATH}"
        fi
    fi
}


#UNSET TEMPORARY VARIABLES

# OPT++
if test $URANIEOPTSYS
then
    unset URANIEOPTSYS
fi

# JSONCPP
if test $URANIEJSONCPPSYS
then
    unset URANIEJSONCPPSYS
fi

source @ROOTSYS@/bin/thisroot.sh

export URANIESYS=@CMAKE_INSTALL_PREFIX@
export URANIEOPTSYS=@OPT_LIBRARY@
URANIEJSONCPPSYS=@JsonCpp_LIBDIR@
MPILIBRARY=@MPI_LIBRARY_PATH@
URANIENLOPTSYS=@NLOPT_LIBS@

URANIEPYTLIB=@Python_LIBRARIES@

#------------------------------------------------
#--- PATH & LD_LIBRARY_PATH 
#------------------------------------------------

export PATH=${URANIESYS}/bin:$PATH

## PYTHONPATH
if test $PYTHONPATH
then
    export PYTHONPATH=${URANIESYS}/lib:$PYTHONPATH
else
    export PYTHONPATH=${URANIESYS}/lib
fi


if test $LD_LIBRARY_PATH
then
    export LD_LIBRARY_PATH=${URANIESYS}/lib:$LD_LIBRARY_PATH
else
    export LD_LIBRARY_PATH=${URANIESYS}/lib
fi

echo "**************************"
echo "**"
echo "**  Environment variables for Uranie v@URANIE_VERSION@" 
echo "**"
echo "**  URANIESYS : ${URANIESYS}"
echo "**   with"
echo "**      ROOT  : v"`root-config --version`


#IF OPT++ is available
if test $URANIEOPTSYS
then
    uranie_add LD_LIBRARY_PATH ${URANIEOPTSYS}
    echo "**      OPT++ libs  : ${URANIEOPTSYS}"
fi

#IF JSONCPP is available
if test $URANIEJSONCPPSYS
then
    uranie_add LD_LIBRARY_PATH ${URANIEJSONCPPSYS}
    echo "**      JSONCPP libs  : ${URANIEJSONCPPSYS}"
fi

#IF MPI is available
if test $MPILIBRARY
then
    uranie_add PATH @MPIEXEC_PATH@
    uranie_add LD_LIBRARY_PATH ${MPILIBRARY}
    echo "**      MPI libs  : ${MPILIBRARY}"
fi

#IF NLopt is available
if test $URANIENLOPTSYS
then
    if [[ -f $URANIENLOPTSYS ]]; then
        URANIENLOPTSYS=$(dirname "$URANIENLOPTSYS")
    else
        URANIENLOPTSYS=${URANIENLOPTSYS}/lib
    fi
    uranie_add LD_LIBRARY_PATH ${URANIENLOPTSYS}
    echo "**      NLOPT lib   : ${URANIENLOPTSYS}"
fi

if test $URANIEPYTLIB
then
    echo "**      PYTHON lib  : ${URANIEPYTLIB}"
fi

echo "**************************"

if [ -f ${URANIESYS}/share/uranie/index.html ]; then
    echo ""
    echo "Documentation can be seen with a web browser at: "
    echo "file://${URANIESYS}/share/uranie/index.html"
fi

if [ -d ${URANIESYS}/share/uranie/macros ]; then
    echo ""
    echo "Use-case macros can be found here: "
    echo "${URANIESYS}/share/uranie/macros"
fi

#------------------------------------------------
#--- COMPILATION SHORTCUT 
#------------------------------------------------
export URANIECPPFLAG="`root-config --cflags` -I${URANIESYS}/include"
ULD="-lUranie"
export URANIELDFLAG=" `root-config --libs` -L${URANIESYS}/lib ${ULD}DataServer ${ULD}Sampler ${ULD}Launcher ${ULD}Relauncher ${ULD}RelauncherMPI  ${ULD}Modeler ${ULD}Sensitivity ${ULD}Reoptimizer ${ULD}Optimizer ${ULD}Reliability ${ULD}XMLProblem "
