//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTDataServer.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TDataServer
 
 Created: nov 21 07:59:24 2007
 
 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTXMLProblem.h"
#include "uranie_commands.h"

// ROOT
#include "TString.h"

#include <iostream>
using namespace std;

TXMLProblemTest::TXMLProblemTest():
  txml1(NULL), tds1(NULL)
{
    _workingTestPath = gSystem->pwd(); 
}

TXMLProblemTest::~TXMLProblemTest()
{
}

void TXMLProblemTest::setUp()
{
//  if ( txml1 )
//    delete txml1;
//  txml1=NULL;
//  if ( tds1 )
//    delete tds1;
//  tds1 = NULL;
//  cout << endl << endl;
}

void TXMLProblemTest::tearDown()
{
//  if ( tds1 )
//    delete tds1;
    if ( txml1 )
	delete txml1;
  
    UranieTests::removeFile(_workingTestPath.Data(),"regline.C");
    UranieTests::removeFile(_workingTestPath.Data(),"_dataserver_.root");
    UranieTests::removeFile(_workingTestPath.Data(),"_exported_.dat");
    UranieTests::removeFile(_workingTestPath.Data(),"_exported_.root");
    UranieTests::removeFile(_workingTestPath.Data(),"_flowrateModel_sampler_lhs_.dat");
    UranieTests::removeFile(_workingTestPath.Data(),"flowrateUniformDesign.root");
    UranieTests::removeFile(_workingTestPath.Data(),"housing.root");
    UranieTests::removeFile(_workingTestPath.Data(),"_lhs_.dat");
    UranieTests::removeFile(_workingTestPath.Data(),"_neuralNetwork_CR_.C");
    UranieTests::removeFile(_workingTestPath.Data(),"_neuralNetwork_MinusOneOne_.C");
    UranieTests::removeFile(_workingTestPath.Data(),"_neuralNetwork_ZeroOne_.C");
    UranieTests::removeFile(_workingTestPath.Data(),"OPT_DEFAULT.out");
    UranieTests::removeFile(_workingTestPath.Data(),"sampler_LHS_10000.root");
    UranieTests::removeFile(_workingTestPath.Data(),"_steamplant_ANN_CR_.dat");
    UranieTests::removeFile(_workingTestPath.Data(),"_steamplant_ANN_CR_.root");
    UranieTests::removeFile(_workingTestPath.Data(),"_steamplant_ANN_MinusOneOne_.dat");
    UranieTests::removeFile(_workingTestPath.Data(),"_steamplant_ANN_MinusOneOne_.root");
    UranieTests::removeFile(_workingTestPath.Data(),"_steamplant_ANN_ZeroOne_.dat");
    UranieTests::removeFile(_workingTestPath.Data(),"_steamplant_ANN_ZeroOne_.root");
    UranieTests::removeFile(_workingTestPath.Data(),"steamplant.root");
    UranieTests::removeFile(_workingTestPath.Data(),"titi.dat");
    UranieTests::removeFile(_workingTestPath.Data(),"_tstOptCalNips_MEDVhat_nh005.out");
    UranieTests::removeFile(_workingTestPath.Data(),"_tstOptCalNips_MEDVhat_nh006.out");
    UranieTests::removeFile(_workingTestPath.Data(),"_tstOptCalNips_MEDVhat_nh008.out");
}


////////////////////////////////
///                          ///                            
///         Tests            ///
///                          ///
////////////////////////////////
void TXMLProblemTest::testEmptyConstructor()
{
  txml1 = new URANIE::XMLProblem::TXMLProblem();
  CPPUNIT_ASSERT(strcmp( txml1->GetName(), "_nom_") == 0);
  CPPUNIT_ASSERT(strcmp( txml1->GetTitle(), "_title_") == 0);

  CPPUNIT_ASSERT_EQUAL(0, txml1->getLevelDebug ());
  txml1->setLevelDebug(2);
  CPPUNIT_ASSERT_EQUAL(2, txml1->getLevelDebug ());

  CPPUNIT_ASSERT(txml1->getDataServer() == NULL);
}

void TXMLProblemTest::testXMLfile()
{
    txml1 = new URANIE::XMLProblem::TXMLProblem("launchFunctionSampling.xml");
    
    CPPUNIT_ASSERT_EQUAL(5, txml1->getLevelDebug ());
    txml1->setLevelDebug(2);
    CPPUNIT_ASSERT_EQUAL(2, txml1->getLevelDebug ());
    
    CPPUNIT_ASSERT(txml1->getLog() == false );
    txml1->setLog();
    CPPUNIT_ASSERT(txml1->getLog() == true );
    txml1->unsetLog();
    CPPUNIT_ASSERT(txml1->getLog() == false );
    txml1->changeLog();
    CPPUNIT_ASSERT(txml1->getLog() == true );
    txml1->changeLog();
    CPPUNIT_ASSERT(txml1->getLog() == false );

    tds1 = txml1->getDataServer();

    CPPUNIT_ASSERT_EQUAL(11, tds1->getNAttributes ());
    CPPUNIT_ASSERT_EQUAL(1000, tds1->getNPatterns());
    tds1->computeStatistic("");

    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("rw")->getMean(), 0.100, 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("rw")->getStd(), 0.028, 0.01);
    CPPUNIT_ASSERT(tds1->isAttribute("zz"));
    CPPUNIT_ASSERT(tds1->isAttribute("flowrateModel"));
    CPPUNIT_ASSERT(tds1->isAttribute("y"));
    CPPUNIT_ASSERT(strcmp(tds1->getAttribute("zz")->getUnit(), "") == 0);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("zz")->getMean(), 0.350, 0.001);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("zz")->getStd(), 0.147, 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getValue("flowrateModel", 0),
            tds1->getValue("y", 0), DBL_PRECISION_COMPARE);
}

void TXMLProblemTest::testSampling()
{
    txml1 = new URANIE::XMLProblem::TXMLProblem("sampling.xml");
    tds1 = txml1->getDataServer();


    CPPUNIT_ASSERT_EQUAL(8, tds1->getNAttributes());
    CPPUNIT_ASSERT_EQUAL(10000, tds1->getNPatterns());

    CPPUNIT_ASSERT(tds1->isAttribute("rw"));
    CPPUNIT_ASSERT(tds1->isAttribute("r"));
    CPPUNIT_ASSERT(tds1->isAttribute("ystar") == kFALSE);
    CPPUNIT_ASSERT(tds1->isAttribute("flowrateModel") == kFALSE);

    tds1->computeStatistic("");

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.100, tds1->getAttribute("rw")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.05, tds1->getAttribute("rw")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(8030.0, tds1->getAttribute("r")->getMean(), 1.00);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(11691.0, tds1->getAttribute("r")->getStd(), 1.00);


    CPPUNIT_ASSERT(txml1->getLevelDebug () > 2);
    TString sTotoFile = Form("sampler_LHS_%d.dat", tds1->getNPatterns());
    TString sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);

    URANIE::DataServer::TDataServer tdsLoad;
    tdsLoad.fileDataRead(sFindFile);

    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNAttributes(), tds1->getNAttributes());
    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNPatterns(), tds1->getNPatterns());

    CPPUNIT_ASSERT(tdsLoad.isAttribute("rw"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("r"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("flowrateModel") == kFALSE);
    CPPUNIT_ASSERT(tdsLoad.isAttribute("ystar") == kFALSE);

    txml1->run();
    sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);

    URANIE::DataServer::TDataServer tdsLoadRun;
    tdsLoadRun.fileDataRead(sFindFile);

    CPPUNIT_ASSERT_EQUAL(tdsLoadRun.getNAttributes(), tds1->getNAttributes());
    CPPUNIT_ASSERT_EQUAL(tdsLoadRun.getNPatterns(), tds1->getNPatterns());

    CPPUNIT_ASSERT(tdsLoadRun.isAttribute("rw"));
    CPPUNIT_ASSERT(tdsLoadRun.isAttribute("r"));
    CPPUNIT_ASSERT(tdsLoadRun.isAttribute("flowrateModel") == kFALSE);
    CPPUNIT_ASSERT(tdsLoadRun.isAttribute("ystar") == kFALSE);

    UranieTests::removeFile("",sTotoFile);
}

void TXMLProblemTest::testUrl()
{
    txml1 = new URANIE::XMLProblem::TXMLProblem("urlWithoutLauncher.xml");
    tds1 = txml1->getDataServer();

    URANIE::DataServer::TDataServer tdsInput;
    tdsInput.fileDataRead(tds1->getDataFileName());

    CPPUNIT_ASSERT_EQUAL(9, tds1->getNAttributes ());
    CPPUNIT_ASSERT_EQUAL(32, tds1->getNPatterns());

    CPPUNIT_ASSERT(tds1->isAttribute("rw"));
    CPPUNIT_ASSERT(tds1->isAttribute("ystar"));
    CPPUNIT_ASSERT(tds1->isAttribute("flowrateModel") == kFALSE);

    tds1->computeStatistic("");

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.100, tds1->getAttribute("rw")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.031, tds1->getAttribute("rw")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(76.452, tds1->getAttribute("ystar")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(45.815, tds1->getAttribute("ystar")->getStd(), 0.001);
}


void TXMLProblemTest::testUrlWithLauncher()
{
    txml1 = new URANIE::XMLProblem::TXMLProblem("urlWithLauncher.xml");
    tds1 = txml1->getDataServer();

    URANIE::DataServer::TDataServer tdsInput;
    tdsInput.fileDataRead(tds1->getDataFileName());

    CPPUNIT_ASSERT_EQUAL(10, tds1->getNAttributes ());
    CPPUNIT_ASSERT_EQUAL(32, tds1->getNPatterns());

    CPPUNIT_ASSERT(tds1->isAttribute("rw"));
    CPPUNIT_ASSERT(tds1->isAttribute("ystar"));
    CPPUNIT_ASSERT(tds1->isAttribute("flowrateModel"));

    tds1->computeStatistic("");

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.100, tds1->getAttribute("rw")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.031, tds1->getAttribute("rw")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(76.452, tds1->getAttribute("ystar")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(45.815, tds1->getAttribute("ystar")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(76.452, tds1->getAttribute("flowrateModel")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(45.811, tds1->getAttribute("flowrateModel")->getStd(), 0.001);

    TString sTotoFile = "_exported_.dat";
    TString sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);

    URANIE::DataServer::TDataServer tdsLoad;
    tdsLoad.fileDataRead(sFindFile);

    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNAttributes(), tds1->getNAttributes ());
    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNPatterns(), tds1->getNPatterns ());

    CPPUNIT_ASSERT(tdsLoad.isAttribute("rw"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("flowrateModel"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("ystar"));
}
void TXMLProblemTest::testModelerSelectedPatterns()
{
    txml1 = new URANIE::XMLProblem::TXMLProblem("steamplantSelectedPatterns.xml");
    tds1 = txml1->getDataServer();

    CPPUNIT_ASSERT_EQUAL(15, tds1->getNAttributes());
    CPPUNIT_ASSERT_EQUAL(314,  tds1->getNPatterns());
}

void TXMLProblemTest::testModelerLinearRegression()
{
    txml1 = new URANIE::XMLProblem::TXMLProblem("steamplantLinearRegression.xml");
    tds1 = txml1->getDataServer();

    URANIE::DataServer::TDataServer tdsInput;
    tdsInput.fileDataRead(tds1->getDataFileName());

    CPPUNIT_ASSERT_EQUAL(tdsInput.getNAttributes()+2, tds1->getNAttributes ());
    CPPUNIT_ASSERT_EQUAL(tdsInput.getNPatterns(), tds1->getNPatterns());

    CPPUNIT_ASSERT(tds1->isAttribute("x1"));
    CPPUNIT_ASSERT(tds1->isAttribute("x2"));
    CPPUNIT_ASSERT(tds1->isAttribute("x10"));
    CPPUNIT_ASSERT(tds1->isAttribute("x1hat"));
    CPPUNIT_ASSERT(tds1->isAttribute("RL"));

    tds1->computeStatistic("");
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.4424, tds1->getAttribute("x2")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8169, tds1->getAttribute("x2")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.424, tds1->getAttribute("x1")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.6306, tds1->getAttribute("x1")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.424, tds1->getAttribute("x1hat")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.567, tds1->getAttribute("x1hat")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("x1hat")->getMean(), tds1->getAttribute("RL")->getMean(), 0.01);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(tds1->getAttribute("x1hat")->getStd(), tds1->getAttribute("RL")->getStd(), 0.001);

    TString sTotoFile = "regline.C";
    TString sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);
}


void TXMLProblemTest::testModelerNeuralNetworkMinusOneOne()
{
    txml1 = new URANIE::XMLProblem::TXMLProblem("steamplantNeuralNetworksMinusOneOne.xml");
    tds1 = txml1->getDataServer();

    URANIE::DataServer::TDataServer tdsInput;
    tdsInput.fileDataRead(txml1->getDataServer()->getDataFileName());

    CPPUNIT_ASSERT_EQUAL(tdsInput.getNAttributes()+1, tds1->getNAttributes());
    CPPUNIT_ASSERT_EQUAL(tdsInput.getNPatterns(), tds1->getNPatterns());

    CPPUNIT_ASSERT(tds1->isAttribute("CRIM"));
    CPPUNIT_ASSERT(tds1->isAttribute("ZN"));
    CPPUNIT_ASSERT(tds1->isAttribute("LSTAT"));
    CPPUNIT_ASSERT(tds1->isAttribute("MEDV"));
    CPPUNIT_ASSERT(tds1->isAttribute("x1hat") == kFALSE);
    CPPUNIT_ASSERT(tds1->isAttribute("yhatANNMOO"));
    CPPUNIT_ASSERT(tds1->isAttribute("RL") == kFALSE);

    tds1->computeStatistic("");
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.61352355, tds1->getAttribute("CRIM")->getMean(), 1.0e-7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(8.60154510, tds1->getAttribute("CRIM")->getStd(),  1.0e-7);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(22.53280632, tds1->getAttribute("MEDV")->getMean(), 1.0e-7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 9.19710408, tds1->getAttribute("MEDV")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(22.53280632, tds1->getAttribute("yhatANNMOO")->getMean(), 0.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 9.19710408, tds1->getAttribute("yhatANNMOO")->getStd(),  1.5);

    TString sTotoFile = "_neuralNetwork_MinusOneOne_.C";
    TString sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);

    sTotoFile = "_steamplant_ANN_MinusOneOne_.dat";
    sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);

    URANIE::DataServer::TDataServer tdsLoad;
    tdsLoad.fileDataRead(sFindFile);

    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNAttributes(), tds1->getNAttributes ());
    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNPatterns(), tds1->getNPatterns ());

    CPPUNIT_ASSERT(tdsLoad.isAttribute("CRIM"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("ZN"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("LSTAT"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("MEDV"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("x1hat") == kFALSE);
    CPPUNIT_ASSERT(tdsLoad.isAttribute("yhatANNMOO"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("RL") == kFALSE);
}

void TXMLProblemTest::testModelerNeuralNetworkZeroOne()
{
    txml1 = new URANIE::XMLProblem::TXMLProblem("steamplantNeuralNetworksZeroOne.xml");
    tds1 = txml1->getDataServer();

    URANIE::DataServer::TDataServer tdsInput;
    tdsInput.fileDataRead(txml1->getDataServer()->getDataFileName());

    CPPUNIT_ASSERT_EQUAL(tdsInput.getNAttributes()+1, tds1->getNAttributes());
    CPPUNIT_ASSERT_EQUAL(tdsInput.getNPatterns(), tds1->getNPatterns());

    CPPUNIT_ASSERT(tds1->isAttribute("CRIM"));
    CPPUNIT_ASSERT(tds1->isAttribute("ZN"));
    CPPUNIT_ASSERT(tds1->isAttribute("LSTAT"));
    CPPUNIT_ASSERT(tds1->isAttribute("MEDV"));
    CPPUNIT_ASSERT(tds1->isAttribute("x1hat") == kFALSE);
    CPPUNIT_ASSERT(tds1->isAttribute("yhatANNZO"));
    CPPUNIT_ASSERT(tds1->isAttribute("RL") == kFALSE);

    tds1->computeStatistic("");
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.61352355, tds1->getAttribute("CRIM")->getMean(), 1.0e-7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(8.60154510, tds1->getAttribute("CRIM")->getStd(),  1.0e-7);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(22.53280632, tds1->getAttribute("MEDV")->getMean(), 1.0e-7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 9.19710408, tds1->getAttribute("MEDV")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(22.53280632, tds1->getAttribute("yhatANNZO")->getMean(), 0.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 9.19710408, tds1->getAttribute("yhatANNZO")->getStd(),  1.0);

    TString sTotoFile = "_neuralNetwork_ZeroOne_.C";
    TString sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);

    sTotoFile = "_steamplant_ANN_ZeroOne_.dat";
    sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);

    URANIE::DataServer::TDataServer tdsLoad;
    tdsLoad.fileDataRead(sFindFile);

    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNAttributes(), tds1->getNAttributes ());
    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNPatterns(), tds1->getNPatterns ());

    CPPUNIT_ASSERT(tdsLoad.isAttribute("CRIM"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("ZN"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("LSTAT"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("MEDV"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("x1hat") == kFALSE);
    CPPUNIT_ASSERT(tdsLoad.isAttribute("yhatANNZO"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("RL") == kFALSE);
}

void TXMLProblemTest::testModelerNeuralNetworkCR()
{
    txml1 = new URANIE::XMLProblem::TXMLProblem("steamplantNeuralNetworksCR.xml");
    tds1 = txml1->getDataServer();

    URANIE::DataServer::TDataServer tdsInput;
    tdsInput.fileDataRead(tds1->getDataFileName());

    CPPUNIT_ASSERT_EQUAL(tdsInput.getNAttributes()+1, tds1->getNAttributes());
    CPPUNIT_ASSERT_EQUAL(tdsInput.getNPatterns(), tds1->getNPatterns());

    CPPUNIT_ASSERT(tds1->isAttribute("CRIM"));
    CPPUNIT_ASSERT(tds1->isAttribute("ZN"));
    CPPUNIT_ASSERT(tds1->isAttribute("LSTAT"));
    CPPUNIT_ASSERT(tds1->isAttribute("MEDV"));
    CPPUNIT_ASSERT(tds1->isAttribute("x1hat") == kFALSE);
    CPPUNIT_ASSERT(tds1->isAttribute("yhatANNCR"));
    CPPUNIT_ASSERT(tds1->isAttribute("RL") == kFALSE);

    tds1->computeStatistic("");
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.61352355, tds1->getAttribute("CRIM")->getMean(), 1.0e-7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(8.60154510, tds1->getAttribute("CRIM")->getStd(),  1.0e-7);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(22.53280632, tds1->getAttribute("MEDV")->getMean(), 1.0e-7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 9.19710408, tds1->getAttribute("MEDV")->getStd(), 0.001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(22.53280632, tds1->getAttribute("yhatANNCR")->getMean(), 0.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 9.19710408, tds1->getAttribute("yhatANNCR")->getStd(),  1.5);

    TString sTotoFile = "_neuralNetwork_CR_.C";
    TString sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);

    sTotoFile = "_steamplant_ANN_CR_.dat";
    sFindFile = gSystem->FindFile(gSystem->pwd(), sTotoFile);
    //cout << " ****** toto[" << sTotoFile << "] sFindFile[" << sFindFile << "]" << endl;
    CPPUNIT_ASSERT(sFindFile.Length() > 0);

    URANIE::DataServer::TDataServer tdsLoad;
    tdsLoad.fileDataRead(sFindFile);

    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNAttributes(), tds1->getNAttributes ());
    CPPUNIT_ASSERT_EQUAL(tdsLoad.getNPatterns(), tds1->getNPatterns ());

    CPPUNIT_ASSERT(tdsLoad.isAttribute("CRIM"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("ZN"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("LSTAT"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("MEDV"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("x1hat") == kFALSE);
    CPPUNIT_ASSERT(tdsLoad.isAttribute("yhatANNCR"));
    CPPUNIT_ASSERT(tdsLoad.isAttribute("RL") == kFALSE);

//    delete txml1;
//    txml1 = NULL;
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////
void TXMLProblemTest::throwExceptionConstructor()
{
  // when the DTD file does not exist
  CPPUNIT_ASSERT_THROW(
        URANIE::XMLProblem::TXMLProblem("samplingNotExistDTD.xml"),
        URANIE::Exceptions::UErrorExceptions);

  // when the ROOT element is not Problem
  CPPUNIT_ASSERT_THROW(
        URANIE::XMLProblem::TXMLProblem("samplingBadROOTElement.xml"),
        URANIE::Exceptions::UErrorExceptions);
}

// End Of File
