//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2023 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTXMLProblem.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test File of the class URANIE::XMLProblem::TXMLProblem
 
 Created: wen nov 21 07:52:44 2007
 
 */
#ifndef CPPUNITEST_TESTTXMLPROBLEM_H
#define CPPUNITEST_TESTTXMLPROBLEM_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

/*!
 @class TXMLProblemTest
 @brief Unit tests for TDataServer.
 
 The units tests are :
 \li Names \ref testNames
 \li Title \ref testTitles
 
 */

// Uranie
#include "../TXMLProblem.h"
#include "TDataServer.h"

class TXMLProblemTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TXMLProblemTest);
    CPPUNIT_TEST (testEmptyConstructor);
    CPPUNIT_TEST (testXMLfile);
    CPPUNIT_TEST (testSampling);
    CPPUNIT_TEST (testUrl);
    CPPUNIT_TEST (testUrlWithLauncher);
    CPPUNIT_TEST (testModelerSelectedPatterns);
    CPPUNIT_TEST (testModelerLinearRegression);
#ifdef WITH_OPT
    CPPUNIT_TEST (testModelerNeuralNetworkZeroOne);
    CPPUNIT_TEST (testModelerNeuralNetworkMinusOneOne);
    CPPUNIT_TEST (testModelerNeuralNetworkCR);
#endif
    CPPUNIT_TEST (throwExceptionConstructor);
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TXMLProblemTest();

    //! Default destructor
    virtual ~TXMLProblemTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test of the XML Problem empty constructor
    void testEmptyConstructor();
    //! Test of the first XML file
    void testXMLfile();
    //! Test of the sampling node
    void testSampling();
    //! Test of the url node alone
    void testUrl();
    //! Test of the url node in DataDictionary with Launcher
    void testUrlWithLauncher();
    //! Test of the Modeler with selected patterns
    void testModelerSelectedPatterns();
    //! Test of the Linear Regression
    void testModelerLinearRegression();
    //! Test of the Neural Network
    void testModelerNeuralNetworkMinusOneOne();
    //! Test of the Neural Network
    void testModelerNeuralNetworkZeroOne();
    //! Test of the Neural Network CR
    void testModelerNeuralNetworkCR();
    //! Throws an exception when constructor (DTD File does not exist + bad root element)
    void  throwExceptionConstructor();
    //@}

private:
    TString _workingTestPath;
    URANIE::XMLProblem::TXMLProblem *txml1;
    URANIE::DataServer::TDataServer *tds1;
};

#endif  // CPPUNITEST_TDATASERVER_H
